package gov.va.med.bhie.common;

import com.sun.jersey.core.util.MultivaluedMapImpl;
import gov.va.med.bhie.beans.QueryBean;
import gov.va.med.foundations.utilities.ARMLog;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.core.MultivaluedMap;
import java.util.Calendar;

public class QueryMapFactory {
    /**
     * Logger.
     */
    private static final Logger LOGGER =
            LoggerFactory.getLogger(QueryMapFactory.class);
    
    /**
     * Creates a query map for use with BHIERestServiceHandler.
     * 
     * @param version BDA call version
     * @param queryBean BHIE Relay QueryBean
     * @param clinicalDomainLoincCode Data type LOINC code. (required)

     * @return Query map.
     */
    public static MultivaluedMap<String, String> create(String version, QueryBean queryBean,
                                                        ClinicalDomainLoincCode clinicalDomainLoincCode) {
        MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();

        queryParams.add("patientid", queryBean.getPatientId());
        queryParams.add("loinc", clinicalDomainLoincCode.getLoincCode());

        queryParams.add("useonepass", "true");

        queryParams.add("version", version);

        if ("4".equals(version)) {
            if (queryBean.getStartDate() != null) {
                queryParams.add("startdate", formatQueryDateStr4(queryBean.getStartDate()));
            }
    
            if (queryBean.getEndDate() != null) {
                queryParams.add("enddate", formatQueryDateStr4(queryBean.getEndDate()));
            }
        } else {
            if (queryBean.getStartDate() != null) {
                queryParams.add("startdate", formatQueryDateStr(queryBean.getStartDate()));
            }
    
            if (queryBean.getEndDate() != null) {
                queryParams.add("enddate", formatQueryDateStr(queryBean.getEndDate()));
            }
        }
        if (queryBean.isForceRefresh()) {
            queryParams.add("refresh", "true");
        } else {
            queryParams.add("refresh", "false");
        }
        queryParams.add("userid", queryBean.getUserId());
        queryParams.add("userfacilityid", AppConfig.getInstance().getUserFacilityId());

        if(StringUtils.isNotEmpty(queryBean.getClassCodes())){
            queryParams.add("classCodes", queryBean.getClassCodes());
        }
        if(StringUtils.isNotEmpty(queryBean.getTypeCodes())){
            queryParams.add("typeCodes", queryBean.getTypeCodes());
        }
        String excludeProp = AppConfig.getInstance().getProperties().getProperty("excludedsources." + clinicalDomainLoincCode.getLoincCode());
        if (excludeProp != null && excludeProp.trim().length() != 0) {
            queryParams.add("excludedsources", excludeProp.trim());            
            LOGGER.info("Override exclude for {}: {}", clinicalDomainLoincCode.getLoincCode(), ARMLog.cleanString(excludeProp));
        } else {
            if (queryBean.getExcludedSources() != null) {
                StringBuilder excludedSourcesSB = new StringBuilder();
                int excludedSourcesSize = queryBean.getExcludedSources().size();
                for (int i = 0; i < excludedSourcesSize; i++) {
                    String source = queryBean.getExcludedSources().get(i);
                    excludedSourcesSB.append(source);
                    if (i < (excludedSourcesSize - 1)) {
                        excludedSourcesSB.append(",");
                    }
                }
    
                queryParams.add("excludedsources", excludedSourcesSB.toString());
            }
        }
        
        String includeProp = AppConfig.getInstance().getProperties().getProperty("includedsources." + clinicalDomainLoincCode.getLoincCode());
        if (includeProp != null && includeProp.trim().length() != 0) {
            queryParams.add("includedsources", includeProp.trim());
            LOGGER.info("Override include for {}: {}", clinicalDomainLoincCode.getLoincCode(), ARMLog.cleanString(includeProp));
        } else {
            if (queryBean.getIncludedSources() != null) {
                StringBuilder includedSourcesSB = new StringBuilder();
                int includedSroucesSize = queryBean.getIncludedSources().size();
                for (int i = 0; i < includedSroucesSize; i++) {
                    String source = queryBean.getIncludedSources().get(i);
                    includedSourcesSB.append(source);
                    if (i < (includedSroucesSize - 1)) {
                        includedSourcesSB.append(",");
                    }
                }
    
                queryParams.add("includedsources", includedSourcesSB.toString());
            }
        }
        return queryParams;
    }

    /**
     * Helper method that formats a Calendar date object into a date query string.
     * @param date Calendar object for format.
     * @return Date query string.
     */
    private static String formatQueryDateStr(Calendar date) {
        StringBuilder rtc = new StringBuilder();
        
        String month = Integer.toString(date.get(Calendar.MONTH) + 1);
        if (month.length() == 1) {
            rtc.append('0');
        }
        rtc.append(month);
        rtc.append('-');
        
        String day = Integer.toString(date.get(Calendar.DAY_OF_MONTH));
        if (day.length() == 1) {
            rtc.append('0');
        }
        rtc.append(day);
        rtc.append('-');

        rtc.append(date.get(Calendar.YEAR));
        
        return rtc.toString();
    }

    /**
     * Helper method that formats a Calendar date object into a date query string.
     * @param date Calendar object for format.
     * @return Date query string.
     */
    private static String formatQueryDateStr4(Calendar date) {
        StringBuilder rtc = new StringBuilder();
        rtc.append(date.get(Calendar.YEAR));
        
        String month = Integer.toString(date.get(Calendar.MONTH) + 1);
        if (month.length() == 1) {
            rtc.append('0');
        }
        rtc.append(month);
        
        String day = Integer.toString(date.get(Calendar.DAY_OF_MONTH));
        if (day.length() == 1) {
            rtc.append('0');
        }
        rtc.append(day);

        return rtc.toString();
    }
}
