/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */

package gov.va.med.bhie.dataservice;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import gov.va.med.bhie.beans.NoteImage;
import gov.va.med.bhie.common.AppConfig;
import gov.va.med.bhie.resultcollection.ResultCollection;
import gov.va.med.bhie.resultcollection.V4ResultCollection;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import javax.ws.rs.core.MultivaluedMap;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class BHIERestServiceHandler {
    /**
     * Logger.
     */
    private static final Logger LOGGER =
            Logger.getLogger(BHIERestServiceHandler.class.getName());

    private static final String REST_ASYNC_QUERY_PATH = AppConfig.getInstance().getRestAsyncQueryPath();
    private static final String REST_ASYNC_COMET_PATH = AppConfig.getInstance().getRestAsyncCometPath();
    private static final String REST_ASYNC_QUERY_PATH3 = AppConfig.getInstance().getRestAsyncQueryPath3();
    private static final String REST_ASYNC_COMET_PATH3 = AppConfig.getInstance().getRestAsyncCometPath3();
    private static final String REST_ASYNC_QUERY_PATH4 = AppConfig.getInstance().getRestAsyncQueryPath4();
    private static final String REST_ASYNC_COMET_PATH4 = AppConfig.getInstance().getRestAsyncCometPath4();
    private static final String REST_ASYNC_QUERY_PATHVLER = AppConfig.getInstance().getRestAsyncQueryPathVLER();
    private static final String REST_ASYNC_COMET_PATHVLER = AppConfig.getInstance().getRestAsyncCometPathVLER();

    private static final int CONNECTION_TIMEOUT_MS = AppConfig.getInstance().getRestConnectionTimeoutMS();
    private static final int READ_TIMEOUT_MS = AppConfig.getInstance().getRestReadTimeoutMS();
    private static final int POLLER_TIMEOUT_MS = AppConfig.getInstance().getRestPollerTimeoutMS();
    private static final int DOWNLOAD_TIMEOUT_MS = AppConfig.getInstance().getDownloadTimeoutMS();

    private static final BHIERestServiceHandler INSTANCE =
            new BHIERestServiceHandler(AppConfig.getInstance().getBhieEndpoint());

    private static final BHIERestServiceHandler VLERINSTANCE =
            new BHIERestServiceHandler(AppConfig.getInstance().getVlerEndpoint());

    private static final String INVALID_CHARS = AppConfig.getInstance().getInvalidChars();

    private String endpoint;
    private ObjectMapper objectMapper;
    private Client restClient;
    private Client docClient;

    /**
     * Creates a new BHIEDataServiceHandler instance.
     *
     * @param endpoint  BHIEDataService end point
     */
    public BHIERestServiceHandler(String endpoint) {
        this.endpoint = endpoint;

        if ('/' != endpoint.charAt(endpoint.length() - 1)) {
            this.endpoint += "/";
        }

        objectMapper = new ObjectMapper();

        //allow unknown data fields
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

        this.restClient = initRestClient(CONNECTION_TIMEOUT_MS, READ_TIMEOUT_MS);
        this.docClient  = initRestClient(CONNECTION_TIMEOUT_MS, DOWNLOAD_TIMEOUT_MS);
    }

    public static BHIERestServiceHandler getInstance() {
        return INSTANCE;
    }

    public static BHIERestServiceHandler getVlerInstance() {
        return VLERINSTANCE;
    }

    /**
     * Initializes REST client.
     *
     * @param connectionTimeout  REST client connection timeout
     * @param readTimeout  REST client read timeout
     * @return Jersey REST Client
     */
    private Client initRestClient(Integer connectionTimeout, Integer readTimeout) {
        Client client = Client.create();
        client.setConnectTimeout(connectionTimeout);
        client.setReadTimeout(readTimeout);

        return client;
    }

    /**
     * Sends a query request to BHIE REST interface and
     * then polls the BHIE REST comet interface with the provided queryID.
     *
     * When the long poller receives valid response, it will bind the json response to a ResultCollection object
     *  and then add the ResultCollection to a List.
     *
     * @param queryParams REST query parameter Map (keys: userId, patientId, startDate, endDate, onepass, loinc)
     * @param resultCollectionCls The ResultCollection class (json response will be bound to an object of this class),
     *        (see resultcollection package, an example of possible ResultCollection classes include
     *        AllergyResultCollection, MedicationResultCollection, etc.)
     * @return A List of ResultCollection objects.
     * @throws BHIERestServiceHandlerException If there is a failure.
     */
    @SuppressWarnings("unchecked")
    public List<ResultCollection> queryAndPollRESTInterface(MultivaluedMap<String, String> queryParams, Class resultCollectionCls) throws BHIERestServiceHandlerException {
        List<ResultCollection> rtc;
        if (queryParams == null) {
            throw new IllegalArgumentException("queryParams cannot be null");
        }
        // Fortify false positive, fortify says null check is always false.  If null is passed
        // in it will be true.
        if (resultCollectionCls == null) {
            throw new IllegalArgumentException("resultCollectionCls cannot be null");
        }

        try {
            String version = queryParams.get("version").get(0);
            if ("VLER".equalsIgnoreCase(version)) {
                String vlerendpoint = AppConfig.getInstance().getProperties().getProperty("vlerendpoint", endpoint + REST_ASYNC_QUERY_PATHVLER);
                if ('/' != vlerendpoint.charAt(vlerendpoint.length() - 1)) {
                    vlerendpoint += "/";
                }
                String path = queryParams.get("userid").get(0);
                path += "/" + queryParams.get("patientid").get(0);
                path += "/" + queryParams.get("loinc").get(0);
                path += "?requestSource=" + AppConfig.getInstance().getUserFacilityId();
                if ("true".equalsIgnoreCase(queryParams.get("refresh").get(0))) {
                    path += "&refresh=true";
                }
                WebResource queryResource = restClient.resource(vlerendpoint + path);

                String queryResp = queryResource.get(String.class);

                Map queryRespMap = objectMapper.readValue(queryResp, Map.class);

                if (queryRespMap.get("error") != null) {
                    throw new BHIERestServiceHandlerException((String) queryRespMap.get("error"));
                }

                String queryId = "";
                if (queryRespMap.get("queryid") != null) {
                    queryId = (String) queryRespMap.get("queryid");
                } else if (queryRespMap.get("queryID") != null) {
                    queryId = (String) queryRespMap.get("queryID");
                }
                rtc = pollRESTInterface(version, queryId, resultCollectionCls);
            } else if ("4".equalsIgnoreCase(version)) {
                // WebResource queryResource = restClient.resource(endpoint + REST_ASYNC_QUERY_PATH4);
                String v4endpoint = endpoint + REST_ASYNC_QUERY_PATH4;
                if ('/' != v4endpoint.charAt(v4endpoint.length() - 1)) {
                    v4endpoint += "/";
                }
                String path = queryParams.get("userid").get(0);
                path += "/" + queryParams.get("patientid").get(0);
                path += "/" + queryParams.get("loinc").get(0);
                path += "?requestSource=" + AppConfig.getInstance().getUserFacilityId();
                if ("51847-2".equals(queryParams.get("loinc").get(0))) {
                    if (queryParams.get("startdate") != null) {
                        if (queryParams.get("startdate").get(0).length() > 8) {
                            path += "&queryStartDate=" + queryParams.get("startdate").get(0).substring(0, 8);
                        } else {
                            path += "&queryStartDate=" + queryParams.get("startdate").get(0);
                        }
                    }
                    if (queryParams.get("enddate") != null) {
                        if (queryParams.get("enddate").get(0).length() > 8) {
                            path += "&queryEndDate=" + queryParams.get("enddate").get(0).substring(0, 8);
                        } else {
                            path += "&queryEndDate=" + queryParams.get("enddate").get(0);
                        }
                    }                    
                } else {
                    if (queryParams.get("startdate") != null) {
                        path += "&queryStartDate=" + queryParams.get("startdate").get(0);
                    }
                    if (queryParams.get("enddate") != null) {
                        path += "&queryEndDate=" + queryParams.get("enddate").get(0);
                    }
                }
                if (queryParams.get("classCodes") != null) {
                    path += "&classCodes=" + queryParams.get("classCodes").get(0);
                }
                if (queryParams.get("typeCodes") != null) {
                    path += "&typeCodes=" + queryParams.get("typeCodes").get(0);
                }
                if (queryParams.get("includedsources") != null) {
                    path += "&includeSources=" + queryParams.get("includedsources").get(0);
                }
                if (queryParams.get("excludedsources") != null) {
                    path += "&excludeSources=" + queryParams.get("excludedsources").get(0);
                }

                WebResource queryResource = restClient.resource(v4endpoint + path);

                //TODO: melcolm debug string
//                LOGGER.error("V4 URL = " + v4endpoint + path);
//                System.out.println("BDAv4 URL = " + queryResource.toString());

                
                String queryResp = queryResource.get(String.class);

                Map queryRespMap = objectMapper.readValue(queryResp, Map.class);

                if (queryRespMap.get("error") != null) {
                    throw new BHIERestServiceHandlerException((String) queryRespMap.get("error"));
                }
                String queryId = "";
                if (queryRespMap.get("queryid") != null) {
                    queryId = (String) queryRespMap.get("queryid");
                } else if (queryRespMap.get("queryID") != null) {
                    queryId = (String) queryRespMap.get("queryID");
                }
                rtc = pollRESTInterface(version, queryId, resultCollectionCls);
            } else {
                String key;
                StringBuilder path = new StringBuilder("?");
                Iterator<String> keyIterator = queryParams.keySet().iterator();
                while (keyIterator.hasNext()) {
                    key = keyIterator.next();
                    // &(attribute)=(value)
                    path.append("&").append(key).append("=").append(queryParams.getFirst(key));
                }

                WebResource queryResource;
                if (StringUtils.equals(version, "2")) {
                    queryResource = restClient.resource(endpoint + REST_ASYNC_QUERY_PATH + path.toString());
//                    System.out.println("BDAv2 URL = " + queryResource.toString());
                }
                else if (StringUtils.equals(version, "3")) {
                    queryResource = restClient.resource(endpoint + REST_ASYNC_QUERY_PATH3 + path.toString());
//                    System.out.println("BDAv3 URL = " + queryResource.toString());
                }
                else {
                    throw new BHIERestServiceHandlerException("Unsupported version: " + version);
                }

                String queryResp = queryResource.get(String.class);

                Map queryRespMap = objectMapper.readValue(queryResp, Map.class);

                if (queryRespMap.get("error") != null) {
                    throw new BHIERestServiceHandlerException((String) queryRespMap.get("error"));
                }
                rtc = pollRESTInterface(version, (String) queryRespMap.get("queryID"), resultCollectionCls);
            }
            return rtc;
        } catch (Exception e) {
            throw new BHIERestServiceHandlerException(e);
        }
    }

    /**
     * Polls the BHIE REST comet interface with the provided query ID.
     *
     * When the long poller receives valid response, it will bind the json response to a ResultCollection object
     * and then add the ResultCollection to a List.
     *
     * @param version BHIE interface query version
     * @param queryId A valid BHIE interface query identifier.
     * @param resultCollectionCls The ResultCollection class (json response will be bound to an object of this class),
     *        (see resultcollection package, an example of possible ResultCollection classes include
     *        AllergyResultCollection, MedicationResultCollection, etc.)
     * @return A List of ResultCollection objects.
     * @throws BHIERestServiceHandlerException If there is a failure.
     */
    @SuppressWarnings("unchecked")
    private List<ResultCollection> pollRESTInterface(String version, String queryId, Class resultCollectionCls) throws BHIERestServiceHandlerException {
        List<ResultCollection> resultCollectionList = new ArrayList<ResultCollection>();
        List<ResultCollection> rtc;
        try {
            if ("VLER".equalsIgnoreCase(version) || "4".equals(version)) {
                String v4endpoint;
                MultivaluedMap<String, String> pollerQuery = new MultivaluedMapImpl();

                if ("4".equals(version)) {
                    v4endpoint = endpoint + REST_ASYNC_COMET_PATH4;
                    if ('/' != v4endpoint.charAt(v4endpoint.length() - 1)) {
                        v4endpoint += "/";
                    }
                    v4endpoint += queryId;
                } else {
                    v4endpoint = AppConfig.getInstance().getProperties().getProperty("vlerendpoint", endpoint + REST_ASYNC_COMET_PATHVLER);
                    if ('/' != v4endpoint.charAt(v4endpoint.length() - 1)) {
                        v4endpoint += "/";
                    }
                    v4endpoint += queryId;
                }

//                System.out.println("##### poller URL: " + v4endpoint);
                WebResource cometResource = restClient.resource(v4endpoint);
                boolean queryComplete = false;
                boolean isPollerTimeout = false;
                long pollerStart = System.currentTimeMillis();
                long cnt = 0;

                //keep hitting service until response includes a queryComplete field
                while (!queryComplete && !isPollerTimeout) {
                    String jsonResp = "";
                    ResultCollection resultCollection = null;

                    try {

                        if ("4".equals(version)) {
                            jsonResp = cometResource.queryParams(pollerQuery).get(String.class);

                        } else {
                            jsonResp = cometResource.get(String.class);
                        }

                        //fix for invalid characters from DES
                        String[] invalidChars = StringUtils.split(INVALID_CHARS, ",");
                        for(String invChar : invalidChars){
                            if(invChar.length() == 4){
                                jsonResp = jsonResp.replace("\\u".concat(invChar), "");
                            }
                        }
                        resultCollection = (ResultCollection) objectMapper.readValue(jsonResp, resultCollectionCls);
                        resultCollectionList.add(resultCollection);
                    } catch (IOException ex) {
                        LOGGER.error("EXCEPTION - unable to map JSON: " + ex.getLocalizedMessage());
                        if(jsonResp.contains("queryComplete")){
                            //most likely an invalid payload, set queryComplete to true, since it exists in payload.
                            //it will not exist if the query isn't complete at the DES-side
                            queryComplete = true;

//                            throw new BHIERestServiceHandlerException("Invalid DES payload detected. Unable to map JSON with queryId: " + queryId);

                            //generate error to show user
                            if(resultCollection == null){
                                resultCollection = new V4ResultCollection();
                                resultCollection.setQueryComplete("true");
                            }

                            if(((V4ResultCollection) resultCollection).getErrorListv4() == null){
                                ((V4ResultCollection) resultCollection).setErrorListv4(new ArrayList<String>());
                            }
                            ((V4ResultCollection) resultCollection).getErrorListv4().add("Query result error. Invalid DES payload detected.");
                            resultCollectionList.add(resultCollection);

                        }
                    }

                    if (resultCollection != null) {
//                        if (resultCollection.getErrorList() != null) {
//                            StringBuilder sb = new StringBuilder();
//                            for(gov.va.med.bhie.beans.Error error : resultCollection.getErrorList() ){
//                                sb.append("Error Code: " + error.getErrorCode() + ", Error Value: " + error.getErrorValue() + "\n");
//                            }
//                            throw new BHIERestServiceHandlerException(sb.toString());
//                        }

                        queryComplete = resultCollection.getQueryComplete() != null && "true".equals(resultCollection.getQueryComplete());

                    }
                    if (!queryComplete) {
                        LOGGER.info("Query not complete sleeping: " + (2000L + (cnt * 500L)));
                        try {
                            Thread.sleep(2000L + (cnt++ * 500L));
                        } catch (InterruptedException ex) {
                            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
                        }
                    }
                    //check for poller timeout
                    if ((System.currentTimeMillis() - pollerStart) > POLLER_TIMEOUT_MS) {
                        isPollerTimeout = true;
                    }
                }

                if (!queryComplete) {
                    if (isPollerTimeout) {
                        throw new BHIERestServiceHandlerException("Query poller timed out for queryID: " + queryId);
                    } else {
                        throw new BHIERestServiceHandlerException("Incomplete query for queryID: " + queryId);
                    }
                }

                rtc = resultCollectionList;
            } else {
                WebResource cometResource = restClient.resource(endpoint + REST_ASYNC_COMET_PATH);
                if ("3".equals(version)) {
                    cometResource = restClient.resource(endpoint + REST_ASYNC_COMET_PATH3);
                }

                MultivaluedMap<String, String> pollerQuery = new MultivaluedMapImpl();
                pollerQuery.add("queryid", queryId);
                pollerQuery.add("version", version);
                pollerQuery.add("userfacilityid", AppConfig.getInstance().getUserFacilityId());

                boolean queryComplete = false;
                boolean isPollerTimeout = false;
                long pollerStart = System.currentTimeMillis();
                long cnt = 0;

                //keep hitting service until response includes a queryComplete field
                while (!queryComplete && !isPollerTimeout) {
                    String jsonResp;
                    ResultCollection resultCollection = null;

                    try {

                        jsonResp = cometResource.queryParams(pollerQuery).get(String.class);

                        //melcolm - fix for null character from BDA
                        jsonResp = jsonResp.replace("\\u0000", "");

                        resultCollection = (ResultCollection) objectMapper.readValue(jsonResp, resultCollectionCls);

                        resultCollectionList.add(resultCollection);
                    } catch (IOException ex) {
                        LOGGER.warn("EXCEPTION: " + ex.getLocalizedMessage());

                    }

                    if (resultCollection != null) {
//                        if (resultCollection.getErrorList() != null) {
//                            StringBuilder sb = new StringBuilder();
//                            for(gov.va.med.bhie.beans.Error error : resultCollection.getErrorList() ){
//                                sb.append("Error Code: " + error.getErrorCode() + ", Error Value: " + error.getErrorValue() + "\n");
//                            }
//                            throw new BHIERestServiceHandlerException(sb.toString());
//                        }

                        queryComplete = resultCollection.getQueryComplete() != null && "true".equals(resultCollection.getQueryComplete());
                    }
                    if (!queryComplete) {
                        LOGGER.info("Query not complete sleeping: " + (2000L + (cnt * 500L)));
                        try {
                            Thread.sleep(2000L + (cnt++ * 500L));
                        } catch (InterruptedException ex) {
                            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
                        }
                    }
                    //check for poller timeout
                    if ((System.currentTimeMillis() - pollerStart) > POLLER_TIMEOUT_MS) {
                        isPollerTimeout = true;
                    }
                }

                if (!queryComplete) {
                    if (isPollerTimeout) {
                        throw new BHIERestServiceHandlerException("Query poller timed out for queryID: " + queryId);
                    } else {
                        throw new BHIERestServiceHandlerException("Incomplete query for queryID: " + queryId);
                    }
                }

                rtc = resultCollectionList;
            }
            return rtc;
        } catch (RuntimeException e) {
            LOGGER.error("EXCEPTION: " + e.getLocalizedMessage(), e);
            throw new BHIERestServiceHandlerException(e);
        } catch (Exception e) {
            LOGGER.error("EXCEPTION: " + e.getLocalizedMessage(), e);
            throw new BHIERestServiceHandlerException(e);
        }
    }

    /**
     * Makes complex note REST query for note image.
     * @param eventId Note's event id
     * @return NoteImage bean containing note binary and header data.
     * @throws BHIERestServiceHandlerException if there is a failure
     */
    public NoteImage queryNoteImage(String eventId) throws BHIERestServiceHandlerException {
        if (eventId == null || eventId.isEmpty()) {
            throw new IllegalArgumentException("Event id cannot be null or empty");
        }

        try {

            String complexNotePath = AppConfig.getInstance().getComplexNotePath();

            if (complexNotePath.charAt(0) == '/') {
                complexNotePath = complexNotePath.substring(1);
            }

            if (!complexNotePath.endsWith("/")) {
                complexNotePath += "/";
            }

            URI url = new URI(endpoint + complexNotePath + eventId);

            return queryNoteImage(url);

        } catch (URISyntaxException e) {
            throw new BHIERestServiceHandlerException(e);
        }
    }

    /**
     * Makes complex note REST query for note image.
     * @param url URL to query complex note.
     * @return NoteImage bean containing note binary and header data.
     * @throws BHIERestServiceHandlerException if there is a failure
     */
    public NoteImage queryNoteImage(URI url) throws BHIERestServiceHandlerException {
        if (url == null) {
            throw new IllegalArgumentException("URL cannot be null or empty");
        }

        try {
            WebResource noteImageResource = docClient.resource(url);
            ClientResponse clientResponse = noteImageResource.get(ClientResponse.class);

            if (clientResponse.getStatus() != 200) {
                throw new BHIERestServiceHandlerException("There was a problem fetching note image, http status: " + clientResponse.getStatus());
            }

            NoteImage noteImage = new NoteImage();

            InputStream istream = clientResponse.getEntityInputStream();
            try {
                noteImage.setNoteBytes(IOUtils.toByteArray(istream));
            } finally {
                istream.close();
            }

            List<String> contentType = clientResponse.getHeaders().get("Content-Type");
            if (contentType != null && !contentType.isEmpty()) {
                noteImage.setContentType(contentType.get(0));
            }

            List<String> contentDisposition = clientResponse.getHeaders().get("Content-Disposition");
            if (contentDisposition != null && !contentDisposition.isEmpty()) {
                noteImage.setContentDisposition(contentDisposition.get(0));
            }

            return noteImage;

        } catch (Exception e) {
            throw new BHIERestServiceHandlerException(e);
        }
    }
}
