package gov.va.med.bhie.webservice;

import gov.va.med.bhie.beans.QueryBean;
import gov.va.med.bhie.common.AppConfig;
import gov.va.med.bhie.resultcollection.ResultCollection;
import gov.va.med.foundations.utilities.ARMLog;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.log4j.Logger;


/**
 * BHIE Relay Limit and Cache.
 *
 * @author <a href="mailto:kweller@hawaiirg.com">kweller</a>
 * @version 1.0
 *
 */
public class BHIELimit {

    /**
     * Logger.
     */
    private static final Logger LOGGER =
        Logger.getLogger(BHIELimit.class.getName());

    /**
     * Tokens to limit number of concurrent calls.
     */
    private static ConcurrentHashMap<String, Date> tokens = new ConcurrentHashMap<String, Date>();

    /**
     * Cache.
     */
    private static ConcurrentHashMap<String, ResultCollection> cachemap = new ConcurrentHashMap<String, ResultCollection>();

    /**
     * Age of cache entries.
     */
    private static ConcurrentHashMap<String, Date> cacheagemap = new ConcurrentHashMap<String, Date>();

    /**
     * Time to live for cache entries.
     */
    private int expiretimesecs = 900; // 15 minute

    /**
     * Constructor.
     */
    private BHIELimit() {
        super();
        String timeout = AppConfig.getInstance().getProperties().getProperty("BHIELimit.timeout", "900");
        try {
            expiretimesecs = Integer.parseInt(timeout);
        } catch (NumberFormatException ex) {
            expiretimesecs = 900;
        }
    }
    
    /**
     * Get singleton class.
     *
     * @return this
     */
    @SuppressWarnings("all")
    public static synchronized BHIELimit getInstance() {
        return BHIELimitHolder.INSTANCE;
    }


    public ResultCollection getCache(QueryBean queryBean, String type) {
        ResultCollection rtc = null;

        String key = getKey(queryBean, type);
        if (cachemap.containsKey(key) && cacheagemap.containsKey(key)) {
            Date ldate = cacheagemap.get(key);
            Date cdate = new Date();
            if (ldate != null && (expiretimesecs == -1 || (Math.abs(cdate.getTime() - ldate.getTime()) <= (1000L * expiretimesecs)))) {
                rtc = cachemap.get(key);
                LOGGER.info("Returning cache for: " + ARMLog.cleanString(key));
            } else {
                cleanMap();
            }
        } else {
            cleanMap();
        }
        return rtc;
    }

    public void putCache(QueryBean queryBean, String type, ResultCollection collection) {
        String key = getKey(queryBean, type);
        putCache(key, collection);
    }

    public void putCache(String key, ResultCollection collection) {
        Date cdate = new Date();
        cacheagemap.put(key, cdate);
        cachemap.put(key, collection);
    }

    public void dropCache(QueryBean queryBean, String type) {
        String key = getKey(queryBean, type);
        cacheagemap.remove(key);
        cachemap.remove(key);
    }

    public String getKey(QueryBean queryBean, String type) {
        StringBuilder rtc = new StringBuilder();
        if (queryBean != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd", Locale.getDefault());
            rtc.append(queryBean.getPatientId());
            rtc.append('|');
            rtc.append(type);
            rtc.append('|');
            if (queryBean.getStartDate() != null) {
                rtc.append(sdf.format(queryBean.getStartDate().getTime()));
            }
            rtc.append('|');
            if (queryBean.getEndDate() != null) {
                rtc.append(sdf.format(queryBean.getEndDate().getTime()));
            }
            rtc.append('|');
            if (queryBean.getIncludedSources() != null) {
                List<String> src = queryBean.getIncludedSources();
                if (CollectionUtils.isNotEmpty(src)) {
                    int srcSize = src.size();
                    for (int i = 0; i < srcSize; ++i) {
                        rtc.append(src.get(i));
                        rtc.append('|');
                    }
                }
            }
            if (queryBean.getExcludedSources() != null) {
                List<String> src = queryBean.getExcludedSources();
                if (CollectionUtils.isNotEmpty(src)) {
                    int srcSize = src.size();
                    for (int i = 0; i < srcSize; ++i) {
                        rtc.append(src.get(i));
                        rtc.append('|');
                    }
                }
            }
        }
        return rtc.toString();
    }

    private void cleanMap() {
        Enumeration<String> keys = cacheagemap.keys();
        Date cdate = new Date();
        while (keys != null && keys.hasMoreElements()) {
            String key = keys.nextElement();
            Date adate = tokens.get(key);
            if (adate != null && cdate != null && (Math.abs(cdate.getTime()
                    - adate.getTime()) > (1000L * (60 + expiretimesecs)))) {
                cacheagemap.remove(key);
                cachemap.remove(key);
            }
        }
    }

    /**
     * Disable cloning.
     *
     * @return nothing
     *
     * @throws CloneNotSupportedException exception
     */
    @SuppressWarnings("all")
    public Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException();
    }

    /**
     * Singleton helper class.
     *
     */
    private static class BHIELimitHolder {
        /**
         * BHIELimit.
         */
        private static final BHIELimit INSTANCE =
            new BHIELimit();

    }
}
