//= require lib/AppKit.js
//= require JLV.js
//= require ./DetailsWindow.js
//= require ./WidgetWindow.js
//= require ./MoreFilter.js

Date.prototype.isValid = function () {
    // An invalid date object returns NaN for getTime() and NaN is the only
    // object not strictly equal to itself.
    return this.getTime() === this.getTime();
};

JLV.WidgetDataModel = Backbone.Model.extend({
    dateFields: [],
    parse: function (data) {
        var count = this.dateFields.length,
            i = 0;

        for (; i < count; i++) {
            if (data[this.dateFields[i]]) {
                data[this.dateFields[i]] = new Date(data[this.dateFields[i]]);
                if (!data[this.dateFields[i]].isValid()) {
                    data[this.dateFields[i]] = null;
                }
            }
            else {
                data[this.dateFields[i]] = null;
            }
        }

        if (data.provider) {
            data.taxonomy = data.provider.providerType;
            data.taxonomyCode = data.provider.taxonomyCode;
            data.providerName = data.provider.name;
        }

        if (data.orderingProvider) {
            if (data.orderingProvider.providerType) {
                data.taxonomy = data.orderingProvider.providerType;
            }

            if (data.orderingProvider.taxonomyCode) {
                data.taxonomyCode = data.orderingProvider.taxonomyCode;
            }

            data.providerName = data.orderingProvider.name;
        }

        return data;
    }
});


JLV.DateFilteredDataCollection = Backbone.Collection.extend({
    doFetch: function (callback, error) {
        this.fetch({
            data: {
                startDate: moment(this.startDate).format("M/D/YY"),
                endDate: moment(this.endDate).format("M/D/YY")
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    }
});

/**
 *  Widget base class
 *
 **/

JLV.Widget = AppKit.View.extend({
    classID: 'jlv-widget',
    normCodeSystems: [],
    portal: null,
    options: {
        name: '',
        data: null,
        closable: true,
        settings: true,
        dateFilter: false,
        vlerDomain: false,
        dasvlerDomain: false,
        print: false,
        refresh: false,
        status: false,
        clipboard: false,
        plus: false,
        moreWindowClass: null,
        columns: undefined,
        defaultSort: null,
        height: 300,
        startDay: -120,
        endDay: 0,
        gridHeaderHeight: 24,
        groupBy: undefined,
        groupFormatter: undefined,
        groupSorter: undefined
    },
    initialize: function (options) {

        var self = this;

        this.dodAdapterSourceReport = [];
        this.dodAdapterStatusRec = [];
        this._dataIsLoading = false; 

        this.$el.attr('tabindex', 0);

        this._header = $('<div role="heading">').addClass('jlv-widget-header');
        this._title = $('<span>').addClass('jlv-widget-title').appendTo(this._header).attr("unselectable", "on");

        this._dateLabel = $('<span tabindex="0">').addClass('jlv-widget-dateLabel jlv-clickable-link')
            .appendTo(this._header).attr("unselectable", "on")
            .button()
            .on('click', _.bind(function () {
                this.toggleDateFilterPanel();
            }, this));


        this._tools = $('<div>').addClass('jlv-widget-tools').prependTo(this._header);

        this._closeBtn = $('<div title="Close" tabindex="0" class="jlv-widget-toolbutton close">')
            .append('<div class="icon-close">')
            .attr('aria-label', 'Close button, press space bar to activate.')
            .button()
            .prependTo(this._tools);


        this._settingsBtn = $('<div title="Column Settings" tabindex="0" class="jlv-widget-toolbutton settings">')
            .append('<div class="icon-columns">')
            .attr('aria-label', 'Column settings button, press space bar to activate.')
            .button()
            .prependTo(this._tools);

        this._setupColumnSettings();


        this._refreshBtn = $('<div title="Refresh" tabindex="0" class="jlv-widget-toolbutton refresh">')
            .append('<div class="icon-refresh">')
            .attr('aria-label', 'Refresh button, press space bar to activate.')
            .button()
            .prependTo(this._tools).on('click', function (evt) {
                if (_.isFunction(self.loadData)) {
                    self.loadData();
                }
            });

        this._printBtn = $('<div title="Print" tabindex="0" class="jlv-widget-toolbutton print">')
            .append('<div class="icon-print">')
            .attr('aria-label', 'Print button, press space bar to activate.')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doPrint)) {
                    self.doPrint();
                }
            }).prependTo(this._tools);

        this._clipboardBtn = $('<div title="Copy to clipboard" tabindex="0" class="jlv-widget-toolbutton clipboard">')
            .append('<div class="icon-clipboard2">')
            .attr('aria-label', 'Copy button, press space bar to activate.')
            .button()
            .on('click', function () {
                self.doClipboard();
            }).prependTo(this._tools);

        this._dateFilterBtn = $('<div title="Date Filter" tabindex="0" class="jlv-widget-toolbutton filter">')
            .append('<div class="icon-filter">')
            .attr('aria-label', 'Date filter toggle button, press space bar to activate.')
            .button()
            .on('click', function () {
                self.toggleDateFilterPanel();
            }).prependTo(this._tools);


        if ((JLV.Session.dodVLER == true) || (JLV.Session.dasVLER == true)) {
            this._vlerLoadBtn = $('<div title="Force VLER Data Refresh - This can take a while" tabindex="0" class="jlv-widget-toolbutton vler">')
                .append('<div class="icon-refresh-alt">')
                .attr('aria-label', 'VLER refresh button, press space bar to activate.')
                .button()
                .on('click', function () {
                    //self.vlerLoadMessage("Community partner data retrieval in progress...");

                    AppKit.AlertDialog.yesNo('Confirm VLER Refresh',
                        'A forced refresh of community partner data may take a few minutes. Click Yes to continue. Click No to cancel.',
                        function (yes) {
                            if (yes) {
                                JLV.Session.forceRefresh = true;
                                if (_.isFunction(JLV.LoadVlerData)) {
                                    JLV.LoadVlerData();
                                }
                            }
                        }
                    );


                }).prependTo(this._tools);
        }

        this._addBtn = $('<div title="Add" tabindex="0" class="jlv-widget-toolbutton add">')
            .append('<div class="icon-plus-big">')
            .attr('aria-label', 'Add button, press space bar to activate.')
            .button()
            .hide()
            .on('click', function () {
                if (_.isFunction(self.onAdd)) {
                    self.onAdd();
                }
            }).prependTo(this._tools)

        this._interfaceStatusBtn = $('<div title="Interface Status" tabindex="0" class="jlv-widget-toolbutton status">')
            .append('<div class="icon-info">')
            .attr('aria-label', 'Status button, press space bar to activate.')
            .button()
            .on('click', function () {
                self.showInterfaceStatus();
            }).prependTo(this._tools);


        this._closeBtn.on('click', function (e) {
            e.stopPropagation();
            if (self.portal) {
                self.portal.removeWidget(self);
            }
        });

        this.$el.append(this._header);

        this._body = $('<div>').addClass('jlv-widget-body').appendTo(this.$el);
        this._footer = $('<div>').addClass('jlv-widget-footer').appendTo(this.$el);

        if (this.options && this.options.columns) {

            var tablePager = new AppKit.Pager();

            this._datagrid = new AppKit.PagedTableView({
                headerHeight: this.options.gridHeaderHeight,
                fitColumns: true,
                controller: this,
                pager: tablePager
            });

            if (_.isFunction(this.itemClicked)) {

                this._datagrid.on('afterRender.click', function () {
                    this.$el.find('.jlv-clickable-link').on('click', function () {
                        var modelId = $(this).data('model-id'),
                            model = self._datagrid.get('source').get(modelId);
                        self.itemClicked(model, $(this));
                        $(this).addClass('visited');
                    });
                });
            }


            this._datagrid.$el.addClass('jlv-widget-table').appendTo(this._body);

            this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {

                this._defaultSort = {
                    asc: sortInfo.asc,
                    id: sortInfo.column.id
                };

                if (_.isFunction(this.savePortalUI)) {
                    this.savePortalUI();
                }
            }, this));


            tablePager.$el.appendTo(this._footer);

            tablePager.on('selection', function () {
                self._updatePageInfoDisplay();
            });

            this._tablePageInfo = $('<span>')
                .addClass('jlv-table-page-info')
                .appendTo(this._footer)
                .hide();
        }

        this._moreBtn = $('<div title="More" tabindex="0" class="jlv-more-button">')
            .text("More >>")
            .button({text: false})
            .appendTo(this._footer)
            .on('click', function () {

                var moreWindow = self.get('moreWindow');

                moreWindow.center();
                if (moreWindow.get('maximized')) {
                    moreWindow.set('maximized', false);
                }
                moreWindow.set('defaultSort', self._defaultSort);

                moreWindow.open();
                moreWindow.focus();
            }).
            on('keydown', function (evt) {
                if (evt.which == $.ui.keyCode.TAB) {
                    if (!evt.shiftKey) {
                        evt.preventDefault();
                        //loop tab
                        self.$el.find('.jlv-widget-toolbutton:visible').first().focus();
                    }

                }
            })
            .hide();

        this.$el.resizable({
            handles: "s",
            stop: function () {
                if (_.isFunction(self.savePortalUI)) {
                    self.savePortalUI();
                }
            }
        });

        if (this.options.data) {
            this.set('_data', this.options.data);
        }

        this._super(options);

        this.set('title', this.get('name'));

        this.$el.addClass(this.get('widgetId'));

        this._setup508Events();

        //make it draggable
        this.$el.draggable({
            revert: 'invalid',
            helper: 'clone',
            handle: '.jlv-widget-header',
            start: function (event, ui) {
                //for performance reasons in IE, do not drag along the table

                $(this).hide();

                ui.helper.parent().css('zIndex', 1000);
                ui.helper.css({
                    opacity: 0.88,
                    zIndex: 1000
                });

                ui.helper.find('.jlv-widget-body, .jlv-widget-footer, .jlv-vler-loading').hide();
            },
            stop: function (event, ui) {

                ui.helper.css({
                    opacity: 1,
                    zIndex: 0
                });
                ui.helper.find('.jlv-widget-body, .jlv-widget-footer, .jlv-vler-loading').show();
                ui.helper.parent().css('zIndex', 0);

                var portal = $(this).data('portal');

                if (portal) {
                    portal.portal('removePlaceholder');
                }

                $(this).show();

            },
            drag: function (event, ui) {

                var portal = $(this).data('portal');

                if (portal) {
                    portal.portal('computeDrop', event, ui, false);
                }
            }
        });


    },

    _setup508Events: function () {

        //508 - tab nav and hot keys
        //508 - tab nav and hot keys
        var self = this;
        this.$el.on('keydown', function (evt) {

            if (evt.keyCode === $.ui.keyCode.ESCAPE) {
                self.$el.focus();
            }

            if (evt.which === 83 && evt.altKey) { //Alt-S skip navigation to more button or tool buttons
                evt.stopPropagation();
                evt.preventDefault();
                if (self._moreBtn && self._moreBtn.is(':visible')) {
                    self._moreBtn.focus();
                }
                else {
                    self.$el.find('.jlv-widget-toolbutton:visible').first().focus();
                }
                return;
            }

            //block bubbling keydown events from child elements
            if (!$(evt.target).hasClass('jlv-widget'))
                return;

            if (evt.keyCode === $.ui.keyCode.TAB) {
                if (!evt.shiftKey) {
                    var next = self.$el.nextInDOM('.jlv-widget');
                    evt.preventDefault();
                    evt.stopPropagation();
                    if (next.length) {
                        if (!next.is(":visible")) {
                            $('#AddWidgetButton').focus();
                        } else {
                            next.focus();
                        }
                    }
                    else {
                        $('#AddWidgetButton').focus();
                    }
                }
                else {

                    var prev = self.$el.prevInDOM('.jlv-widget');
                    evt.preventDefault();
                    evt.stopPropagation();

                    if (prev.length) {
                        if (!prev.is(":visible")) {
                            $('#JLV-Desktop').find('.jlv-tabview-add').focus();
                        } else {
                            prev.focus();
                        }
                    }
                    else {

                        prev = self.$el.prevInDOM('.jlv-insurance-link');
                        if (prev.length) {
                            prev.focus();
                        }
                        else {
                            $('#JLV-Desktop').find('.jlv-tabview-add').focus();
                        }
                    }
                }
            }
            else if (evt.keyCode === $.ui.keyCode.RIGHT) { //page next

                if (self._datagrid) {
                    var pager = self._datagrid.get('pager'),
                        current = pager.get('selectedPage'),
                        nextPage = Math.min(current + 1, pager.get('pageCount'));

                    if (nextPage > current)
                        pager.set('selectedPage', nextPage);

                    self._datagrid._changePage();
                }

            }
            else if (evt.keyCode === $.ui.keyCode.LEFT) { //page back

                if (self._datagrid) {
                    var pager = self._datagrid.get('pager'),
                        current = pager.get('selectedPage'),
                        prevPage = Math.max(current - 1, 1);

                    if (prevPage < current)
                        pager.set('selectedPage', prevPage);

                    self._datagrid._changePage();
                }
            }
            else if (evt.keyCode === $.ui.keyCode.DOWN) { // resize widget +

                var height = self.get('height');

                height += 5;

                self.set('height', height);


            }
            else if (evt.keyCode === $.ui.keyCode.UP) { // resize widget -

                var height = self.get('height');

                height -= 5;

                self.set('height', Math.max(200, height));


            }
            else if (evt.keyCode === $.ui.keyCode.ENTER) { //focus in
                evt.preventDefault();
                //focus on toolbar buttons
                self.$el.find('.jlv-widget-toolbutton:visible').first().focus();
            }


        });


    },

    _setupColumnSettings: function () {

        this._settingsBtn.on('click', _.bind(function (evt) {

            evt.stopPropagation();

            if (this._settingsPopover) {
                this._settingsPopover.remove();
                this._settingsPopover = null;
                $(document).off('click.columnSettings, keydown.columnSettings');
                return;
            }

            this._settingsPopover = $('<div>')
                .addClass('ui-tooltip bottom arrow').hide().appendTo('body');


            var content = $('<div>').addClass('content').on('click', function (evt) {
                evt.stopPropagation();
            });

            var columns = this._datagrid.get('columns'),
                count = columns.length,
                i = 0;

            var table = $('<table>'),
                tr = null;
            for (; i < count; i++) {

                if (i % 2 == 0) {

                    if (tr !== null) {
                        table.append(tr);
                    }

                    tr = $('<tr>')
                }

                var col = columns[i];

                var td = $('<td>');

                td.append(
                    $(String.format('<input type="checkbox" value="{0}" {1} />',
                        col.id, (col.isHidden ? '' : 'checked')))
                );

                td.append($(String.format('<span>&nbsp;{0}</span>', col.title)));
                tr.append(td);
            }

            if (tr !== null) {
                table.append(tr);
            }

            content.append(table);

            content.append(
                $('<div class="primary" tabindex="0" style="margin-top:14px;;height:25px;width:80px;">Apply</div>')
                    .button()
                    .on('keydown', _.bind(function (evt) {
                        if (evt.which == $.ui.keyCode.TAB) {
                            evt.preventDefault();
                            this._settingsPopover.find('input').first().focus();
                        }


                    }, this))
                    .on('click', _.bind(function () {

                        this._settingsPopover.remove();
                        this._settingsPopover = null;

                        var notChecked = content.find('input:checkbox:not(:checked)');

                        var toHide = [];
                        $.each(notChecked, function (idx, el) {

                            var id = $(el).val();
                            toHide.push(id);

                        });

                        this.set('hiddenColumnsMinimized', toHide);
                        this.$el.trigger('columnsHidden', [toHide]);

                        this.savePortalUI();

                        this._settingsBtn.focus();
                        $(document).off('click.columnSettings');
                        $(document).off('keydown.columnSettings');
                    }, this))
            );

            this._settingsPopover.append(content);

            this._settingsPopover.css({
                left: (this._settingsBtn.offset().left -
                this._settingsPopover.outerWidth() / 2 + 12),
                top: this._settingsBtn.offset().top + 24
            });

            this._settingsPopover.show();
            this._settingsPopover.find('input').first().focus();

            var self = this;


            $(document).on('click.columnSettings', function (evt) {
                if (self._settingsPopover) {
                    self._settingsPopover.remove();
                }

                self._settingsBtn.focus();
                self._settingsPopover = null;
                $(document).off('click.columnSettings');
                $(document).off('keydown.columnSettings');
            });

            $(document).on('keydown.columnSettings', function (evt) {
                if (evt.which == $.ui.keyCode.ESCAPE) {
                    if (self._settingsPopover) {
                        self._settingsPopover.remove();
                    }

                    self._settingsBtn.focus();
                    self._settingsPopover = null;
                    $(document).off('click.columnSettings');
                    $(document).off('keydown.columnSettings');
                }

            });

        }, this));


    },

    /**
     * creates the more window
     * @private
     */
    _createMoreWindow: function () {
        var moreWindowClassName = this.get('moreWindowClass');
        if (moreWindowClassName) {
            if (!this._moreWindow) {
                var Class = AppKit.get(moreWindowClassName);
                this._moreWindow = new Class({
                    data: this.get('data'),
                    widget: this,
                    hiddenColumns: this.get('hiddenColumnsExpanded')
                });
            }
            else {
                this._moreWindow.renderData();
            }
        }
    },

    moreWindow: function () {
        if (!this._moreWindow) {
            this._createMoreWindow();
        }

        return this._moreWindow;
    },

    close: function () {

        this.destroy();

        if (_.isFunction(this.savePortalUI)) {
            this.savePortalUI();
        }

        this.$el.trigger('close');
    },


    destroy: function () {
        //remove all notifications if they exist
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataFetchedNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataRefreshNotification);

        if (this._moreWindow) {
            this._moreWindow.destroy();
            this._moreWindow = null;
        }

        if (this._settingsPopover) {
            this._settingsPopover.remove();
            this._settingsPopover = null;
        }

        setTimeout(_.bind(function () {
            this.$el.remove();
        }, this), 0);
    },


    adjustHeight: function(){
        var newHeight = this._header.outerHeight();
        if(this._vlerLoadPanel)
            newHeight += this._vlerLoadPanel.el.offsetHeight;

        this._body.css({
            top: newHeight,
            bottom: this._footer.outerHeight()
        });
    },
    /**
     * Update the pages displayed information in the widget footer
     */
    _updatePageInfoDisplay: function () {

        if (this._datagrid) {
            var range = this._datagrid.displayedRange(),
                count = this._datagrid.get('rowCount');

            if (range && count) {
                this._tablePageInfo.html(_.template('Displaying {{start}}-{{end}} of {{total}}')({
                    start: range.start + 1,
                    end: range.end,
                    total: count
                })).show();
            }
            else {
                this._tablePageInfo.hide();
            }

        }

        setTimeout(_.bind(function () {
            this.adjustHeight();
        }, this), 0);

    },
    isLoading: function () {
        return this._dataIsLoading;
    },
    loadData: function (callback) {


        /* loading is starting, set flag to true, disable apply filter */
        this._dataIsLoading = true;

        if (this._dateFilterPanel) {
            this._dateFilterPanel.set('disabled', true);
        }

        if (this._moreWindow) {
            this._moreWindow.disableFilterPanel(true);
        }

        /* set up widget for loading */
        this.set('title', this.get('name'));
        this.dodAdapterSourceReport = [];
        this.dodAdapterStatusRec = [];
        this._moreBtn.hide();

        if (this._datagrid)
            this._datagrid.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');

        if (this._moreWindow) {
            if (this._moreWindow._datagrid) {
                this._moreWindow._datagrid.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');
            }
        }

        this._updatePageInfoDisplay();

        JLV.VLERDataAvailable();

    },
    doClipboard: function () {

        if (this._dataIsLoading) {
            return;
        }

        if (this._datagrid) {

            var clone = this._datagrid.$el.clone();
            clone.find('.appkit-table-column, .ui-datatable-loading-rows, .dummy-row-1, .dummy-column, .hidden').remove();
            JLV.common.copyGridPanelHTML(clone.html());

        }
    },
    vlerLoadMessage: function (mesg){
        if (!this._vlerLoadPanel) {
            var data = this.get('data');
            if (data) {

                this._vlerLoadPanel = new JLV.Widget.VlerLoadPanel({
                    widget: this
                });

                this._vlerLoadPanel.$el.insertAfter(this._header);
                this._vlerLoadPanel.$el.hide();
            }
        }

        if (this._vlerLoadPanel) {

            var $loading = this.$el.find('.jlv-vler-loading');

            $loading.html(mesg);
            $loading.removeClass('jlv-vler-loading-img');

            if (this._vlerLoadPanel.get('hidden')) {
                this._vlerLoadPanel.set('hidden', false);
            }

            this.adjustHeight();
        }
    },


    hideVlerLoadMessage: function (){
        if (this._vlerLoadPanel) {
            this._vlerLoadPanel.set('hidden', true);
            this.adjustHeight();
        }
    },


    toggleDateFilterPanel: function () {

        if (!this._dateFilterPanel) {
            var data = this.get('data');
            if (data) {
                if (data.startDate !== undefined && data.endDate !== undefined) {
                    this._dateFilterPanel = new JLV.Widget.DateFilterPanel({
                        widget: this,
                        startDate: data.startDate.toDate(),
                        endDate: data.endDate.toDate()
                    });

                    this._dateFilterPanel.$el.prependTo(this._body);
                    this._dateFilterPanel.$el.hide();
                }
            }
        }

        if (this._dateFilterPanel) {

            var filterHeight = this._dateFilterPanel.$el.outerHeight(),
                top = this._datagrid.el.offsetTop;

            if (this._dateFilterPanel.get('hidden')) {
                this._dateFilterPanel.set('hidden', false);
                this._dateFilterPanel.$el.prependTo(this._body);
                if (this._datagrid)
                    this._datagrid.$el.css('top', top + filterHeight);
            }
            else {

                if (this._datagrid) {
                    this._datagrid.$el.css('top',
                        Math.max(0, top - filterHeight));
                }

                this._dateFilterPanel.set('hidden', true);
            }

            this._dateFilterPanel.set('disabled', this._dataIsLoading);
        }
    },

    showInterfaceStatus: function (aBOOL) {
        if (!this._interfaceStatusWindow) {
            this._interfaceStatusWindow = new AppKit.Dialog({
                title: 'Interface Status ~ ' + this.get('name'),
                width: 600,
                height: 500,
                maximizable: true,
                minimizable: true,
                resizable: true,
                modal: false,
                className: 'jlv-interface-status'
            });

            JLV.Desktop.registerWindow(this._interfaceStatusWindow);
        }

        var sourceStatus = JLV.common.DODADAPTER_STATUS_NO_REPORT_MSG;

        var data = this.get('data');

        if (data.dodAdapterSourceReport.length) {
            sourceStatus = '';

            for (var i = 0; i < data.dodAdapterSourceReport.length; i++) {
                sourceStatus += data.dodAdapterSourceReport[i] + '<br/><br/>';
            }
        }

        if (Object.keys(data.connectionErrors).length > 0
            || (JLV.Session.dodVLERFail && this.options.vlerDomain)
            || (JLV.Session.dasVLERFail && this.options.dasvlerDomain)) {
            sourceStatus += "<table><thead><th>Site Name</th><th>Site Status</th></thead>";
            for (i in data.connectionErrors) {
                if(data.connectionErrors[i].site.moniker != JLV.common.DODADAPTER_STATUS_REPORT_FLAG){
                    sourceStatus += "<tr><td>" + data.connectionErrors[i].site.moniker + '</td><td> Connection Unavailable </td></tr>';
                }
            }

            if (JLV.Session.dodVLERFail && this.options.vlerDomain) {
                sourceStatus += "<tr><td>DOD VLER</td><td> Connection Unavailable </td></tr>";
            }

            if (JLV.Session.dasVLERFail && this.options.dasvlerDomain) {
                sourceStatus += "<tr><td>VA VLER</td><td> Connection Unavailable </td></tr>";
            }

            sourceStatus += "</table>";

        }

        this._interfaceStatusWindow.$el.html('<div class="jlv-dodadapter-report">' + sourceStatus + '</div>');
        if (aBOOL === false) {
            this._interfaceStatusWindow.close();
        }
        else {
            this._interfaceStatusWindow.center();
            this._interfaceStatusWindow.open();

        }
    },
    /**
     * Columns setter/getter
     * @param anArray - an Array of column config objects
     * @returns anArray of column configurations
     */
    columns: function (anArray) {
        if (anArray !== undefined) {

            var colcount = anArray.length,
                i = 0;

            if (this._datagrid)
                this._datagrid.removeAllColumns();

            for (; i < colcount; i++) {

                var colInfo = anArray[i];
                if (this._datagrid)
                    this._datagrid.addColumn(colInfo);
            }

            this._columns = anArray;
        }

        return this._columns;
    },

    hiddenColumnsMinimized: function (anArrayOfColumnIds) {

        if (this._datagrid) {
            if (anArrayOfColumnIds !== undefined) {
                this._datagrid.set('hiddenColumns', anArrayOfColumnIds);
            }

            return this._datagrid.get('hiddenColumns');
        }

        return [];

    },

    hiddenColumnsExpanded: function (anArrayOfColumnIds) {

        if (anArrayOfColumnIds !== undefined) {
            this._hiddenColumnsExpanded = anArrayOfColumnIds;
        }

        return this._hiddenColumnsExpanded;
    },


    /** public getter **/
    data: function () {
        if (this._datagrid)
            return this._datagrid.get('source');
    },
    /**
     * private setter/getter Collection for data shown in the widget's table
     * @param aBBCollection - a Backbone collection
     * @returns a Backbone collection
     */
    _data: function (aBBCollection) {
        if (this._datagrid) {
            if (_.isFunction(aBBCollection)) {
                this._datagrid.set('source', aBBCollection ? new aBBCollection() : null);
            }
            else {
                this._datagrid.set('source', aBBCollection);
            }

            return this._datagrid.get('source');
        }

    },
    /**
     * widget title getter/setter shown in widget header
     * @param aString
     * @returns {*}
     */
    title: function (aString) {
        if (aString) {
            this._title.html(aString);
            this.$el.attr('aria-label', aString + " - Press Enter to navigate into a widget.");
            this.adjustHeight();
        }

        return this._title.text();
    },
    /**
     * is widget closable getter/setter
     * @param aBOOL
     * @returns {*}
     */
    closeable: function (aBOOL) {
        if (aBOOL === true) {
            this._closeBtn.show();
        }
        else if (aBOOL === false) {
            this._closeBtn.hide();
        }

        return this._closeBtn.is(':visible');

    },


    /**
     * does widget have column settings getter/setter
     * @param aBOOL
     * @returns {*}
     */
    settings: function (aBOOL) {
        if (aBOOL === true) {
            this._settingsBtn.show();
        }
        else if (aBOOL === false) {
            this._settingsBtn.hide();
        }

        return this._settingsBtn.is(':visible');

    },


    /**
     * widget clipboard tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    clipboard: function (aBOOL) {
        if (aBOOL === true) {
            this._clipboardBtn.show();
        }
        else if (aBOOL === false) {
            this._clipboardBtn.hide();
        }

        return this._clipboardBtn.is(':visible');
    },
    /**
     * widget date filter tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    dateFilter: function (aBOOL) {
        if (aBOOL === true) {
            this._dateFilterBtn.show();
            this._dateLabel.show();
            this.adjustHeight();
        }
        else if (aBOOL === false) {
            this._dateFilterBtn.hide();
            this._dateLabel.hide();
            this.adjustHeight();
        }
        return this._dateFilterBtn.is(':visible');

    },
    /**
     * DOD VLER toggle
     * @param aBOOL
     * @returns {*}
     */
    vlerDomain: function (aBOOL) {

        if (this._vlerLoadBtn) {

            if (aBOOL === true) {

                this._vlerDomain = true;
                this._vlerLoadBtn.show();
                //add vler-related notifications
                JLV.NotificationCenter.addObserverForNotification({
                    observer: this,
                    name: JLV.VlerDataFetchedNotification,
                    action: _.bind(function () {
                        this.vlerLoadMessage(JLV.common.DOD_VLER_LOADING_MSG);
                        this.$el.find('.jlv-vler-loading').addClass('jlv-vler-loading-img');
                        if (this._moreWindow) {
                            this._moreWindow.vlerLoadMessage(JLV.common.DOD_VLER_LOADING_MSG);
                            this._moreWindow.$el.find('.jlv-vler-loading').addClass('jlv-vler-loading-img');
                        }
                    }, this)
                });

                JLV.NotificationCenter.addObserverForNotification({
                    observer: this,
                    name: JLV.VlerDataRefreshNotification,
                    action: _.bind(function () {

                        if (this._moreWindow) {
                            this.loadData(_.bind(function () {
                                this._moreWindow.renderData();
                            }, this));
                            this._moreWindow.hideVlerLoadMessage();
                        } else {
                            this.loadData();
                        }

                        this.hideVlerLoadMessage();

                    }, this)
                });


            }
            else if (aBOOL === false) {
                this._vlerDomain = false;
                this._vlerLoadBtn.hide();

            }

            if (!this._vlerDomain) {
                //remove all vler notifications if they exist
                JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataFetchedNotification);
                JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataRefreshNotification);
            }

            return this._vlerDomain;
        }

        return false;

    },
    /**
     * DAS VA VLER toggle
     * @param aBOOL
     * @returns {*}
     */
    dasvlerDomain: function (aBOOL) {

        if (this._vlerLoadBtn) {

            if (aBOOL === true) {

                this._dasvlerDomain = true;
                this._vlerLoadBtn.show();
                //add vler-related notifications
                JLV.NotificationCenter.addObserverForNotification({
                    observer: this,
                    name: JLV.DasVlerDataFetchedNotification,
                    action: _.bind(function () {
                        this.vlerLoadMessage(JLV.common.DOD_VLER_LOADING_MSG);
                        this.$el.find('.jlv-vler-loading').addClass('jlv-vler-loading-img');
                        if (this._moreWindow) {
                            this._moreWindow.vlerLoadMessage(JLV.common.DOD_VLER_LOADING_MSG);
                            this._moreWindow.$el.find('.jlv-vler-loading').addClass('jlv-vler-loading-img');
                        }
                    }, this)
                });

                JLV.NotificationCenter.addObserverForNotification({
                    observer: this,
                    name: JLV.DasVlerDataRefreshNotification,
                    action: _.bind(function () {

                        if (this._moreWindow) {
                            this.loadData(_.bind(function () {
                                this._moreWindow.renderData();
                            }, this));
                            this._moreWindow.hideVlerLoadMessage();
                        } else {
                            this.loadData();
                        }

                        this.hideVlerLoadMessage();

                    }, this)
                });


            }
            else if (aBOOL === false) {
                this._dasvlerDomain = false;
                this._vlerLoadBtn.hide();

            }

            if (!this._dasvlerDomain) {
                //remove all vler notifications if they exist
                JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.DasVlerDataFetchedNotification);
                JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.DasVlerDataRefreshNotification);
            }

            return this._dasvlerDomain;
        }

        return false;

    },
    defaultSort: function (sort) {
        if (sort !== undefined) {
            this._defaultSort = sort;
        }

        return this._defaultSort;
    },

    /**
     * widget print tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    print: function (aBOOL) {
        if (aBOOL === true) {
            this._printBtn.show();
        }
        else if (aBOOL === false) {
            this._printBtn.hide();
        }

        return this._printBtn.is(':visible');

    },
    /**
     * widget refresh tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    refresh: function (aBOOL) {
        if (aBOOL === true) {
            this._refreshBtn.show();
        }
        else if (aBOOL === false) {
            this._refreshBtn.hide();
        }

        return this._refreshBtn.is(':visible');
    },
    /**
     * widget status tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    status: function (aBOOL) {

        if (aBOOL === true) {
            this._interfaceStatusBtn.show();
        }
        else if (aBOOL === false) {
            this._interfaceStatusBtn.hide();
        }
        return this._interfaceStatusBtn.is(':visible');
    },

    /* converts status icon to yellow alert icon */
    statusAlert: function (aBOOL) {

        if (aBOOL === true) {
            this._interfaceStatusBtn.find('.icon-info').addClass('yellow');
        }
        else if (aBOOL === false) {
            this._interfaceStatusBtn.find('.icon-info').removeClass('yellow');

        }

        return this._interfaceStatusBtn.find('.icon-info').hasClass('yellow');

    },

    addBanner: function (config) {

        if (this._body) {
            if (!this._bannerHeight) {
                this._bannerHeight = 0;
            }

            this.clearBanner(config.id);

            var b = $('<div>')
                .attr('id', config.id || _.uniqueId())
                .attr('tabindex', 0)
                .css({
                    height: config.height || 32,
                    background: config.bgcolor || '#d0d0d0',
                    color: config.fgcolor || '#111'
                }).addClass('jlv-widget-banner')
                .html(config.text || '');


            var dateFilter = this._body.find('.jlv-date-filter-panel:visible');
            if (dateFilter.length) {
                b.insertAfter(dateFilter);
            }
            else {
                b.prependTo(this._body)
            }


            if (config.closable) {

                $('<div>')
                    .attr('title', 'Close Banner')
                    .attr('tabindex', 0)
                    .addClass('appkit-button jlv-widget-banner-close')
                    .awaken('appkit-button')
                    .html('<div class="jlv-banner-close"></div>')
                    .on('click', _.bind(function () {
                        this.clearBanner(config.id)

                    }, this))
                    .appendTo(b);


            }

            var extra = 1;

            if (dateFilter.length) {
                extra += dateFilter.height();
            }

            this._bannerHeight += b.outerHeight();


            if (this._datagrid) {
                this._datagrid.$el.css('top', this._bannerHeight + extra);
            }
        }
    },

    clearBanner: function (id) {

        if (id) {

            if (this._body) {

                var b = this._body.find('#' + id);

                if (b) {

                    var h = b.outerHeight();

                    b.remove();

                    if (this._bannerHeight) {
                        this._bannerHeight -= h;
                    }

                    var dateFilter = this._body.find('.jlv-date-filter-panel:visible');
                    var extra = 1;


                    if (dateFilter.length) {
                        extra += dateFilter.height();
                    }

                    if (this._datagrid) {
                        this._datagrid.$el.css('top', this._bannerHeight + extra);
                    }
                }

            }
        }
    },

    clearBanners: function () {

        if (this._body) {
            this._body.find(".jlv-widget-banner").remove();


            var dateFilter = this._body.find('.jlv-date-filter-panel:visible');
            var extra = 32;
            if (dateFilter.length) {
                extra = dateFilter.height() + 1;
            }


            if (this._datagrid) {
                this._datagrid.$el.css('top', extra);
            }
        }
    },


    /**
     * widget plus/add tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    plus: function (aBOOL) {
        if (aBOOL === true) {
            this._addBtn.show();
        }
        else if (aBOOL === false) {
            this._addBtn.hide();
        }

        return this._addBtn.is(':visible');
    },
    groupBy: function (aString) {
        if (this._datagrid)
            return this._datagrid.set('groupBy', aString);
    },
    groupFormatter: function (aFunction) {
        if (this._datagrid)
            return this._datagrid.set('groupFormatter', aFunction);
    },
    groupSorter: function (aFunction) {
        if (this._datagrid)
            return this._datagrid.set('groupSorter', aFunction);
    },
    /**
     * called when widget is resized
     */
    sizer: function () {
        if (this._datagrid) {
            this._datagrid.sizer();
        }

        this.adjustHeight();
    },
    height: function (aNumber) {
        if (aNumber !== undefined) {
            this.$el.height(aNumber);
        }

        return this.$el.height();
    },

    updateDateLabel: function () {

        var data = this.get('data');
        if (data && data.startDate && data.endDate) {

            var beginDate = data.startDate.format("M/D/YY"),
                endDate = data.endDate.format("M/D/YY");

            this._dateLabel.text('Filtered date range: ' + beginDate + " - " + endDate);
            this.adjustHeight();
        }

        if (this._moreWindow) {
            this._moreWindow.updateDateLabel();
        }
    },

    startDay: function (days) {

        var collection = this.get('data');
        if (days !== undefined) {
            if (collection) {
                collection.startDate = moment(moment().add(days, 'days'), "M/D/YY");
                this._startDay = days;

                if (this._dateFilterPanel) {
                    this._dateFilterPanel.set('startDate', collection.startDate);
                    this._dateFilterPanel.updateDatePickers();
                }
            }

            this.updateDateLabel();

            return days;

        }
        if (collection) {
            if (!collection.startDate) {
                collection.startDate = moment(moment().add(this._startDay, 'days'), "M/D/YY");
            }

            return this._startDay;
        }

    },
    endDay: function (days) {

        var collection = this.get('data');
        if (days !== undefined) {
            if (collection) {
                collection.endDate = moment(moment().add(days, 'days'), "M/D/YY");
                this._endDay = days;

                if (this._dateFilterPanel) {
                    this._dateFilterPanel.set('endDate', collection.endDate);
                    this._dateFilterPanel.updateDatePickers();
                }
            }

            this.updateDateLabel();

            return days;
        }

        if (collection) {

            if (!collection.endDate) {
                collection.endDate = moment(moment().add(this._endDay, 'days'), "M/D/YY");
            }

            return this._endDay;
        }
    },
    datagrid: function () {
        return this._datagrid;
    },
    gridHeaderHeight: function (aNumber) {
        if (this._datagrid) {
            if (_.isNumber(aNumber)) {
                this._datagrid.set('headerHeight', aNumber);
            }

            return this._datagrid.get('headerHeight');
        }
    },
    /**
     * widget id getter/setter
     * @param aString
     * @returns {*}
     */
    widgetId: function (aString) {
        if (aString !== undefined)
            this._widgetId = aString;

        return this._widgetId;
    }
});

$(function () {
    JLV.Widget.DateFilterPanel = AppKit.View.extend({
        className: 'jlv-date-filter-panel',
        template: $('#WidgetDateFilterTemplate').template(),
        widget: null,
        startDate: null,
        endDate: null,
        initialize: function (options) {
            this._super(options);

            var self = this;
            this.subview('.jlv-date-filter-close').on('click', function () {
                if (self.widget) {
                    self.widget.toggleDateFilterPanel();
                }
            }).on('keydown', function (evt) {
                if (evt.which === $.ui.keyCode.SPACE) {
                    if (self.widget) {
                        self.widget.toggleDateFilterPanel();
                        setTimeout(function () {
                            self.widget.$el.find('.filter').focus();
                        }, 0);
                    }
                }
            });


            this.subview('.jlv-date-filter-apply').on('click', function () {
                if (self.widget) {
                    var data = self.widget.get('data');
                    if (data) {

                        data.startDate = moment(self._startDateInput.get('date'));
                        data.endDate = moment(self._endDateInput.get('date'));

                        var startDay = data.startDate.diff(moment(), 'days');
                        if (startDay > 0) {//fix diff rounding error
                            startDay++;
                        }

                        self.widget.startDay(startDay);

                        var endDay = data.endDate.diff(moment(), 'days');
                        if (endDay > 0) {
                            endDay++;
                        }

                        self.widget.endDay(endDay);

                        self.widget.loadData();
                    }
                    if (self.widget.savePortalUI) {
                        self.widget.savePortalUI();
                    }
                }
            });


            this.$el.find('.preset')
                .on('click', function () {
                    self.applyPreset($(this).data('value'));

                })
                .on('keydown', function (evt) {
                    if (evt.which === $.ui.keyCode.SPACE || evt.which === $.ui.keyCode.ENTER) {
                        self.applyPreset($(this).data('value'));
                    }
                });


            this._startDateInput = this.subview('.jlv-start-date-input');
            this._endDateInput = this.subview('.jlv-end-date-input');

            this.updateDatePickers();

            this._startDateInput.on('dateSet', function () {
                self.startDate = moment(this.get('date'));
                self._endDateInput.set('minDate', self.startDate.toDate());
            });

            this._endDateInput.on('dateSet', function () {
                self.endDate = moment(this.get('date'));
                self._startDateInput.set('maxDate', self.endDate.toDate());
            });
        },
        disabled: function (aFlag) {
            if (aFlag !== undefined) {
                this.subview('.jlv-date-filter-apply').set('enabled', !aFlag);
                this._startDateInput.set('enabled', !aFlag);
                this._endDateInput.set('enabled', !aFlag);
            }
        },
        applyPreset: function (value) {
            if (this.widget && !this.widget._dataIsLoading) {

                var data = this.widget.get('data');
                if (data) {

                    data.endDate = moment();
                    data.startDate = moment();

                    if (value === '1w') {
                        data.startDate.subtract(7, 'days');
                    }
                    else if (value === '2w') {
                        data.startDate.subtract(14, 'days');
                    }
                    else if (value === '1m') {
                        data.startDate.subtract(1, 'months');
                    }
                    else if (value === '3m') {
                        data.startDate.subtract(3, 'months');
                    }
                    else if (value === '6m') {
                        data.startDate.subtract(6, 'months');
                    }
                    else if (value === '1y') {
                        data.startDate.subtract(1, 'years');
                    }
                    else if (value === '2y') {
                        data.startDate.subtract(2, 'years');
                    }
                    else if (value === '3y') {
                        data.startDate.subtract(3, 'years');
                    }

                    this.widget.startDay(data.startDate.diff(moment(),'days'));
                    this.widget.endDay(data.endDate.diff(moment(),'days'));

                    this.startDate = data.startDate;
                    this.endDate = data.endDate;

                    this.updateDatePickers();

                    this.widget.loadData();
                    if (this.widget.savePortalUI) {
                        this.widget.savePortalUI();
                    }
                }
            }
        },
        updateDatePickers: function () {
            if (this._startDateInput) {
                if (this.startDate.toDate) {
                    this._startDateInput.set('date', this.startDate.toDate());
                } else {
                    this._startDateInput.set('date', this.startDate);
                }
            }

            if (this._endDateInput) {
                if (this.endDate.toDate) {
                    this._endDateInput.set('date', this.endDate.toDate());
                } else {
                    this._endDateInput.set('date', this.endDate);
                }
            }
        }
    });
});

$(function () {
    JLV.Widget.VlerLoadPanel = AppKit.View.extend({
        className: 'jlv-vler-loading-panel',
        template: $('#WidgetVlerLoadingTemplate').template(),
        widget: null,
        initialize: function (options) {
            this._super(options);

            var self = this;
            this.subview('.jlv-vler-loading').on('click', function () {
                if (self.widget) {
                    //self.widget.hideVlerLoadMessage();
                }
            });

        }
    });


    JLV.Widget.PrintConfirm = function (callback) {

        var dialog = new AppKit.AlertDialog({

            confirmLabel: 'Print',
            cancelLabel: 'Cancel',
            cancelButton: true,
            title: 'Printable PDF Ready',
            message: 'The printable PDF is ready to download. Click Print to proceed.',
            callback: callback

        });

        dialog.center();
        dialog.open();

        return dialog;

    };

    JLV.Widget.DownloadConfirm = function (callback) {


        var dialog = new AppKit.AlertDialog({
            confirmLabel: 'Add to Report Builder',
            cancelLabel: 'Cancel',
            cancelButton: true,
            thirdButton: true,
            thirdLabel: 'Open',
            title: 'Document Ready',
            message: 'Patient Document is ready to download or add to Report Builder',
            callback: callback

        });

        dialog.center();
        dialog.open();

        return dialog;

    };

});


