//= require ./PatientWidget.js
//= require ./windows/PatientInpatientMedicationsWidgetWindow.js


JLV.InpatientMedicationsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'drugName',
    url: 'patientInpatientMedications/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['stopDate', 'dispenseDate', 'startDate', 'orderDate'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system === '2.16.840.1.113883.6.88' ||
                            data.codes[i].system.toUpperCase().indexOf("RXNORM") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            if (!data.orderId) {
                data.orderId = '';
            }

            return this._super(data);
        }
    })
});


JLV.PatientInpatientMedications = JLV.PatientWidget.extend({
    options: {
        widgetId: 'inpatientmedications_widget',
        name: 'Inpatient Medications',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        groupBy: 'orderId',
        groupFormatter: function (value) {
            if (value && (value !== 'null')) {
                return 'Order Number: &nbsp;' + value;
            } else {
                return 'Order Number: none';
            }
        },
        moreWindowClass: 'JLV.PatientInpatientMedicationsWidgetWindow',
        startDay: -365,
        endDay: 0,
        columns: [
            {
                title: 'Order Number',
                id: 'orderId',
                sortable: true,
                width: 100
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 150,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    return JLV.common.linkFormatter(value, model, null, "report");
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                'RxNorm Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 100,
                sortable: true
            },
            {
                title: 'Order<br/>Stop Date',
                id: 'stopDate',
                width: 130,
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm')
                },
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                width: 80,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'orderId',
            asc: false
        },
        data: JLV.InpatientMedicationsCollection
    },
    initialize: function (options) {
        this._super(options);

        var defaultSort = this.get('defaultSort');
        if (defaultSort && defaultSort.id === 'orderId') {
            if (defaultSort.asc) {
                this.set('groupSorter', function (val1, val2) {
                    return (val1 >= val2) ? 1 : -1;
                });
            }
            else {
                this.set('groupSorter', function (val1, val2) {
                    return (val1 <= val2) ? 1 : -1;
                });
            }
        }

        this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {
            if (sortInfo.column.id === 'orderId') {
                if (sortInfo.asc) {
                    this.set('groupSorter', function (val1, val2) {
                        return (val1 >= val2) ? 1 : -1;
                    });
                }
                else {
                    this.set('groupSorter', function (val1, val2) {
                        return (val1 <= val2) ? 1 : -1;
                    });
                }
            }
        }, this));

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.checkConnectionUnavailable();

            }, this)
        });
    },
    itemClicked: function (model, item) {
        if (item.data('route') === 'report') {
            if (model.get('recordDetailsToken')) {
                var dialog = new JLV.DetailsWindow({
                    title: String.format("Inpatient Medication Detail View ~ {0}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                    url: 'patientInpatientMedications/details.json',
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken')
                    },
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function (response) {
                        return '<pre>' + response.medDetail + '</pre>';
                    }
                });

                dialog.center();
                dialog.open();

            }
            else {

                var dialog = new JLV.DetailsWindow({
                    title: String.format("Inpatient Medication Detail View ~ {0}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function () {
                        try {
                            var data = model.toJSON();

                            if (data.orderDate) {
                                data.orderDate = moment(data.orderDate).format("M/D/YY");
                            }

                            return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                            '<td>Ordering Provider Comments:</td><td>{{comment}}</td></tr><tr>' +
                            '<td>Order Date:</td><td>{{orderDate}}</td></tr><tr>' +
                            '<td>Status:</td><td><pre>{{status}}</pre></td></tr><tr>' +
                            '<td>Order Number:</td><td><pre>{{orderId}}</pre></td></tr></table>')(data);
                        } catch (e) {
                        }

                        return '<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                            '<td>Ordering Provider Comments:</td><td></td></tr><tr>' +
                            '<td>Order Date:</td><td></td></tr><tr>' +
                            '<td>Status:</td><td><pre></pre></td></tr><tr>' +
                            '<td>Order Number:</td><td><pre></pre></td></tr></table>';


                    }
                });

                dialog.center();
                dialog.open();
            }
        }
    }
});

JLV.registerJLVWidget('inpatientmedications_widget', JLV.PatientInpatientMedications);