//= require ./PatientWidget.js
//= require ./windows/PatientPrescriptionFillsWindow.js
//= require ./windows/PatientOutpatientMedicationsWidgetWindow.js

JLV.PrescriptionFillsModel = JLV.PatientDataModel.extend({
    dateFields: ['lastFilledDate', 'expirationDate']
});


JLV.OutpatientMedicationsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'drugName',
    url: 'patientOutpatientMedications/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['lastFilledDate', 'stopDate', 'dispenseDate', 'startDate', 'orderDate'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system === '2.16.840.1.113883.6.88' ||
                            data.codes[i].system.toUpperCase().indexOf("RXNORM") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            if (data.prescriptionFills) {
                var count = data.prescriptionFills.length,
                    i = 0;
                for (; i < count; i++) {
                    data.prescriptionFills[i] = new JLV.PrescriptionFillsModel(
                        _.extend(data.prescriptionFills[i], {sig: data.sig}),
                        {parse: true}
                    );
                }
            }

            if (data.childResistant === true) {
                data.childResistant = "YES";
            } else if (data.childResistant === false) {
                data.childResistant = "NO";
            }

            if (data.quantity) {
                data.quantity = parseInt(data.quantity, 10);
            }

            return this._super(data);
        }
    })
});


JLV.PatientOutpatientMedications = JLV.PatientWidget.extend({
    options: {
        widgetId: 'outpatientmedications_widget',
        name: 'Outpatient Medications',
        dateFilter: true,
        status: true,
        clipboard: true,
        copyActiveMeds: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientOutpatientMedicationsWidgetWindow',
        startDay: -365,
        endDay: 0,
        columns: [
            {
                title: 'Last Fill',
                id: 'lastFilledDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 130
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 150,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model.get('manuallyEntered')) {
                        return JLV.common.linkFormatter(value, model, null, "report") + ' <span class="jlv-widget-self-reported-med">(Patient/Family Reported)</span>';
                    }
                    else {
                        return JLV.common.linkFormatter(value, model, null, "report");
                    }

                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                'RxNorm Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 120,
                sortable: true
            },
            {
                title: 'Expires',
                id: 'stopDate',
                width: 130,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                width: 100,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'fillOrderDate',
            asc: false
        },
        data: JLV.OutpatientMedicationsCollection
    },
    initialize: function (options) {
        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.checkConnectionUnavailable();

            }, this)
        });
    },
    itemClicked: function (model, item) {

        var route = item.data('route');
        if (route === "report") {
            if (model.get('recordDetailsToken')) {

                var dialog = new JLV.DetailsWindow({
                    title: String.format("Outpatient Medication Detail View ~ {0}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                    url: 'patientOutpatientMedications/details.json',
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken')
                    },
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function (response) {
                        return '<pre>' + response.medDetail + '</pre>';
                    }
                });

                dialog.center();
                dialog.open();

            }
            else {

                var dialog = new JLV.DetailsWindow({
                    height: 500,
                    title: String.format("Outpatient Medication Detail View ~ {0}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function () {

                        var data = model.toJSON();

                        if (data.orderDate) {
                            data.orderDate = moment(data.orderDate).format("D MMMM YYYY");
                        }

                        data.ndcCode = "";
                        if (data.codes && data.codes.length > 0) {
                            for (var i in data.codes) {
                                if (data.codes[i].system && data.codes[i].code && data.codes[i].system ===
                                    '2.16.840.1.113883.6.69') {
                                    data.ndcCode = data.codes[i].code
                                }
                            }

                        }

                        if (data.dispenseDate) {
                            data.dispenseDate = moment(data.dispenseDate).format("D MMMM YYYY");
                        }

                        if (data.currentSupplyEndDate) {
                            data.currentSupplyEndDate = moment(data.currentSupplyEndDate).format("D MMMM YYYY");
                        }

                        if (data.stopDate) {
                            data.stopDate = moment(data.stopDate).format("D MMMM YYYY");
                        }

                        if (data.lastFilledDate) {
                            data.lastFilledDate = moment(data.lastFilledDate).format("D MMMM YYYY");
                        }

                        try {
                            var notDOD = (JLV.ActiveProvider.agency !== JLV.common.AGENCY_DOD);

                            //data = null; //uncomment to test error display below

                            var html = _.template(
                                '<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table" width="100%">' +
                                '<tbody>' +
                                '<tr><td colspan="2" ><span style="font-weight: bold;">RX Number:</span> {{rxNumber}}</td></tr>' +
                                '<tr>' +
                                '<td><span style="font-weight: bold;">Drug Name:</span> {{drugName}}' +
                                '</td>' +
                                '<td><span style="font-weight: bold;">Units:</span> {{units}}' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td><span style="font-weight: bold;">Sig:</span> {{sig}}' +
                                '</td>' +
                                '<td><span style="font-weight: bold;">Quantity:</span> {{quantity}}' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td><span style="font-weight: bold;">Order Date:</span> {{orderDate}}' +
                                '</td>' +
                                '<td><span style="font-weight: bold;">Order Status:</span> {{status}}' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '{[ if(refillsRemaining && refills) { ]}' +
                                '<td><span style="font-weight: bold;">Refills left:</span> {{refillsRemaining}} of {{refills}}' +
                                '{[ } else { ]}' +
                                '<td><span style="font-weight: bold;">Refills left:</span> {{refillsRemaining}}' +
                                '{[ } ]}' +
                                '</td>' +
                                '<td><span style="font-weight: bold;">Exp Date:</span> {{stopDate}}' +
                                '</td>' +
                                '</tr>' +
                                //((notDOD) ?
                                '<tr>' +
                                '<td colspan=2><span style="font-weight: bold;">Dispense Date:</span> {{lastFilledDate}}' +
                                '</td> ' +
                                '</tr>' +
                                    //: '' ) +
                                '<tr>' +
                                '<td colspan=2><span style="font-weight: bold;">Dispensing Pharmacy:</span> {{pharmacy}}' +
                                '</td> ' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan=2><span style="font-weight: bold;">Current Supply End Date:</span> {{currentSupplyEndDate}} ' +
                                '</td> ' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan=2><span style="font-weight: bold;">Child Resistant Container:</span> {{childResistant}}' +
                                '</td> ' +
                                '</tr>' +
                                '<tr>' +
                                '<td><span style="font-weight: bold;">NDC:</span> {{ndc}}' +
                                '</td>' +
                                '<td><span style="font-weight: bold;">Order Number:</span> {{orderId}}' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan=2><span style="font-weight: bold;">' + ((notDOD) ? 'Ordering Provider Comments:</span>' : 'Prescription Comments:</span>' ) + ' {{comment}}' +
                                '</td>' +
                                '</tr>' +
                                '</tbody>' +
                                '</table></div>'
                            )(data);

                            //html += '<div style="padding:15px;"><table width="100%" height="100%">' +
                            //    '<thead style="border-bottom: solid 1px #3E3F40;">' +
                            //    '<th style="text-align: left;">Fill Date</th><th style="text-align: left;">Dispensing Pharmacy</th></thead>' +
                            //    '<tr><td colspan=2 style="border-top: solid 1px #3E3F40; "></td></tr>' //use empty row for border below headers
                            //;
                            //
                            //for (var pf = data.prescriptionFills.length - 1; pf >=0; pf--) {
                            //    if (data.prescriptionFills[pf] && data.prescriptionFills[pf].attributes && data.prescriptionFills[pf].attributes.lastFilledDate) {
                            //        data.prescriptionFills[pf].attributes.lastFilledDate =
                            //            moment(data.prescriptionFills[pf].attributes.lastFilledDate).format("D MMMM YYYY");
                            //    }
                            //    html += _.template(
                            //        '<tr>' +
                            //        '<td style="text-align: left;">{{lastFilledDate}}</td>' +
                            //        '<td style="text-align: left;">{{dispensingPharmacy}}</td>' +
                            //        '</tr>'
                            //    )(data.prescriptionFills[pf].attributes);
                            //}
                            //
                            //html += '</table>&nbsp;</div>';

                            return html;

                        } catch (e) {
                        }

                        var notDOD = false; //in error display VA form by default

                        return '<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table" width="100%" >' +
                            '<tbody>' +
                            '<tr><td colspan="2" ><span style="font-weight: bold;">RX Number:</span> </td></tr>' +
                            '<tr>' +
                            '<td><span style="font-weight: bold;">Drug Name:</span> ' +
                            '</td>' +
                            '<td><span style="font-weight: bold;">Units:</span> ' +
                            '</td>' +
                            '</tr>' +
                            '<tr>' +
                            '<td><span style="font-weight: bold;">Sig:</span> ' +
                            '</td>' +
                            '<td><span style="font-weight: bold;">Quantity:</span> ' +
                            '</td>' +
                            '</tr>' +
                            '<tr>' +
                            '<td><span style="font-weight: bold;">Order Date:</span> ' +
                            '</td>' +
                            '<td><span style="font-weight: bold;">Order Status:</span> ' +
                            '</td>' +
                            '</tr>' +
                            '<tr>' +
                            '<td><span style="font-weight: bold;">Refills left:</span> ' +
                            '</td>' +
                            '<td><span style="font-weight: bold;">Exp Date:</span> ' +
                            '</td>' +
                            '</tr>' +
                            //((notDOD) ?
                            '<tr>' +
                            '<td colspan=2><span style="font-weight: bold;">Dispense Date:</span> ' +
                            '</td> ' +
                            '</tr>' +
                                //: '' ) +
                            '<tr>' +
                            '<td colspan=2><span style="font-weight: bold;">Dispensing Pharmacy:</span>' +
                            '</td> ' +
                            '</tr>' +
                            '<tr>' +
                            '<td colspan=2><span style="font-weight: bold;">Current Supply Date:</span>  ' +
                            '</td> ' +
                            '</tr>' +
                            '<tr>' +
                            '<td colspan=2><span style="font-weight: bold;">Child Resistant Container:</span> ' +
                            '</td> ' +
                            '</tr>' +
                            '<tr>' +
                            '<td><span style="font-weight: bold;">NDC:</span> ' +
                            '</td>' +
                            '<td><span style="font-weight: bold;">Order Number:</span> ' +
                            '</td>' +
                            '</tr>' +
                            '<tr>' +
                            '<td colspan=2><span style="font-weight: bold;">' + ((notDOD) ? 'Ordering Provider Comments:</span>' : 'Prescription Comments:</span>' ) +
                            '</td>' +
                            '</tr>' +
                            '</tbody>' +
                            '</table></div>' ;
                            //'<div style="padding:15px;"><table width="100%" height="100%">' +
                            //'<thead style="border-bottom: solid 1px #3E3F40;">' +
                            //'<th style="text-align: left;">Fill Date</th><th style="text-align: left;">Dispensing Pharmacy</th></thead>' +
                            //'<tr><td colspan=2 style="border-top: solid 1px #3E3F40; "></td></tr>' + //use empty row for border below headers
                            //'<tr height=100% ><td height=100%></td></tr></table></div>';

                    }
                });

                dialog.center();
                dialog.open();
                $.post('auditUserAction/auditDetails', {
                    recordType: "MedicationDetail",
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recordSiteMoniker: model.get('site').moniker,
                    recordSiteAgency: model.get('site').agency,
                    info: model.get('drugName'),
                    recordItemId: model.get('orderId')
                });
            }
        }
        else {

            if (model.get('prescriptionFills') &&
                model.get('lastFilledDate')) {

                //send ajax rx fills class
                jQuery.post('patientOutpatientMedications/auditRxFillsQuery', {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recordDate: model.get('lastFilledDate').toString(),
                    recordSiteMoniker: model.get('site').moniker,
                    recordSiteAgency: model.get('site').agency
                });

                var dialog = new JLV.PatientPrescriptionFillsWindow({
                    data: new Backbone.Collection(model.get('prescriptionFills'))
                });

                dialog.center();
                dialog.open();
            }
        }

    },
    copyActiveMeds: function (aBool) {
        if (aBool === true) {

            if (!this._copyActiveMedsBtn) {
                var self = this;
                this._copyActiveMedsBtn = $('<div title="Copy active Rxs in date range to clipboard" tabindex="0" class="jlv-widget-toolbutton">')
                    .append('<div class="icon-med-drugs">')
                    .button()
                    .insertBefore(self.$el.find('.clipboard'))
                    .on('click', function () {
                        self._doCopyActiveMeds(true, self.options.columns);
                    });
            }

            this._copyActiveMedsBtn.show();
        }
        else if (aBool === false) {
            this._copyActiveMedsBtn.hide();
        }

        return this._copyActiveMedsBtn.is(':visible');
    },
    _doCopyActiveMeds: function (minimized, allColumns) {

        if (window.clipboardData && window.clipboardData.setData) {

            var returnData = "For Official Use Only\r\n";

            var columns = allColumns,
                hiddenCols = minimized ? this.get('hiddenColumnsMinimized') : this.get('hiddenColumnsExpanded'),
                count = columns.length,
                i = 0;

            for (; i < count; i++) {
                var col = columns[i];
                if (hiddenCols.indexOf(col.id) === -1) {
                    if (i > 0) {
                        returnData += '\t';
                    }
                    returnData += ("" + _.text(col.title));
                }
            }

            returnData += '\r\n';

            var data = this._datagrid.get('source');


            data.each(function (model) {
                var status = model.get('status').toUpperCase()
                if (_.includes(status, 'ACTIVE')) {
                    i = 0;
                    for (; i < count; i++) {
                        var col = columns[i];
                        if (hiddenCols.indexOf(col.id) === -1) {
                            if (col.id === 'lastFilledDate' || col.id === 'stopDate') {
                                returnData += JLV.common.dateFormatter(model.get(col.id)) + "\t";
                            }
                            else if (col.id === 'site') {
                                returnData += model.get('site').moniker + "\r\n";
                            }
                            else {
                                returnData += model.get(col.id) + "\t";
                            }
                        }
                    }
                }
            });

            window.clipboardData.setData('text', returnData);
        }
        else {
            console.info("Copying to clipboard only works in IE8+");
        }
    }
});

JLV.registerJLVWidget('outpatientmedications_widget', JLV.PatientOutpatientMedications);
