/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.AdmissionDetail;
import gov.va.med.jmeadows.webservice.Code;
import gov.va.med.jmeadows.webservice.PatientAdmission;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 *
 *
 */
@Component
public class AdmissionDao extends DaoBase {

    private static final Logger logger = Logger.getLogger(AdmissionDao.class.getName());

    public List<GUIAdmission> getPatientAdmissions(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate) {

        try {
            PatientAdmission[] admissions = getServiceHandler().getPatientAdmissions(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIAdmission> guiAdms = new ArrayList<GUIAdmission>();

            for (PatientAdmission adm : admissions) {
                String admDate = "";
                String dcDate = "";
                String ward = "";
                String regnum = "";
                String admitId = "";
                String providerName = "";
                String diagnosisText = "";
                String diagnosisCode = "";
                String specialty = "";
                String division = "";

                if (adm.getAdmissionDate() != null) {
                    admDate = Utils.formatDisplayDateTime(adm.getAdmissionDate());
                }

                if (adm.getDispositionDate() != null) {
                    dcDate = Utils.formatDisplayDateTime(adm.getDispositionDate());
                }

                if (adm.getWard() != null) {
                    ward = adm.getWard();
                }

                if (adm.getSpecialty() != null) {
                    specialty = adm.getSpecialty();
                }

                if (adm.getDivision() != null) {
                    division = adm.getDivision();
                }

                if (adm.getRegisterNumber() != null) {
                    regnum = adm.getRegisterNumber();
                }

                if (adm.getAdmissionId() != null) {
                    admitId = adm.getAdmissionId();
                }

                if (adm.getAdmittingProvider() != null) {
                    providerName = adm.getAdmittingProvider().getName();
                }

                // if the admitting provider was blank look in provider field
                if ("".equalsIgnoreCase(providerName) && adm.getProvider() != null) {
                    providerName = adm.getProvider().getName();
                }

                // Use the ICD9 code display as the diagnosis text.
                if (adm.getCodes() != null && adm.getCodes().size() > 0)
                {
                    for (Code code : adm.getCodes()) {
                        if (code.getSystem() != null && "ICD9".equalsIgnoreCase(code.getSystem()))
                        {
                            diagnosisCode = code.getDisplay();
                        }
                    }
                }

                if (adm.getDrgDescription() != null) {
                    diagnosisText = adm.getDrgDescription();
                }

                GUIAdmission guiAdm = new GUIAdmission();
                guiAdm.setAdmissionDate(admDate);
                guiAdm.setDischargeDate(dcDate);
                guiAdm.setAdmissionId(admitId);
                guiAdm.setWard(ward);
                guiAdm.setRegistrationNum(regnum);
                guiAdm.setDiagnosisText(diagnosisText);
                guiAdm.setDiagnosisCode(diagnosisCode);
                guiAdm.setProviderName(providerName);
                guiAdm.setSpecialty(specialty);
                guiAdm.setDivision(division);

                if (adm.getProvider() != null) {
                    guiAdm.setProvider(dozerBeanMapper.map(adm.getProvider(), GUIProvider.class));
                }


                addSiteDataToGUIBean(guiAdm, adm);

                List<GUICode> guiCodes = new ArrayList<GUICode>();
                for (Code code : adm.getCodes()) {
                    guiCodes.add(dozerBeanMapper.map(code, GUICode.class));
                }
                guiAdm.setCodes(guiCodes);

                guiAdms.add(guiAdm);
            }
            if (logger.isDebugEnabled()) {
                logger.debug("End");
            }

            return guiAdms;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIAdmissionDetail getPatientAdmissionDetails(GUIUser user, GUIPatient patient, String admissionId,
                                                         String recordSiteCode) {
        try {
            AdmissionDetail admissionDetail = getServiceHandler().
                    getAdmissionDetails(

                            jMeadowsQueryBuilderFactory.getBuilder()
                                    .user(user)
                                    .patient(patient)
                                    .itemId(admissionId)
                                    .recordSiteCode(recordSiteCode)
                                    .build());


            GUIAdmissionDetail guiAdmissionDetail = new GUIAdmissionDetail();
            guiAdmissionDetail.setDrgDescription(admissionDetail.getDrgDescription());
            addSiteDataToGUIBean(guiAdmissionDetail, admissionDetail);

            return guiAdmissionDetail;


        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIAdmissionDetail getPatientAdmissionDetails(GUIUser user, GUIPatient patient, String admissionId, String recordSiteCode, Calendar startDate, Calendar endDate) {
        try {
            AdmissionDetail admissionDetail = getServiceHandler().
                    getAdmissionDetails(
                            jMeadowsQueryBuilderFactory.getBuilder()
                                    .user(user)
                                    .patient(patient)
                                    .startDate(startDate)
                                    .endDate(endDate)
                                    .itemId(admissionId)
                                    .recordSiteCode(recordSiteCode)
                                    .build());

            GUIAdmissionDetail guiAdmissionDetail = new GUIAdmissionDetail();
            guiAdmissionDetail.setDetailText(StringEscapeUtils.escapeHtml4(admissionDetail.getDetailText()));
            addSiteDataToGUIBean(guiAdmissionDetail, admissionDetail);

            return guiAdmissionDetail;


        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public List<GUIAdmission> getProviderAdmissions(
            GUIUser user,
            Calendar startDate,
            Calendar endDate,
            byte[] encryptSecurityKey) {
        try {
            PatientAdmission[] admissions = getServiceHandler().getProviderAdmissions(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIAdmission> guiAdms = new ArrayList<GUIAdmission>();

            for (PatientAdmission adm : admissions) {
                String admDate = "";
                String dcDate = "";
                String ward = "";
                String regnum = "";
                String admitId = "";
                String providerName = "";
                String diagnosisText = "";
                String patName = "";
                String patToken = "";

                if (adm.getAdmissionDate() != null) {
                    admDate = Utils.formatDisplayDateTime(adm.getAdmissionDate());
                }

                if (adm.getDispositionDate() != null) {
                    dcDate = Utils.formatDisplayDateTime(adm.getDispositionDate());
                }

                if (adm.getWard() != null) {
                    ward = adm.getWard();
                }

                if (adm.getRegisterNumber() != null) {
                    regnum = adm.getRegisterNumber();
                }

                if (adm.getAdmissionId() != null) {
                    admitId = adm.getAdmissionId();
                }

                if (adm.getAdmittingProvider() != null) {
                    providerName = adm.getAdmittingProvider().getName();
                }

                if (adm.getDrgDescription() != null) {
                    diagnosisText = adm.getDrgDescription();
                }

                if (adm.getPatientName() != null) {
                    patName = adm.getPatientName();
                }

                patToken = getProvQueryPatientToken(encryptSecurityKey, adm);

                GUIAdmission guiAdm = new GUIAdmission();
                guiAdm.setAdmissionDate(admDate);
                guiAdm.setDischargeDate(dcDate);
                guiAdm.setAdmissionId(admitId);
                guiAdm.setWard(ward);
                guiAdm.setRegistrationNum(regnum);
                guiAdm.setDiagnosisText(diagnosisText);
                guiAdm.setProviderName(providerName);
                addSiteDataToGUIBean(guiAdm, adm);
                guiAdm.setPatientName(patName);
                guiAdm.setPatientToken(patToken);

                guiAdms.add(guiAdm);
            }

            return guiAdms;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }
}
