/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */

package gov.va.med.dao;

import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.Allergy;
import gov.va.med.jmeadows.webservice.AllergyDetail;
import gov.va.med.jmeadows.webservice.Code;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.xml.datatype.XMLGregorianCalendar;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

@Component
public class AllergyDao extends DaoBase {

    /**
     * Retrieves a list of patient allergies from all available patient locations.
     *
     * @param user    {@link gov.va.med.guibeans.GUIUser} making the request.
     * @param patient {@link gov.va.med.guibeans.GUIPatient} patient retrieving allergies from this patient record.
     * @return A list of patient allergies.
     */
    public List<GUIAllergy> getPatientAllergies(GUIUser user, GUIPatient patient, boolean includeVLER, boolean includeDASVLER) {
        try {
            Allergy[] allergies = getServiceHandler().getPatientAllergies(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .includeVLER(includeVLER)
                            .includeDASVLER(includeDASVLER)
                            .build());

            List<GUIAllergy> guiAllergies = new ArrayList<GUIAllergy>();

            for (Allergy allergy : allergies) {
                GUIAllergy guiAllergy = mapToGUIAllergy(allergy);


//                List<GUICode> guiAllergyCodes = new ArrayList<GUICode>();
//                for (Code allergyCode : allergy.getCodes()) {
//                    guiAllergyCodes.add(dozerBeanMapper.map(allergyCode, GUICode.class));
//                }
//                guiAllergy.setCodes(guiAllergyCodes);
                guiAllergies.add(guiAllergy);

            }

            return guiAllergies;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    /**
     * Retrieves a patient allergy details from the provided record location.
     *
     * @param user           {@link gov.va.med.guibeans.GUIUser} making the request.
     * @param patient        {@link gov.va.med.guibeans.GUIPatient} patient retrieving allergy details from this patient record.
     * @param allergyId      Internal entry number of the patient allergy file.
     * @param recordSiteCode Site code of allergy record.
     * @return AllergyDetail containing allergy report text.
     */
    public GUIAllergyDetail getAllergyDetail(GUIUser user, GUIPatient patient,
                                             String allergyId, String recordSiteCode) {
        try {
            AllergyDetail allergyDetail = getServiceHandler().
                    getAllergyDetail(
                            jMeadowsQueryBuilderFactory.getBuilder()
                                    .user(user)
                                    .patient(patient)
                                    .itemId(allergyId)
                                    .recordSiteCode(recordSiteCode)
                                    .build());

            String reportText = allergyDetail.getReportText();
            allergyDetail.setReportText(StringEscapeUtils.escapeHtml4(reportText));
            return dozerBeanMapper.map(allergyDetail, GUIAllergyDetail.class);

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    private GUIAllergy mapToGUIAllergy(Allergy allergy) {
        GUIAllergy guiAllergy = dozerBeanMapper.map(allergy, GUIAllergy.class);

        String enteredDate = "";
        String onsetDate = "";
        StringBuilder reactionSB = new StringBuilder();

        if (allergy.getEnteredDate() != null) {
            enteredDate = Utils.formatDisplayDateTime(allergy.getEnteredDate());
        }

        if (allergy.getOnsetDate() != null) {
            XMLGregorianCalendar calX = allergy.getOnsetDate();
            SimpleDateFormat sdf = new SimpleDateFormat("MMM dd, yyyy");
            sdf.setTimeZone(calX.getTimeZone(-2147483648));
            onsetDate = sdf.format(calX.toGregorianCalendar().getTime());
        }

        if (allergy.getReactions() != null) {
            List<Code> reactions = allergy.getReactions();
            for (int i = 0; i < reactions.size(); i++) {
                if (i > 0) {
                    reactionSB.append("; ");
                }
                reactionSB.append(reactions.get(i).getDisplay());
            }
        }

        guiAllergy.setEnteredDate(enteredDate);
        guiAllergy.setOnsetDate(onsetDate);
        guiAllergy.setReaction(reactionSB.toString());

        for(GUICode guicode: guiAllergy.getCodes()){
            guiAllergy.setCodeSystem("RXNORM");
            if(guicode != null && StringUtils.isNotEmpty(guicode.getSystem())) {
                if ("2.16.840.1.113883.6.88".equals(guicode.getSystem()) || "RXNORM".equalsIgnoreCase(guicode.getSystem())) {
                    guiAllergy.setCodeName(guicode.getDisplay());
                    guiAllergy.setCodeNumber(guicode.getCode());
                } else if ("2.16.840.1.113883.6.96".equals(guicode.getSystem())) {
                    guiAllergy.setCodeName(guicode.getDisplay());
                    guiAllergy.setCodeNumber(guicode.getCode());
                    guiAllergy.setCodeSystem("SNOMED");
                } else if ("2.16.840.1.113883.3.26.1.5".equals(guicode.getSystem())) {
                    guiAllergy.setCodeName(guicode.getDisplay());
                    guiAllergy.setCodeNumber(guicode.getCode());
                    guiAllergy.setCodeSystem("NDFRT");
                }
            }
        }

        return guiAllergy;

    }
}
