/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.enterprise.bean.ResponseGUIPatientQuery;
import gov.va.med.enterprise.bean.ResponseGUIPatientSelect;
import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.*;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import javax.xml.soap.SOAPException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 *
 *
 */
@Component
public class PatientDao extends DaoBase {
    private static final Logger logger = Logger.getLogger(PatientDao.class.getName());
    private static final SimpleDateFormat simpleDateFormatOrganDonorInd = new SimpleDateFormat("MMMM d, yyyy");

    public ResponseGUIPatientQuery lookupPatientPDWS(GUIUser user, PdwsQueryBean queryBean, byte[] encryptSecurityKey) {
        if (user.isRestrictedUser()) {
            throw new DaoException(new Exception("You have a restricted user account that does not have authorization to perform a PDWS patient lookup"));
        }

        try {
            User querybeanUser = dozerBeanMapper.map(user, User.class);
            if (CollectionUtils.isNotEmpty(user.getRestrictedSiteCodesList())) {
                querybeanUser.getRestrictedSiteCodesList().addAll(user.getRestrictedSiteCodesList());
            }
            queryBean.setUser(querybeanUser);
            queryBean.setRequestingApp(appConfig.getAppName());

            ResponsePatientQuery respPatQuery = getServiceHandler().lookupPatientPDWS(queryBean);
            List<Patient> patients = respPatQuery.getPatients();

            ResponseGUIPatientQuery respGUIPatQuery = new ResponseGUIPatientQuery();
            List<GUILookupPatient> guiLookupPatients = new ArrayList<GUILookupPatient>();

            // Going to compare user's SSN with patient's SSN to prevent user from viewing their own medical records
            String userSSN = null;
            if (user.getEncryptedFederatedUid() != null)
                userSSN = EncryptUtil.decrypt(user.getEncryptedFederatedUid());
            String dob;
            String gender;

            for (Patient patient : patients) {
                String fmpssn = patient.getSSN();
                fmpssn = Utils.formattedSSN(fmpssn);
                dob = patient.getDob();
                if (dob == null || dob.isEmpty()) {
                    dob = "N/A";
                }
                gender = patient.getGender();
                if (gender == null || gender.isEmpty()) {
                    gender = "N/A";
                }

                String patientToken = generatePatientToken(encryptSecurityKey, appConfig.getIdTypeEDIPI(), patient.getEDIPI());

                GUILookupPatient guiLookupPatient = new GUILookupPatient();
                guiLookupPatient.setName(patient.getName());
                guiLookupPatient.setEdipi(patient.getEDIPI());
                guiLookupPatient.setDob(dob);
                guiLookupPatient.setFmpssn(fmpssn);
                guiLookupPatient.setSponsorSsn(patient.getSponsorSSN());
                guiLookupPatient.setGender(gender);
                guiLookupPatient.setSensitive(patient.isSensitive());
                guiLookupPatient.setPatientToken(patientToken);

                // User is VA
                if ("VA".equalsIgnoreCase(user.getAgency())) {
                    // User is in list of searched patients
                    if (userSSN != null && userSSN.equalsIgnoreCase(fmpssn)) {
                        // Prefix patientToken to indicate this patient is attempting to select self.
                        // Still need token; later selected patient is used to query vista for DG Sensitive alert level 3.
                        guiLookupPatient.setPatientToken("selfselectaccessdenied_" + patientToken);
                    }
                }

                guiLookupPatients.add(guiLookupPatient);
            }

            respGUIPatQuery.setSuccess(respPatQuery.isSuccess());
            respGUIPatQuery.setErrorMsg(respPatQuery.getErrorMsg());
            respGUIPatQuery.setGuiPatients(guiLookupPatients);

            return respGUIPatQuery;
        } catch (SOAPException ex) {
            throw new DaoException(ex);
        }
    }

    public ResponseGUIPatientQuery lookupPatientMVI(GUIUser user, PdwsQueryBean queryBean, byte[] encryptSecurityKey) {
        if (user.isRestrictedUser()) {
            throw new DaoException(new Exception("You have a restricted user account that does not have authorization to perform a patient lookup"));
        }
        ResponseGUIPatientQuery respGUIPatQuery = new ResponseGUIPatientQuery();
        try {
            User querybeanUser = dozerBeanMapper.map(user, User.class);
            querybeanUser.getRestrictedSiteCodesList().addAll(user.getRestrictedSiteCodesList());
            queryBean.setUser(querybeanUser);
            queryBean.setRequestingApp(appConfig.getAppName());

            ResponsePatientQuery respPatQuery = getServiceHandler().lookupPatientMVI(queryBean);
            List<Patient> patients = respPatQuery.getPatients();


            List<GUILookupPatient> guiLookupPatients = new ArrayList<GUILookupPatient>();

            // Going to compare user's SSN with patient's SSN to prevent user from viewing their own medical records
            String userSSN = null;
            if (user.getEncryptedFederatedUid() != null)
                userSSN = EncryptUtil.decrypt(user.getEncryptedFederatedUid());

            for (Patient patient : patients) {
                String dob = "";
                String gender = "";
                String icn = "";
                String ien = "";
                String siteCode = "";

                String fmpssn = patient.getSSN();
                fmpssn = Utils.formattedSSN(fmpssn);
                dob = patient.getDob();
                if (dob == null || dob.isEmpty()) {
                    dob = "N/A";
                }
                gender = patient.getGender();
                if (gender == null || gender.isEmpty()) {
                    gender = "N/A";
                }
                String patientToken = "";

                if (patient.getPatientIens() != null && !patient.getPatientIens().isEmpty()) {
                    siteCode = patient.getPatientIens().get(0).getKey().toString();
                    ien = patient.getPatientIens().get(0).getValue().toString();
                }

                if (StringUtils.isNotEmpty(patient.getICN())) {
                    icn = patient.getICN();
                }

                if (StringUtils.isNotEmpty(patient.getEDIPI())) {
                    patientToken = generatePatientToken(encryptSecurityKey, appConfig.getIdTypeEDIPI(), patient.getEDIPI());
                } else if (StringUtils.isNotEmpty(patient.getICN())) {
                    patientToken = generatePatientToken(encryptSecurityKey, appConfig.getIdTypeICN(), patient.getICN());
                } else if (patient.getPatientIens() != null && !patient.getPatientIens().isEmpty()) {
                    patientToken = tokenUtils.generatePatientToken(encryptSecurityKey, appConfig.getIdTypeVAIen(), ien, siteCode);
                }

                GUILookupPatient guiLookupPatient = new GUILookupPatient();
                guiLookupPatient.setName(patient.getName());
                guiLookupPatient.setEdipi(patient.getEDIPI());
                guiLookupPatient.setDob(dob);
                guiLookupPatient.setFmpssn(fmpssn);
                guiLookupPatient.setSponsorSsn(patient.getSponsorSSN());
                guiLookupPatient.setGender(gender);
                guiLookupPatient.setSensitive(patient.isSensitive());
                guiLookupPatient.setPatientToken(patientToken);
                guiLookupPatient.setIcn(icn);
                guiLookupPatient.setIen(ien);
                guiLookupPatient.setSiteCode(siteCode);

                if (userSSN != null && userSSN.equalsIgnoreCase(fmpssn)) {
                    // Prefix patientToken to indicate this patient is attempting to select self.
                    // Still need token; later selected patient is used to query vista for DG Sensitive alert level 3.
                    guiLookupPatient.setPatientToken("selfselectaccessdenied_" + patientToken);
                }

                guiLookupPatients.add(guiLookupPatient);
            }

            respGUIPatQuery.setSuccess(respPatQuery.isSuccess());
            respGUIPatQuery.setErrorMsg(respPatQuery.getErrorMsg());
            respGUIPatQuery.setGuiPatients(guiLookupPatients);


        } catch (SOAPException ex) {
            throw new DaoException(ex);
        } catch (Exception e){
            logger.error("MVI Lookup error: " + e.getMessage());
            throw new DaoException(e);
        }
        return respGUIPatQuery;
    }

    public ResponseGUIPatientSelect selectPatient(GUIUser user, Patient patient, String patientId, String idType, String patientSiteCode) {
        try {
            if (user.isRestrictedUser()) {
                boolean validSelect = false;

                foundId:
                if ((idType.equals("VAIEN") || idType.equals("ICN")) && StringUtils.isNotBlank(patientId)) {
                    //todo add a check by icn here for RU list
                    for (GUIPatient gp : user.getRestrictedPatients()) {
                        if (patientId.equals(gp.getICN())) {
                            validSelect = true;
                            break foundId;
                        }
                    }
                }
                if (!validSelect)
                    throw new DaoException(new Exception("You have a restricted user account that does not have authorization to view this patient"));
            }

            JMeadowsQuery queryBean =
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .itemId(patientId)
                            .status(idType)
                            .recordSiteCode(patientSiteCode)
                            .build();

            // Selected patient's info to be stored in the user's recently viewed patients table
            if (patient != null) {
                queryBean.setPatient(patient);
            }

            ResponsePatientSelect response = getServiceHandler().selectPatientMVI(queryBean);

            ResponseGUIPatientSelect guiResponse = new ResponseGUIPatientSelect();

            Patient responsePatient = response.getPatient();

            guiResponse.setSuccess(response.isSuccess());
            guiResponse.setVARestricted(response.isVARestricted());
            guiResponse.setAllowVAAccess(response.isAllowVAAccess());
            guiResponse.setErrorMsg(response.getErrorMsg());
            guiResponse.setGUIPatient(
                    dozerBeanMapper.map(responsePatient, GUIPatient.class));

            guiResponse.getGUIPatient().setHieSharingOptIn(responsePatient.isHIESharingOptIn());
            if (responsePatient.getHIESharingOptInLastUpdateDate() != null) {
                guiResponse.getGUIPatient().setHieSharingOptInLastUpdateDate(
                        Utils.formatDisplayDate(Utils.XMLGregorianCalendarToDate(responsePatient.getHIESharingOptInLastUpdateDate())));
            }
            List<GUIExternalID> externalIDs = new ArrayList<GUIExternalID>();
            for (ExternalID eid : response.getExternalIDs()) {
                externalIDs.add(dozerBeanMapper.map(eid, GUIExternalID.class));
            }

            guiResponse.setExternalIDs(externalIDs);

            guiResponse.setSensitivePatientAlerts(response.getSensitivePatientAlerts());

            return guiResponse;
        } catch (SOAPException ex) {
            throw new DaoException(ex);
        }
    }

    public void selectPatientForVASensitive(GUIUser user, GUIPatient patient) {
        try {
            getServiceHandler().selectPatientForVASensitive(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .build());
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }


    public Alert[] getVASensitivePatientAlerts(GUIUser user, GUIPatient patient) {
        try {
            Alert[] alerts = getServiceHandler().getVASensitivePatientAlerts(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .build());
            return alerts;
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public void auditVARestrictedAccess(GUIUser user, GUIPatient patient, String auditText) {
        try {
            JMeadowsQuery queryBean =
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .build();

            queryBean.setInfo(auditText);

            getServiceHandler().auditVARestrictedAccess(queryBean);
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public List<GUIPatientDemographics> getPatientDemographics(GUIUser user, GUIPatient patient, boolean includeVLER, boolean includeDASVLER) {
        try {
            List<PatientDemographics> patDemos = getServiceHandler().getPatientDemographics(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .includeVLER(includeVLER)
                            .includeDASVLER(includeDASVLER)
                            .build());

            List<GUIPatientDemographics> guiPatDemos = new ArrayList<GUIPatientDemographics>();

            for (PatientDemographics patDemo : patDemos) {
                String name = "";
                String address1 = "";
                String address2 = "";
                String city = "";
                String state = "";
                String zipCode = "";
                String addressType = "";
                String fmpssn = "";
                String edipi = "";
                String pcm = "";
                String pcmPhone = "";
                String pcmDate = "";
                String pcmClinic = "";
                String pcTeam = "";
                String age = "";
                String dob = "";
                String gender = "";
                String race = "";
                String phone1 = "";
                String phone2 = "";
                boolean isCodeGray = false;
                String codeGrayMsg = "";
                String percentServiceConnected = "";
                String clinic = "";
                String rank = "";
                String fmp = "";
                String sponsorName = "";
                String languages = "";
                String religion = "";
                String maritalStatus = "";


                if (patDemo.getName() != null) {
                    name = patDemo.getName();
                }

                if (patDemo.getAddress() != null && patDemo.getAddress().size() > 0 && patDemo.getAddress().get(0) != null) {
                    if (patDemo.getAddress().get(0).getStreet1() != null) {
                        address1 = patDemo.getAddress().get(0).getStreet1();
                    }

                    if (patDemo.getAddress().get(0).getStreet2() != null) {
                        address2 = patDemo.getAddress().get(0).getStreet2();
                    }

                    if (patDemo.getAddress().get(0).getCity() != null) {
                        city = patDemo.getAddress().get(0).getCity();
                    }

                    if (patDemo.getAddress().get(0).getStateProvince() != null) {
                        state = patDemo.getAddress().get(0).getStateProvince();
                    }

                    if (patDemo.getAddress().get(0).getPostalCode() != null) {
                        zipCode = patDemo.getAddress().get(0).getPostalCode();
                    }
                    if (patDemo.getAddress().get(0).getType() != null) {
                        addressType = patDemo.getAddress().get(0).getType();
                    }
                }

                if (patDemo.getSSN() != null) {
                    fmpssn = patDemo.getSSN();
                }

                if (patDemo.getPrimaryProvider() != null) {
                    pcm = patDemo.getPrimaryProvider();
                }

                if (patDemo.getPcmPhone() != null) {
                    pcmPhone = patDemo.getPcmPhone();
                }

                if (patDemo.getPcmDates() != null) {
                    pcmDate = Utils.formatDisplayDate(Utils.XMLGregorianCalendarToDate(patDemo.getPcmDates()));
                }

                if (patDemo.getPcmClinic() != null) {
                    pcmClinic = patDemo.getPcmClinic();
                }

                if (patDemo.getPcTeam() != null) {
                    pcTeam = patDemo.getPcTeam();
                }

                if (patDemo.getAge() != null) {
                    age = patDemo.getAge();
                }

                if (patDemo.getDob() != null) {
                    dob = patDemo.getDob();
                }

                if (patDemo.getGender() != null) {
                    gender = patDemo.getGender();
                }

                if (patDemo.getRace() != null && !patDemo.getRace().isEmpty()) {
                    race = patDemo.getRace().get(0).getDisplay();
                }

                if (patDemo.getPhone1() != null) {
                    phone1 = patDemo.getPhone1();
                }

                if (patDemo.getPhone2() != null) {
                    phone2 = patDemo.getPhone2();
                }

                if (patDemo.getCodeGreen() != null || (patDemo.getFlags() != null && !patDemo.getFlags().isEmpty())) {
                    if ("DODADAPTER".equals(patDemo.getSourceProtocol())) {
                        for (Flag flag : patDemo.getFlags()) {
                            //avoid counting the DES status record
                            if (!StringUtils.isEmpty(flag.getName()) && !flag.getName().toLowerCase().contains("bdastatus")) {
                                isCodeGray = true;
                                break;
                            }
                        }
                    } else if (!patDemo.getCodeGreen().equals("0") && patDemo.getCodeGreen().length() > 0) {
                        isCodeGray = true;
                        codeGrayMsg = patDemo.getCodeGreen();
                    }
                }

                if (patDemo.getPercentServiceConnected() != null) {
                    percentServiceConnected = patDemo.getPercentServiceConnected();
                }

                if (patDemo.getClinic() != null) {
                    clinic = patDemo.getClinic();
                }

                if (patDemo.getFMP() != null) {
                    fmp = patDemo.getFMP();
                }

                if (patDemo.getEDIPI() != null) {
                    edipi = patDemo.getEDIPI();
                }

                if (patDemo.getRank() != null) {
                    rank = patDemo.getRank();
                }

                if (patDemo.getSponsorName() != null) {
                    sponsorName = patDemo.getSponsorName();
                }
//
//                String languages = "";
//                String religion = "";
//                String maritalStatus = "";

                if (patDemo.getLanguages() != null) {
                    StringBuilder langSb = new StringBuilder();
                    int count = 0;
                    for (String language : patDemo.getLanguages()) {
                        if (count > 0) {
                            langSb.append("; ");
                        }
                        langSb.append(language);
                        count++;

                    }
                    languages = langSb.toString();
                }

                if (patDemo.getReligion() != null) {
                    StringBuilder relSb = new StringBuilder();
                    int count = 0;
                    for (String rel : patDemo.getReligion()) {
                        if (count > 0) {
                            relSb.append("; ");
                        }
                        relSb.append(rel);
                        count++;

                    }
                    religion = relSb.toString();
                }

                List<GUIPerson> emergencyContacts = new ArrayList<GUIPerson>();
                if (patDemo.getEmergencyContact() != null && patDemo.getEmergencyContact().size() > 0) {
                    for (Person person : patDemo.getEmergencyContact()) {
                        emergencyContacts.add(dozerBeanMapper.map(person, GUIPerson.class));
                    }
                }

                List<GUIPerson> nextOfKin = new ArrayList<GUIPerson>();
                if (patDemo.getNextOfKin() != null && patDemo.getNextOfKin().size() > 0) {
                    for (Person person : patDemo.getNextOfKin()) {
                        nextOfKin.add(dozerBeanMapper.map(person, GUIPerson.class));
                    }
                }

                List<GUIPerson> sponsors = new ArrayList<GUIPerson>();
                if (patDemo.getSponsors() != null && patDemo.getSponsors().size() > 0) {
                    for (Person person : patDemo.getSponsors()) {
                        sponsors.add(dozerBeanMapper.map(person, GUIPerson.class));
                    }
                }

                GUIPatientDemographics guiPatDemo = new GUIPatientDemographics();
                guiPatDemo.setName(name);
                guiPatDemo.setAddress1(address1);
                guiPatDemo.setAddress2(address2);
                guiPatDemo.setCity(city);
                guiPatDemo.setState(state);
                if (addressType.length() > 1) {
                    guiPatDemo.setZipCode(zipCode + " (" + addressType + ")");
                } else {
                    guiPatDemo.setZipCode(zipCode);
                }
                guiPatDemo.setFmpssn(fmpssn);
                guiPatDemo.setPcm(pcm);
                guiPatDemo.setPcmPhone(pcmPhone);
                guiPatDemo.setPcmDate(pcmDate);
                guiPatDemo.setPcmClinic(pcmClinic);
                guiPatDemo.setPcTeam(pcTeam);
                guiPatDemo.setAge(age);
                guiPatDemo.setDob(dob);
                guiPatDemo.setGender(gender);
                guiPatDemo.setRace(race);
                guiPatDemo.setPhone1(phone1);
                guiPatDemo.setPhone2(phone2);
                guiPatDemo.setCodeGray(isCodeGray);
                guiPatDemo.setCodeGrayMessage(codeGrayMsg);
                guiPatDemo.setPercentServiceConnected(percentServiceConnected);
                guiPatDemo.setClinic(clinic);
                guiPatDemo.setEdipi(edipi);

                guiPatDemo.setReligion(religion);
                guiPatDemo.setMaritalStatus(patDemo.getMaritalStatus());
                guiPatDemo.setLanguages(languages);

                guiPatDemo.setEmergencyContact(emergencyContacts);
                guiPatDemo.setNextOfKin(nextOfKin);
                guiPatDemo.setSponsors(sponsors);
                guiPatDemo.setRank(rank);
                guiPatDemo.setFmp(fmp);
                guiPatDemo.setSponsorName(sponsorName);

                List<GUICode> guiCodes = new ArrayList<GUICode>();
                for (Code code : patDemo.getRace()) {
                    guiCodes.add(dozerBeanMapper.map(code, GUICode.class));
                }
                guiPatDemo.setCodes(guiCodes);

                List<GUIFlag> guiFlags = new ArrayList<GUIFlag>();
                for (Flag flag : patDemo.getFlags()) {
                    GUIFlag guiFlag = new GUIFlag();
                    guiFlag.setName(flag.getName());
                    guiFlag.setContent(StringEscapeUtils.escapeHtml4(flag.getContent()));
                    guiFlag.setComment(StringEscapeUtils.escapeHtml4(flag.getComment()));
                    if (flag.getSite() != null) {
                        guiFlag.setSite(dozerBeanMapper.map(flag.getSite(), GUISite.class));
                    }
                    guiFlags.add(guiFlag);
                }
                guiPatDemo.setFlags(guiFlags);

                addSiteDataToGUIBean(guiPatDemo, patDemo);
                guiPatDemos.add(guiPatDemo);

                if (patDemo.getOrganDonorInd() != null) {
                    StringBuilder indicatorAndDateSB = new StringBuilder();
                    if ("true".equalsIgnoreCase(patDemo.getOrganDonorInd())) {
                        indicatorAndDateSB.append("Yes");
                    }
                    else {
                        indicatorAndDateSB.append("No");
                    }
                    if (patDemo.getOrganDonorIndDate() != null) {
                        try {
                            String dstr = simpleDateFormatOrganDonorInd.format(patDemo.getOrganDonorIndDate().toGregorianCalendar().getTime());
                            if (dstr != null) {
                                indicatorAndDateSB.append(" (");
                                indicatorAndDateSB.append(dstr);
                                indicatorAndDateSB.append(")");
                            }
                        } catch (NumberFormatException e) {
                            logger.error("error formatting organ donor indication date: " + e);
                        }
                    }

                    guiPatDemo.setOrganDonorIndDateDisplay(indicatorAndDateSB.toString());
                }
                if (patDemo.getBloodType() != null)
                    guiPatDemo.setBloodType(patDemo.getBloodType());

            }

            //sort by site moniker
            Collections.sort(guiPatDemos, new SiteAlphaComparator());

            //push lookup site to the top of the list
            String hostSiteCode = user.getHostSite().getSiteCode();

            int localSiteIdx = -1;
            int demosSize = guiPatDemos.size();
            for (int i = 0; i < demosSize; i++) {
                GUIPatientDemographics guiPatDemo = guiPatDemos.get(i);
                if (guiPatDemo.getSite().getSiteCode().equals(hostSiteCode)) {
                    localSiteIdx = i;
                    break;
                }
            }

            if (localSiteIdx > -1) {
                GUIPatientDemographics guiPatDemo = guiPatDemos.get(localSiteIdx);
                guiPatDemos.remove(localSiteIdx);
                guiPatDemos.add(0, guiPatDemo);
            }

            return guiPatDemos;
        } catch (SOAPException ex) {
            throw new DaoException(ex);
        }
    }

    public List<GUIPatientDemographicDetail> getPatientDemographicDetails(GUIUser user, GUIPatient patient) {
        if (user.isRestrictedUser()) {
            throw new DaoException(new Exception("You have a restricted user account that does not have authorization to perform this patient demographics detail select"));
        }
        try {
            List<PatientDemographicsDetail> patDemoDetails = getServiceHandler()
                    .getAllPatientDemographicDetails(
                            jMeadowsQueryBuilderFactory.getBuilder()
                                    .user(user)
                                    .patient(patient)
                                    .build());
            List<GUIPatientDemographicDetail> guiPatDemoDetails = new ArrayList<GUIPatientDemographicDetail>();

            for (PatientDemographicsDetail patDemoDetail : patDemoDetails) {
                GUIPatientDemographicDetail guiDemoDetail = new GUIPatientDemographicDetail();
                guiDemoDetail.setReport(StringEscapeUtils.escapeHtml4(patDemoDetail.getReport()));

                addSiteDataToGUIBean(guiDemoDetail, patDemoDetail);

                guiPatDemoDetails.add(guiDemoDetail);
            }

            //sort by site moniker
            Collections.sort(guiPatDemoDetails, new SiteAlphaComparator());

            //push lookup site to the top of the list
            String hostSiteCode = user.getHostSite().getSiteCode();

            int localSiteIdx = -1;
            int detailsSize = guiPatDemoDetails.size();
            for (int i = 0; i < detailsSize; i++) {
                GUIPatientDemographicDetail guiPatDemo = guiPatDemoDetails.get(i);
                if (guiPatDemo.getSite().getSiteCode().equals(hostSiteCode)) {
                    localSiteIdx = i;
                    break;
                }
            }

            if (localSiteIdx > -1) {
                GUIPatientDemographicDetail guiPatDemo = guiPatDemoDetails.get(localSiteIdx);
                guiPatDemoDetails.remove(localSiteIdx);
                guiPatDemoDetails.add(0, guiPatDemo);
            }

            return guiPatDemoDetails;
        } catch (SOAPException ex) {
            throw new DaoException(ex);
        }
    }
}
