/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.Code;
import gov.va.med.jmeadows.webservice.Problem;
import gov.va.med.jmeadows.webservice.ProblemDetail;
import gov.va.med.jmeadows.webservice.ProblemNote;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 *
 *
 */
@Component
public class ProblemDao extends DaoBase {

    private static final Logger logger = Logger.getLogger(ProblemDao.class.getName());

    public List<GUIProblem> getPatientProblemList(GUIUser user, GUIPatient patient, String status, boolean includeVLER, boolean includeDASVLER) {
        try {
            Problem[] problems = getServiceHandler().getPatientProblemList(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .status(status)
                            .includeVLER(includeVLER)
                            .includeDASVLER(includeDASVLER)
                            .build());
            List<GUIProblem> guiProblems = new ArrayList<GUIProblem>();
            for (int i = 0; i < problems.length; i++) {
                Problem problem = problems[i];

                GUIProblem guiProblem = new GUIProblem();

                mapProblemData(problem, guiProblem);

                if (appConfig.getProtocolDODAdapter().equalsIgnoreCase(problem.getSourceProtocol())) {
                    //problems that already contain detail
                    //information will be cached within the user's session
                    guiProblem.setSessionCachedRecord(true);

                    //set identifier to position iteration position
                    guiProblem.setProblemId(appConfig.getCachedTag() + i);

                    mapProblemDetailsData((ProblemDetail) problem, guiProblem);
                }

                guiProblems.add(guiProblem);
            }

            return guiProblems;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIProblem getProblemDetails(GUIUser user, GUIPatient patient, String problemId, String recordSiteCode) {
        try {
            ProblemDetail problemDetail = getServiceHandler().getProblemDetail(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .itemId(problemId)
                            .recordSiteCode(recordSiteCode)
                            .build());

            GUIProblem guiProblem = null;

            if (problemDetail != null) {
                //One GUI problem bean is used to house both general and details information
                //in this case, just populate the details related data of the GUI problem bean
                guiProblem = new GUIProblem();

                mapProblemData(problemDetail, guiProblem);
                mapProblemDetailsData(problemDetail, guiProblem);
            }

            return guiProblem;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    private void mapProblemData(Problem problem, GUIProblem guiProblem) {
        String onsetDate = "";
        String icdCode = "";
        String status = "";
        String lastUpdatedDate = "";
        String description = "";
        String priority = "";
        String problemId = "";
        String hospitalLocation = "";
        String responsibleProvider = "";
        String recordedDate = "";

        if (problem.getOnsetDate() != null) {
            onsetDate = problem.getOnsetDate();
        }

        if (problem.getLastModifiedDate() != null) {
            lastUpdatedDate = Utils.formatDisplayDate(problem.getLastModifiedDate());
        }

        if(problem.getNarrativeId() != null)
            guiProblem.setNarrativeId(problem.getNarrativeId());

        if(problem.getNarrative() != null)
            guiProblem.setNarrative(problem.getNarrative());

        if (problem.getIcdCode() != null) {
            icdCode = problem.getIcdCode();
        }

        if (problem.getStatus() != null) {

            if (problem.getStatus().equals("A")) {
                status = "Active";
            } else if (problem.getStatus().equals("I")) {
                status = "Inactive";
            } else {
                status = problem.getStatus();
            }
        }

        if (problem.getDescription() != null) {
            description = problem.getDescription();
        }

        if (problem.getAcuity() != null) {
            priority = problem.getAcuity();
        }

        if (problem.getId() != null) {
            problemId = problem.getId();
        }

        if (problem.getHospitalLocation() != null) {
            hospitalLocation = problem.getHospitalLocation();
        }

        if (problem.getResponsibleProvider() != null) {
            responsibleProvider = problem.getResponsibleProvider();
        }

        if (problem.getRecordedDate() != null) {
            recordedDate = Utils.formatDisplayDate(problem.getRecordedDate());
        }

        guiProblem.setOnsetDate(onsetDate);
        guiProblem.setDateTime(onsetDate);
        guiProblem.setIcdCode(icdCode);
        guiProblem.setStatus(status);
        guiProblem.setLastUpdated(lastUpdatedDate);
        guiProblem.setDescription(description);
        guiProblem.setPriority(priority);
        guiProblem.setProblemId(problemId);
        guiProblem.setHospitalLocation(hospitalLocation);
        guiProblem.setResponsibleProvider(responsibleProvider);
        guiProblem.setRecordedDate(recordedDate);
        guiProblem.setType(problem.getType());

        List<GUICode> guiCodes = new ArrayList<GUICode>();
        for (Code code : problem.getCodes()) {
            guiCodes.add(dozerBeanMapper.map(code, GUICode.class));
        }
        guiProblem.setCodes(guiCodes);

        addSiteDataToGUIBean(guiProblem, problem);
    }

    private void mapProblemDetailsData(ProblemDetail problemDetail, GUIProblem guiProblemDetail) {
        String hospitalLocation = "";
        String responsibleProvider = "";
        String enteredDate = "";
        String enteredBy = "";
        String recordedDate = "";
        String recordedBy = "";
        List<GUIProblemNote> guiNotes = new ArrayList<GUIProblemNote>();
        String detailText = "";

        if (problemDetail.getHospitalLocation() != null) {
            hospitalLocation = problemDetail.getHospitalLocation();
        }

        if (problemDetail.getResponsibleProvider() != null) {
            responsibleProvider = problemDetail.getResponsibleProvider();
        }

        if (problemDetail.getEnteredDate() != null) {
            enteredDate = Utils.formatDisplayDate(problemDetail.getEnteredDate());
        }

        if (problemDetail.getEnteredBy() != null) {
            enteredBy = problemDetail.getEnteredBy();
        }

        if (problemDetail.getRecordedDate() != null) {
            recordedDate = Utils.formatDisplayDate(problemDetail.getRecordedDate());
        }

        if (problemDetail.getRecordedBy() != null) {
            recordedBy = problemDetail.getRecordedBy();
        }

        if (problemDetail.getNotes() != null) {
            for (ProblemNote note : problemDetail.getNotes()) {
                String noteDate = "";
                String noteEnteredBy = "";
                String noteText = "";

                if (note.getNoteDate() != null) {
                    noteDate = Utils.formatDisplayDate(note.getNoteDate());
                }

                if (note.getNoteEnteredBy() != null) {
                    noteEnteredBy = note.getNoteEnteredBy();
                }

                if (note.getNoteText() != null) {
                    noteText = StringEscapeUtils.escapeHtml4(note.getNoteText());
                }

                GUIProblemNote guiNote = new GUIProblemNote();
                guiNote.setNoteDate(noteDate);
                guiNote.setNoteEnteredBy(noteEnteredBy);
                guiNote.setNoteText(noteText);

                guiNotes.add(guiNote);
            }
        }

        if (problemDetail.getDetailText() != null) {
            detailText = StringEscapeUtils.escapeHtml4(problemDetail.getDetailText());
        }

        guiProblemDetail.setHospitalLocation(hospitalLocation);
        guiProblemDetail.setEnteredDate(enteredDate);
        guiProblemDetail.setEnteredBy(enteredBy);
        guiProblemDetail.setRecordedDate(recordedDate);
        guiProblemDetail.setRecordedBy(recordedBy);
        guiProblemDetail.setResponsibleProvider(responsibleProvider);
        guiProblemDetail.setDetailText(detailText);
        guiProblemDetail.setNotes(guiNotes);
    }
}
