package gov.va.med.ssoi;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateExceptionHandler;
import gov.va.med.common.AppConfig;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.codehaus.groovy.grails.web.context.ServletContextHolder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.xml.soap.*;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

@Component
public class SSOiSecurityTokenServiceHandlerImpl implements SSOiSecurityTokenServiceHandler{

    private static final Logger logger = Logger.getLogger(SSOiSecurityTokenServiceHandlerImpl.class.getName());
    private static final String SAML_TAG_START = "<saml:Assertion";
    private static final String SAML_TAG_END = "</saml:Assertion>";

    private Configuration cfg;
    @Autowired
    private AppConfig appConfig;

    private void initTemplateConfiguration() {
        cfg = new Configuration(Configuration.VERSION_2_3_25);
        cfg.setServletContextForTemplateLoading(ServletContextHolder.getServletContext(), "WEB-INF/templates");
        cfg.setDefaultEncoding("UTF-8");
        cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
        cfg.setLogTemplateExceptions(false);
    }

    @Override
    public String getSAMLToken(String ssoiCookie) throws SOAPException {
        if (cfg == null) {
            initTemplateConfiguration();
        }

        String request = createRequest(ssoiCookie);
        SOAPConnectionFactory soapConnectionFactory = SOAPConnectionFactory.newInstance();
        URL endpoint;
        try {
            endpoint = new URL(appConfig.getSsoiRetrieveTokenURL());
        } catch (MalformedURLException e) {
            logger.error("invalid URL  ERROR: " + e.getMessage(), e);
            throw new SOAPException(e);
        }
        SOAPConnection connection = soapConnectionFactory.createConnection();
        MessageFactory factory = MessageFactory.newInstance();
        SOAPMessage response;
        try (ByteArrayInputStream inputStream = new ByteArrayInputStream(request.getBytes(StandardCharsets.UTF_8))){
            MimeHeaders mimeHeaders = new MimeHeaders();
            mimeHeaders.setHeader("Content-Type", SOAPConstants.SOAP_1_1_CONTENT_TYPE);
            SOAPMessage message = factory.createMessage(mimeHeaders, inputStream);
            response = connection.call(message, endpoint);
        } catch (IOException e) {
            logger.error("create SOAP message ERROR: " + e.getMessage(), e);
            throw new SOAPException(e);
        } finally {
            connection.close();
        }

        try (ByteArrayOutputStream out = new ByteArrayOutputStream()){
            response.writeTo(out);
            return parseResponse(new String(out.toByteArray()));
        } catch (IOException e) {
            logger.error("SOAP response ERROR: " + e.getMessage(), e);
            throw new SOAPException(e);
        }
    }

    private String createRequest(String ssoiCookie) throws SOAPException {
        Map<String, Object> root = new HashMap<>();
        root.put("IAM_SESSION", ssoiCookie);

        Template template;
        try {
            template = cfg.getTemplate("SSOiSecurityTokenServiceRequest.ftlx");
        } catch (IOException e) {
            logger.error("get SSOi STS request template ERROR: " + e.getMessage(), e);
            throw new SOAPException(e);
        }

        try (Writer out = new StringWriter()) {
            template.process(root, out);
            return out.toString();
        } catch (TemplateException|IOException e) {
            logger.error("process SSOi STS template ERROR: " + e.getMessage(), e);
            throw new SOAPException(e);
        }
    }

    private String parseResponse(String response) throws SOAPException {
        if (StringUtils.isNotEmpty(response)) {
            String betweenTags = StringUtils.substringBetween(response, SAML_TAG_START, SAML_TAG_END);
            if (StringUtils.isEmpty(betweenTags)) {
                logger.error("parse response ERROR: invalid response - " + response);
                throw new SOAPException("invalid SSOi STS response");
            } else {
                return SAML_TAG_START + betweenTags + SAML_TAG_END;
            }
        } else {
            logger.error("parse response ERROR: empty response");
            throw new SOAPException("empty SSOi STS response");
        }
    }

}
