package gov.va.med.monitor;

import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.site.Sites;
import gov.va.med.vds.webservice.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public abstract class AbstractServiceMonitor {

    private static final Logger logger = LoggerFactory.getLogger(AbstractServiceMonitor.class);

    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();

    protected Map<String, ServiceReport> serviceReportMap;
    protected List<ServiceReport> logServiceReportList;
    protected Timer emailTimer;
    protected User user;
    protected String timestamp;

    //The Vista status implementation of the service handler will want exclude the BHIE Share checks.
    protected boolean allowBhieShareCheck;

    protected String emailSubject;

    public AbstractServiceMonitor()
    {
        serviceReportMap = new TreeMap<String, ServiceReport>();
        logServiceReportList = new ArrayList<ServiceReport>();
        emailTimer = null;

        allowBhieShareCheck = true;

        //default email subject is JLVQoS Report (VistA implementation email subject will vary slightly)
        emailSubject = " - JLVQoS Report: ";

        setUser();
    }

    protected abstract List<ServiceBase> populateMonitorList();

    public void run()
    {
        // Clear log report list
        logServiceReportList.clear();

        timestamp = (new Date()).toString();

        List<ServiceBase> monitorList = populateMonitorList();

        logger.debug("Service monitors added to list");

        // Set max number of concurrent threads
        ExecutorService executorService = Executors.newFixedThreadPool(monitorList.size());

        for (ServiceBase monitor : monitorList)
        {
            monitor.init();
            executorService.execute(monitor);
        }

        logger.debug("Threads created for service monitors");

        // No new threads can be added and finish all threads in queue
        executorService.shutdown();

        try {
            long timeout = APP_CONFIG.getConnectionTimeoutMS() + APP_CONFIG.getRequestTimeoutMS();
            // Wait a while for existing tasks to terminate
            if(!executorService.awaitTermination(timeout, TimeUnit.MILLISECONDS)) {
                // Cancel currently executing tasks
                executorService.shutdownNow();

                // Wait a while for tasks to respond to being canceled
                if(!executorService.awaitTermination(timeout, TimeUnit.MILLISECONDS)) {
                    logger.error("Pool did not terminate");
                }
            }
        } catch(InterruptedException e) {
            // (Re-)Cancel if current thread also interrupted
            executorService.shutdownNow();
        }

        // Check reports per service including all share sites.
        // Calling getReport() will check if report is NULL. NULL report indicates that a report
        // was not generated during the services run() method (didn't finish for some reason).
        // getReport() will generate an error message if report is null, then add that to the list
        // for logging into the database.
        if (allowBhieShareCheck && APP_CONFIG.isServiceMonitorTestShare()) {
            ShareService.getInstance().checkReports();
        }
        for (ServiceBase monitor : monitorList)
        {
            monitor.getReport();
        }

        logger.debug("Reports checked for NULL");

        // Log reports to database
        // DatabaseHandler is a special case service that happens after all services (threads)
        // have been monitored and completed. It is not added to the threads above because it
        // needs to wait till all services have been monitored, then it can log. During the
        // logging is when the database will be monitored and a service report is generated in
        // logReports() rather than the run() method. It is added to the monitorList at this
        // point so that the business logic below for emailing reports can factor in the
        // database status also.
        logger.debug("Logging to database...");
        DatabaseHandler.getInstance().logReports(logServiceReportList);
        monitorList.add(DatabaseHandler.getInstance());

        AppConfig appConfig = AppConfig.getInstance();
        ServiceReport currentReport;
        ServiceReport previousReport;
        boolean isEmailReport = false;

        logger.debug("Emailing report...");
        if (appConfig.isEmailStatusChangeOnly())
        {
            for (ServiceBase monitor : monitorList)
            {
                currentReport = monitor.getReport();
                previousReport = serviceReportMap.put(currentReport.getType(), currentReport);

                if (!currentReport.isEqual(previousReport))
                {
                    isEmailReport = true;
                }
            }

            if (isEmailReport)
            {
                EmailHandler.getInstance().email(emailSubject, generateEmailReport());
            }
        }
        else
        {
            boolean isDeltaReport = false;

            for (ServiceBase monitor : monitorList)
            {
                currentReport = monitor.getReport();
                previousReport = serviceReportMap.put(currentReport.getType(), currentReport);

                if (!currentReport.isEqual(previousReport))
                {
                    isDeltaReport = true;
                }

                if (!ServiceStatus.OK.toString().equals(currentReport.getStatus()))
                {
                    isEmailReport = true;
                }
            }

            if (!appConfig.isEmailErrorAtEveryCheck() && !isDeltaReport)
            {
                isEmailReport = false;
            }

            if(isDeltaReport){
                isEmailReport = true;
            }

            if (emailTimer == null || isEmailReport)
            {
                cancelEmailTimer();

                emailTimer = new Timer();
                emailTimer.schedule(new EmailTask(this), 0, appConfig.getEmailSendTaskTimerPeriod());
            }
        }
    }

    private void setUser()
    {
        AppConfig appConfig = AppConfig.getInstance();

        user = new User();
        user.setAgency(appConfig.getAgencyVA());
        user.setName(appConfig.getUserName());
        user.setUserIen(appConfig.getUserIEN());
        user.setEncryptedAvCode(appConfig.getEncryptedUserAVCode());
        user.setEncryptedFederatedUid(appConfig.getEncryptedUserFederatedUID());
        user.setHostSite(Sites.getInstance().getSite(appConfig.getUserSiteCode()));

        logger.debug("User created");
    }

    public String generateEmailReport() {
        StringBuilder email = new StringBuilder();
        String lineSeparator = System.getProperty("line.separator");

        email.append("Services were last monitored on " + timestamp + lineSeparator + lineSeparator);

        int index = 0;

        for (ServiceReport report : serviceReportMap.values())
        {
            if(!"OK".equalsIgnoreCase(report.getStatus())) {
                index++;
                // Service type
                email.append("Service: " + report.getType() + lineSeparator);

                // Status of service
                email.append("Status: " + report.getStatus() + lineSeparator);

                // Message of service
                email.append("Message: " + report.getMessage() + lineSeparator + lineSeparator);
            }
        }
        if(index == 0){
            email.append("No errors detected." + lineSeparator + lineSeparator);
        }
        return email.toString();
    }


    public synchronized void addReport(ServiceReport report)
    {
        logServiceReportList.add(report);

        logger.debug("Report added: " + report.getType());
    }

    public User getUser()
    {
        return user;
    }

    public void cancelEmailTimer()
    {
        if (emailTimer != null)
        {
            emailTimer.cancel();
            emailTimer.purge();
        }
    }
}
