package gov.va.med.monitor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class ShareService extends ServiceBase
{
    private static final Logger logger = LoggerFactory.getLogger(ShareService.class);
    private static final ShareService INSTANCE = new ShareService();
    private Map<String, ShareSiteService> shareServiceMonitorMap = null;

    private ShareService()
    {
        super(ServiceType.ShareService);

        ShareSiteService monitor;
        List<ShareSiteInfo> shareSiteInfoList = DatabaseHandler.getInstance().getShareEndpoints();

        if (shareSiteInfoList != null)
        {
            shareServiceMonitorMap = new TreeMap<String, ShareSiteService>();

            for (ShareSiteInfo shareSiteInfo : shareSiteInfoList)
            {
                // Create new service monitor
                monitor = new ShareSiteService(
                        shareSiteInfo.getHostURL(),
                        shareSiteInfo.getSiteCode(),
                        shareSiteInfo.getName(),
                        shareSiteInfo.getEncryptedUsername(),
                        shareSiteInfo.getEncryptedPassword());

                // Add service monitor to map
                shareServiceMonitorMap.put(shareSiteInfo.getSiteCode(), monitor);
            }
        }
    }

    public static ShareService getInstance()
    {
        return INSTANCE;
    }

    public void checkReports()
    {
        for (Map.Entry<String, ShareSiteService> entry : shareServiceMonitorMap.entrySet())
        {
            entry.getValue().getReport();
        }
    }

    @Override
    public void run()
    {
        if (shareServiceMonitorMap == null)
        {
            status = ServiceStatus.UNKNOWN;
            message.append("Database did not return any share endpoints to test.");
        }
        else
        {
            int recordsCount = shareServiceMonitorMap.size();

            if (recordsCount > 0)
            {
                status = ServiceStatus.OK;
                message.append("Review the following for individual share site statuses:");

                // Set max number of concurrent threads
                ExecutorService executorService = Executors.newFixedThreadPool(recordsCount);

                for (Map.Entry<String, ShareSiteService> entry : shareServiceMonitorMap.entrySet())
                {
                    entry.getValue().init();
                    executorService.execute(entry.getValue());
                }

                // No new threads can be added and finish all threads in queue
                executorService.shutdown();

                try {
                    long timeout = APP_CONFIG.getConnectionTimeoutMS() + APP_CONFIG.getRequestTimeoutMS();
                    // Wait a while for existing tasks to terminate
                    if(!executorService.awaitTermination(timeout, TimeUnit.MILLISECONDS)) {
                        // Cancel currently executing tasks
                        executorService.shutdownNow();

                        // Wait a while for tasks to respond to being canceled
                        if(!executorService.awaitTermination(timeout, TimeUnit.MILLISECONDS)) {
                            logger.error("Pool did not terminate");
                        }
                    }
                } catch(InterruptedException e) {
                    // (Re-)Cancel if current thread also interrupted
                    executorService.shutdownNow();
                    logger.error("QoS Error: Interrupted Exception : \n" + e.getMessage());
                }

                ServiceReport report;
                int index = 0;

                for (Map.Entry<String, ShareSiteService> entry : shareServiceMonitorMap.entrySet())
                {
                    report = entry.getValue().getReport();

                    if (!"OK".equals(report.getStatus()))
                    {
                        index++;
                        status = ServiceStatus.ERROR;

                        message.append(LINE_SEPARATOR + index + ") "
                                + "Site: " + entry.getValue().getSiteName() + ", "
                                + "Status: " + report.getStatus() + ", "
                                + "Message: " + report.getMessage());
                    }

                }
                //check to see if there were any errors.
                if(index == 0){
                    //clear message
                    message.setLength(0);
                    message.append("Running as expected.");
                }
            }
            else
            {
                // The database did not return any records
                status = ServiceStatus.UNKNOWN;
                message.append("Database did not return any share endpoints to test.");
            }
        }

        // Report status
        report(type.toString(), status.toString(), message.toString());
    }
}
