/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */


package gov.va.med.rpc;

import gov.va.med.rpc.site.SiteUtils;
import gov.va.med.vds.vistabeans.Endpoint;
import gov.va.med.vds.vistabeans.Site;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.LinkedHashMap;
import java.util.concurrent.ExecutorService;

/**
 *
 *
 */
public class RPCConnection {
    private static final Logger LOGGER = LoggerFactory.getLogger(RPCConnection.class);

    private static final int SOCKET_TIMEOUT_MS = 15000;
    private static final int SOCKET_SO_TIMEOUT_MS = 60000;
    
    private RPCConnectionStrategy connectionStrategy;

    private Site site;
    private Endpoint rpcEndpoint;
    private boolean isVisitCxn;
    private boolean isTestSource;
    private boolean remote;

    private String errorMsg;
    private String userIen;
    private Site userHostSite;
    private String patientIen;
    private String patientIcn;

    private Socket socket;
    private String currentContext;

    private LinkedHashMap<String, LinkedHashMap<String, String>> lookupTables;

    private ThreadPoolContainer parserThreadPool;

    public RPCConnection(Site site) {
        if (site == null) {
            throw new IllegalArgumentException("Site is null");
        }

        this.site = site;
        rpcEndpoint = SiteUtils.getRPCEndpoint(site);
        if (rpcEndpoint == null) {
            throw new IllegalArgumentException("Cannot find RPC endpoint within Site");
        }
        lookupTables = new LinkedHashMap<String, LinkedHashMap<String, String>>();
    }
    
    public String getSiteCode() {
        return site.getSiteCode();
    }

    public Socket getRPCSocket() {
        return socket;
    }

    public void connect() throws RPCException {
        //by default utilize connection connectionStrategy national
        if (connectionStrategy == null || connectionStrategy == RPCConnectionStrategy.NATIONAL) {
            try {
                connectionStrategy = RPCConnectionStrategy.NATIONAL;
                connectStrategyNational();
            } catch (RPCException e) {
                //fallback on direct RPC connectionStrategy if national fails
                connectionStrategy = RPCConnectionStrategy.DIRECT;
                connectStrategyDirect();
            }
        } else if (connectionStrategy == RPCConnectionStrategy.DIRECT) {
            connectStrategyDirect();
        }
    }

    private void connectStrategyNational() throws RPCException {
        if (!isConnected()) {
            if (rpcEndpoint == null) {
                throw new RPCException("No data source");
            }
            String hostname = rpcEndpoint.getHost();
            if (hostname.isEmpty()) {
                throw new RPCException("No provider (hostname)");
            }
            int vistaPort = rpcEndpoint.getPort();
            if (-1 == vistaPort) {
                StringBuilder exceptionMsg = new StringBuilder();
                exceptionMsg.append("DataSource doesn't have a valid VistA broker:");
                exceptionMsg.append(rpcEndpoint.getHost());
                throw new RPCException(exceptionMsg.toString());
            }
    
            //make socket connection
            socket = new Socket();
            try {
                socket.connect(new InetSocketAddress(hostname, vistaPort), SOCKET_TIMEOUT_MS);
            } catch (IOException e) {
                try {
                    socket.close();
                    socket = null;
                } catch (IOException e2) {
                    socket = null;
                }
    
                throw new RPCException(e);
            }
    
            if (!socket.isConnected()) {
                StringBuilder exceptionMsg = new StringBuilder();
                exceptionMsg.append("Socket was unable to connect to ");
                exceptionMsg.append(hostname);
                exceptionMsg.append(":");
                exceptionMsg.append(vistaPort);
                throw new RPCException(exceptionMsg.toString());
            }
    
            //Build the request message
            int COUNT_WIDTH = 3;
    
            StringBuilder requestSB = new StringBuilder("[XWB]10")
                    .append(COUNT_WIDTH)
                    .append("04\nTCPConnect50")
                    .append(StringUtils.strPack(socket.getLocalAddress().toString().replace("/", ""), COUNT_WIDTH))
                    .append("f0")
                    .append(StringUtils.strPack("0", COUNT_WIDTH))
                    .append("f0")
                    .append(StringUtils.strPack(socket.getLocalAddress().toString().replace("/", ""), COUNT_WIDTH))
                    .append("f\u0004");
    
            String reply = (String) query(requestSB.toString());
            if (!"accept".equals(reply)) {
                try {
                    socket.close();
                } catch (IOException e) {
    
                    throw new RPCException(e);
                }
                StringBuilder exceptionMsg = new StringBuilder();
                exceptionMsg.append("Unaccepted by ");
                exceptionMsg.append(hostname);
                throw new RPCException(exceptionMsg.toString());
            }
    
            connectionStrategy = RPCConnectionStrategy.NATIONAL;
            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("Utilizing RPC broker protocol version 1.1");
            }
        }
    }


    public void connectStrategyDirect() throws RPCException {
        ServerSocket listener = null;

        if (!isConnected()) {
            try {
                if (rpcEndpoint == null) {
                    throw new RPCException("No data source");
                }
                String hostname = rpcEndpoint.getHost();
                if (hostname.isEmpty()) {
                    throw new RPCException("No provider (hostname)");
                }
                int vistaPort = rpcEndpoint.getPort();
                if (-1 == vistaPort) {
                    StringBuilder exceptionMsg = new StringBuilder();
                    exceptionMsg.append("DataSource doesn't have a valid VistA broker:");
                    exceptionMsg.append(rpcEndpoint.getHost());
                    throw new RPCException(exceptionMsg.toString());
                }
    
    
                listener = new ServerSocket(0, 2);
    
                listener.setSoTimeout(SOCKET_SO_TIMEOUT_MS);
    
                //make socket connection
                socket = new Socket();
                socket.connect(new InetSocketAddress(hostname, vistaPort), SOCKET_TIMEOUT_MS);
                if (!socket.isConnected()) {
                    StringBuilder exceptionMsg = new StringBuilder();
                    exceptionMsg.append("Socket was unable to connect to ");
                    exceptionMsg.append(hostname);
                    exceptionMsg.append(":");
                    exceptionMsg.append(vistaPort);
                    throw new RPCException(exceptionMsg.toString());
                }
    
                //Build the request message
                StringBuilder requestSB = new StringBuilder("TCPconnect^")
                    .append(listener.getLocalSocketAddress().toString())
                    .append('^')
                    .append(listener.getLocalPort())
                    .append('^')
                    .append(hostname);
    
                StringBuilder querySB = new StringBuilder("{XWB}");
                querySB.append(StringUtils.strPack(requestSB.toString(), 5));
    
                String reply = (String) query(querySB.toString());
                if (!"accept".equals(reply)) {
                    socket.close();
                    StringBuilder exceptionMsg = new StringBuilder();
                    exceptionMsg.append("Unaccepted by ");
                    exceptionMsg.append(hostname);
                    throw new RPCException(exceptionMsg.toString());
                }
                socket.close();
                socket = listener.accept();
                socket.setSoTimeout(SOCKET_SO_TIMEOUT_MS);
    
                connectionStrategy = RPCConnectionStrategy.DIRECT;
    
                if (LOGGER.isDebugEnabled()) {
                    LOGGER.debug("Utilizing RPC broker protocol version 1.0");
                }
            } catch (RPCException ex) {
                LOGGER.error(ex.getMessage());
            } catch (Exception ex) {
                throw new RPCException(ex);
            } finally {
                if (listener != null) {
                    try {
                        listener.close();
                    } catch (IOException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
    }

    public void disconnect() throws RPCException {
        if (connectionStrategy == RPCConnectionStrategy.NATIONAL) {
            disconnectStrategyNational();
        } else if (connectionStrategy == RPCConnectionStrategy.DIRECT) {
            disconnectStrategyDirect();
        }
    }

    public void disconnectStrategyNational() throws RPCException {
        if (isConnected()) {
            try {
                query("[XWB]10304\u0005#BYE#\u0004");
                socket.close();
            } catch (Exception e) {
                throw new RPCException(e);
            }
        }
    }

    public void disconnectStrategyDirect() throws RPCException {
        if (isConnected()) {
            try {
                String msg = StringUtils.strPack(StringUtils.strPack("#BYE#", 5), 5);
                msg = (String) query(msg);
                //todo msg should contain "BYE" - do anything if it doesn't?
                socket.close();
            } catch (Exception ex) {
                throw new RPCException(ex);
            }
        }
    }

    public String getCurrentContext() {
        return currentContext;
    }

    public void setCurrentContext(String currentContext) {
        this.currentContext = currentContext;
    }

    public VistaResponse query(VistaQuery vquery) throws RPCException, IllegalStateException {
        String result = (String) query(vquery.buildMessage());
        return new VistaResponse(result);
    }

    public Object query(String request) throws RPCException {
        Object result = null;

        /**
         * If the socket is not connected then connect; don't worry
         * about authorization as a program could perform a simple
         * impl query such as get welcome message which doesn't require
         * login.
         */
        boolean disconnectAfterQuery = false;
        if (!isConnected()) {
            disconnectAfterQuery = true;
            connect();
        }

        try {
            result = queryRequest(request);
        } catch (RPCException ex) {
            //retry query on re-authentication errors
            String exMsgRetry = "You are re-authenticating an already authenticated session";
            if (!ex.getMessage().contains(exMsgRetry)) {
                throw ex;
            } else {
                boolean isRetry = true;
                int retryCnt = 0;
                while (isRetry && retryCnt < 2) {
                    isRetry = false;
                    this.disconnect();
                    this.connect();
                    try {
                        result = queryRequest(request);
                    } catch (RPCException ex2) {
                        if (!ex2.getMessage().contains(exMsgRetry) || retryCnt >= 2) {
                            throw ex2;
                        } else {
                            isRetry = true;
                            retryCnt++;
                        }
                    }
                }
            }
        }

        if (disconnectAfterQuery) {
            disconnect();
        }
        return result;
    }

    private Object queryRequest(String request) throws RPCException {
        OutputStream out = null;
        try {
            byte[] bytesSent = request.getBytes(VistaConstants.CHARSET_UTF8);
            byte[] bytesReceived = new byte[256];
            out = socket.getOutputStream();
            out.write(bytesSent);
            int bytes = 0;
            StringBuilder replySB = new StringBuilder("");
            boolean isHdr = true;
            boolean isErrorMsg = false;
            int endIdx = -1;
            StringBuilder exceptionMsg = new StringBuilder();
            StringBuilder batchStr = new StringBuilder();
            do {
                InputStream inpt = socket.getInputStream();
                bytes = inpt.read(bytesReceived, 0, bytesReceived.length);

                if (bytes < 0) {
                    exceptionMsg.setLength(0);
                    exceptionMsg.append("An error occurred while communicating with RPC host: ");
                    exceptionMsg.append(socket.getInetAddress());
                    throw new RPCException(exceptionMsg.toString());
                }

                String thisBatch = new String(bytesReceived, 0, bytes, VistaConstants.CHARSET_UTF8);
                char eot = 0x04;
                endIdx = thisBatch.indexOf(eot);
                if (endIdx != -1) {
                    thisBatch = StringUtils.CSubstring(thisBatch, 0, endIdx);
                }
                if (isHdr) {
                    if (bytesReceived[0] != 0) {
                        String msg = StringUtils.CSubstring(thisBatch, 1, bytesReceived[0]);
                        if (msg.startsWith("The remote procedure XUS SIGNON SETUP is not registered to the option XUS SIGNON.")) {
                            msg = "You are re-authenticating an already authenticated session";
                        }
                        batchStr.setLength(0);
                        batchStr.append("Security Error: ");
                        batchStr.append(msg);
                        thisBatch = batchStr.toString();
                        isErrorMsg = true;
                    } else if (bytesReceived[1] != 0) {
                        batchStr.setLength(0);
                        batchStr.append("Application Error: ");
                        batchStr.append(thisBatch.substring(2));
                        thisBatch = batchStr.toString();
                        isErrorMsg = true;
                    } else {
                        thisBatch = thisBatch.substring(2);
                    }
                    isHdr = false;
                }
                replySB.append(thisBatch);
            }
            while (endIdx == -1);
            if (isErrorMsg || replySB.toString().contains("M  ERROR")) {
                replySB.append(" (This message has come directly from ");
                replySB.append(this.rpcEndpoint.getHost());
                replySB.append(").");
                throw new RPCException(replySB.toString());
            }
            return replySB.toString();
        } catch (IOException ex) {
            throw new RPCException(ex);
        }
    }

    public boolean setContext(String value) {
        /*VistaUserDao dao = new VistaUserDao(this);
        boolean result = dao.setContext(value);
        if (result)
        {
                currentContext = value;
        }
        return result;*/
        currentContext = value;
        //return false;
        return true;
    }

    public LinkedHashMap<String, String> getLookupTable(String fileNum) throws RPCException {
        if (!lookupTables.containsKey(fileNum)) {
            DdrLister query = new DdrLister(this);
            query.setIenNameQuery(fileNum);
//            DdrLister query = DdrLister.buildIenNameQuery(this, fileNum);
            String[] response = query.execute();
            lookupTables.put(fileNum, RPCUtils.toStringDictionary(response));
        }
        return lookupTables.get(fileNum);
    }


    public Site getSite() {
        return site;
    }

    public void setSite(Site site) {
        this.site = site;
    }

    public String getErrorMsg() {
        return errorMsg;
    }

    public void setErrorMsg(String errorMsg) {
        this.errorMsg = errorMsg;
    }

    public boolean isConnected() {
        boolean rtc = false;
        if (socket != null && (socket.isConnected() && socket.isBound() && !socket.isClosed())) {
            rtc = true;
        }

        return rtc;
    }

    public String getPatientIen() {
        return patientIen;
    }

    public void setPatientIen(String patientIen) {
        this.patientIen = patientIen;
    }

    public String getUserIen() {
        return userIen;
    }

    public void setUserIen(String userIen) {
        this.userIen = userIen;
    }

    public Site getUserHostSite() {
        return userHostSite;
    }

    public void setUserHostSite(Site userHostSite) {
        this.userHostSite = userHostSite;
    }

    public boolean isVisitCxn() {
        return isVisitCxn;
    }

    public void setIsVisitCxn(boolean visitCxn) {
        this.isVisitCxn = visitCxn;
    }

    public Endpoint getRPCEndpoint() {
        return rpcEndpoint;
    }

    public void setRPCEndpoint(Endpoint rpcEndpoint) {
        this.rpcEndpoint = rpcEndpoint;
    }

    public boolean isTestSource() {
        return isTestSource;
    }

    public void setTestSource(boolean testSource) {
        isTestSource = testSource;
    }

    public boolean isRemote() {
        return remote;
    }

    public void setRemote(boolean remote) {
        this.remote = remote;
    }

    public RPCConnectionStrategy getConnectionStrategy() {
        return connectionStrategy;
    }

    public String getPatientIcn() {
        return patientIcn;
    }

    public void setPatientIcn(String patientIcn) {
        this.patientIcn = patientIcn;
    }

    public ThreadPoolContainer getParserThreadPool() {
        return parserThreadPool;
    }

    public void setParserThreadPool(ThreadPoolContainer parserThreadPool) {
        this.parserThreadPool = parserThreadPool;
    }
}
