package gov.va.med.vds.dataservicehandler;


import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.vistabeans.vix.StudyDetail;
import gov.va.med.vds.vistabeans.vix.StudyQuery;
import gov.va.med.vds.vistabeans.vix.StudyQueryHeader;
import gov.va.med.vhahon.common.Utils;
import org.apache.log4j.Logger;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.IOException;

public class VixServiceHandler extends DataServiceHandler {
    private static final String STUDYQUERY_URL = APP_CONFIG.getEvixStudyqueryUrl();
    private static final int REQUEST_TIMEOUT = APP_CONFIG.getEvixRequestTimeout();
    private static final int CONNECTION_TIMEOUT = APP_CONFIG.getEvixConnectionTimeout();
    private static final Logger LOGGER = Logger.getLogger(VixServiceHandler.class.getName());
    private static final VixServiceHandler INSTANCE = new VixServiceHandler();

    private ObjectMapper objectMapper;
    private Client restClient;

    /**
     * Creates a new VixServiceHandler instance
     */
    private VixServiceHandler()
    {
        super();

        // Initialize the object mapper and allow unknown data fields
        objectMapper = new ObjectMapper();
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);

        // Inialize REST client
        restClient = Client.create();
        restClient.setConnectTimeout(CONNECTION_TIMEOUT);
        restClient.setReadTimeout(REQUEST_TIMEOUT);
    }

    /**
     * Gets the VixServiceHandler instance
     * @return VixServiceHandler instance
     */
    public static VixServiceHandler getInstance()
    {
        return INSTANCE;
    }

    /**
     * Sends a POST query request to the eVix REST interface to retrieve studies
     * @param studyQueryHeader VistA user information sent in the request header
     * @param studyQuery The query request that defines what documents to query for images
     * @return The query response that contains a list of studies with images that matches the query request
     * @throws VDSException If there was an error or exception
     */
    public StudyQuery postStudyQuery(StudyQueryHeader studyQueryHeader, StudyQuery studyQuery) throws VDSException
    {
        String requestJson;

        // Map the studyQuery object into a JSON string to send as the POST request body
        try {
            requestJson = objectMapper.writeValueAsString(studyQuery);
            if(!Utils.isJSONValid(requestJson)){
                throw new IOException("Json failed to valid");
            }
        }
        catch(JsonProcessingException e) {
            LOGGER.error("Failed to process JSON request", e);
            throw new VDSException(e);
        }
        catch(IOException e){
            LOGGER.error("Failed to process JSON request", e);
            throw new VDSException(e);
        }

        if (StringUtils.isEmpty(STUDYQUERY_URL)) {
            return null;
        }

        // Send a POST request to the eVix studyquery REST interface
        WebResource resource = restClient.resource(STUDYQUERY_URL);
        ClientResponse response = resource
                .header("Content-Type", "application/json")
                .header("xxx-fullname", studyQueryHeader.getFullName())
                .header("xxx-duz", studyQueryHeader.getDuz())
                .header("xxx-ssn", studyQueryHeader.getSsn())
                .header("xxx-sitename", studyQueryHeader.getSiteName())
                .header("xxx-sitenumber", studyQueryHeader.getSiteNumber())
                .header("xxx-securityToken", studyQueryHeader.getSecurityToken())
                .accept("application/json")
                .post(ClientResponse.class, requestJson);

        // Ensure the eVix REST interface returned a 200 (OK) status in the response
        if (response.getStatus() != 200) {
            LOGGER.error("Vix server returned HTTP error code: " + response.getStatus());
            throw new VDSException("Vix server returned HTTP error code: " + response.getStatus());
        }

        // Map the response JSON back into the studyQuery object
        try {
            studyQuery = objectMapper.readValue(response.getEntityInputStream(), StudyQuery.class);
        }
        catch (JsonParseException e) {
            LOGGER.error("Failed to parse JSON", e);
            throw new VDSException(e);
        }
        catch (JsonMappingException e) {
            LOGGER.error("Failed to map JSON", e);
            throw new VDSException(e);
        }
        catch (IOException e) {
            LOGGER.error("Failed getting JSON data", e);
            throw new VDSException(e);
        }

        return studyQuery;
    }

    /**
     * Sends a GET query request to the eVix REST interface to retrieve in-depth details on a study
     * @param detailsUrl The complete study details URL returned from the studyquery POST response
     * @return The study details of the study
     * @throws VDSException If there was an error or exception
     */
    public StudyDetail getStudyDetails(String detailsUrl) throws VDSException
    {
        StudyDetail details;

        // Send a GET request to the eVix studydetails REST interface
        WebResource resource = restClient.resource(detailsUrl);
        ClientResponse response = resource.accept("application/json").get(ClientResponse.class);

        // Ensure the eVix REST interface returned a 200 (OK) status in the response
        if (response.getStatus() != 200) {
            LOGGER.error("Vix server returned HTTP error code: " + response.getStatus());
            throw new VDSException("Vix server returned HTTP error code: " + response.getStatus());
        }

        // Map the response JSON back into the studyQuery object
        try {
            details = objectMapper.readValue(response.getEntityInputStream(), StudyDetail.class);
        }
        catch (JsonParseException e) {
            LOGGER.error("Failed to parse JSON", e);
            throw new VDSException(e);
        }
        catch (JsonMappingException e) {
            LOGGER.error("Failed to map JSON", e);
            throw new VDSException(e);
        }
        catch (IOException e) {
            LOGGER.error("Failed getting JSON data", e);
            throw new VDSException(e);
        }

        return details;
    }

    /**
     * Sends a GET query request to the eVix REST interface to retrieve a study's detail image thumbnail
     * @param thumbnailUrl The complete thumbnail URL returned from the studydetail GET response
     * @return The buffered image in JPEG format
     * @throws VDSException If there was an error or exception
     */
    public BufferedImage getThumbnails(String thumbnailUrl) throws VDSException
    {
        // Send a GET request to the eVix thumbnails REST interface
        WebResource resource = restClient.resource(thumbnailUrl);
        ClientResponse response = resource.get(ClientResponse.class);

        // Ensure the eVix REST interface returned a 200 (OK) status in the response
        if (response.getStatus() != 200) {
            LOGGER.error("Vix server returned HTTP error code: " + response.getStatus());
            throw new VDSException("Vix server returned HTTP error code: " + response.getStatus());
        }

        // Read the JPEG data input stream into a buffered image object
        BufferedImage thumbnail;

        try {
            thumbnail = ImageIO.read(response.getEntityInputStream());
        }
        catch (IOException e) {
            LOGGER.error("Failed reading thumbnail data", e);
            throw new VDSException(e);
        }

        return thumbnail;
    }
}
