/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.patient.dao.rpc.PatientRPC;
import gov.va.med.vds.patient.dataservice.PatientDataService;
import gov.va.med.vds.vistabeans.*;

/**
 *
 *
 */
public class PatientDao extends DaoBase {

    public PatientDao(RPCConnection cxn) {
        super(cxn);
    }

    public Patient[] lookupPatient(QueryBean queryBean) throws RPCException {
        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.match(queryBean.getItemId());
    }

    public Patient selectPatient() throws RPCException {
        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.select(cxn.getPatientIen());
    }

    public Boolean issueSensitivePatientBulletin() throws RPCException {
        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.issueSensitivePatientBulletin(cxn.getPatientIen());
    }

    public PatientDemographics getPatientDemographics(QueryBean queryBean) throws RPCException {
        //SB: no longer using cache demographics routine due cache server load issues

        PatientDemographics demo = null;
        PatientRPC rpc = new PatientRPC(cxn);
        try {
            demo = rpc.getPatientDemographicsVPR(queryBean.getUser());
        } catch (RPCException e) {
            if ("No such patient".equalsIgnoreCase(e.getMessage())) {
                demo = createUnknownPatientBean(cxn.getSite(), PROTOCOL_RPC/*, AGENCY_VA*/);
            } else {
                throw e;
            }
        }

        return demo;
    }

    private PatientDemographics createUnknownPatientBean(Site site,
                                                         String protocol/*, String agency*/) {
        PatientDemographics demo = new PatientDemographics();
        demo.setName(PatientDataService.UNKNOWN_PATIENT_MSG);
        demo.setAge("");
        demo.setClinic("");
        demo.setDob("");
        demo.setGender("");
//        demo.setPatientId("");
        demo.setPrimaryProvider("");
        demo.setPhone1("");
        demo.setPhone2("");
        demo.setSSN("");
        demo.setCodeGreen("0");
        demo.setSite(site);
        demo.setSourceProtocol(protocol);
        demo.getAddress().clear();

        return demo;
    }

    public PatientDemographicsDetail getPatientDemographicsDetail(QueryBean queryBean) throws RPCException {

        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.getPatientDemographicsDetail();
    }

    /**
     * Determines if a patient is flagged as sensitive in VistA and sends the alert level and message.
     *
     * If the patient record is sensitive the alertId will be non-zero.
     *
     * -1 RPC/API failed. Required variable not defined <br/>
     * 0 No display/action required. Not an employee, not sensitive or not accessing own Patient record <br/>
     * 1-Display warning message. Sensitive - inpatient or a DG SENSITIVITY key holder or Employee and
     *   DG SECURITY OFFICER key holder
     * 2-Display warning message, require OK to continue and call. DG SENSITIVE RECORD BULLETIN RPC to update
     *   DG Security Log file and generate Sensitive Record Access mail message. Sensitive - not an inpatient and not
     *   a key holder or Employee/not a DG SECURITY OFFICER key holder
     * 3-Access to record denied. Accessing own Patient file record
     * 4-Access to Patient file (#2) records denied. SSN not defined.
     *
     * The message text for the sensitivity can be retrieved with Alert.getMessageText()
     *
     * @param queryBean Required queryBean params: User, patient
     * @return The sensitive Alert. If the alertId is 0, then the patient record is not sensitive.
     * @throws gov.va.med.vds.base.dataservice.VDSException
     *          if an error has occurred.
     */
    public Alert getSensitivePatientAlert(QueryBean queryBean) throws RPCException {
        PatientRPC patientRPC = new PatientRPC(cxn);
        return patientRPC.getSensitivePatientAlert();
    }
}
