package gov.va.med.vds.patient.dao;


import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservicehandler.VixServiceHandler;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import gov.va.med.vds.vistabeans.vix.StudyDetail;
import gov.va.med.vds.vistabeans.vix.StudyQuery;
import gov.va.med.vds.vistabeans.vix.StudyQueryHeader;
import gov.va.med.vds.vistabeans.vix.StudyQueryStudy;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.log4j.Logger;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

public class VixDao {
    private static final Logger LOGGER = Logger.getLogger(VixDao.class);
    private static final String ITEM_ID_DELIMITER = ";";
    private static final String CONTEXT_ID_DELIMITER = ":";

    public VixDao()
    {

    }

    public StudyQuery getStudyQueryFromSite(QueryBean queryBean) throws VDSException
    {
        User user = queryBean.getUser();
        Site site = user.getHostSite();

        // Set Vix study query header
        StudyQueryHeader studyQueryHeader = new StudyQueryHeader();
        studyQueryHeader.setFullName(user.getName());
        studyQueryHeader.setDuz(user.getUserIen());
        if (!StringUtils.isEmpty(user.getEncryptedFederatedUid())) {
            studyQueryHeader.setSsn(EncryptUtil.decrypt(user.getEncryptedFederatedUid()));
        } else {
            studyQueryHeader.setSsn(VistaConstants.PSUEDO_FEDERATED_UID);
        }
        studyQueryHeader.setSiteName(site.getName());
        studyQueryHeader.setSiteNumber(site.getSiteCode());
        studyQueryHeader.setSecurityToken(user.getBseToken());

        // Set Vix study query body
        StudyQuery studyQuery = new StudyQuery();
        Patient patient = queryBean.getPatient();
        studyQuery.setPatientIcn(patient.getICN());
        studyQuery.setSiteNumber(queryBean.getItemId());

        try {
            studyQuery = VixServiceHandler.getInstance().postStudyQuery(studyQueryHeader, studyQuery);
        }
        catch (VDSException e) {
            LOGGER.error("Failed to post Vix study query");
            throw new VDSException(e);
        }

        if (studyQuery != null && studyQuery.getStudies() != null) {
            StudyDetail studyDetail;

            for (StudyQueryStudy studyQueryStudy : studyQuery.getStudies()) {
                // Get study details
                queryBean.setItemId(studyQueryStudy.getDetailsUrl());
                try {
                    studyDetail = getStudyDetail(queryBean);
                    studyQueryStudy.setStudyDetail(studyDetail);
                }
                catch (VDSException e) {
                    // Continue to process other studies
                    LOGGER.debug("Failed to retrieve study detail: " + e.getLocalizedMessage());
                }
            }
        }

        return studyQuery;
    }


    public StudyQuery getStudyQueryWithDetails(QueryBean queryBean) throws VDSException
    {
        StudyQuery studyQuery = null;

        try {
            studyQuery = getStudyQueryWithoutDetails(queryBean);

            // Get studies
            if (studyQuery != null && studyQuery.getStudies() != null) {
                StudyDetail studyDetail;

                for (StudyQueryStudy studyQueryStudy : studyQuery.getStudies()) {
                    // Status code 0 is returned when you send a studyquery request with no context ID(s). This means
                    // that all images for the patient is being requested at specified site only. Both, status 0 and 200
                    // means success.
                    if ("0".equals(studyQueryStudy.getStatusCode()) || "200".equals(studyQueryStudy.getStatusCode())) {
                        // Get study details
                        queryBean.setItemId(studyQueryStudy.getDetailsUrl());
                        try {
                            studyDetail = getStudyDetail(queryBean);
                            studyQueryStudy.setStudyDetail(studyDetail);
                        }
                        catch (VDSException e) {
                            // continue to process other studies
                            LOGGER.debug("Failed to retrieve study detail: " + e.getLocalizedMessage());
                        }
                    }
                }
            }
        }
        catch (VDSException e) {
            LOGGER.error("Failed to get vix studies with details", e);
        }

        return studyQuery;
    }

    public StudyQuery getStudyQueryWithoutDetails(QueryBean queryBean) throws VDSException
    {
        User user = queryBean.getUser();
        Site site = user.getHostSite();

        // Set Vix study query header
        StudyQueryHeader studyQueryHeader = new StudyQueryHeader();
        studyQueryHeader.setFullName(user.getName());
        studyQueryHeader.setDuz(user.getUserIen());
        if (!StringUtils.isEmpty(user.getEncryptedFederatedUid())) {
            studyQueryHeader.setSsn(EncryptUtil.decrypt(user.getEncryptedFederatedUid()));
        } else {
            studyQueryHeader.setSsn(VistaConstants.PSUEDO_FEDERATED_UID);
        }
        studyQueryHeader.setSiteName(site.getName());
        studyQueryHeader.setSiteNumber(site.getSiteCode());
        studyQueryHeader.setSecurityToken(user.getBseToken());

        // Set Vix study query body
        StudyQuery studyQuery = new StudyQuery();
        Patient patient = queryBean.getPatient();
        studyQuery.setPatientIcn(patient.getICN());
        studyQuery.setSiteNumber(site.getSiteCode());

        // Set context IDs if they exist
        if (!StringUtils.isEmpty(queryBean.getItemId())) {
            String[] itemIds = queryBean.getItemId().split(ITEM_ID_DELIMITER);

            if (itemIds.length > 0) {
                List<StudyQueryStudy> studies = new ArrayList<StudyQueryStudy>();
                StudyQueryStudy study;
                String[] contextIds;

                for (String itemId : itemIds) {
                    contextIds = itemId.split(CONTEXT_ID_DELIMITER);

                    study = new StudyQueryStudy();
                    study.setContextId(contextIds[0]);

                    // Study site may be attached to context ID delimited by ":" (semicolon)
                    if (contextIds.length > 1) {
                        study.setSiteNumber(contextIds[1]);
                    }

                    studies.add(study);
                }

                studyQuery.setStudies(studies);
            }
        }

        try {
            studyQuery = VixServiceHandler.getInstance().postStudyQuery(studyQueryHeader, studyQuery);
        }
        catch (VDSException e) {
            LOGGER.error("Failed to post Vix study query");
            throw new VDSException(e);
        }

        return studyQuery;
    }

    public StudyDetail getStudyDetail(QueryBean queryBean) throws VDSException
    {
        StudyDetail studyDetail;

        try {
            studyDetail = VixServiceHandler.getInstance().getStudyDetails(queryBean.getItemId());
        }
        catch (VDSException e) {
            LOGGER.error("Failed to get Vix study detail");
            throw new VDSException(e);
        }

        return studyDetail;
    }

    public BufferedImage getThumbnail(QueryBean queryBean) throws VDSException
    {
        BufferedImage bufferedImage;

        try {
            bufferedImage = VixServiceHandler.getInstance().getThumbnails(queryBean.getItemId());
        }
        catch (VDSException e) {
            LOGGER.error("Failed to get Vix thumbnail");
            throw new VDSException(e);
        }

        return bufferedImage;
    }

    public String getStudyViewerUrl(QueryBean queryBean) throws VDSException {
        User user = queryBean.getUser();
        Site site = user.getHostSite();

        // Set Vix study query header
        StudyQueryHeader studyQueryHeader = new StudyQueryHeader();
        studyQueryHeader.setFullName(user.getName());
        studyQueryHeader.setDuz(user.getUserIen());
        if (!StringUtils.isEmpty(user.getEncryptedFederatedUid())) {
            studyQueryHeader.setSsn(EncryptUtil.decrypt(user.getEncryptedFederatedUid()));
        } else {
            studyQueryHeader.setSsn(VistaConstants.PSUEDO_FEDERATED_UID);
        }
        studyQueryHeader.setSiteName(site.getName());
        studyQueryHeader.setSiteNumber(site.getSiteCode());
        studyQueryHeader.setSecurityToken(user.getBseToken());

        // Set Vix study query body
        StudyQuery studyQuery = new StudyQuery();
        Patient patient = queryBean.getPatient();
        studyQuery.setPatientIcn(patient.getICN());
        studyQuery.setSiteNumber(site.getSiteCode());

        // Set context ID if exist
        if (!StringUtils.isEmpty(queryBean.getItemId())) {
            List<StudyQueryStudy> studies = new ArrayList<StudyQueryStudy>();
            StudyQueryStudy study = new StudyQueryStudy();
            String[] contextIds = queryBean.getItemId().split(CONTEXT_ID_DELIMITER);
            study.setContextId(contextIds[0]);

            // Study site may be attached to context ID delimited by ":" (colon)
            if (contextIds.length > 1) {
                study.setSiteNumber(contextIds[1]);
            }

            studies.add(study);
            studyQuery.setStudies(studies);

            try {
                studyQuery = VixServiceHandler.getInstance().postStudyQuery(studyQueryHeader, studyQuery);
                if (studyQuery != null && CollectionUtils.isNotEmpty(studyQuery.getStudies())) {
                    StudyQueryStudy responseStudy = studyQuery.getStudies().get(0);
                    if (responseStudy.getStatusCode().equals("200")) {
                        return responseStudy.getViewerUrl();
                    } else {
                        LOGGER.error(responseStudy.getStatus());
                    }
                }
            } catch (VDSException e) {
                LOGGER.error("Failed to post Vix study query");
                throw new VDSException(e);
            }
        }
        return null;
    }
}
