/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.ParserBatchManager;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.Timestamp;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.util.*;
import java.util.concurrent.*;


/**
 *
 *
 */
public class EncountersRPC extends BaseRPC {

    private static final Logger LOGGER = LoggerFactory.getLogger(EncountersRPC.class);
    
    private static final String XPATH_VISIT = "//visit";
    private static final String XPATH_ARRIVAL_DATETIME = "./arrivalDateTime/@value";
    private static final String XPATH_DEPARTURE_DATETIME = "./departureDateTime/@value";
    private static final String XPATH_ROOMBED = "./roomBed/@value";
    private static final String XPATH_SPECIALTY = "./specialty/@value";
    private static final String XPATH_ADMISSION = "./admission/@value";
    private static final String XPATH_DATETIME = "./dateTime/@value";
    private static final String XPATH_ID = "./id/@value";
    private static final String XPATH_LOCATION = "./location/@value";
    private static final String XPATH_PATIENT_CLASS = "./patientClass/@value";
    private static final String XPATH_SERVICE = "./service/@value";
    private static final String XPATH_SERVICE_CAT_CODE = "./serviceCategory/@code";
    private static final String XPATH_SERVICE_CAT_NAME = "./serviceCategory/@name";
    private static final String XPATH_VISIT_STRING = "./visitString/@value";
    private static final String XPATH_CREDIT_STOP_CODE_CODE = "./creditStopCode/@code";
    private static final String XPATH_CREDIT_STOP_CODE_NAME = "./creditStopCode/@name";
    private static final String XPATH_STOP_CODE_CODE = "./stopCode/@code";
    private static final String XPATH_STOP_CODE_NAME = "./stopCode/@name";
    private static final String XPATH_TYPE_NAME = "./type/@name";
    private static final String XPATH_REASON_CODE = "./reason/@code";
    private static final String XPATH_REASON_NAME = "./reason/@name";
    private static final String XPATH_REASON_NARRATIVE = "./reason/@narrative";
    private static final String XPATH_PROVIDERS = "./providers";
    private static final String XPATH_PROVIDER = "./provider";
    private static final String XPATH_PROVIDER_CODE = "./@code";
    private static final String XPATH_PROVIDER_NAME = "./@name";
    private static final String XPATH_PROVIDER_ROLE = "./@role";
    private static final String XPATH_PROVIDER_TAXONOMY_CODE = "./@taxonomyCode";
    private static final String XPATH_PROVIDER_PROVIDER_TYPE = "./@providerType";
    private static final String XPATH_PROVIDER_CLASSIFICATION = "./@classification";
    private static final String XPATH_PROVIDER_SPECIALIZATION = "./@specialization";
    private static final String XPATH_SERVICE_ATTR = "./@service";
    private static final String XPATH_OFFICE_PHONE = "./@officePhone";
    private static final String XPATH_ANALOG_PAGER = "./@analogPager";
    private static final String XPATH_EMAIL = "./@email";
    private static final String XPATH_CPTS = "./cpts";
    private static final String XPATH_CPT = "./cpt";
    private static final String XPATH_CPT_NAME = "./@name";
    private static final String XPATH_CPT_CODE = "./@code";
    private static final String XPATH_ICDS = "./icds";
    private static final String XPATH_ICD = "./icd";
    private static final String XPATH_ICD_NAME = "./@name";
    private static final String XPATH_ICD_CODE = "./@code";
    private static final String XPATH_ICD_NARRATIVE = "./@narrative";
    private static final String XPATH_ICD_RANKING = "./ranking";
    private static final String XPATH_DOCUMENTS = "./documents";
    private static final String XPATH_DOCUMENT = "./document";
    private static final String XPATH_DOCUMENT_ID = "./@id";
    private static final String XPATH_DOCUMENT_VUID = "./@vuid";
    private static final String XPATH_DOCUMENT_LOCAL_TITLE = "./@localTitle";
    private static final String XPATH_DOCUMENT_NATIONAL_TITLE = "./@nationalTitle";
    private static final String XPATH_DOCUMENT_CONTENT = "./content";

    private static final XPathExpression exprVisit = XPathUtil.xpathCompile(XPATH_VISIT);
    private static final XPathExpression exprArrivalDateTime = XPathUtil.xpathCompile(XPATH_ARRIVAL_DATETIME);
    private static final XPathExpression exprDepartureDateTime = XPathUtil.xpathCompile(XPATH_DEPARTURE_DATETIME);
    private static final XPathExpression exprRoomBed = XPathUtil.xpathCompile(XPATH_ROOMBED);
    private static final XPathExpression exprSpecialty = XPathUtil.xpathCompile(XPATH_SPECIALTY);
    private static final XPathExpression exprAdmission = XPathUtil.xpathCompile(XPATH_ADMISSION);
    private static final XPathExpression exprDateTime = XPathUtil.xpathCompile(XPATH_DATETIME);
    private static final XPathExpression exprId = XPathUtil.xpathCompile(XPATH_ID);
    private static final XPathExpression exprLocation = XPathUtil.xpathCompile(XPATH_LOCATION);
    private static final XPathExpression exprPatientClass = XPathUtil.xpathCompile(XPATH_PATIENT_CLASS);
    private static final XPathExpression exprService = XPathUtil.xpathCompile(XPATH_SERVICE);
    private static final XPathExpression exprServiceCatCode = XPathUtil.xpathCompile(XPATH_SERVICE_CAT_CODE);
    private static final XPathExpression exprServiceCatName = XPathUtil.xpathCompile(XPATH_SERVICE_CAT_NAME);
    private static final XPathExpression exprVisitString = XPathUtil.xpathCompile(XPATH_VISIT_STRING);
    private static final XPathExpression exprTypeName = XPathUtil.xpathCompile(XPATH_TYPE_NAME);
    private static final XPathExpression exprCreditStopCode = XPathUtil.xpathCompile(XPATH_CREDIT_STOP_CODE_CODE);
    private static final XPathExpression exprCreditStopName = XPathUtil.xpathCompile(XPATH_CREDIT_STOP_CODE_NAME);
    private static final XPathExpression exprStopCodeCode = XPathUtil.xpathCompile(XPATH_STOP_CODE_CODE);
    private static final XPathExpression exprStopCodeName = XPathUtil.xpathCompile(XPATH_STOP_CODE_NAME);
    private static final XPathExpression exprReasonCode = XPathUtil.xpathCompile(XPATH_REASON_CODE);
    private static final XPathExpression exprReasonName = XPathUtil.xpathCompile(XPATH_REASON_NAME);
    private static final XPathExpression expreReasonNarrative = XPathUtil.xpathCompile(XPATH_REASON_NARRATIVE);
    private static final XPathExpression exprProviders = XPathUtil.xpathCompile(XPATH_PROVIDERS);
    private static final XPathExpression exprProvider = XPathUtil.xpathCompile(XPATH_PROVIDER);
    private static final XPathExpression exprProviderCode = XPathUtil.xpathCompile(XPATH_PROVIDER_CODE);
    private static final XPathExpression exprProviderName = XPathUtil.xpathCompile(XPATH_PROVIDER_NAME);
    private static final XPathExpression exprProviderRole = XPathUtil.xpathCompile(XPATH_PROVIDER_ROLE);
    private static final XPathExpression exprProviderTaxonomyCode = XPathUtil.xpathCompile(XPATH_PROVIDER_TAXONOMY_CODE);
    private static final XPathExpression exprProviderProviderType = XPathUtil.xpathCompile(XPATH_PROVIDER_PROVIDER_TYPE);
    private static final XPathExpression exprProviderClassification = XPathUtil.xpathCompile(XPATH_PROVIDER_CLASSIFICATION);
    private static final XPathExpression exprProviderSpecialization = XPathUtil.xpathCompile(XPATH_PROVIDER_SPECIALIZATION);
    private static final XPathExpression exprServiceAttr = XPathUtil.xpathCompile(XPATH_SERVICE_ATTR);
    private static final XPathExpression exprOfficePhone = XPathUtil.xpathCompile(XPATH_OFFICE_PHONE);
    private static final XPathExpression exprFax = XPathUtil.xpathCompile(XPATH_PROVIDER_SPECIALIZATION);
    private static final XPathExpression exprAnalogPager = XPathUtil.xpathCompile(XPATH_ANALOG_PAGER);
    private static final XPathExpression exprEmail = XPathUtil.xpathCompile(XPATH_EMAIL);
    private static final XPathExpression exprCpts = XPathUtil.xpathCompile(XPATH_CPTS);
    private static final XPathExpression exprCpt = XPathUtil.xpathCompile(XPATH_CPT);
    private static final XPathExpression exprCptName = XPathUtil.xpathCompile(XPATH_CPT_NAME);
    private static final XPathExpression exprCptCode = XPathUtil.xpathCompile(XPATH_CPT_CODE);
    private static final XPathExpression exprIcds = XPathUtil.xpathCompile(XPATH_ICDS);
    private static final XPathExpression exprIcd = XPathUtil.xpathCompile(XPATH_ICD);
    private static final XPathExpression exprIcdName = XPathUtil.xpathCompile(XPATH_ICD_NAME);
    private static final XPathExpression exprIcdCode = XPathUtil.xpathCompile(XPATH_ICD_CODE);
    private static final XPathExpression exprIcdNarrative = XPathUtil.xpathCompile(XPATH_ICD_NARRATIVE);
    private static final XPathExpression exprIcdRanking = XPathUtil.xpathCompile(XPATH_ICD_RANKING);
    private static final XPathExpression exprDocuments = XPathUtil.xpathCompile(XPATH_DOCUMENTS);
    private static final XPathExpression exprDocument = XPathUtil.xpathCompile(XPATH_DOCUMENT);
    private static final XPathExpression exprDocumentId = XPathUtil.xpathCompile(XPATH_DOCUMENT_ID);
    private static final XPathExpression exprDocumentLocalTitle = XPathUtil.xpathCompile(XPATH_DOCUMENT_LOCAL_TITLE);
    private static final XPathExpression exprDocumentNationalTitle = XPathUtil.xpathCompile(XPATH_DOCUMENT_NATIONAL_TITLE);
    private static final XPathExpression exprDocumentContent = XPathUtil.xpathCompile(XPATH_DOCUMENT_CONTENT);
    private static final XPathExpression exprDocumentVuid = XPathUtil.xpathCompile(XPATH_DOCUMENT_VUID);
    private static final int PARSER_BATCH_SIZE = APP_CONFIG.getParserBatchSizeEncounters();

    public EncountersRPC(RPCConnection cxn) {
        super(cxn);
    }

    /**
     * Uses the VPR RPC with "visits" as the type.
     *
     * @param startDate  The start of the date range for the list of encounters
     * @param endDate    The end of the date range for the list of encounters
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter[] getPatientEncounters(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientEncounters(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Uses the VPR RPC with "appointments" as the type.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of encounters
     * @param endDate    The end of the date range for the list of encounters
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter[] getPatientEncounters(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetEncountersRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        Encounter[] encounters = parseEncounters(response, false);

        if (encounters.length > 0) {
            // get the documents padding 3 months to end date. Cross reference against visits for hasImage
            NotesRPC notesRPC = new NotesRPC(cxn);
            Calendar docsEndDate = Calendar.getInstance();
            docsEndDate.setTime(endDate.getTime());
            docsEndDate.add(Calendar.MONTH, 3);
            ProgressNote[] notes = notesRPC.getProgressNotesVPRImagesFieldOnly(startDate, docsEndDate);
            // put notes in a HashMap using the id as the key
            if (notes.length > 0) {
                HashMap<String, ProgressNote> notesMap = new HashMap<String, ProgressNote>();
                for (int noteNum = 0; noteNum < notes.length; noteNum++) {
                    notesMap.put(notes[noteNum].getNoteId(), notes[noteNum]);
                }
                // check each encounter document for hasImage
                for (Encounter encounter: encounters) {
                    if (encounter.getDocuments() != null) {
                        for (EncounterDocument encounterDocument : encounter.getDocuments()) {
                            if (notesMap.containsKey(encounterDocument.getId())) {
                                encounterDocument.setImages(notesMap.get(encounterDocument.getId()).getImages());
                            }
                        }
                    }
                }
            }
        }

        return encounters;
    }

    public Encounter[] getPatientEncountersNoDocuments(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetEncountersRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        Encounter[] encounters = parseEncounters(response, false);

        return encounters;
    }


    /**
     * Uses the underlying call to VPR visits used in getting the Encounters and filters for the Admissions.
     * Admissions in the VPR visits are noted by those that have an arrivalDate.
     *
     * @param startDate  The start of the date range for the list of encounters
     * @param endDate    The end of the date range for the list of encounters
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter[] getPatientAdmissionsFromEncounters(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientAdmissionsFromEncounters(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Uses the underlying call to VPR visits used in getting the Encounters and filters for the Admissions.
     * Admissions in the VPR visits are noted by those that have an arrivalDate.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of encounters
     * @param endDate    The end of the date range for the list of encounters
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter[] getPatientAdmissionsFromEncounters(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetEncountersRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        Encounter[] encounters = parseEncounters(response, true);

        return encounters;
    }


    /**
     * Uses the underlying call to VPR visits used in getting the Encounters and filters for the Admissions.
     * Admissions in the VPR visits are noted by those that have an arrivalDate.
     *
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter getPatientAdmission(String encounterId) throws RPCException {
        return getPatientAdmission(cxn.getPatientIen(), encounterId);
    }

    /**
     * Uses the underlying call to VPR visits used in getting the Encounters and filters for the Admissions.
     * Admissions in the VPR visits are noted by those that have an arrivalDate.
     *
     * @param patientIen The patient IEN
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter getPatientAdmission(String patientIen, String encounterId) throws RPCException {
        Encounter rtc;
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, VPRUtil.VPRType.VISITS, encounterId);
        String response = (String) cxn.query(request);

        Encounter[] encounters = parseEncounters(response, true);
        if (encounters != null && encounters.length > 0) {
            rtc = encounters[0];
        } else {
            rtc = null;
        }
        return rtc;
    }


    /**
     * Uses the underlying call to VPR visits used in getting the Encounters and filters for the non-Admissions.
     * Admissions in the VPR visits are noted by those that have an arrivalDate. The non-admissions are all other
     * encounters.
     *
     * @param startDate  The start of the date range for the list of encounters
     * @param endDate    The end of the date range for the list of encounters
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter[] getPatientNonAdmissionsFromEncounters(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientNonAdmissionsFromEncounters(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Uses the underlying call to VPR visits used in getting the Encounters and filters for the non-Admissions.
     * Admissions in the VPR visits are noted by those that have an arrivalDate. The non-admissions are all other
     * encounters.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of encounters
     * @param endDate    The end of the date range for the list of encounters
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public Encounter[] getPatientNonAdmissionsFromEncounters(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        Encounter[] encounters = getPatientEncountersNoDocuments(patientIen, startDate, endDate);

        List<Encounter> encounterList = new ArrayList<Encounter>(Arrays.asList(encounters));
        Iterator<Encounter> encounterIterator = encounterList.iterator();
        while (encounterIterator.hasNext()) {
            Encounter encounter = encounterIterator.next();
            if (encounter.getArrivalDateTime() != null) {
                encounterIterator.remove();
            }
        }

        return encounterList.toArray(new Encounter[0]);
    }


    /**
     * Builds the VPR RPC request string using "appointments" as the type. The problems contain the list of
     * appointments. The RPC will fail if the "VPR APPLICATION PROXY" context is not set. That is set when
     * setting up the connection.
     *
     * @param patientIen Patient IEN
     * @param startDate  Start date of range
     * @param endDate    End date of range
     * @return The RPC response message string
     */
    private String buildGetEncountersRequest(String patientIen, Calendar startDate, Calendar endDate) {
        return VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.VISITS);
    }

    private Encounter[] parseEncounters(String response, boolean admissionsOnly) throws RPCException {
        Encounter[] rtc;
        if ("".equals(response)) {
            rtc = new Encounter[0];
        } else {
    
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            ArrayList<Encounter> encountersList = new ArrayList<Encounter>();
    
            Document document = null;
    
            try {
                document = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList visitsNodeList = null;
            try {
                visitsNodeList = (NodeList) exprVisit.evaluate(document, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);

            int visitNodeListLength = visitsNodeList.getLength();
            for (int visitNum = 0; visitNum < visitNodeListLength; visitNum++) {
                parserBatchManager.addNode(visitsNodeList.item(visitNum));
            }

            List<VPRVisitsParser> visitsParserList = new ArrayList<VPRVisitsParser>();

            for (List<Node> visitsBatch : parserBatchManager.getParserBatchList()) {
                visitsParserList.add(new VPRVisitsParser(visitsBatch, admissionsOnly));
            }
            
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<Encounter>>> futures = null;
            try {
                futures = executorService.invokeAll(visitsParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                XPathUtil.logErrorMessage(LOGGER, e);
                throw new RPCException(e);
            }
    
            Iterator<Future<List<Encounter>>> futureIterator = futures.iterator();
    
            while (futureIterator.hasNext()) {
                try {
                    Future<List<Encounter>> future = futureIterator.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Visit(Encounter) Data");
                    } else {
                        encountersList.addAll(future.get());
                    }
                } catch (InterruptedException ie) {
                    XPathUtil.logErrorMessage(LOGGER, ie);
                } catch (ExecutionException ee) {
                    XPathUtil.logErrorMessage(LOGGER, ee);
                }
            }
    
            rtc = encountersList.toArray(new Encounter[0]);
        }
        return rtc;
    }

    private class VPRVisitsParser implements Callable<List<Encounter>> {
        private List<Node> visitNodeList;
        private boolean admissionsOnly;

        public VPRVisitsParser(List<Node> visitNodeList, boolean admissionsOnly) {
            this.visitNodeList = visitNodeList;
            this.admissionsOnly = admissionsOnly;
        }

        public List<Encounter> call() throws Exception {
            List<Encounter> encounters = new ArrayList<Encounter>();

            for (Node visitNode : visitNodeList) {
                // Admission parsing
                String arrivalDateTime = null;
                Calendar arrivalDate = null;
                arrivalDateTime = XPathUtil.evaluateNodeToString(visitNode, exprArrivalDateTime, LOGGER);
                if (arrivalDateTime != null) {
                    arrivalDate = Timestamp.toDateTime(arrivalDateTime);
                }
                String departureDateTime = null;
                Calendar departureDate = null;
                departureDateTime = XPathUtil.evaluateNodeToString(visitNode, exprDepartureDateTime, LOGGER);
                if (departureDateTime != null) {
                    departureDate = Timestamp.toDateTime(departureDateTime);
                }
                String roomBed = XPathUtil.evaluateNodeToString(visitNode, exprRoomBed, LOGGER);
                String specialty = XPathUtil.evaluateNodeToString(visitNode, exprSpecialty, LOGGER);
                String admission = XPathUtil.evaluateNodeToString(visitNode, exprAdmission, LOGGER);

                // If not an admission and we are looking for admissionOnly, we don't need to continue
                if (admissionsOnly && arrivalDate == null) {
                    continue;
                } else {
                    Encounter encounter = new Encounter();
                    fillCommonDataBeanValues(encounter);

                    // Other encounter parsing
                    String dateTime = null;
                    Calendar visitDate = null;
                    dateTime = XPathUtil.evaluateNodeToString(visitNode, exprDateTime, LOGGER);
                    if (dateTime != null) {
                        visitDate = Timestamp.toDateTime(dateTime);
                        
                    }
                    String id = XPathUtil.evaluateNodeToString(visitNode, exprId, LOGGER);
                    String location = XPathUtil.evaluateNodeToString(visitNode, exprLocation, LOGGER);
                    String patientClass = XPathUtil.evaluateNodeToString(visitNode, exprPatientClass, LOGGER);
                    String serviceCategoryCode = XPathUtil.evaluateNodeToString(visitNode, exprServiceCatCode, LOGGER);
                    String serviceCategoryName = XPathUtil.evaluateNodeToString(visitNode, exprServiceCatName, LOGGER);
                    String visitString = XPathUtil.evaluateNodeToString(visitNode, exprVisitString, LOGGER);
                    //            String typeCode = XPathUtil.evaluateNodeToString(visitNode, exprTypeCode, logger);
                    String typeName = XPathUtil.evaluateNodeToString(visitNode, exprTypeName, LOGGER);
                    String service = XPathUtil.evaluateNodeToString(visitNode, exprService, LOGGER);
                    String creditStopCode = XPathUtil.evaluateNodeToString(visitNode, exprCreditStopCode, LOGGER);
                    String creditStopName = XPathUtil.evaluateNodeToString(visitNode, exprCreditStopName, LOGGER);
                    String stopCodeCode = XPathUtil.evaluateNodeToString(visitNode, exprStopCodeCode, LOGGER);
                    String stopCodeName = XPathUtil.evaluateNodeToString(visitNode, exprStopCodeName, LOGGER);
                    String reasonCode = XPathUtil.evaluateNodeToString(visitNode, exprReasonCode, LOGGER);
                    String reasonName = XPathUtil.evaluateNodeToString(visitNode, exprReasonName, LOGGER);
                    String reasonNarrative = XPathUtil.evaluateNodeToString(visitNode, expreReasonNarrative, LOGGER);

                    Node providersNode = XPathUtil.evaluateNodeToNode(visitNode, exprProviders, LOGGER);
                    if (providersNode != null) {
                        NodeList providerNodeList = XPathUtil.evaluateNodeToNodeList(providersNode, exprProvider, LOGGER);

                        if (providerNodeList != null) {
                            int providersNodeListLength = providerNodeList.getLength();

                            for (int providerNum = 0; providerNum < providersNodeListLength; providerNum++) {
                                Provider provider = new Provider();
                                Node providerNode = providerNodeList.item(providerNum);
                                providerNode.getParentNode().removeChild(providerNode);

                                String providerRole = XPathUtil.evaluateNodeToString(providerNode, exprProviderRole, LOGGER);
                                if ("P".equals(providerRole)) {
                                    provider.setPrimary(true);
                                } else if (arrivalDate == null) {
                                    // this is not an admission so we can continue to look for only the primary provider
                                    continue;
                                }

                                String providerCode = XPathUtil.evaluateNodeToString(providerNode, exprProviderCode, LOGGER);
                                String providerName = XPathUtil.evaluateNodeToString(providerNode, exprProviderName, LOGGER);
                                String providerTaxonomyCode = XPathUtil.evaluateNodeToString(providerNode, exprProviderTaxonomyCode, LOGGER);
                                String providerType = XPathUtil.evaluateNodeToString(providerNode, exprProviderProviderType, LOGGER);
                                String providerClassification = XPathUtil.evaluateNodeToString(providerNode, exprProviderClassification, LOGGER);
                                String providerSpecialization = XPathUtil.evaluateNodeToString(providerNode, exprProviderSpecialization, LOGGER);
                                String providerService = XPathUtil.evaluateNodeToString(providerNode, exprServiceAttr, LOGGER);
                                String providerOfficePhone = XPathUtil.evaluateNodeToString(providerNode, exprOfficePhone, LOGGER);
                                String providerAnalogPager = XPathUtil.evaluateNodeToString(providerNode, exprAnalogPager, LOGGER);
                                String providerFax = XPathUtil.evaluateNodeToString(providerNode, exprFax, LOGGER);
                                String providerEmail = XPathUtil.evaluateNodeToString(providerNode, exprEmail, LOGGER);

                                provider.setCode(providerCode);
                                provider.setName(providerName);
                                provider.setRole(providerRole);
                                provider.setTaxonomyCode(providerTaxonomyCode);
                                provider.setProviderType(providerType);
                                provider.setClassification(providerClassification);
                                provider.setSpecialization(providerSpecialization);
                                provider.setService(providerService);
                                provider.setPhone(providerOfficePhone);
                                provider.setAnalogPager(providerAnalogPager);
                                provider.setFax(providerFax);
                                provider.setEmail(providerEmail);

                                if (provider.getName() != null) {
                                    if (provider.isPrimary()) {
                                        encounter.setProvider(provider);
                                        if (arrivalDate == null) {
                                            // we found the primary provider for the non-admission so we can leave the for loop
                                            break;
                                        }
                                    } else if (arrivalDate != null && "A".equalsIgnoreCase(providerRole)) {
                                        // this is encounter is an admission and the current provider is the admitting provider
                                        encounter.setAdmittingProvider(provider);
                                    }
                                }
                            }

                        }
                    }

                    List<Code> codes = new ArrayList<Code>();

                    Node cptsNode = XPathUtil.evaluateNodeToNode(visitNode, exprCpts, LOGGER);
                    if (cptsNode != null) {
                        NodeList cptNodeList = XPathUtil.evaluateNodeToNodeList(cptsNode, exprCpt, LOGGER);

                        if (cptNodeList != null) {
                            int cptNodeListLength = cptNodeList.getLength();
                            List<Code> cptList = new ArrayList<Code>();
                            List<Procedure> procedureList = new ArrayList<Procedure>();
                            for (int cptNum = 0; cptNum < cptNodeListLength; cptNum++) {
                                Node cptNode = cptNodeList.item(cptNum);
                                cptNode.getParentNode().removeChild(cptNode);

                                Code cpt = new Code();

                                cpt.setSystem("CPT");

                                String cptName = XPathUtil.evaluateNodeToString(cptNode, exprCptName, LOGGER);
                                String cptCode = XPathUtil.evaluateNodeToString(cptNode, exprCptCode, LOGGER);

                                cpt.setDisplay(cptName);
                                cpt.setCode(cptCode);

                                Procedure proc = new Procedure();
                                fillCommonDataBeanValues(proc);
                                proc.setCode(cptCode);
                                proc.setLocation(location);
                                proc.setCodeSystem("CPT");
                                proc.setDescription(cptName);
                                proc.setProvider(encounter.getProvider());
                                proc.setEncounterId(id);

                                // If this is an admission, set as inpatient procedure, else set as out
                                if (arrivalDate != null) {
                                    proc.setInOutpatient("I");
                                } else {
                                    proc.setInOutpatient("O");
                                }

                                proc.setDate(encounter.getApptDate());

                                procedureList.add(proc);
                                cptList.add(cpt);
                            }

                            if (!cptList.isEmpty()) {
                                codes.addAll(cptList);
                                encounter.setProcedures(procedureList.toArray(new Procedure[0]));
                            }
                        }
                    }

                    Node icdsNode = XPathUtil.evaluateNodeToNode(visitNode, exprIcds, LOGGER);
                    if (icdsNode != null) {
                        NodeList icdNodeList = XPathUtil.evaluateNodeToNodeList(icdsNode, exprIcd, LOGGER);

                        if (icdNodeList != null) {
                            int icdNodeListLength = icdNodeList.getLength();
                            List<Code> icdList = new ArrayList<Code>();
                            List<Diagnosis> diagnosisList = new ArrayList<Diagnosis>();
                            for (int icdNum = 0; icdNum < icdNodeListLength; icdNum++) {
                                Node icdNode = icdNodeList.item(icdNum);
                                icdNode.getParentNode().removeChild(icdNode);

                                Code icd = new Code();
                                icd.setSystem("ICD9");

                                String icdName = XPathUtil.evaluateNodeToString(icdNode, exprIcdName, LOGGER);
                                String icdCode = XPathUtil.evaluateNodeToString(icdNode, exprIcdCode, LOGGER);
                                String icdNarrative = XPathUtil.evaluateNodeToString(icdNode, exprIcdNarrative, LOGGER);
                                String icdRanking = XPathUtil.evaluateNodeToString(icdNode, exprIcdRanking, LOGGER);

                                icd.setDisplay(icdName);
                                icd.setCode(icdCode);
                                icd.setNarrative(icdNarrative);
                                icd.setRanking(icdRanking);

                                Diagnosis diag = new Diagnosis();
                                fillCommonDataBeanValues(diag);
                                diag.setCode(icdCode);
                                diag.setDescription(icdName);
                                diag.setCodeSystem("ICD9");


                                icdList.add(icd);
                                diagnosisList.add(diag);
                            }

                            if (!icdList.isEmpty()) {
                                codes.addAll(icdList);
                                encounter.setDiagnosis(diagnosisList.toArray(new Diagnosis[0]));
                            }
                        }
                    }

                    Node documentsNode = XPathUtil.evaluateNodeToNode(visitNode, exprDocuments, LOGGER);
                    if (documentsNode != null) {
                        NodeList documentNodeList = XPathUtil.evaluateNodeToNodeList(documentsNode, exprDocument, LOGGER);

                        if (documentNodeList != null) {
                            int documentNodeListLength = documentNodeList.getLength();
                            List<EncounterDocument> documentList = new ArrayList<EncounterDocument>();
                            for (int documentNum = 0; documentNum < documentNodeListLength; documentNum++) {
                                Node documentNode = documentNodeList.item(documentNum);
                                documentNode.getParentNode().removeChild(documentNode);

                                EncounterDocument document = new EncounterDocument();
                                fillCommonDataBeanValues(document);

                                String documentId = XPathUtil.evaluateNodeToString(documentNode, exprDocumentId, LOGGER);
                                String documentLocalTitle = XPathUtil.evaluateNodeToString(documentNode, exprDocumentLocalTitle, LOGGER);
                                String documentNationalTitle = XPathUtil.evaluateNodeToString(documentNode, exprDocumentNationalTitle, LOGGER);
                                String documentContent = XPathUtil.evaluateNodeToString(documentNode, exprDocumentContent, LOGGER);
                                String documentVuid = XPathUtil.evaluateNodeToString(documentNode, exprDocumentVuid, LOGGER);

                                document.setId(documentId);
                                document.setLocalTitle(documentLocalTitle);
                                document.setNationalTitle(documentNationalTitle);
                                document.setContent(documentContent);
                                document.setVuid(documentVuid);

                                documentList.add(document);
                            }

                            if (!documentList.isEmpty()) {
                                encounter.setDocuments(documentList.toArray(new EncounterDocument[0]));
                            }
                        }
                    }

                    encounter.setApptDate(visitDate);
                    encounter.setApptId(id);
                    encounter.setEncounterId(id);
                    encounter.setClinic(location);
                    encounter.setApptType(typeName);
                    encounter.setPatientClass(patientClass);
                    encounter.setService(service);
                    encounter.setServiceCategoryCode(serviceCategoryCode);
                    encounter.setServiceCategoryName(serviceCategoryName);
                    encounter.setVisitString(visitString);
                    encounter.setClinicStopCode(stopCodeCode);
                    encounter.setClinicStopName(stopCodeName);
                    encounter.setCreditStopCode(creditStopCode);
                    encounter.setCreditStopName(creditStopName);
                    encounter.setCodes(codes.toArray(new Code[0]));
                    encounter.setReasonCode(reasonCode);
                    encounter.setReason(reasonName);
                    encounter.setReasonNarrative(reasonNarrative);

                    // Admission fields
                    encounter.setArrivalDateTime(arrivalDate);
                    encounter.setDepartureDateTime(departureDate);
                    encounter.setRoomBed(roomBed);
                    encounter.setSpecialty(specialty);
                    encounter.setAdmissionRegistrationNum(admission);

                    encounters.add(encounter);
                }
                
            }
            return encounters;

        }
    }

}
