/**
 * Copyright (c) 2017 Hokukahu LLC. All Rights Reserved.
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.rpc.VistaQuery;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.HealthSummaryTypeList;
import gov.va.med.vds.vistabeans.HealthSummaryType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * Allergy RPC data access object.
 */
public class HealthSummariesRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(HealthSummariesRPC.class);
    
    public HealthSummariesRPC(RPCConnection cxn) {
        super(cxn);
    }

    /**
     * Retrieves a list of patient health summary types.
     *
     * @return List of patient health summary types.
     * @throws RPCException if there was an error..
     */
    public HealthSummaryTypeList getHealthSummaryTypes() throws RPCException {
        VistaQuery vistaQuery = vistaQueryFactory.getVistaQuery("ORWRP REPORT LISTS");
        String request = vistaQuery.buildMessage();
        String response = (String) cxn.query(request);
        
        return toHealthSummaryTypes(response);
    }

    /**
     * Retrieves the health summary.
     * 
     * @param healthSummaryIen the IEN to the health summary
     * @return the health summary as a FreeTextReport
     * @throws RPCException
     */
    public FreeTextReport getPatientHealthSummary(String healthSummaryIen) throws RPCException {
        return getPatientHealthSummary(cxn.getPatientIen(), healthSummaryIen);
    }

    /**
     * Retrieves patient health summary.
     *
     * @param patientIen Patient data file number (or IEN).
     * @param healthSummaryIen Internal entry number of the patient allergy file.
     * @return the health summary as a FreeTextReport.
     * @throws RPCException if there was an error..
     */
    public FreeTextReport getPatientHealthSummary(String patientIen, String healthSummaryIen) throws RPCException {
        //build request
        VistaQuery vistaQuery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vistaQuery.addParameter(VistaQuery.LITERAL, patientIen);
        vistaQuery.addParameter(VistaQuery.LITERAL, "1~");
        vistaQuery.addParameter(VistaQuery.LITERAL, healthSummaryIen);
        String request = vistaQuery.buildMessage();

        String response = (String) cxn.query(request);

        FreeTextReport healthSummary = new FreeTextReport();
        healthSummary.setReportText(response);
        fillCommonDataBeanValues(healthSummary);

        return healthSummary;
    }

    private static final String HEALTH_SUMMARY_TYPES_START_MARKER = "[HEALTH SUMMARY TYPES]";
    private static final String HEALTH_SUMMARY_TYPES_END_MARKER = "$$END";
    private HealthSummaryTypeList toHealthSummaryTypes(String response) {
        HealthSummaryTypeList rtc = new HealthSummaryTypeList();
        HealthSummaryType[] healthSummaryTypes;
        fillCommonDataBeanValues(rtc);

        if ("".equals(response)) {
            healthSummaryTypes = new HealthSummaryType[0];
        } else {
            // find where the Health Summary Types start
            int healthSummaryTypesIndex = response.indexOf(HEALTH_SUMMARY_TYPES_START_MARKER);
            if (healthSummaryTypesIndex > -1) {
                response = response.substring(healthSummaryTypesIndex + HEALTH_SUMMARY_TYPES_START_MARKER.length());
            }
            // split the reamining string into lines
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            List<HealthSummaryType> healthSummaryTypeList = new ArrayList<HealthSummaryType>();
            
            for (String line : lines) {
                if (HEALTH_SUMMARY_TYPES_END_MARKER.equalsIgnoreCase(line)) {
                    // stop when we reach the next $$END of section marker
                    break;
                }
                
                HealthSummaryType healthSummaryType = new HealthSummaryType();
                fillCommonDataBeanValues(healthSummaryType);

                String[] pieces = StringUtils.split(line, "^");
    
                if (pieces.length < 2) {
                    continue;
                }
                // the first piece is the IEN prepended by h
                if (!pieces[0].startsWith("h")) {
                    continue;
                } else {
                    // remove the leading h to get the IEN
                    String ien = pieces[0].substring(1);    
                
                    if ("0".equalsIgnoreCase(ien)) {
                        // don't include adhoc reports which is IEN = 0
                        continue;
                    } else {
                        healthSummaryType.setId(ien);

                        // National HS IENs are in the 5000000 range
                        if (healthSummaryType.getId().startsWith("5") && healthSummaryType.getId().length() == 7) {
                            healthSummaryType.setLocale(VistaConstants.NATIONAL_STRING);
                        } else {
                            healthSummaryType.setLocale(VistaConstants.LOCAL_STRING);
                        }
                    }
                }
                // the second should be the name
                healthSummaryType.setName(pieces[1]);
    
                healthSummaryTypeList.add(healthSummaryType);
            }
    
            healthSummaryTypes = healthSummaryTypeList.toArray(new HealthSummaryType[0]);
        }
        
        rtc.setHealthSummaryTypes(healthSummaryTypes);
        return rtc;
    }
}
