/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.Code;
import gov.va.med.vds.vistabeans.Immunization;
import gov.va.med.vds.vistabeans.PCE;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.util.*;
import java.util.concurrent.*;


/**
 *
 *
 */
public class ImmunizationsRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(ImmunizationsRPC.class);

    private static final String XPATH_IMMUNIZATION = "//immunization";
    private static final String XPATH_ADMINISTERED = "./administered/@value";
    private static final String XPATH_CONTRAINDICATED = "./contraindicated/@value";
    private static final String XPATH_CPT_CODE = "./cpt/@code";
    private static final String XPATH_CPT_NAME = "./cpt/@name";
    private static final String XPATH_ID = "./id/@value";
    private static final String XPATH_NAME = "./name/@value";
    private static final String XPATH_PROVIDER_NAME = "./provider/@name";
    private static final String XPATH_REACTION = "./reaction/@value";
    private static final String XPATH_SERIES = "./series/@value";
    private static final String XPATH_BODY_SITE = "./bodySite/@value";
    private static final String XPATH_CVX = "./cvx/@value";
    private static final String XPATH_DOCUMENTED_BY_NAME = "./documentedBy/@name";
    private static final String XPATH_DOSE = "./dose/@value";
    private static final String XPATH_EXPIRATION_DATE = "./expirationDate/@value";
    private static final String XPATH_LOT = "./lot/@value";
    private static final String XPATH_MANUFACTURER = "./manufacturer/@value";
    private static final String XPATH_ORDERING_PROVIDER = "./orderingProvider/@name";
    private static final String XPATH_ROUTE = "./route/@value";
    private static final String XPATH_SOURCE_NAME = "./source/@name";
    private static final String XPATH_LOCATION = "./location/@value";
    private static final String XPATH_COMMENT = "./comment";
    
    private static final XPathExpression exprName = XPathUtil.xpathCompile(XPATH_NAME);
    private static final XPathExpression exprId = XPathUtil.xpathCompile(XPATH_ID);
    private static final XPathExpression exprAdministered = XPathUtil.xpathCompile(XPATH_ADMINISTERED);
    private static final XPathExpression exprContraindicated = XPathUtil.xpathCompile(XPATH_CONTRAINDICATED);
    private static final XPathExpression exprCptCode = XPathUtil.xpathCompile(XPATH_CPT_CODE);
    private static final XPathExpression exprCptName = XPathUtil.xpathCompile(XPATH_CPT_NAME);
    private static final XPathExpression exprProviderName = XPathUtil.xpathCompile(XPATH_PROVIDER_NAME);
    private static final XPathExpression exprReaction = XPathUtil.xpathCompile(XPATH_REACTION);
    private static final XPathExpression exprSeries = XPathUtil.xpathCompile(XPATH_SERIES);
    private static final XPathExpression exprBodySite = XPathUtil.xpathCompile(XPATH_BODY_SITE);
    private static final XPathExpression exprCvx = XPathUtil.xpathCompile(XPATH_CVX);
    private static final XPathExpression exprDocumentedByName = XPathUtil.xpathCompile(XPATH_DOCUMENTED_BY_NAME);
    private static final XPathExpression exprDose = XPathUtil.xpathCompile(XPATH_DOSE);
    private static final XPathExpression exprExpirationDate = XPathUtil.xpathCompile(XPATH_EXPIRATION_DATE);
    private static final XPathExpression exprLot = XPathUtil.xpathCompile(XPATH_LOT);
    private static final XPathExpression exprManufacturer = XPathUtil.xpathCompile(XPATH_MANUFACTURER);
    private static final XPathExpression exprOrderingProvider = XPathUtil.xpathCompile(XPATH_ORDERING_PROVIDER);
    private static final XPathExpression exprRoute = XPathUtil.xpathCompile(XPATH_ROUTE);
    private static final XPathExpression exprSourceName = XPathUtil.xpathCompile(XPATH_SOURCE_NAME);
    private static final XPathExpression exprComment = XPathUtil.xpathCompile(XPATH_COMMENT);
    private static final XPathExpression exprLocation = XPathUtil.xpathCompile(XPATH_LOCATION);
    private static final XPathExpression exprImmunization = XPathUtil.xpathCompile(XPATH_IMMUNIZATION);

    private static final int PARSER_BATCH_SIZE = APP_CONFIG.getParserBatchSizeImmunizations();

    public ImmunizationsRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Immunization[] getImmunizationList() throws RPCException {
        return getImmunizationList(cxn.getPatientIen());
    }

    //SB: missing reaction and series
    public Immunization[] getImmunizationList(String patientIen) throws RPCException {
        String request = buildGetImmunizationsListRequest(patientIen);
        String response = (String) cxn.query(request);
        return toImmunizationList(response/*, patientIen*/);
    }

    private String buildGetImmunizationsListRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQPX IMMUN LIST");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

    private Immunization[] toImmunizationList(String response/*, String patientIen*/) {
        Immunization[] rtc;
        if ("".equals(response)) {
            rtc = new Immunization[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);

            ArrayList<Immunization> lst = new ArrayList<Immunization>();
            Immunization rec = null;
            for (int i = 0; i < lines.length; i++) {
                if ("".equals(lines[i])) {
                    continue;
                }
                String[] flds = StringUtils.split(lines[i], StringUtils.CARET);

                rec = new Immunization();
                rec.setId(flds[0]);
                rec.setName(flds[1]);
                Calendar dateTimeCal = Utils.getCal(Timestamp.toUtcString(flds[2]));
                rec.setDateTime(Utils.formatDisplayDate(dateTimeCal));
                fillCommonDataBeanValues(rec);

                lst.add(rec);
            }

            rtc = lst.toArray(new Immunization[0]);
        }
        return rtc;
    }

    /**
     * Uses the VPR RPC with "immunizations" as the type.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of immunizations
     * @param endDate    The end of the date range for the list of immunizations
     * @return The list of immunizations for the specified date range
     * @throws RPCException
     */
    public Immunization[] getImmunizationsVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.IMMUNIZATIONS);
        String response = (String) cxn.query(request);

        Immunization[] immunizations = parseImmunizationsFromVPR(response);
        ArrayList<Immunization> rtc = new ArrayList<Immunization>(Arrays.asList(immunizations));

//        // Add the skinTests
//        PCERPC pcerpc = new PCERPC(cxn);
//        PCE[] skinTests = pcerpc.getSkinTests(patientIen, startDate, endDate);
//        if (skinTests != null && skinTests.length > 0) {
//            // map each skinTest into an Immunization
//            for (PCE skinTest : skinTests) {
//                Immunization immunization = new Immunization();
//                fillCommonDataBeanValues(immunization);
//
//                immunization.setName(skinTest.getName());
//                String administered = Timestamp.fromDateTime(skinTest.getReadDate());
//                String formattedAdministered = Timestamp.formatDisplayDateNoBadMonthOrDay(administered);
//                immunization.setDateTime(formattedAdministered);
//                immunization.setResult(skinTest.getResult());
//                immunization.setUnits(skinTest.getReading());
//
//                rtc.add(immunization);
//            }
//        }

        return rtc.toArray(new Immunization[0]);
    }

    /**
     * Retrieves Skin Tests
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of immunizations
     * @param endDate    The end of the date range for the list of immunizations
     * @return The list of immunizations for the specified date range
     * @throws RPCException
     */
    public Immunization[] getImmunizationSkinTests(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        ArrayList<Immunization> rtc = new ArrayList<Immunization>();

        // Add the skinTests
        PCERPC pcerpc = new PCERPC(cxn);
        PCE[] skinTests = pcerpc.getSkinTests(patientIen, startDate, endDate);
        if (skinTests != null && skinTests.length > 0) {
            // map each skinTest into an Immunization
            for (PCE skinTest : skinTests) {
                Immunization immunization = new Immunization();
                fillCommonDataBeanValues(immunization);

                immunization.setName(skinTest.getName());
                String administered = Timestamp.fromDateTime(skinTest.getEnteredDate());
                String formattedAdministered = Timestamp.formatDisplayDateNoBadMonthOrDay(administered);
                immunization.setDateTime(formattedAdministered);
                StringBuilder immunizationResultSB = new StringBuilder();
                immunizationResultSB.append(skinTest.getReading());
                immunizationResultSB.append("  ");
                immunizationResultSB.append(skinTest.getResult());
                immunization.setResult(immunizationResultSB.toString());
                immunization.setUnits(skinTest.getReading());

                rtc.add(immunization);
            }
        }

        return rtc.toArray(new Immunization[0]);
    }


    /**
     * Uses the VPR RPC with "immunizations" as the type.
     *
     * @param startDate The start of the date range for the list of immunizations
     * @param endDate   The end of the date range for the list of immunizations
     * @return The list of immunizations for the specified date range
     * @throws RPCException
     */
    public Immunization[] getImmunizationsVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getImmunizationsVPR(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Retrieves Skin Tests.
     *
     * @param startDate The start of the date range for the list of immunizations
     * @param endDate   The end of the date range for the list of immunizations
     * @return The list of immunizations for the specified date range
     * @throws RPCException
     */
    public Immunization[] getImmunizationSkinTests(Calendar startDate, Calendar endDate) throws RPCException {
        return getImmunizationSkinTests(cxn.getPatientIen(), startDate, endDate);
    }

    private Immunization[] parseImmunizationsFromVPR(String response) throws RPCException {
        Immunization[] rtc;
        if ("".equals(response)) {
            rtc = new Immunization[0];
        } else {

            List<Immunization> immunizationList = new ArrayList<Immunization>();

            Document doc = null;

            try {
                doc = parseXMLDocument(response);
            } catch (Exception e) {
                throw new RPCException(e);
            }

            NodeList immunizationsNodeList = null;
            try {
                immunizationsNodeList = (NodeList) exprImmunization.evaluate(doc, XPathConstants.NODESET);
            } catch (Exception e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);
            int listLength = immunizationsNodeList.getLength();
            for (int immunizationNum = 0; immunizationNum < listLength; immunizationNum++) {
                parserBatchManager.addNode(immunizationsNodeList.item(immunizationNum));
            }

            List<VPRImmunizationParser> immunizationParserList = new ArrayList<VPRImmunizationParser>();
            for(List<Node> parserBatch : parserBatchManager.getParserBatchList()) {
                immunizationParserList.add(new VPRImmunizationParser(parserBatch));
            }

            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<Immunization>>> futures = null;
            try {
                futures = executorService.invokeAll(immunizationParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);

            } catch (InterruptedException e) {
                LOGGER.error(e.getMessage(), e);
                throw new RPCException(e);
            }

            Iterator<Future<List<Immunization>>> futuresItr = futures.iterator();

            while (futuresItr.hasNext()) {
                try {
                    Future<List<Immunization>> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Immunization Data");
                    } else {
                        immunizationList.addAll(future.get());
                    }

                } catch (InterruptedException e) {
                    LOGGER.error(e.getMessage(), e);
                } catch (ExecutionException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }

            rtc = immunizationList.toArray(new Immunization[0]);
        }
        return rtc;
    }

    private class VPRImmunizationParser implements Callable<List<Immunization>> {
        private List<Node> immunizationNodeList;

        public VPRImmunizationParser(List<Node> immunizationNodeList) {
            this.immunizationNodeList = immunizationNodeList;
        }

        public List<Immunization> call() throws RPCException {
            List<Immunization> immunizations = new ArrayList<Immunization>();

            for(Node immunizationNode : immunizationNodeList) {
                Immunization immunization = new Immunization();
                String immunizationName = null;
                String immunizationIen = null;
                String contraindicatedString = null;
                String cptCode = null;
                String cptName = null;
                String providerName = null;
                //providerCode = null,
                String reaction = null;
                String series = null;
                String administered = null;

                try {
                    immunizationName = (String) exprName.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    immunizationName = null;
                    LOGGER.error(e.getMessage(), e);
                }
                try {
                    immunizationIen = (String) exprId.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    immunizationIen = null;
                    LOGGER.error(e.getMessage(), e);
                }
                try {
                    contraindicatedString = (String) exprContraindicated.evaluate(immunizationNode, XPathConstants.STRING);

                } catch (XPathExpressionException e) {
                    contraindicatedString = null;
                    LOGGER.error(e.getMessage(), e);
                }
                try {
                    cptCode = (String) exprCptCode.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    cptCode = null;
                    LOGGER.error(e.getMessage(), e);
                }
                try {
                    cptName = (String) exprCptName.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    cptName = null;
                    LOGGER.error(e.getMessage(), e);
                }
                try {
                    providerName = (String) exprProviderName.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    providerName = null;
                    LOGGER.error(e.getMessage(), e);
                }
/*
            try {
                providerCode = (String) exprProviderCode.evaluate( immunizationNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                providerCode = null;
                logger.error(e.getMessage(), e);
            }
*/
                try {
                    reaction = (String) exprReaction.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    reaction = null;
                    LOGGER.error(e.getMessage(), e);
                }
                try {
                    series = (String) exprSeries.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    series = null;
                    LOGGER.error(e.getMessage(), e);
                }
                try {
                    administered = (String) exprAdministered.evaluate(immunizationNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    administered = null;
                    LOGGER.error(e.getMessage(), e);
                }

                String bodySite = XPathUtil.evaluateNodeToString(immunizationNode, exprBodySite, LOGGER);
                String cvx = XPathUtil.evaluateNodeToString(immunizationNode, exprCvx, LOGGER);
                String documentedByName = XPathUtil.evaluateNodeToString(immunizationNode, exprDocumentedByName, LOGGER);
                String dose = XPathUtil.evaluateNodeToString(immunizationNode, exprDose, LOGGER);
                String expirationDateString = XPathUtil.evaluateNodeToString(immunizationNode, exprExpirationDate, LOGGER);
                String lot = XPathUtil.evaluateNodeToString(immunizationNode, exprLot, LOGGER);
                String manufacturer = XPathUtil.evaluateNodeToString(immunizationNode, exprManufacturer, LOGGER);
                String orderingProvider = XPathUtil.evaluateNodeToString(immunizationNode, exprOrderingProvider, LOGGER);
                String route = XPathUtil.evaluateNodeToString(immunizationNode, exprRoute, LOGGER);
                String sourceName = XPathUtil.evaluateNodeToString(immunizationNode, exprSourceName, LOGGER);
                String comment = XPathUtil.evaluateNodeToString(immunizationNode, exprComment, LOGGER);
                String location = XPathUtil.evaluateNodeToString(immunizationNode, exprLocation, LOGGER);

                immunization.setBodySite(bodySite);
                immunization.setCvx(cvx);
                immunization.setDocumentedBy(documentedByName);
                immunization.setDose(dose);
                Calendar expirationDate = null;
                if (expirationDateString != null) {
                    expirationDate = Timestamp.toDateTime(expirationDateString);
                }
                immunization.setLotExpirationDate(expirationDate);
                immunization.setLot(lot);
                immunization.setManufacturerName(manufacturer);
                immunization.setOrderingProvider(orderingProvider);
                immunization.setRoute(route);
                immunization.setSource(sourceName);
                immunization.setComment(comment);
                immunization.setVisitLocation(location);

                immunization.setName(immunizationName);
                immunization.setId(immunizationIen);
                immunization.setEncounterProvider(providerName);
                immunization.setSeries(series);
                immunization.setReaction(reaction);
                //placeholder for DoD results
                immunization.setResult("");
                if (contraindicatedString != null) {
                    if ("1".equals(contraindicatedString)) {
                        immunization.setContraindicated(true);
                    } else {
                        immunization.setContraindicated(false);
                    }
                }

                if (!StringUtils.isEmpty(administered)) {
//                Calendar administeredDate;
//                administeredDate = Timestamp.formatDisplayDateNoBadMonthOrDay(administered);
                    String formattedAdministered = Timestamp.formatDisplayDateNoBadMonthOrDay(administered);
                    immunization.setDateTime(formattedAdministered);
                }
                List<Code> codeList = new ArrayList<Code>();
                if (!StringUtils.isEmpty(cptCode)) {
                    Code immunizationCode = new Code();
                    immunizationCode.setCode(cptCode);
                    immunizationCode.setDisplay(cptName);
                    immunizationCode.setSystem("CPT");
                    codeList.add(immunizationCode);
                }

                if (!StringUtils.isEmpty(cvx)) {
                    Code immunizationCode = new Code();
                    immunizationCode.setCode(cvx);
                    immunizationCode.setDisplay("");
                    immunizationCode.setSystem("CVX");
                    codeList.add(immunizationCode);
                }

                Code immunizationCode = new Code();
                immunizationCode.setCode(immunizationName);
                immunizationCode.setDisplay(immunizationName);
                immunizationCode.setSystem("TEXT");
                codeList.add(immunizationCode);

                immunization.setCodes(codeList.toArray(new Code[0]));

                fillCommonDataBeanValues(immunization);

                immunizations.add(immunization);
            }

            return immunizations;
        }
    }
}
