/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.Code;
import gov.va.med.vds.vistabeans.InsuranceBean;
import gov.va.med.vds.vistabeans.Telecom;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 *
 *
 */
public class InsuranceRPC extends BaseRPC {

    private static final Logger LOGGER = LoggerFactory.getLogger(InsuranceRPC.class);

    private static final String XPATH_INSURANCE_POLICY = "//insurancePolicy";
    private static final String XPATH_EFFECTIVE_DATE = "./effectiveDate/@value";
    private static final String XPATH_EXPIRATION_DATE = "./expirationDate/@value";
    private static final String XPATH_ID = "./id/@value";
    private static final String XPATH_COMPANY = "./company";
    private static final String XPATH_SUBSCRIBER = "./subscriber";
    private static final String XPATH_CODE = "./@code";
    private static final String XPATH_NAME = "./@name";
    private static final String XPATH_ADDRESS = "./address";
    private static final String XPATH_DOB = "./dob";
    private static final String XPATH_TELECOM_LIST = "./telecomList";
    private static final String XPATH_STREET_LINE1 = "./@streetLine1";
    private static final String XPATH_STREET_LINE2 = "./@streetLine2";
    private static final String XPATH_STREET_LINE3 = "./@streetLine3";
    private static final String XPATH_CITY = "./@city";
    private static final String XPATH_STATE_PROVINCE = "./@stateProvince";
    private static final String XPATH_POSTAL_CODE = "./@postalCode";
    private static final String XPATH_TELECOM = "./telecom";
    private static final String XPATH_USAGE_TYPE = "./@usageType";
    private static final String XPATH_VALUE = "./@value";
    private static final String XPATH_GROUP_NAME = "./groupName/@value";
    private static final String XPATH_GROUP_NUMBER = "./groupNumber/@value";
    private static final String XPATH_INSURANCE_TYPE_CODE = "./insuranceType/@code";
    private static final String XPATH_INSURANCE_TYPE_NAME = "./insuranceType/@name";
    private static final String XPATH_MEMBER_ID = "./memberID/@value";
    private static final String XPATH_RELATIONSHIP = "./relationship/@value";

    private static XPathExpression exprInsurancePolicy = XPathUtil.xpathCompile(XPATH_INSURANCE_POLICY);
    private static XPathExpression exprEffectiveDate = XPathUtil.xpathCompile(XPATH_EFFECTIVE_DATE);
    private static XPathExpression exprExpirationDate = XPathUtil.xpathCompile(XPATH_EXPIRATION_DATE);
    private static XPathExpression exprId = XPathUtil.xpathCompile(XPATH_ID);
    private static XPathExpression exprCompany = XPathUtil.xpathCompile(XPATH_COMPANY);
    private static XPathExpression exprSubscriber = XPathUtil.xpathCompile(XPATH_SUBSCRIBER);
    private static XPathExpression exprCode = XPathUtil.xpathCompile(XPATH_CODE);
    private static XPathExpression exprName = XPathUtil.xpathCompile(XPATH_NAME);
    private static XPathExpression exprDob = XPathUtil.xpathCompile(XPATH_DOB);
    private static XPathExpression exprAddress = XPathUtil.xpathCompile(XPATH_ADDRESS);
    private static XPathExpression exprTelecomList = XPathUtil.xpathCompile(XPATH_TELECOM_LIST);
    private static XPathExpression exprStreetLine1 = XPathUtil.xpathCompile(XPATH_STREET_LINE1);
    private static XPathExpression exprStreetLine2 = XPathUtil.xpathCompile(XPATH_STREET_LINE2);
    private static XPathExpression exprStreetLine3 = XPathUtil.xpathCompile(XPATH_STREET_LINE3);
    private static XPathExpression exprCity = XPathUtil.xpathCompile(XPATH_CITY);
    private static XPathExpression exprStateProvince = XPathUtil.xpathCompile(XPATH_STATE_PROVINCE);
    private static XPathExpression exprPostalCode = XPathUtil.xpathCompile(XPATH_POSTAL_CODE);
    private static XPathExpression exprTelecom = XPathUtil.xpathCompile(XPATH_TELECOM);
    private static XPathExpression exprUsageType = XPathUtil.xpathCompile(XPATH_USAGE_TYPE);
    private static XPathExpression exprValue = XPathUtil.xpathCompile(XPATH_VALUE);
    private static XPathExpression exprGroupName = XPathUtil.xpathCompile(XPATH_GROUP_NAME);
    private static XPathExpression exprGroupNumber = XPathUtil.xpathCompile(XPATH_GROUP_NUMBER);
    private static XPathExpression exprInsuranceTypeCode = XPathUtil.xpathCompile(XPATH_INSURANCE_TYPE_CODE);
    private static XPathExpression exprInsuranceTypeName = XPathUtil.xpathCompile(XPATH_INSURANCE_TYPE_NAME);
    private static XPathExpression exprMemberId = XPathUtil.xpathCompile(XPATH_MEMBER_ID);
    private static XPathExpression exprRelationship = XPathUtil.xpathCompile(XPATH_RELATIONSHIP);

    private static final int PARSER_BATCH_SIZE = APP_CONFIG.getParserBatchSizeInsurance();

    public InsuranceRPC(RPCConnection cxn) {
        super(cxn);
    }

    /**
     * Uses the VPR RPC with "insurancePolicies" as the type.
     *
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public InsuranceBean[] getPatientInsurance(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientInsurance(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Uses the VPR RPC with "insurancePolicies" as the type.
     *
     * @param patientIen The patient IEN
     * @return The list of appointments for the specified date range
     * @throws gov.va.med.rpc.RPCException
     */
    public InsuranceBean[] getPatientInsurance(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetEncountersRequest(patientIen/*, startDate, endDate*/);
        String response = (String) cxn.query(request);

        InsuranceBean[] policies = parseInsurance(response);

        return policies;
    }

    /**
     * Builds the VPR RPC request string using "insurancePolicies" as the type. The problems contain the list of
     * appointments. The RPC will fail if the "VPR APPLICATION PROXY" context is not set. That is set when
     * setting up the connection.
     *
     * @param patientIen Patient IEN
     * @param startDate  Start date of range
     * @param endDate    End date of range
     * @return The RPC response message string
     */
    private String buildGetEncountersRequest(String patientIen/*, Calendar startDate, Calendar endDate*/) {
        return VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, VPRUtil.VPRType.INSURANCE_POLICIES);
    }

    private InsuranceBean[] parseInsurance(String response) throws RPCException {
        InsuranceBean[] rtc;
        if ("".equals(response)) {
            rtc = new InsuranceBean[0];
        } else {
    
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            ArrayList<InsuranceBean> insuranceList = new ArrayList<InsuranceBean>();
    
            Document document = null;

            try {
                document = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList policiesNodeList = null;
            try {
                policiesNodeList = (NodeList) exprInsurancePolicy.evaluate(document, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);
            int policiesNodeListLength = policiesNodeList.getLength();
            for (int policyNum = 0; policyNum < policiesNodeListLength; policyNum++) {
                parserBatchManager.addNode(policiesNodeList.item(policyNum));
            }

            List<VPRInsurancePoliciesParser> policiesParserList = new ArrayList<VPRInsurancePoliciesParser>();
            for(List<Node> batchList : parserBatchManager.getParserBatchList()) {
                policiesParserList.add(new VPRInsurancePoliciesParser(batchList));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<InsuranceBean>>> futures = null;
            try {
                futures = executorService.invokeAll(policiesParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                XPathUtil.logErrorMessage(LOGGER, e);
                throw new RPCException(e);
            }
    
            Iterator<Future<List<InsuranceBean>>> futureIterator = futures.iterator();
    
            while (futureIterator.hasNext()) {
                try {
                    Future<List<InsuranceBean>> future = futureIterator.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Insurance Data");
                    } else {
                        insuranceList.addAll(future.get());
                    }
                } catch (InterruptedException ie) {
                    XPathUtil.logErrorMessage(LOGGER, ie);
                } catch (ExecutionException ee) {
                    XPathUtil.logErrorMessage(LOGGER, ee);
                }
            }
    
            rtc = insuranceList.toArray(new InsuranceBean[0]);
        }
        return rtc;
    }

    private class VPRInsurancePoliciesParser implements Callable<List<InsuranceBean>> {
        private static final String ALPHA_ONLY_REGEX = "[a-zA-Z]+";
        private static final String NUMERIC_ONLY_REGEX = "[0-9]+";
        private List<Node> policyNodeList;
        private Pattern numbersOnlyPattern =  Pattern.compile(NUMERIC_ONLY_REGEX);
        private Pattern alphaOnlyPattern = Pattern.compile(ALPHA_ONLY_REGEX);

        public VPRInsurancePoliciesParser(List<Node> policyNodeList) {
            this.policyNodeList = policyNodeList;
        }

        public List<InsuranceBean> call() throws Exception {

            List<InsuranceBean> policyList = new ArrayList<InsuranceBean>();
            for(Node policyNode : policyNodeList) {
                InsuranceBean policy = new InsuranceBean();
                fillCommonDataBeanValues(policy);

                String effectiveDateString = null;
                Calendar effectiveDateCalendar = null;
                try {
                    effectiveDateString = (String) exprEffectiveDate.evaluate(policyNode, XPathConstants.STRING);
                    if (effectiveDateString != null) {
                        effectiveDateCalendar = Timestamp.toDateTime(effectiveDateString);
                    }
                } catch (XPathExpressionException e) {
                    XPathUtil.logErrorMessage(LOGGER, e);
                }
                String expirationDateString = null;
                Calendar expirationDateCalendar = null;
                try {
                    expirationDateString = (String) exprExpirationDate.evaluate(policyNode, XPathConstants.STRING);
                    if (expirationDateString != null) {
                        expirationDateCalendar = Timestamp.toDateTime(expirationDateString);
                    }
                } catch (XPathExpressionException e) {
                    XPathUtil.logErrorMessage(LOGGER, e);
                }

//            String id = XPathUtil.evaluateNodeToString(policyNode, exprId);
                String groupName = XPathUtil.evaluateNodeToString(policyNode, exprGroupName, LOGGER);
                String groupNumber = XPathUtil.evaluateNodeToString(policyNode, exprGroupNumber, LOGGER);
                String insuranceTypeCode = XPathUtil.evaluateNodeToString(policyNode, exprInsuranceTypeCode, LOGGER);
                String insuranceTypeName = XPathUtil.evaluateNodeToString(policyNode, exprInsuranceTypeName, LOGGER);
                String memberId = XPathUtil.evaluateNodeToString(policyNode, exprMemberId, LOGGER);
                String relationship = XPathUtil.evaluateNodeToString(policyNode, exprRelationship, LOGGER);

                Node subscriberNode = XPathUtil.evaluateNodeToNode(policyNode, exprSubscriber, LOGGER);
                String subscriberName = null;
                String subscriberId = null;
                String subscriberDobString = null;
                Calendar subscriberDob = null;
                if (subscriberNode != null) {
                    subscriberName = XPathUtil.evaluateNodeToString(subscriberNode, exprName, LOGGER);
                    subscriberId = XPathUtil.evaluateNodeToString(subscriberNode, exprCode, LOGGER);
                    subscriberDobString = XPathUtil.evaluateNodeToString(subscriberNode, exprDob, LOGGER);
                    if (subscriberDobString != null) {
                        subscriberDob = Timestamp.toDateTime(subscriberDobString);
                    }
                }

                Node companyNode = XPathUtil.evaluateNodeToNode(policyNode, exprCompany, LOGGER);
                String companyName = null;
                String companyId = null;
                String companyStreetLine1 = null;
                String companyStreetLine2 = null;
                String companyStreetLine3 = null;
                String companyCity = null;
                String companyStateProvince = null;
                String companyPostalCode = null;
                if (companyNode != null) {
                    companyName = XPathUtil.evaluateNodeToString(companyNode, exprName, LOGGER);
                    companyId = XPathUtil.evaluateNodeToString(companyNode, exprId, LOGGER);

                    Node companyAddressNode = XPathUtil.evaluateNodeToNode(companyNode, exprAddress, LOGGER);
                    if (companyAddressNode != null) {
                        companyStreetLine1 = XPathUtil.evaluateNodeToString(companyAddressNode, exprStreetLine1, LOGGER);
                        companyStreetLine2 = XPathUtil.evaluateNodeToString(companyAddressNode, exprStreetLine2, LOGGER);
                        companyStreetLine3 = XPathUtil.evaluateNodeToString(companyAddressNode, exprStreetLine3, LOGGER);
                        companyCity = XPathUtil.evaluateNodeToString(companyAddressNode, exprCity, LOGGER);
                        companyStateProvince = XPathUtil.evaluateNodeToString(companyAddressNode, exprStateProvince, LOGGER);
                        companyPostalCode = XPathUtil.evaluateNodeToString(companyAddressNode, exprPostalCode, LOGGER);

                        // Due to a bug in the VPR RPC, the postalCode is the State name and the
                        // stateProvince is the IEN for the state. For now we'll "fix" the data by putting the
                        // postalCode as the state and have no postalCode.
                        Matcher stateProvAsNumbersMatcher = numbersOnlyPattern.matcher(companyStateProvince);
                        Matcher postalCodeAsAlphaMatcher = alphaOnlyPattern.matcher(companyPostalCode);
                        if (stateProvAsNumbersMatcher.matches() && postalCodeAsAlphaMatcher.matches()) {
                            companyStateProvince = companyPostalCode;
                            companyPostalCode = "";
                        }

                        policy.setCompanyAddressLine1(companyStreetLine1);
                        policy.setCompanyAddressLine2(companyStreetLine2);
                        policy.setCompanyAddressLine3(companyStreetLine3);
                        policy.setCompanyCity(companyCity);
                        policy.setCompanyStateProvince(companyStateProvince);
                        policy.setCompanyPostalCode(companyPostalCode);
                    }

                    Node telecomListNode = XPathUtil.evaluateNodeToNode(companyNode, exprTelecomList, LOGGER);
                    if (telecomListNode != null) {
                        NodeList telecomNodeList = XPathUtil.evaluateNodeToNodeList(telecomListNode, exprTelecom, LOGGER);

                        if (telecomNodeList != null) {
                            int telecomNodeListLength = telecomNodeList.getLength();
                            List<Telecom> telecomList = new ArrayList<Telecom>();
                            for (int telecomNum = 0; telecomNum < telecomNodeListLength; telecomNum++) {
                                Node telecomNode = telecomNodeList.item(telecomNum);
                                telecomNode.getParentNode().removeChild(telecomNode);

                                String telecomValue = XPathUtil.evaluateNodeToString(telecomNode, exprValue, LOGGER);
                                String telecomUsageType = XPathUtil.evaluateNodeToString(telecomNode, exprUsageType, LOGGER);

                                Telecom telecom = new Telecom();
                                telecom.setValue(telecomValue);
                                telecom.setUsageType(telecomUsageType);

                                telecomList.add(telecom);
                            }

                            if (!telecomList.isEmpty()) {
                                policy.setCompanyTelecom(telecomList.toArray(new Telecom[0]));
                            }
                        }
                    }

                    policy.setCompanyName(companyName);
                    policy.setCompanyIen(companyId);

                }

                policy.setEffectiveDate(effectiveDateCalendar);
                policy.setExpirationDate(expirationDateCalendar);

                policy.setPlanName(groupName);
                policy.setGroupNumber(groupNumber);
                policy.setInsuranceType(insuranceTypeName);
                policy.setInsuranceTypeId(insuranceTypeCode);
                policy.setRelationShip(relationship);
                policy.setSubscriberId(subscriberId);
                policy.setSubscriberDOB(subscriberDob);
                policy.setPolicyHolder(subscriberName);
                policy.setMemberId(memberId);

                List<Code> codeList = new ArrayList<Code>();

                if (insuranceTypeCode != null && !StringUtils.isEmpty(insuranceTypeCode)) {
                    Code insType = new Code();
                    insType.setSystem("VUID");
                    insType.setCode(insuranceTypeCode);
                    codeList.add(insType);
                }

                if(insuranceTypeName != null && !StringUtils.isEmpty(insuranceTypeName)){
                    Code insType = new Code();
                    insType.setSystem("TEXT");
                    insType.setCode(insuranceTypeName);
                    codeList.add(insType);
                }

                if(codeList.size() > 0) {
                    policy.setCodes(codeList.toArray(new Code[codeList.size()]));
                }


                policyList.add(policy);
            }

            return policyList;
        }
    }
}
