/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.util.*;
import java.util.concurrent.*;


/**
 *
 *
 */
public class MedicationsRPC extends BaseRPC {

    private static final Logger LOGGER = LoggerFactory.getLogger(MedicationsRPC.class);

    private static final String INPATIENT_SORT = "I";
    private static final String OUTPATIENT_SORT = "O";
//    private static final String ALL_SORT = "A";

    private static final String XPATH_MED = "//med";
    private static final String XPATH_DAYS_SUPPLY = "./daysSupply/@value";                 // O
    private static final String XPATH_FILLS_REMAINING = "./fillsRemaining/@value";         // O
    private static final String XPATH_FILLS_ALLOWED = "./fillsAllowed/@value";         // O
    private static final String XPATH_LAST_FILLED = "./lastFilled/@value";                 // O
    private static final String XPATH_MED_ID = "./medID/@value";
    private static final String XPATH_NAME = "./name/@value";
    private static final String XPATH_ORDER_ID = "./orderID/@value";
    private static final String XPATH_ORDERED = "./ordered/@value";
    private static final String XPATH_ORDERING_PROVIDER = "./orderingProvider";
    private static final String XPATH_CURRENT_PROVIDER = "./currentProvider";
    private static final String XPATH_CODE = "./@code";
    private static final String XPATH_NAME_ATTRIB = "./@name";
    private static final String XPATH_TAXONOMY_CODE = "./@taxonomyCode";
    private static final String XPATH_PROVIDER_TYPE = "./@providerType";
    private static final String XPATH_CLASSIFICATION = "./@classification";
    private static final String XPATH_SPECIALIZATION = "./@specialization";
    private static final String XPATH_SERVICE = "./@service";
    private static final String XPATH_OFFICE_PHONE = "./@officePhone";
    private static final String XPATH_FAX = "./@fax";
    private static final String XPATH_ANALOG_PAGER = "./@analogPager";
    private static final String XPATH_EMAIL = "./@email";
    private static final String XPATH_PHARMACIST_CODE = "./pharmacist/@code";              // O, I, V
    private static final String XPATH_PHARMACIST_NAME = "./pharmacist/@name";
    private static final String XPATH_PRESCRIPTION = "./prescription/@value";              // O
    private static final String XPATH_PT_INSTRUCTIONS = "./ptInstructions";                // O
    private static final String XPATH_QUANTITY = "./quantity/@value";                      // O
    private static final String XPATH_SIG = "./sig";                                       // O, N, I
    private static final String XPATH_RATE = "./rate/@value";                                       // V
    private static final String XPATH_START = "./start/@value";
    private static final String XPATH_STOP = "./stop/@value";                     // O "Prescription", N "OTC"
    private static final String XPATH_VA_STATUS = "./vaStatus/@value";                     // "ACTIVE, DISCONTINUED, EXPIRED"
    private static final String XPATH_VA_TYPE = "./vaType/@value";                         // "O, N, I, V"
    private static final String XPATH_DOSES = "./doses";
    private static final String XPATH_DOSE = "./dose";
    private static final String XPATH_DOSE_ROUTE = "./@route";
    private static final String XPATH_DOSE_SCHEDULE = "./@schedule";
    private static final String XPATH_FILLS = "./fills";
    private static final String XPATH_FILL = "./fill";
    private static final String XPATH_FILL_FILL_DATE = "./@fillDate";
    private static final String XPATH_FILL_FILL_ROUTING = "./@fillRouting";
    private static final String XPATH_FILL_RELEASE_DATE = "./@releaseDate";
    private static final String XPATH_FILL_FILL_QUANTITY = "./@fillQuantity";
    private static final String XPATH_FILL_FILL_DAYS_SUPPLY = "./@fillDaysSupply";
    private static final String XPATH_FILL_PARTIAL = "./@partial";
    private static final String XPATH_PRODUCTS = "./products";
    private static final String XPATH_PRODUCT = "./product";
    private static final String XPATH_PRODUCT_NAME = "./@name";
    private static final String XPATH_PRODUCT_VA_PRODUCT_NAME = "./vaProduct/@name";
    private static final String XPATH_PRODUCT_VA_PRODUCT_VUID = "./vaProduct/@vuid";
    
    private static XPathExpression exprMed = XPathUtil.xpathCompile(XPATH_MED);
    private static XPathExpression exprDaysSupply = XPathUtil.xpathCompile(XPATH_DAYS_SUPPLY);
    private static XPathExpression exprFillsRemaining = XPathUtil.xpathCompile(XPATH_FILLS_REMAINING);
    private static XPathExpression exprFillsAllowed = XPathUtil.xpathCompile(XPATH_FILLS_ALLOWED);
    private static XPathExpression exprLastFilled = XPathUtil.xpathCompile(XPATH_LAST_FILLED);
    private static XPathExpression exprMedId = XPathUtil.xpathCompile(XPATH_MED_ID);
    private static XPathExpression exprName = XPathUtil.xpathCompile(XPATH_NAME);
    private static XPathExpression exprOrderId = XPathUtil.xpathCompile(XPATH_ORDER_ID);
    private static XPathExpression exprOrdered = XPathUtil.xpathCompile(XPATH_ORDERED);
    private static XPathExpression exprOrderingProvider = XPathUtil.xpathCompile(XPATH_ORDERING_PROVIDER);
    private static XPathExpression exprCurrentProvider = XPathUtil.xpathCompile(XPATH_CURRENT_PROVIDER);
    private static XPathExpression exprNameAttrib = XPathUtil.xpathCompile(XPATH_NAME_ATTRIB);
    private static XPathExpression exprCode = XPathUtil.xpathCompile(XPATH_CODE);
    private static XPathExpression exprTaxonomyCode = XPathUtil.xpathCompile(XPATH_TAXONOMY_CODE);
    private static XPathExpression exprProviderType = XPathUtil.xpathCompile(XPATH_PROVIDER_TYPE);
    private static XPathExpression exprClassification = XPathUtil.xpathCompile(XPATH_CLASSIFICATION);
    private static XPathExpression exprSpecialization = XPathUtil.xpathCompile(XPATH_SPECIALIZATION);
    private static XPathExpression exprService = XPathUtil.xpathCompile(XPATH_SERVICE);
    private static XPathExpression exprOfficePhone = XPathUtil.xpathCompile(XPATH_OFFICE_PHONE);
    private static XPathExpression exprAnalogPager = XPathUtil.xpathCompile(XPATH_ANALOG_PAGER);
    private static XPathExpression exprFax = XPathUtil.xpathCompile(XPATH_FAX);
    private static XPathExpression exprEmail = XPathUtil.xpathCompile(XPATH_EMAIL);
    private static XPathExpression exprPharmacistCode = XPathUtil.xpathCompile(XPATH_PHARMACIST_CODE);
    private static XPathExpression exprPharmacistName = XPathUtil.xpathCompile(XPATH_PHARMACIST_NAME);
    private static XPathExpression exprPrescription = XPathUtil.xpathCompile(XPATH_PRESCRIPTION);
    private static XPathExpression exprPtInstructions = XPathUtil.xpathCompile(XPATH_PT_INSTRUCTIONS);
    private static XPathExpression exprQuantity = XPathUtil.xpathCompile(XPATH_QUANTITY);
    private static XPathExpression exprSig = XPathUtil.xpathCompile(XPATH_SIG);
    private static XPathExpression exprRate = XPathUtil.xpathCompile(XPATH_RATE);
    private static XPathExpression exprStart = XPathUtil.xpathCompile(XPATH_START);
    private static XPathExpression exprStop = XPathUtil.xpathCompile(XPATH_STOP);
    private static XPathExpression exprVaStatus = XPathUtil.xpathCompile(XPATH_VA_STATUS);
    private static XPathExpression exprVaType = XPathUtil.xpathCompile(XPATH_VA_TYPE);
    private static XPathExpression exprDoses = XPathUtil.xpathCompile(XPATH_DOSES);
    private static XPathExpression exprDose = XPathUtil.xpathCompile(XPATH_DOSE);
    private static XPathExpression exprDoseRoute = XPathUtil.xpathCompile(XPATH_DOSE_ROUTE);
    private static XPathExpression exprDoseSchedule = XPathUtil.xpathCompile(XPATH_DOSE_SCHEDULE);
    private static XPathExpression exprFills = XPathUtil.xpathCompile(XPATH_FILLS);
    private static XPathExpression exprFill = XPathUtil.xpathCompile(XPATH_FILL);
    private static XPathExpression exprFillFillDate = XPathUtil.xpathCompile(XPATH_FILL_FILL_DATE);
    private static XPathExpression exprFillFillQuantity = XPathUtil.xpathCompile(XPATH_FILL_FILL_QUANTITY);
    private static XPathExpression exprFillFillRouting = XPathUtil.xpathCompile(XPATH_FILL_FILL_ROUTING);
    private static XPathExpression exprFillReleaseDate = XPathUtil.xpathCompile(XPATH_FILL_RELEASE_DATE);
    private static XPathExpression exprFillFillDaysSupply = XPathUtil.xpathCompile(XPATH_FILL_FILL_DAYS_SUPPLY);
    private static XPathExpression exprFillPartial = XPathUtil.xpathCompile(XPATH_FILL_PARTIAL);
    private static XPathExpression exprProducts = XPathUtil.xpathCompile(XPATH_PRODUCTS);
    private static XPathExpression exprProduct = XPathUtil.xpathCompile(XPATH_PRODUCT);
    private static XPathExpression exprProductName = XPathUtil.xpathCompile(XPATH_PRODUCT_NAME);
    private static XPathExpression exprProductVaProductName = XPathUtil.xpathCompile(XPATH_PRODUCT_VA_PRODUCT_NAME);
    private static XPathExpression exprProductVaProductVuid = XPathUtil.xpathCompile(XPATH_PRODUCT_VA_PRODUCT_VUID);

    private static final int PARSER_BATCH_SIZE = APP_CONFIG.getParserBatchSizeMedications();

    public MedicationsRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Medication[] getOutpatientMedications(Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(startDate, endDate, OUTPATIENT_SORT);
    }

    //SB: Missing refills
    public Medication[] getOutpatientMedications(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(patientIen, startDate, endDate, OUTPATIENT_SORT);
    }

    public Medication[] getInpatientMedications(Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(startDate, endDate, INPATIENT_SORT);
    }

    public Medication[] getInpatientMedications(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(patientIen, startDate, endDate, INPATIENT_SORT);
    }


    public Medication[] getNonVAMeds(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        Medication[] rtc;
        String request = buildGetNonVAMedsRequest(patientIen);
        String response = (String) cxn.query(request);
        Medication[] meds = toNonVAMeds(response/*, patientIen*/);
        if (meds == null) {
            rtc = new Medication[0];
        } else {
            rtc = filterMedsByDate(meds, startDate, endDate);
        }
        return rtc;
    }

    private Medication[] filterMedsByDate(Medication[] meds, Calendar startDate, Calendar endDate) {
        ArrayList<Medication> medsFiltered = new ArrayList<Medication>();

        //filter by date range
        for (Medication med : meds) {
            Calendar medDate = med.getOrderDate();

            if (medDate.compareTo(startDate) >= 0 && medDate.compareTo(endDate) <= 0) {
                medsFiltered.add(med);
            }
        }
        Medication[] rtc;
        if (medsFiltered.isEmpty()) {
            rtc = new Medication[0];
        } else {
            rtc = medsFiltered.toArray(new Medication[0]);

        }
        return rtc;
    }

    private String buildGetNonVAMedsRequest(String patientIen) {
        return RPCUtils.buildReportTextRequest(vistaQueryFactory, patientIen, "", "", 0, "OR_RXN:HERBAL/OTC/NON-VA MEDS~NVA;ORDV06A;0;");
    }

    private Medication[] toNonVAMeds(String response/*, String patientIen*/) {
        Medication[] rtc;
        if ("".equals(response)) {
            rtc = null;
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<Medication> list = new ArrayList<Medication>();
            Medication rec = null;
            StringBuilder sig = new StringBuilder();
//            String dose = "";
            StringBuilder comment = new StringBuilder();
            for (int i = 0; i < lines.length; i++) {
                if ("".equals(lines[i])) {
                    continue;
                }
                String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
                int fldnum = Integer.parseInt(flds[0]);
                switch (fldnum) {
                    case 1:
                        if (rec != null) {
                            rec.setComment(comment.toString());
                            rec.setSigCode(sig.toString());
                            list.add(rec);
                        }
                        rec = new Medication();
                        //rec.IsOutpatient = true;
                        //rec.IsNonVA = true;
                        //rec.Type = "NV";
                        rec.setMedType("O");
                        sig.setLength(0);
//                        dose = "";
                        comment.setLength(0);
                        fillCommonDataBeanValues(rec);
                        break;
                    case 2:
                        if (flds.length == 2 && rec != null) {
                            rec.setDrugName(flds[1]);
                        }
                        break;
                    case 3:
                        if (flds.length == 2 && rec != null) {
                            rec.setActive(flds[1]);
                        }
                            //rec.Status = flds[1];
                        break;
                    case 4:
//                        if (flds.length == 2) {
//                            dose = flds[1];
//                            //rec.Dose = flds[1];
//                        }
                        break;
                    case 5:
                        if (flds.length == 2 && rec != null) {
                            rec.setOrderDate(Utils.getCal(Timestamp.toUtcFromRdv(flds[1])));
                        }
                            //rec.DateDocumented = VistaTimestamp.toUtcFromRdv(flds[1]);
                        break;
                    case 6:
                        if (flds.length == 2 && rec != null) {
                            Provider orderingProvider = new Provider();
                            orderingProvider.setName(flds[1]);
                            rec.setOrderingProvider(orderingProvider);
                        }
                            //rec.Documentor = new Author("",flds[1],"");
                        break;
                    case 7:
                        if (flds.length == 2 && rec != null) {
                            rec.setStopDate(Utils.getCal(Timestamp.toUtcFromRdv(flds[1])));
                        }
                            //rec.StopDate = flds[1];
                        break;
                    case 8:
                        if (flds.length == 2) {
                            sig.append(flds[1]);
                            sig.append('\n');
                        }
                        break;
                    case 10:
                        if (flds.length == 2) {
                            comment.append(flds[1]);
                            comment.append('\n');
                        }
                        break;
                    default:
                        LOGGER.error("Invalid length");
                        break;
                }
            }
            if (rec != null) {
                if (comment.length() > 0) {
                    rec.setComment(comment.toString());
                }
                if (sig.length() > 0) {
                    rec.setSigCode(sig.toString());
                }
                list.add(rec);
            }
            rtc = (Medication[]) list.toArray(new Medication[0]);
        }
        return rtc;
    }

    /**
     * Returns report text for the given medID using ORWPS DETAIL using the patientIen associated with the connection
     *
     * @param medID Medication Identifier that dynamically points to the proper pharmacy file depending on the type of medication.
     * @return Report Formatted Data
     * @throws RPCException
     */
    public FreeTextReport getMedicationDetail(String medID) throws RPCException {
        return getMedicationDetail(cxn.getPatientIen(), medID);
    }

    /**
     * Returns report text for the given patientIen and medID using ORWPS DETAIL.
     *
     * @param patientIen     Patient data file number
     * @param medAndOrderID  Medication Identifier that dynamically points to the proper pharmacy file depending on the type of medication.
     * @return Report Formatted Data
     * @throws RPCException
     */
    public FreeTextReport getMedicationDetail(String patientIen, String medAndOrderID) throws RPCException {
        String medID;
        String orderID;
        StringBuilder reportTextSB = new StringBuilder();

        String[] ids = StringUtils.split(medAndOrderID, StringUtils.CARET);

        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPS DETAIL");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, ids[0]);
        String request = vquery.buildMessage();
        reportTextSB.append((String) cxn.query(request));
        reportTextSB.append("\n==========================================================================\n\n");

        if (ids.length > 1) {
            vquery = vistaQueryFactory.getVistaQuery("ORWPS MEDHIST");
            vquery.addParameter(VistaQuery.LITERAL, patientIen);
            vquery.addParameter(VistaQuery.LITERAL, ids[1]);
            request = vquery.buildMessage();
            reportTextSB.append((String) cxn.query(request));
        }

        FreeTextReport freeTextReport = new FreeTextReport();
        freeTextReport.setReportText(reportTextSB.toString());
        fillCommonDataBeanValues(freeTextReport);

        return freeTextReport;
    }

    /**
     * Uses the VPR RPC with "meds" as the type.
     *
     * @param startDate The start of the date range for the list of problems
     * @param endDate   The end of the date range for the list of problems
     * @return The list of medications for the specified date range
     * @throws RPCException
     */
    public Medication[] getMedicationsVPR(Calendar startDate, Calendar endDate, String inptOrOutpt) throws RPCException {
        return getMedicationsVPR(cxn.getPatientIen(), startDate, endDate, inptOrOutpt);
    }

    /**
     * Uses the VPR RPC with "meds" as the type.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of problems
     * @param endDate    The end of the date range for the list of problems
     * @return The list of medications for the specified date range
     * @throws RPCException
     */
    public Medication[] getMedicationsVPR(String patientIen, Calendar startDate, Calendar endDate, String inptOrOutpt) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.MEDS);
        String response = (String) cxn.query(request);

        Medication[] meds = parseMedsFromVPR(/*patientIen, */response, inptOrOutpt);
        Medication[] dateFilteredMeds;
        if (meds != null && meds.length > 0) {
            dateFilteredMeds = filterMedsByDate(meds, startDate, endDate);
        } else {
            dateFilteredMeds = meds;
        }


        return dateFilteredMeds;
    }

    private Medication[] parseMedsFromVPR(/*String patientIen, */String response, String inptOrOutpt) throws RPCException {
        Medication[] rtc;
        if ("".equals(response)) {
            rtc = new Medication[0];
        } else {
    
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            List<Medication> medsList = new ArrayList<Medication>();
    
            Document doc = null;
            try {
                doc = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList medsNodeList = null;
            try {
                medsNodeList = (NodeList) exprMed.evaluate(doc, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);
            int medsNodeListLength = medsNodeList.getLength();
            for (int medNum = 0; medNum < medsNodeListLength; medNum++) {
                parserBatchManager.addNode(medsNodeList.item(medNum));
            }

            List<VPRMedsParser> medsParserList = new ArrayList<VPRMedsParser>();
            for(List<Node> parserBatch : parserBatchManager.getParserBatchList()) {
                medsParserList.add(new VPRMedsParser(parserBatch, inptOrOutpt));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<Medication>>> futures = null;
            try {
                futures = executorService.invokeAll(medsParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                logErrorMessage(e);
                throw new RPCException(e);
            }
    
            Iterator<Future<List<Medication>>> futuresItr = futures.iterator();
    
            while (futuresItr.hasNext()) {
                try {
                    Future<List<Medication>> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Medication Data");
                    } else {
                        medsList.addAll(future.get());
                    }
    
                } catch (InterruptedException e) {
                    logErrorMessage(e);
                } catch (ExecutionException e) {
                    logErrorMessage(e);
                }
            }
    
            rtc = medsList.toArray(new Medication[0]);
        }
        return rtc;
    }

    private void logErrorMessage(Exception except) {
        LOGGER.error(except.getMessage(), except);
    }

    private class VPRMedsParser implements Callable<List<Medication>> {
        String inptOrOutpt;
        private List<Node> medNodeList;

        public VPRMedsParser(List<Node> medNodeList, String inptOrOutpt) {
            this.medNodeList = medNodeList;
            this.inptOrOutpt = inptOrOutpt;
        }

        public List<Medication> call() throws RPCException {
            List<Medication> medications = new ArrayList<Medication>();

            for(Node medNode : medNodeList) {
                Medication rtc;

                String vaType = null;
                try {
                    vaType = (String) exprVaType.evaluate(medNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                // skip parsing this node if type doesn't match
                // inpatient covers all non-outpatient and non-nonVA
                if (INPATIENT_SORT.equals(inptOrOutpt) && ("O".equals(vaType) || "N".equals(vaType))) {
                    rtc = null;
                } else if (OUTPATIENT_SORT.equals(inptOrOutpt) && (!"O".equals(vaType) && !"N".equals(vaType))) {                // outpatient is only O or N
                    rtc = null;
                } else {

                    Medication medication = new Medication();
                    fillCommonDataBeanValues(medication);

                    String daysSupply = null;
                    try {
                        daysSupply = (String) exprDaysSupply.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String fillsRemaining = null;
                    try {
                        fillsRemaining = (String) exprFillsRemaining.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String fillsAllowed = null;
                    try {
                        fillsAllowed = (String) exprFillsAllowed.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String lastFilled = null;
                    try {
                        lastFilled = (String) exprLastFilled.evaluate(medNode, XPathConstants.STRING);
                        // Zero date returned from VPR call. Converting it to empty string so no date is displayed
                        if ("0".equalsIgnoreCase(lastFilled)) {
                            lastFilled = "";
                        }
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String medId = null;
                    try {
                        medId = (String) exprMedId.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String name = null;
                    try {
                        name = (String) exprName.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String orderId = null;
                    try {
                        orderId = (String) exprOrderId.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String ordered = null;
                    try {
                        ordered = (String) exprOrdered.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }

                    Node orderingProviderNode = XPathUtil.evaluateNodeToNode(medNode, exprOrderingProvider, LOGGER);
                    if (orderingProviderNode != null) {
                        String orderingProviderName = XPathUtil.evaluateNodeToString(orderingProviderNode, exprNameAttrib, LOGGER);
                        String orderingProviderCode = XPathUtil.evaluateNodeToString(orderingProviderNode, exprCode, LOGGER);
                        String orderingProviderTaxonomyCode = XPathUtil.evaluateNodeToString(orderingProviderNode, exprTaxonomyCode, LOGGER);
                        String orderingProviderProviderType = XPathUtil.evaluateNodeToString(orderingProviderNode, exprProviderType, LOGGER);
                        String orderingProviderClassification = XPathUtil.evaluateNodeToString(orderingProviderNode, exprClassification, LOGGER);
                        String orderingProviderSpecialization = XPathUtil.evaluateNodeToString(orderingProviderNode, exprSpecialization, LOGGER);
                        String orderingProviderService = XPathUtil.evaluateNodeToString(orderingProviderNode, exprService, LOGGER);
                        String orderingProviderOfficePhone = XPathUtil.evaluateNodeToString(orderingProviderNode, exprOfficePhone, LOGGER);
                        String orderingProviderAnalogPager = XPathUtil.evaluateNodeToString(orderingProviderNode, exprAnalogPager, LOGGER);
                        String orderingProviderFax = XPathUtil.evaluateNodeToString(orderingProviderNode, exprFax, LOGGER);
                        String orderingProviderEmail = XPathUtil.evaluateNodeToString(orderingProviderNode, exprEmail, LOGGER);

                        Provider orderingProvider = new Provider();
                        orderingProvider.setName(orderingProviderName);
                        orderingProvider.setCode(orderingProviderCode);
                        orderingProvider.setTaxonomyCode(orderingProviderTaxonomyCode);
                        orderingProvider.setProviderType(orderingProviderProviderType);
                        orderingProvider.setClassification(orderingProviderClassification);
                        orderingProvider.setSpecialization(orderingProviderSpecialization);
                        orderingProvider.setService(orderingProviderService);
                        orderingProvider.setPhone(orderingProviderOfficePhone);
                        orderingProvider.setAnalogPager(orderingProviderAnalogPager);
                        orderingProvider.setFax(orderingProviderFax);
                        orderingProvider.setEmail(orderingProviderEmail);
                        medication.setOrderingProvider(orderingProvider);
                    }

                    Node currentProviderNode = XPathUtil.evaluateNodeToNode(medNode, exprCurrentProvider, LOGGER);
                    if (currentProviderNode != null) {
                        String currentProviderName = XPathUtil.evaluateNodeToString(currentProviderNode, exprNameAttrib, LOGGER);
                        String currentProviderCode = XPathUtil.evaluateNodeToString(currentProviderNode, exprCode, LOGGER);
                        String currentProviderTaxonomyCode = XPathUtil.evaluateNodeToString(currentProviderNode, exprTaxonomyCode, LOGGER);
                        String currentProviderProviderType = XPathUtil.evaluateNodeToString(currentProviderNode, exprProviderType, LOGGER);
                        String currentProviderClassification = XPathUtil.evaluateNodeToString(currentProviderNode, exprClassification, LOGGER);
                        String currentProviderSpecialization = XPathUtil.evaluateNodeToString(currentProviderNode, exprSpecialization, LOGGER);
                        String currentProviderService = XPathUtil.evaluateNodeToString(currentProviderNode, exprService, LOGGER);
                        String currentProviderOfficePhone = XPathUtil.evaluateNodeToString(currentProviderNode, exprOfficePhone, LOGGER);
                        String currentProviderAnalogPager = XPathUtil.evaluateNodeToString(currentProviderNode, exprAnalogPager, LOGGER);
                        String currentProviderFax = XPathUtil.evaluateNodeToString(currentProviderNode, exprFax, LOGGER);
                        String currentProviderEmail = XPathUtil.evaluateNodeToString(currentProviderNode, exprEmail, LOGGER);

                        Provider currentProvider = new Provider();
                        currentProvider.setName(currentProviderName);
                        currentProvider.setCode(currentProviderCode);
                        currentProvider.setTaxonomyCode(currentProviderTaxonomyCode);
                        currentProvider.setProviderType(currentProviderProviderType);
                        currentProvider.setClassification(currentProviderClassification);
                        currentProvider.setSpecialization(currentProviderSpecialization);
                        currentProvider.setService(currentProviderService);
                        currentProvider.setPhone(currentProviderOfficePhone);
                        currentProvider.setAnalogPager(currentProviderAnalogPager);
                        currentProvider.setFax(currentProviderFax);
                        currentProvider.setEmail(currentProviderEmail);
                        medication.setCurrentProvider(currentProvider);
                    }

                    String pharmacistCode = null;
                    try {
                        pharmacistCode = (String) exprPharmacistCode.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String pharmacistName = null;
                    try {
                        pharmacistName = (String) exprPharmacistName.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String prescription = null;
                    try {
                        prescription = (String) exprPrescription.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String quantity = null;
                    try {
                        quantity = (String) exprQuantity.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String start = null;
                    try {
                        start = (String) exprStart.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String stop = null;
                    try {
                        stop = (String) exprStop.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String vaStatus = null;
                    try {
                        vaStatus = (String) exprVaStatus.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String ptInstructions = null;
                    try {
                        ptInstructions = (String) exprPtInstructions.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    String sig = null;
                    try {
                        sig = (String) exprSig.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }

                    Node fillsNode = null;
                    try {
                        fillsNode = (Node) exprFills.evaluate(medNode, XPathConstants.NODE);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }

                    if (fillsNode != null) {
                        NodeList fillNodeList = null;
                        try {
                            fillNodeList = (NodeList) exprFill.evaluate(fillsNode, XPathConstants.NODESET);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }

                        if (fillNodeList != null) {
                            List<PrescriptionFill> rxFillList = new ArrayList<PrescriptionFill>();
                            int fillNodeListLength = fillNodeList.getLength();
                            for (int fillNum = 0; fillNum < fillNodeListLength; fillNum++) {
                                Node fillNode = fillNodeList.item(fillNum);
                                fillNode.getParentNode().removeChild(fillNode);

                                PrescriptionFill rxFill = new PrescriptionFill();
                                fillCommonDataBeanValues(rxFill);

                                String fillDateString = null;
                                try {
                                    fillDateString = (String) exprFillFillDate.evaluate(fillNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                Calendar fillDate = null;
                                if (fillDateString != null) {
                                    fillDate = Timestamp.toDateTime(fillDateString);
                                }
                                String fillQuantity = null;
                                try {
                                    fillQuantity = (String) exprFillFillQuantity.evaluate(fillNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                String fillReleaseDateString = null;
                                try {
                                    fillReleaseDateString = (String) exprFillReleaseDate.evaluate(fillNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                Calendar fillReleaseDate = null;
                                if (fillReleaseDateString != null) {
                                    fillReleaseDate = Timestamp.toDateTime(fillReleaseDateString);
                                }
                                String fillRouting = null;
                                try {
                                    fillRouting = (String) exprFillFillRouting.evaluate(fillNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                String fillDaysSupply = null;
                                try {
                                    fillDaysSupply = (String) exprFillFillDaysSupply.evaluate(fillNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                String fillPartial = null;
                                try {
                                    fillPartial = (String) exprFillPartial.evaluate(fillNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }

                                rxFill.setDispenseDate(fillDate);
                                rxFill.setDispensingQuantity(fillQuantity);
                                rxFill.setReleaseDate(fillReleaseDate);
                                rxFill.setDaysSupply(fillDaysSupply);
                                rxFill.setRouting(fillRouting);
                                if (fillPartial != null && "1".equals(fillPartial)) {
                                    rxFill.setPartial(true);
                                }

                                rxFillList.add(rxFill);
                            }

                            if (!rxFillList.isEmpty()) {
                                medication.setPrescriptionFills(rxFillList.toArray(new PrescriptionFill[0]));
                            }
                        }
                    }

                    Node productsNode = null;
                    try {
                        productsNode = (Node) exprProducts.evaluate(medNode, XPathConstants.NODE);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                    if (productsNode != null) {
                        NodeList productNodeList = null;
                        try {
                            productNodeList = (NodeList) exprProduct.evaluate(productsNode, XPathConstants.NODESET);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        if (productNodeList != null) {
                            StringBuilder productNameSB = new StringBuilder("");
                            String productVaProductName = null;
                            String productVaProductVuid = null;
                            List<Code> codeList = new ArrayList<Code>();
                            int productNodeListLength = productNodeList.getLength();
                            for (int productNum = 0; productNum < productNodeListLength; productNum++) {
                                Node productNode = productNodeList.item(productNum);
                                productNode.getParentNode().removeChild(productNode);

                                if (productNameSB.length() > 0) {
                                    productNameSB.append("; ");
                                }
                                try {
                                    productNameSB.append((String) exprProductName.evaluate(productNode, XPathConstants.STRING));
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                try {
                                    productVaProductName = (String) exprProductVaProductName.evaluate(productNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                try {
                                    productVaProductVuid = (String) exprProductVaProductVuid.evaluate(productNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                if (!StringUtils.isEmpty(productVaProductVuid)) {
                                    Code code = new Code();
                                    code.setSystem("VUID");
                                    code.setCode(productVaProductVuid);
                                    if (!StringUtils.isEmpty(productVaProductName)) {
                                        code.setDisplay(productVaProductName);
                                    }
                                    codeList.add(code);
                                    productVaProductVuid = null;
                                }
                            }
                            medication.setDrugName(productNameSB.toString());
                            medication.setCodes(codeList.toArray(new Code[0]));
                        }
                    }

                    Calendar lastFilledDate = null;
                    Calendar orderedDate;
                    Calendar stopDate;
                    Calendar startDate;

                    if (!StringUtils.isEmpty(stop)) {
                        stopDate = Timestamp.toDateTime(stop);
                        medication.setStopDate(stopDate);
                    }
                    if (!StringUtils.isEmpty(start)) {
                        startDate = Timestamp.toDateTime(start);
                        medication.setStartDate(startDate);
                    }

                    medication.setMedId(medId);
                    
                    // prepend "Active " to non-verified, refill, hold, drug interactions, suspended
                    if (vaStatus != null) {
                        vaStatus = vaStatus.trim();
                        if ("non-verified".equalsIgnoreCase(vaStatus)
                                || "refill".equalsIgnoreCase(vaStatus)
                                || "hold".equalsIgnoreCase(vaStatus)
                                || "drug interactions".equalsIgnoreCase(vaStatus)
                                || "suspended".equalsIgnoreCase(vaStatus)) {
                            StringBuilder vaStatusSB = new StringBuilder("Active: ");
                            vaStatusSB.append(vaStatus);
                            vaStatus = vaStatusSB.toString();
                        }
                    }
                    medication.setStatus(vaStatus.toUpperCase());
                    medication.setMedType(vaType);

                    medication.setSigCode(sig);
                    medication.setOrderId(orderId);
                    medication.setLastDispensingPharmacy(pharmacistName);
                    medication.setPharmacyId(pharmacistCode);

                    if (StringUtils.isEmpty(medication.getDrugName())) {
                        medication.setDrugName(name);
                    }

                    if (!StringUtils.isEmpty(ordered)) {
                        orderedDate = Timestamp.toDateTime(ordered);
                        medication.setOrderDate(orderedDate);
                    }

                    if ("O".equalsIgnoreCase(vaType) || "N".equalsIgnoreCase(vaType)) {
                        medication.setRefills(fillsAllowed);
                        medication.setRefillsRemaining(fillsRemaining);
                        medication.setDaysSupply(daysSupply);

                        if (!StringUtils.isEmpty(lastFilled)) {
                            lastFilledDate = Timestamp.toDateTime(lastFilled);
                            medication.setLastFilledDate(lastFilledDate);
                        }
                        medication.setRxNumber(prescription);
                        medication.setComment(ptInstructions);
                        medication.setQuantity(quantity);

                        if ("N".equalsIgnoreCase(vaType)) {
                            StringBuilder nonVADrugName = new StringBuilder(name);
                            nonVADrugName.insert(0, "NON-VA ");
                            medication.setDrugName(nonVADrugName.toString());
                            medication.setManuallyEntered(true);
                        }
                    } else {
                        // type is "I" or "V"

// set lastFilledDate first with most recent fill relesedDate, else the lastFilledDate, else
                        // the startDate
                        Date lastReleasedDate = new Date(Long.MIN_VALUE);
                        Calendar lastReleaseCal = Calendar.getInstance();
                        lastReleaseCal.setTime(lastReleasedDate);
                        if (medication.getPrescriptionFills() != null) {
                            for (PrescriptionFill pfill : medication.getPrescriptionFills()) {
                                if (pfill.getReleaseDate() != null) {
                                    if (pfill.getReleaseDate().compareTo(lastReleaseCal) > 0) {
                                        lastReleaseCal = pfill.getReleaseDate();
                                        medication.setLastFilledDate(pfill.getReleaseDate());
                                    }
                                }
                            }
                        }
                        if (medication.getLastFilledDate() == null) {
                            if (!StringUtils.isEmpty(lastFilled)) {
                                lastFilledDate = Timestamp.toDateTime(lastFilled);
                                medication.setLastFilledDate(lastFilledDate);
                            } else {
                                medication.setLastFilledDate(medication.getStartDate());
                            }
                        }
                        if ("V".equalsIgnoreCase(vaType)) {
                            // V does not have a sig so we'll build it from dose route schedule and rate
                            Node dosesNode = null;
                            try {
                                dosesNode = (Node) exprDoses.evaluate(medNode, XPathConstants.NODE);
                            } catch (XPathExpressionException e) {
                                logErrorMessage(e);
                            }

                            String doseRoute = "";
                            String doseSchedule = "";

                            if (dosesNode != null) {
                                NodeList doseNodeList = null;
                                try {
                                    doseNodeList = (NodeList) exprDose.evaluate(dosesNode, XPathConstants.NODESET);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }

                                if (doseNodeList != null) {
                                    Node doseNode = doseNodeList.item(0);

                                    if (doseNode != null) {
                                        if (doseNode.getParentNode() != null) {
                                            doseNode.getParentNode().removeChild(doseNode);
                                        }
                                        try {
                                            doseRoute = (String) exprDoseRoute.evaluate(doseNode, XPathConstants.STRING);
                                        } catch (XPathExpressionException e) {
                                            logErrorMessage(e);
                                        }
                                        try {
                                            doseSchedule = (String) exprDoseSchedule.evaluate(doseNode, XPathConstants.STRING);
                                        } catch (XPathExpressionException e) {
                                            logErrorMessage(e);
                                        }
                                    }
                                }

                            }

                            String rate = null;
                            try {
                                rate = (String) exprRate.evaluate(medNode, XPathConstants.STRING);
                            } catch (XPathExpressionException e) {
                                logErrorMessage(e);
                            }

                            StringBuilder sigCodeSB = new StringBuilder();
                            if (!StringUtils.isEmpty(doseRoute)) {
                                sigCodeSB.append("Route: ");
                                sigCodeSB.append(doseRoute);
                            }
                            if (!StringUtils.isEmpty(doseSchedule)) {
                                if (sigCodeSB.length() > 0)
                                    sigCodeSB.append("; ");
                                sigCodeSB.append("Schedule: ");
                                sigCodeSB.append(doseSchedule);
                            }
                            if (!StringUtils.isEmpty(rate)) {
                                if (sigCodeSB.length() > 0)
                                    sigCodeSB.append("; ");
                                sigCodeSB.append("Rate: ");
                                sigCodeSB.append(rate);
                            }
                            medication.setSigCode(sigCodeSB.toString());
                        }
                    }

                    rtc = medication;
                }

                if (rtc != null) {
                    medications.add(rtc);
                }
            }

            return medications;
        }
    }

}
