/**
 * Copyright (c) 2017 Hokukahu LLC. All Rights Reserved.
 */

package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.HealthSummariesDao;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.HealthSummaryTypeList;
import gov.va.med.vds.vistabeans.HealthSummaryType;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Health Summaries Data Service
 */
public class HealthSummariesDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(HealthSummariesDataService.class.getName());

    /**
     * Retrieves list of patient health summary types
     * @param queryBean Query parameters.
     *                  Required params: User, Patient
     * @return A list of the patient's health summaries types
     * @throws VDSException if there was an error..
     */
    public List<HealthSummaryTypeList> getPatientHealthSummaryTypes(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> healthSummaryTypeMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_HEALTH_SUMMARIES,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_HEALTH_SUMMARY_TYPES, new Object[]{queryBean});
            
            Set<Map.Entry<Site, Object>> entries = healthSummaryTypeMap.entrySet();
            List<HealthSummaryTypeList> typeLists = new ArrayList<HealthSummaryTypeList>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object healthSummaryTypeList = entry.getValue();
                if (healthSummaryTypeList instanceof Exception) {
                    healthSummaryTypeList = generateConnectionUnavailableBean(entry.getKey(), (Exception) healthSummaryTypeList);
                }
                typeLists.add((HealthSummaryTypeList) healthSummaryTypeList);
            }

            return typeLists;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    /**
     * Gets the Health Summary
     * 
     * @param queryBean needs to contain the IEN to the health summary as the itemId
     * @return the summary report in a FreeTextReport
     * @throws VDSException
     */
    public FreeTextReport getPatientHealthSummary(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            String itemId = queryBean.getItemId();
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (itemId == null || itemId.length() <= 0) {
                throw new IllegalArgumentException("Item Id cannot be null or empty");
            } else if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            HealthSummariesDao healthSummariesDao = new HealthSummariesDao(cxn);
            FreeTextReport healthSummary = healthSummariesDao.getPatientHealthSummary(queryBean);

            return healthSummary;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private HealthSummaryTypeList generateConnectionUnavailableBean(Site site, Exception except) {
        
        HealthSummaryTypeList healthSummaryTypeList = new HealthSummaryTypeList();
        HealthSummaryType healthSummaryType = new HealthSummaryType();
        healthSummaryType.setName(CONNECTION_UNAVAILABLE);
        healthSummaryType.setSite(site);
        healthSummaryTypeList.setSite(site);
        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            healthSummaryType.setName(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }
        HealthSummaryType[] error = {healthSummaryType};
        healthSummaryTypeList.setHealthSummaryTypes(error);
        return healthSummaryTypeList;
    }
}
