/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.site.VistaSiteException;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.LabsDao;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.LabOrder;
import gov.va.med.vds.vistabeans.LabResult;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class LabsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(LabsDataService.class.getName());

    public List<LabResult> getLabOrderResult(QueryBean queryBean) throws VDSException {
        List<LabResult> rtc;
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            cxn = setupVPRContextSingleRemoteSourceQuery(queryBean);
            LabsDao labsDao = new LabsDao(cxn);
            LabResult[] labs = labsDao.getLabOrderResult(queryBean);

            if (labs == null) {
                rtc = null;
            } else {
                rtc = Arrays.asList(labs);
            }
            return rtc;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                if (cxn.getParserThreadPool() != null && cxn.getParserThreadPool().getThreadPool() != null && !cxn.getParserThreadPool().getThreadPool().isShutdown()) {
                    try {
                        cxn.getParserThreadPool().getThreadPool().shutdown();
                    } catch (SecurityException ex) {
                        LOGGER.error("Failed to shutdown parserExecutorService (thread pool)");
                    }
                }
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }

    }

    public LabResult[] getPatientLabTestResults(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            cxn = setupVPRContextSingleRemoteSourceQuery(queryBean);
            LabsDao labsDao = new LabsDao(cxn);
            LabResult[] labs = labsDao.getPatientLabTestResults(queryBean);

            return labs;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                if (cxn.getParserThreadPool() != null && cxn.getParserThreadPool().getThreadPool() != null && !cxn.getParserThreadPool().getThreadPool().isShutdown()) {
                    try {
                        cxn.getParserThreadPool().getThreadPool().shutdown();
                    } catch (SecurityException ex) {
                        LOGGER.error("Failed to shutdown parserExecutorService (thread pool)");
                    }
                }
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }

    }

    public List<LabOrder> getPatientLabs(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msqVPR = null;
        VistaRPCConnectionManager msq = null;

        try {
            msqVPR = setupVPRContextMultiSourceQuery(queryBean, APP_CONFIG.getParserMaxThreadPoolSizeLabs());
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msqVPR.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LABS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabOrder> labsList = new ArrayList<LabOrder>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labsList.addAll(Arrays.asList((LabOrder[]) labs));
            }

            // Get CPRS micro lab results
            msq = setupMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> labsMicroMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> microEntries = labsMicroMap.entrySet();
            List<LabResult> microLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> microEntry : microEntries) {
                Object labs = microEntry.getValue();
                if (!(labs instanceof Exception)) {
                    microLabResultList.addAll(Arrays.asList((LabResult[]) labs));
                }
            }
            // add results from microLabResultList to incomplete MIs in labResultList
            for (ListIterator<LabOrder> labsListIterator = labsList.listIterator(); labsListIterator.hasNext(); ) {
                LabOrder nextLabOrder = labsListIterator.next();
                // first check if it is micro lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabOrder.getSubscript()) && "MI".equalsIgnoreCase(nextLabOrder.getSubscript())) {
                    for (ListIterator<LabResult> microLabListIterator = microLabResultList.listIterator(); microLabListIterator.hasNext(); ) {
                        LabResult nextMicroLabResult = microLabListIterator.next();
                        // compare the accession number
                        if (nextLabOrder.getLabResults() != null && nextLabOrder.getLabResults().length > 0 &&
                                !StringUtils.isEmpty(nextLabOrder.getLabResults()[0].getAccession()) && !StringUtils.isEmpty(nextMicroLabResult.getAccession())
                                && nextLabOrder.getLabResults()[0].getAccession().equalsIgnoreCase(nextMicroLabResult.getAccession())) {
                            nextLabOrder.getLabResults()[0].setResult(nextMicroLabResult.getResult());
                            nextLabOrder.getLabResults()[0].setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            LinkedHashMap<Site, Object> labsAnaPathMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> anaPathEntries = labsAnaPathMap.entrySet();
            List<LabResult> anaPathLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> anaPathEntry : anaPathEntries) {
                Object labs = anaPathEntry.getValue();
                if (!(labs instanceof Exception)) {
                    anaPathLabResultList.addAll(Arrays.asList((LabResult[]) labs));
                }
            }
            // add results from anaPathLabResultList to incomplete APs in labResultList
            for (ListIterator<LabOrder> labsListIterator = labsList.listIterator(); labsListIterator.hasNext(); ) {
                LabOrder nextLabOrder = labsListIterator.next();
                // first check if it is anatomic pathology lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabOrder.getSubscript())
                        && ("SP".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("CY".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("EM".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("AU".equalsIgnoreCase(nextLabOrder.getSubscript()))) {
                    for (ListIterator<LabResult> anaPathLabListIterator = anaPathLabResultList.listIterator(); anaPathLabListIterator.hasNext(); ) {
                        LabResult nextAnaPathLabResult = anaPathLabListIterator.next();
                        // compare the accession number
                        if (nextLabOrder.getLabResults() != null && nextLabOrder.getLabResults().length > 0 &&
                                !StringUtils.isEmpty(nextLabOrder.getLabResults()[0].getAccession()) && !StringUtils.isEmpty(nextAnaPathLabResult.getAccession())
                                && nextLabOrder.getLabResults()[0].getAccession().equalsIgnoreCase(nextAnaPathLabResult.getAccession())) {
                            nextLabOrder.getLabResults()[0].setResult(nextAnaPathLabResult.getResult());
                            nextLabOrder.getLabResults()[0].setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            // Get Autopsy
            LinkedHashMap<Site, Object> autopsyMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LABS_AUTOPSY, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> autopsyEntries = autopsyMap.entrySet();

            for (Map.Entry<Site, Object> entry : autopsyEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labsList.addAll(Arrays.asList((LabOrder[]) labs));
            }

            return labsList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
            if (msqVPR != null) {
                try {
                    msqVPR.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }

        }
    }

    public List<LabOrder> getPatientLabsBse(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean, APP_CONFIG.getParserMaxThreadPoolSizeLabs());
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LABS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabOrder> labsList = new ArrayList<LabOrder>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labsList.addAll(Arrays.asList((LabOrder[]) labs));
            }

            // Get CPRS micro lab results
            LinkedHashMap<Site, Object> labsMicroMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> microEntries = labsMicroMap.entrySet();
            List<LabResult> microLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> microEntry : microEntries) {
                Object labs = microEntry.getValue();
                if (!(labs instanceof Exception)) {
                    microLabResultList.addAll(Arrays.asList((LabResult[]) labs));
                }
            }
            // add results from microLabResultList to incomplete MIs in labResultList
            for (ListIterator<LabOrder> labsListIterator = labsList.listIterator(); labsListIterator.hasNext(); ) {
                LabOrder nextLabOrder = labsListIterator.next();
                // first check if it is micro lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabOrder.getSubscript()) && "MI".equalsIgnoreCase(nextLabOrder.getSubscript())) {
                    for (ListIterator<LabResult> microLabListIterator = microLabResultList.listIterator(); microLabListIterator.hasNext(); ) {
                        LabResult nextMicroLabResult = microLabListIterator.next();
                        // compare the accession number
                        if (nextLabOrder.getLabResults() != null && nextLabOrder.getLabResults().length > 0 &&
                                !StringUtils.isEmpty(nextLabOrder.getLabResults()[0].getAccession()) && !StringUtils.isEmpty(nextMicroLabResult.getAccession())
                                && nextLabOrder.getLabResults()[0].getAccession().equalsIgnoreCase(nextMicroLabResult.getAccession())) {
                            nextLabOrder.getLabResults()[0].setResult(nextMicroLabResult.getResult());
                            nextLabOrder.getLabResults()[0].setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            LinkedHashMap<Site, Object> labsAnaPathMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> anaPathEntries = labsAnaPathMap.entrySet();
            List<LabResult> anaPathLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> anaPathEntry : anaPathEntries) {
                Object labs = anaPathEntry.getValue();
                if (!(labs instanceof Exception)) {
                    anaPathLabResultList.addAll(Arrays.asList((LabResult[]) labs));
                }
            }
            // add results from anaPathLabResultList to incomplete APs in labResultList
            for (ListIterator<LabOrder> labsListIterator = labsList.listIterator(); labsListIterator.hasNext(); ) {
                LabOrder nextLabOrder = labsListIterator.next();
                // first check if it is anatomic pathology lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabOrder.getSubscript())
                        && ("SP".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("CY".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("EM".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("AU".equalsIgnoreCase(nextLabOrder.getSubscript()))) {
                    for (ListIterator<LabResult> anaPathLabListIterator = anaPathLabResultList.listIterator(); anaPathLabListIterator.hasNext(); ) {
                        LabResult nextAnaPathLabResult = anaPathLabListIterator.next();
                        // compare the accession number
                        if (nextLabOrder.getLabResults() != null && nextLabOrder.getLabResults().length > 0 &&
                                !StringUtils.isEmpty(nextLabOrder.getLabResults()[0].getAccession()) && !StringUtils.isEmpty(nextAnaPathLabResult.getAccession())
                                && nextLabOrder.getLabResults()[0].getAccession().equalsIgnoreCase(nextAnaPathLabResult.getAccession())) {
                            nextLabOrder.getLabResults()[0].setResult(nextAnaPathLabResult.getResult());
                            nextLabOrder.getLabResults()[0].setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            // Get Autopsy
            LinkedHashMap<Site, Object> autopsyMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LABS_AUTOPSY, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> autopsyEntries = autopsyMap.entrySet();

            for (Map.Entry<Site, Object> entry : autopsyEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labsList.addAll(Arrays.asList((LabOrder[]) labs));
            }
            
            return labsList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }

        }
    }

    public List<LabResult> getPatientLabResults(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;
        VistaRPCConnectionManager msqVPR = null;

        try {
            // Get lab results
            msqVPR = setupVPRContextMultiSourceQuery(queryBean, APP_CONFIG.getParserMaxThreadPoolSizeLabs());
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msqVPR.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabResult> labResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // Get CPRS micro lab results
            msq = setupMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> labsMicroMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> microEntries = labsMicroMap.entrySet();
            List<LabResult> microLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> microEntry : microEntries) {
                Object labs = microEntry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(microEntry.getKey(), (Exception) labs);
                }
                microLabResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // add results from microLabResultList to incomplete MIs in labResultList
            for (ListIterator<LabResult> labsListIterator = labResultList.listIterator(); labsListIterator.hasNext(); ) {
                LabResult nextLabResult = labsListIterator.next();
                // first check if it is micro lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabResult.getSubscript()) && "MI".equalsIgnoreCase(nextLabResult.getSubscript())) {
                    for (ListIterator<LabResult> microLabListIterator = microLabResultList.listIterator(); microLabListIterator.hasNext(); ) {
                        LabResult nextMicroLabResult = microLabListIterator.next();
                        // compare the accession number
                        if (!StringUtils.isEmpty(nextLabResult.getAccession()) && !StringUtils.isEmpty(nextMicroLabResult.getAccession())
                                && nextLabResult.getAccession().equalsIgnoreCase(nextMicroLabResult.getAccession())) {
                            nextLabResult.setResult(nextMicroLabResult.getResult());
                            nextLabResult.setResultStatus("completed");
                            break;
                        }
                    }
                }
            }
            
            LinkedHashMap<Site, Object> labsAnaPathMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> anaPathEntries = labsAnaPathMap.entrySet();
            List<LabResult> anaPathLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> anaPathEntry : anaPathEntries) {
                Object labs = anaPathEntry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(anaPathEntry.getKey(), (Exception) labs);
                }
                anaPathLabResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // add results from anaPathLabResultList to incomplete APs in labResultList
            for (ListIterator<LabResult> labsListIterator = labResultList.listIterator(); labsListIterator.hasNext(); ) {
                LabResult nextLabResult = labsListIterator.next();
                // first check if it is anatomic pathology lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabResult.getSubscript())
                        && ("SP".equalsIgnoreCase(nextLabResult.getSubscript()))
                            || ("CY".equalsIgnoreCase(nextLabResult.getSubscript()))
                            || ("EM".equalsIgnoreCase(nextLabResult.getSubscript()))
                            || ("AU".equalsIgnoreCase(nextLabResult.getSubscript()))) {
                    for (ListIterator<LabResult> anaPathLabListIterator = anaPathLabResultList.listIterator(); anaPathLabListIterator.hasNext(); ) {
                        LabResult nextAnaPathLabResult = anaPathLabListIterator.next();
                        // compare the accession number
                        if (!StringUtils.isEmpty(nextLabResult.getAccession()) && !StringUtils.isEmpty(nextAnaPathLabResult.getAccession())
                                && nextLabResult.getAccession().equalsIgnoreCase(nextAnaPathLabResult.getAccession())) {
                            nextLabResult.setResult(nextAnaPathLabResult.getResult());
                            nextLabResult.setResultStatus("completed");
                            break;
                        }
                    }
                }
            }
            
            // Get Autopsy
            LinkedHashMap<Site, Object> autopsyMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_AUTOPSY, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> autopsyEntries = autopsyMap.entrySet();

            for (Map.Entry<Site, Object> entry : autopsyEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            // Get Skin Tests
            LinkedHashMap<Site, Object> skinTestsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_SKIN_TESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> skinTestsEntries = skinTestsMap.entrySet();

            for (Map.Entry<Site, Object> entry : skinTestsEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            // Get Blood transfusions
            LinkedHashMap<Site, Object> bloodTypeTestsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_BLOOD_TYPE_TESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> bloodTypeTestsEntries = bloodTypeTestsMap.entrySet();

            for (Map.Entry<Site, Object> entry : bloodTypeTestsEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            return labResultList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
            if (msqVPR != null) {
                try {
                    msqVPR.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }

        }
    }

    public List<LabResult> getPatientLabResultsBse(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            // Get lab results
            msq = setupVPRContextMultiSourceQuery(queryBean, APP_CONFIG.getParserMaxThreadPoolSizeLabs());
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabResult> labResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // Get CPRS micro lab results
            LinkedHashMap<Site, Object> labsMicroMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> microEntries = labsMicroMap.entrySet();
            List<LabResult> microLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> microEntry : microEntries) {
                Object labs = microEntry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(microEntry.getKey(), (Exception) labs);
                }
                microLabResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // add results from microLabResultList to incomplete MIs in labResultList
            for (ListIterator<LabResult> labsListIterator = labResultList.listIterator(); labsListIterator.hasNext(); ) {
                LabResult nextLabResult = labsListIterator.next();
                // first check if it is micro lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabResult.getSubscript()) && "MI".equalsIgnoreCase(nextLabResult.getSubscript())) {
                    for (ListIterator<LabResult> microLabListIterator = microLabResultList.listIterator(); microLabListIterator.hasNext(); ) {
                        LabResult nextMicroLabResult = microLabListIterator.next();
                        // compare the accession number
                        if (!StringUtils.isEmpty(nextLabResult.getAccession()) && !StringUtils.isEmpty(nextMicroLabResult.getAccession())
                                && nextLabResult.getAccession().equalsIgnoreCase(nextMicroLabResult.getAccession())) {
                            nextLabResult.setResult(nextMicroLabResult.getResult());
                            nextLabResult.setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            LinkedHashMap<Site, Object> labsAnaPathMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> anaPathEntries = labsAnaPathMap.entrySet();
            List<LabResult> anaPathLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> anaPathEntry : anaPathEntries) {
                Object labs = anaPathEntry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(anaPathEntry.getKey(), (Exception) labs);
                }
                anaPathLabResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // add results from anaPathLabResultList to incomplete APs in labResultList
            for (ListIterator<LabResult> labsListIterator = labResultList.listIterator(); labsListIterator.hasNext(); ) {
                LabResult nextLabResult = labsListIterator.next();
                // first check if it is anatomic pathology lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabResult.getSubscript())
                        && ("SP".equalsIgnoreCase(nextLabResult.getSubscript()))
                        || ("CY".equalsIgnoreCase(nextLabResult.getSubscript()))
                        || ("EM".equalsIgnoreCase(nextLabResult.getSubscript()))
                        || ("AU".equalsIgnoreCase(nextLabResult.getSubscript()))) {
                    for (ListIterator<LabResult> anaPathLabListIterator = anaPathLabResultList.listIterator(); anaPathLabListIterator.hasNext(); ) {
                        LabResult nextAnaPathLabResult = anaPathLabListIterator.next();
                        // compare the accession number
                        if (!StringUtils.isEmpty(nextLabResult.getAccession()) && !StringUtils.isEmpty(nextAnaPathLabResult.getAccession())
                                && nextLabResult.getAccession().equalsIgnoreCase(nextAnaPathLabResult.getAccession())) {
                            nextLabResult.setResult(nextAnaPathLabResult.getResult());
                            nextLabResult.setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            // Get Autopsy
            LinkedHashMap<Site, Object> autopsyMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_AUTOPSY, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> autopsyEntries = autopsyMap.entrySet();

            for (Map.Entry<Site, Object> entry : autopsyEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            // Get Skin Tests
            LinkedHashMap<Site, Object> skinTestsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_SKIN_TESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> skinTestsEntries = skinTestsMap.entrySet();

            for (Map.Entry<Site, Object> entry : skinTestsEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            // Get Blood transfusions
            LinkedHashMap<Site, Object> bloodTypeTestsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_BLOOD_TYPE_TESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> bloodTypeTestsEntries = bloodTypeTestsMap.entrySet();

            for (Map.Entry<Site, Object> entry : bloodTypeTestsEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            return labResultList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }

        }
    }

    public List<LabOrder> getPatientLabsOneContext(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean, APP_CONFIG.getParserMaxThreadPoolSizeLabs());
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LABS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabOrder> labsList = new ArrayList<LabOrder>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labsList.addAll(Arrays.asList((LabOrder[]) labs));
            }

            // Get CPRS micro lab results
            LinkedHashMap<Site, Object> labsMicroMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS_SET_CONTEXT, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> microEntries = labsMicroMap.entrySet();
            List<LabResult> microLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> microEntry : microEntries) {
                Object labs = microEntry.getValue();
                if (!(labs instanceof Exception)) {
                    microLabResultList.addAll(Arrays.asList((LabResult[]) labs));
                }
            }
            // add results from microLabResultList to incomplete MIs in labResultList
            for (ListIterator<LabOrder> labsListIterator = labsList.listIterator(); labsListIterator.hasNext(); ) {
                LabOrder nextLabOrder = labsListIterator.next();
                // first check if it is micro lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabOrder.getSubscript()) && "MI".equalsIgnoreCase(nextLabOrder.getSubscript())) {
                    for (ListIterator<LabResult> microLabListIterator = microLabResultList.listIterator(); microLabListIterator.hasNext(); ) {
                        LabResult nextMicroLabResult = microLabListIterator.next();
                        // compare the accession number
                        if (nextLabOrder.getLabResults() != null && nextLabOrder.getLabResults().length > 0 &&
                                !StringUtils.isEmpty(nextLabOrder.getLabResults()[0].getAccession()) && !StringUtils.isEmpty(nextMicroLabResult.getAccession())
                                && nextLabOrder.getLabResults()[0].getAccession().equalsIgnoreCase(nextMicroLabResult.getAccession())) {
                            nextLabOrder.getLabResults()[0].setResult(nextMicroLabResult.getResult());
                            nextLabOrder.getLabResults()[0].setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            LinkedHashMap<Site, Object> labsAnaPathMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> anaPathEntries = labsAnaPathMap.entrySet();
            List<LabResult> anaPathLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> anaPathEntry : anaPathEntries) {
                Object labs = anaPathEntry.getValue();
                if (!(labs instanceof Exception)) {
                    anaPathLabResultList.addAll(Arrays.asList((LabResult[]) labs));
                }
            }
            // add results from anaPathLabResultList to incomplete APs in labResultList
            for (ListIterator<LabOrder> labsListIterator = labsList.listIterator(); labsListIterator.hasNext(); ) {
                LabOrder nextLabOrder = labsListIterator.next();
                // first check if it is anatomic pathology lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabOrder.getSubscript())
                        && ("SP".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("CY".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("EM".equalsIgnoreCase(nextLabOrder.getSubscript()))
                        || ("AU".equalsIgnoreCase(nextLabOrder.getSubscript()))) {
                    for (ListIterator<LabResult> anaPathLabListIterator = anaPathLabResultList.listIterator(); anaPathLabListIterator.hasNext(); ) {
                        LabResult nextAnaPathLabResult = anaPathLabListIterator.next();
                        // compare the accession number
                        if (nextLabOrder.getLabResults() != null && nextLabOrder.getLabResults().length > 0 &&
                                !StringUtils.isEmpty(nextLabOrder.getLabResults()[0].getAccession()) && !StringUtils.isEmpty(nextAnaPathLabResult.getAccession())
                                && nextLabOrder.getLabResults()[0].getAccession().equalsIgnoreCase(nextAnaPathLabResult.getAccession())) {
                            nextLabOrder.getLabResults()[0].setResult(nextAnaPathLabResult.getResult());
                            nextLabOrder.getLabResults()[0].setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            // Get Autopsy
            LinkedHashMap<Site, Object> autopsyMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LABS_AUTOPSY, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> autopsyEntries = autopsyMap.entrySet();

            for (Map.Entry<Site, Object> entry : autopsyEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labsList.addAll(Arrays.asList((LabOrder[]) labs));
            }
            
            return labsList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<LabResult> getPatientLabResultsOneContext(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            // Get lab results
            msq = setupVPRContextMultiSourceQuery(queryBean, APP_CONFIG.getParserMaxThreadPoolSizeLabs());
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabResult> labResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // Get CPRS micro lab results
            LinkedHashMap<Site, Object> labsMicroMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS_SET_CONTEXT, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> microEntries = labsMicroMap.entrySet();
            List<LabResult> microLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> microEntry : microEntries) {
                Object labs = microEntry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(microEntry.getKey(), (Exception) labs);
                }
                microLabResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // add results from microLabResultList to incomplete MIs in labResultList
            for (ListIterator<LabResult> labsListIterator = labResultList.listIterator(); labsListIterator.hasNext(); ) {
                LabResult nextLabResult = labsListIterator.next();
                // first check if it is micro lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabResult.getSubscript()) && "MI".equalsIgnoreCase(nextLabResult.getSubscript())) {
                    for (ListIterator<LabResult> microLabListIterator = microLabResultList.listIterator(); microLabListIterator.hasNext(); ) {
                        LabResult nextMicroLabResult = microLabListIterator.next();
                        // compare the accession number
                        if (!StringUtils.isEmpty(nextLabResult.getAccession()) && !StringUtils.isEmpty(nextMicroLabResult.getAccession())
                                && nextLabResult.getAccession().equalsIgnoreCase(nextMicroLabResult.getAccession())) {
                            nextLabResult.setResult(nextMicroLabResult.getResult());
                            nextLabResult.setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            LinkedHashMap<Site, Object> labsAnaPathMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> anaPathEntries = labsAnaPathMap.entrySet();
            List<LabResult> anaPathLabResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> anaPathEntry : anaPathEntries) {
                Object labs = anaPathEntry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(anaPathEntry.getKey(), (Exception) labs);
                }
                anaPathLabResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            // add results from anaPathLabResultList to incomplete APs in labResultList
            for (ListIterator<LabResult> labsListIterator = labResultList.listIterator(); labsListIterator.hasNext(); ) {
                LabResult nextLabResult = labsListIterator.next();
                // first check if it is anatomic pathology lab result since that is all we are modifying
                if (!StringUtils.isEmpty(nextLabResult.getSubscript())
                        && ("SP".equalsIgnoreCase(nextLabResult.getSubscript()))
                        || ("CY".equalsIgnoreCase(nextLabResult.getSubscript()))
                        || ("EM".equalsIgnoreCase(nextLabResult.getSubscript()))
                        || ("AU".equalsIgnoreCase(nextLabResult.getSubscript()))) {
                    for (ListIterator<LabResult> anaPathLabListIterator = anaPathLabResultList.listIterator(); anaPathLabListIterator.hasNext(); ) {
                        LabResult nextAnaPathLabResult = anaPathLabListIterator.next();
                        // compare the accession number
                        if (!StringUtils.isEmpty(nextLabResult.getAccession()) && !StringUtils.isEmpty(nextAnaPathLabResult.getAccession())
                                && nextLabResult.getAccession().equalsIgnoreCase(nextAnaPathLabResult.getAccession())) {
                            nextLabResult.setResult(nextAnaPathLabResult.getResult());
                            nextLabResult.setResultStatus("completed");
                            break;
                        }
                    }
                }
            }

            // Get Autopsy
            LinkedHashMap<Site, Object> autopsyMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_AUTOPSY, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> autopsyEntries = autopsyMap.entrySet();

            for (Map.Entry<Site, Object> entry : autopsyEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            // Get Skin Tests
            LinkedHashMap<Site, Object> skinTestsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_SKIN_TESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> skinTestsEntries = skinTestsMap.entrySet();

            for (Map.Entry<Site, Object> entry : skinTestsEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }

            // Get Blood transfusions
            LinkedHashMap<Site, Object> bloodTypeTestsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_BLOOD_TYPE_TESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> bloodTypeTestsEntries = bloodTypeTestsMap.entrySet();

            for (Map.Entry<Site, Object> entry : bloodTypeTestsEntries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabResult(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
            return labResultList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getPatientLabReport(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        String itemId = queryBean.getItemId();
        String siteCode = queryBean.getRecordSite().getSiteCode();

        RPCConnection cxn = null;

        if (itemId == null || itemId.length() <= 0) {
            throw new IllegalArgumentException("Item Id cannot be null or empty. This is the accession number");
        } else if (queryBean.getStartDate() == null) {
            throw new IllegalArgumentException("Start Date cannot be null. This is the collectedDate");
        } else if (StringUtils.isEmpty(siteCode)) {
            throw new IllegalArgumentException("Missing record site code");
        }
        try {
            cxn = setupSingleRemoteSourceQuery(queryBean);
            LabsDao labsDao = new LabsDao(cxn);
            return labsDao.getPatientLabReport(queryBean);
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }

    }

    private LabOrder[] generateConnectionUnavailableBeanLabOrder(Site site, Exception except) {
        LabOrder lab = new LabOrder();
        lab.setTestName(CONNECTION_UNAVAILABLE);
        lab.setCollectionDate(Utils.backDate().toGregorianCalendar());
        lab.setSite(site);
        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            lab.setTestName(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }


        return new LabOrder[]{lab};
    }

    private LabResult[] generateConnectionUnavailableBeanLabResult(Site site, Exception except) {
        LabResult lab = new LabResult();
        lab.setTestName(CONNECTION_UNAVAILABLE);
        lab.setCollectedDate(Utils.backDate().toGregorianCalendar());
        lab.setSite(site);
        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            lab.setTestName(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }


        return new LabResult[]{lab};
    }

}
