/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.site.VistaSiteException;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.Vitals;
import gov.va.med.vhahon.common.Utils;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class VitalsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(VitalsDataService.class.getName());

    public List<Vitals> getPatientVitals(QueryBean queryBean) throws VDSException
    {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> vitalsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_VITALS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_VITALS, new Object[]{queryBean});

            List<Vitals> vitalsList = processVitalsMap(vitalsMap);

            return vitalsList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<Vitals> getPatientCurrentVitals(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> vitalsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_VITALS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_CURRENT_VITALS, new Object[]{queryBean});

            List<Vitals> vitalsList = processVitalsMap(vitalsMap);

            return vitalsList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private List<Vitals> processVitalsMap(LinkedHashMap<Site, Object> vitalsMap) throws VistaSiteException {
        Set<Map.Entry<Site, Object>> entries = vitalsMap.entrySet();
        List<Vitals> vitalsList = new ArrayList<Vitals>();

        for (Map.Entry<Site, Object> entry : entries) {
            Object vitals = entry.getValue();
            if (vitals instanceof Exception) {
                vitals = generateConnectionUnavailableBean(entry.getKey(), (Exception) vitals);
            }
            vitalsList.addAll(Arrays.asList((Vitals[]) vitals));
        }

        return vitalsList;
    }

    private Vitals[] generateConnectionUnavailableBean(Site site, Exception except) {

        Vitals vital = new Vitals();
        vital.setVitalType(CONNECTION_UNAVAILABLE);
        vital.setDateTimeTaken(Utils.backDate().toGregorianCalendar());
        vital.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            vital.setVitalType(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }


        return new Vitals[] {vital};
    }
}
