/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.provider.dao.rpc;

import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.rpc.DdrFiler;
import gov.va.med.rpc.DdrLister;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.RPCUserOption;
import gov.va.med.rpc.RPCUtils;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.Timestamp;
import gov.va.med.rpc.UserOption;
import gov.va.med.rpc.UserSecurityKey;
import gov.va.med.rpc.UtilityRPCS;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.rpc.VistaQuery;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.patient.dao.rpc.UserAuthenticationException;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;

/**
 *
 *
 */
public class UserRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(UserRPC.class);

    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();

    public UserRPC(RPCConnection cxn) {
        super(cxn);
    }

    /**
     * Legacy login of VA user with access/verify and retrieves older BSE token using XUS GET TOKEN.
     *
     * @param accessCode
     * @param verifyCode
     * @param incontext
     * @return
     * @throws RPCException
     */
    public User login(String accessCode, String verifyCode, String incontext) throws RPCException {

        String context = incontext;
        authenticate(accessCode, verifyCode);
        if (StringUtils.isEmpty(context)) {
            context = VistaConstants.CPRS_CONTEXT;
        }
        if (!setContext(context)) {
            throw new RPCException("Unable to set context " + context);
        }
        User user = new User();

        String[] parts = null;
        String rtn = null;

        parts = getUserInfoAndRestrictedList(user);
        // Get the FederatedUid for CAPRI style calls; not needed for BSE visit calls
        user.setUserIen(parts[0]);
        user.setName(parts[1]);

        // Get the BSE Token
        if (APP_CONFIG.bseTokenEnabled() && !"VBA".equalsIgnoreCase(cxn.getSite().getMoniker())) {
            user.setBseToken(getBseToken(user.getName(), cxn.getSite().getMoniker()));
        } else {

            if (StringUtils.isEmpty(user.getUserIen())) {
                throw new RPCException("No user ID for site");
            }
            String arg = "@\"^VA(200," + user.getUserIen() + ",1)\"";
            rtn = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            if (StringUtils.isEmpty(rtn)) {
                throw new RPCException("Unable to get federated user id");
            }
            parts = StringUtils.split(rtn, StringUtils.CARET);

            if (parts != null && parts.length > 8) {
                user.setEncryptedFederatedUid(EncryptUtil.encrypt(parts[8]));
            }

            VistaQuery vistaQuery = vistaQueryFactory.getVistaQuery("XUS GET TOKEN");
            rtn = (String) cxn.query(vistaQuery.buildMessage());
            if (!StringUtils.isEmpty(rtn)) {
                user.setBseToken(rtn);
            }
        }

        user.setAgency(AGENCY_VA);
        user.setHostSite(cxn.getSite());
        cxn.setUserIen(user.getUserIen());
        return user;
    }
    
    /**
     * Login method that uses a SAML token instead of access/verify to authenticate. Retrieves the BSE token using 
     * XUS BSE TOKEN
     *
     * @param samlToken
     * @return
     * @throws RPCException
     */
    public User loginSaml(String samlToken) throws RPCException {
        // call XUS SIGNON SETUP then XUS AV CODE
        authenticateSaml(samlToken);

        if (!setContext(VistaConstants.JLV_CONTEXT)) {
            throw new RPCException("Unable to set context " + VistaConstants.JLV_CONTEXT);
        }

        User user = new User();

        String[] parts = null;
        String rtn = null;

        rtn = getBseToken(user.getName(),  cxn.getSite().getMoniker());
        user.setBseToken(rtn);
        
        parts = getUserInfoAndRestrictedList(user);

        user.setUserIen(parts[0]);

        user.setName(parts[1]);
        if (StringUtils.isEmpty(user.getUserIen())) {
            throw new RPCException("No user ID for site");
        }

        user.setAgency(AGENCY_VA);
        user.setHostSite(cxn.getSite());
        cxn.setUserIen(user.getUserIen());
        return user;
    }

    /**
     * Private Helper method that makes RPC call to XUS BSE TOKEN to retrieve the BSE token given the
     * JLV passphrase (hardcoded)
     *
     * @param username
     * @param siteMoniker
     * @return
     * @throws RPCException
     */
    private String getBseToken(String username, String siteMoniker) throws RPCException {
        String rtn = null;
        String[] parts = null;

        VistaQuery vistaQuery = vistaQueryFactory.getVistaQuery("XUS BSE TOKEN");
        // add the JLV passphrase as the argument
        vistaQuery.addParameter(VistaQuery.LITERAL, "janusJLVD0n0tl00K");
        rtn = (String) cxn.query(vistaQuery.buildMessage());

        if (rtn == null) {
            throw new RPCException("Unable to get BSE Token for user: " + username + " from site: " + siteMoniker);
        }
        parts = StringUtils.split(rtn, StringUtils.CARET);
        if (!"-35".equalsIgnoreCase(parts[0])) {
            throw new RPCException("Invalid BSE Token for user:" + username + " from site: " + siteMoniker);
        }

        return rtn;
    }

    /**
     * Helper method for logins that get user info and restricted patient lists.
     *
     * The input parameter User is modified by side-effect of this method, since the parameter is passed as copy of the
     * reference to the User object.
     *
     * @param user User is modified in the method as side-effect.
     * @return the user info String array
     * @throws UserAuthenticationException
     * @throws RPCException
     */
    private String[] getUserInfoAndRestrictedList(User user) throws UserAuthenticationException, RPCException {
        String[] parts = null;
        String rtn = null;

        //VBA users will utilize XUS GET USER INFO to get around CPRS context requirement
        if ("VBA".equalsIgnoreCase(cxn.getSite().getMoniker())) {
            VistaQuery vquery = vistaQueryFactory.getVistaQuery("XUS GET USER INFO");
            rtn = (String) cxn.query(vquery.buildMessage());
            if (rtn == null) {
                throw new RPCException("Unable to get user info");
            }

            parts = StringUtils.split(rtn, StringUtils.CRLF);

            List<Patient> capriRestrictedPatientsList = buildCapriRestrictedPatientsList(parts[0]);
            if (capriRestrictedPatientsList.size() > 0) {
                // there were patients in CAPRI restricted list so add them to the user
                user.setRestrictedUser(true);
                user.setRestrictedPatients(capriRestrictedPatientsList);
            }
            if (!user.isRestrictedUser()) {
                // if there is no restricted list, check for restricted sites
                List<String> sitesStringList = buildCapriRestrictedSitesList(parts[0]);

                if (sitesStringList.size() > 0) {
                    user.setRestrictedSiteCodesList(sitesStringList);
                }
            }
        } else {
            VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWU USERINFO");
            rtn = (String) cxn.query(vquery.buildMessage());
            if (rtn == null) {
                throw new RPCException("Unable to get user info");
            }

            parts = StringUtils.split(rtn, StringUtils.CARET);

            if ("1".equals(parts[19])) {
                vquery = vistaQueryFactory.getVistaQuery("ORQPT TEAM PATIENTS");
                vquery.addParameter(VistaQuery.LITERAL, parts[20]); // set the TEAM IEN from the ORWU USERINFO response
                rtn = (String) cxn.query(vquery.buildMessage());
                if (rtn == null) {
                    throw new UserAuthenticationException("You have a restricted VA Account. Unable to retrieve restricted patient list. Access to JLV is Unauthorized.");
                }

                user.setRestrictedUser(true);
                List<Patient> restrictedPatients = buildRestrictedPatientsList(rtn);
                user.setRestrictedPatients(restrictedPatients);
            }
        }

        return parts;
    }

    private List<Patient> buildRestrictedPatientsList(String response) {
        String[] patientLines = StringUtils.split(response, StringUtils.CRLF);

        ArrayList<Patient> patients = new ArrayList<Patient>();

        for (String patientLine : patientLines) {
            String[] fields = StringUtils.split(patientLine, StringUtils.CARET);
            Patient patient = new Patient();

            String [] idInfoFields = null;

            try {
                VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT ID INFO");
                vquery.addParameter(VistaQuery.LITERAL, fields[0]);
                String rtn = (String) cxn.query(vquery.buildMessage());
                idInfoFields = StringUtils.split(rtn, StringUtils.CARET);

            } catch (RPCException rpce) {
                // log exception
                LOGGER.error("RPC Error on ORWPT ID INFO for patientIEN: {} from site: {}", fields[0], cxn.getSiteCode());
            }

            if (idInfoFields != null && idInfoFields.length > 7) {
                patient.setSSN(idInfoFields[0]);
                patient.setDob(Timestamp.formatDisplayDateNoBadMonthOrDay(idInfoFields[1]));
                patient.setGender(idInfoFields[2]);
                if ("Y".equalsIgnoreCase(idInfoFields[3])) {
                    patient.setIsVeteran(true);
                }
                patient.setScPercent(idInfoFields[4]);
                patient.setPatientLocation(idInfoFields[5]);
                patient.setPatientRoomBed(idInfoFields[6]);
            }

            patient.setICN(fields[0]);
            patient.setName(fields[1]);

            patients.add(patient);
        }

        return patients;
    }

    /**
     * Calls and parses response from RSTLIST^DVBABFRM, RPC DVBAB RESTRICTED LIST PATIENTS
     *
     * RSTLIST(DVBABY,IEN) ;RPC DVBAB RESTRICTED LIST PATIENTS
     *  ; RETURNS A LIST OF PATIENTS IN THE USER'S CLAIMS SYSTEM
     *  ; RESTRICTED LIST IN THE FORMAT
     *  ; LISTNAME (OR NULL)^PTIEN^PT NAME^EXP DATE^EXP DAT INTERNAL^SSN^DOB^ICN
     *  ;  $$GETICN^MPIF001 - supported ICR #2701
     *
     * @param userIen The IEN of the User at this site
     * @return a List of Patients
     */
    private List<Patient> buildCapriRestrictedPatientsList(String userIen) throws  RPCException {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("DVBAB RESTRICTED LIST PATIENTS");
        vquery.addParameter(VistaQuery.LITERAL, userIen); // set the TEAM IEN from the ORWU USERINFO response
        String response = (String) cxn.query(vquery.buildMessage());

        ArrayList<Patient> patients = new ArrayList<Patient>();

        if (!StringUtils.isEmpty(response)) {
            String[] patientLines = StringUtils.split(response, StringUtils.CR);

            for (String patientLine : patientLines) {
                String[] fields = StringUtils.split(patientLine, StringUtils.CARET);
                Patient patient = new Patient();

                patient.setICN(fields[7]); // we use ICN for VBA restricted patients instead of VAIEN as with VHA RPL
                patient.setName(fields[2]);
                patient.setSSN(fields[5]);
                patient.setDob(Timestamp.formatDisplayDateNoBadMonthOrDay(fields[6]));

                patients.add(patient);
            }
        }
        return patients;
    }

    /**
     * Gets the list of restricted sites for a CAPRI user as a list of (String) siteCodes.
     *
     * @param userIen
     * @return a List of sites as Strings
     * @throws RPCException
     */
    private List<String> buildCapriRestrictedSitesList(String userIen) throws  RPCException {
        List<String> sitesList = new ArrayList<String>();

        String d0 = RPCUtils.getVariableValue(vistaQueryFactory, cxn, "$O(^DIZ(396.96,\"B\"," + userIen + ",0))");
        if (!StringUtils.isEmpty(d0)) {
            // parse the response for sites list
            boolean moreSites = false;
            int nextIndex = 0;
            // keep doing a get variable with the reference until it comes back empty.
            do {
                String nextSiteInfoIenIndexString = RPCUtils.getVariableValue(vistaQueryFactory, cxn, "$O(^DIZ(396.96," + d0 + ",1," + nextIndex +"))");
                if (!StringUtils.isEmpty(nextSiteInfoIenIndexString)) {
                    try {
                        nextIndex = Integer.parseInt(nextSiteInfoIenIndexString);
                    } catch (NumberFormatException ne) {
                        break;
                    }
                } else {
                    break;
                }

                String nextSiteInfoIen = RPCUtils.getVariableValue(vistaQueryFactory, cxn, "$P($G(^DIZ(396.96," + d0 + ",1," + nextIndex + ",0)),U,1)");

                if (!StringUtils.isEmpty(nextSiteInfoIen)) {
                    boolean error = false;
                    // there is a site in the restricted list on the VBA server
                    String siteIen = RPCUtils.getVariableValue(vistaQueryFactory, cxn, "$G(^DIZ(396.97," + nextSiteInfoIen +",0))");

                    if (!StringUtils.isEmpty(siteIen)) {
                        // caret delimited, e.g., 516^BAY PINES VAMC^C.W. BILL YOUNG DEPT OF VAMC^^V^^1 
                        String site = RPCUtils.getVariableValue(vistaQueryFactory, cxn, "$G(^DIC(4," + siteIen + ",99))");

                        if (!StringUtils.isEmpty(site)) {
                            String[] siteFields = StringUtils.split(site, StringUtils.CARET);
                            // Only add a site code if we could find the station number in the INSTITUTIONS FILE
                            sitesList.add(siteFields[0]);
                        } else {
                            error = true;
                        }
                    } else {
                        error = true;
                    }

                    if (error) {
                        LOGGER.error("Couldn't find the station number from VBA server for restricted site in list, siteInfoIen: {}", nextSiteInfoIen);
                    }

                    // There was a restricted site so check if there are more
                    moreSites = true;
                } else {
                    moreSites = false;
                }
            } while (moreSites);
        }

        if (!sitesList.isEmpty() && APP_CONFIG.useEnhancedDebug()) {
            StringBuilder debugMessageSB = new StringBuilder("!!! VBAUSER SITERESTRICTED UserIEN: ");
            debugMessageSB.append(userIen);
            debugMessageSB.append("  Sites: ");
            for (String siteCode: sitesList) {
                debugMessageSB.append(siteCode);
                debugMessageSB.append(" ");
            }
            LOGGER.debug(debugMessageSB.toString());
        }
        
        return sitesList;
    }

    /**
     * This method does a XUS SIGNON SETUP with no arguments RPC call, then passes the XUS AV CODE. If successful,
     * it parses out the DUZ and returns it.
     *
     * @param accessCode
     * @param verifyCode
     * @return DUZ of the user from XUS AV CODE RPC
     * @throws RPCException
     */
    /**
     * This method does a XUS SIGNON SETUP with no arguments RPC call, then passes the XUS AV CODE. If successful,
     * it parses out the DUZ and returns it.
     *
     * @param accessCode
     * @param verifyCode
     * @return DUZ of the user from XUS AV CODE RPC
     * @throws RPCException
     */
    public String authenticate(String accessCode, String verifyCode) throws RPCException {
        String rtc;
        if (StringUtils.isEmpty(accessCode) || StringUtils.isEmpty(verifyCode)) {
            throw new IllegalArgumentException("Missing access/verify codes");
        }
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("XUS SIGNON SETUP");
        String rtn = (String) cxn.query(vquery.buildMessage());
        if (rtn == null) {
            throw new RPCException("Unable to setup authentication");
        }
        vquery = vistaQueryFactory.getVistaQuery("XUS AV CODE");

        String decryptAccessCode = VistaKernelHash.decrypt(accessCode);
        String decryptVerifyCode = VistaKernelHash.decrypt(verifyCode);

        vquery.addEncryptedParameter(VistaQuery.LITERAL, decryptAccessCode + ';' + decryptVerifyCode);
        rtn = (String) cxn.query(vquery.buildMessage());
        //TODO - need to catch renew verify id error
        String[] flds = StringUtils.split(rtn, StringUtils.CRLF);
        if ("0".equals(flds[0])) {
            // the DUZ returned is zero (privileged user)
            throw new UserAuthenticationException(flds[3]);
        }
        if (flds.length > 7) {
            rtc = flds[0];
            rtc = flds[0];  // get the DUZ
        } else {
            rtc = "OK";
        }
        return rtc;
    }

    /**
     * Uses XUS ESSO VALIDATE and the samlToken to authenticate instead of A/V.
     *
     * Mirrors authenticate(accessCode, verifyCode) method.
     *
     * @param samlToken
     * @return the DUZ parsed from the response
     * @throws RPCException
     */
    private String authenticateSaml(String samlToken) throws RPCException {
        String rtc;
        if (StringUtils.isEmpty(samlToken)) {
            throw new IllegalArgumentException("Missing samlToken");
        }
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("XUS SIGNON SETUP");
        String rtn = (String) cxn.query(vquery.buildMessage());
        if (rtn == null) {
            throw new RPCException("Unable to setup authentication");
        }
        vquery = vistaQueryFactory.getVistaQuery("XUS ESSO VALIDATE");
        vquery.addParameter(VistaQuery.GLOBAL, StringUtils.stringArrayToDictionaryHashList(StringUtils.divideString(samlToken, 200)));

        rtn = (String) cxn.query(vquery.buildMessage());

        String[] flds = StringUtils.split(rtn, StringUtils.CRLF);
        if ("0".equals(flds[0])) {
            // the DUZ returned is zero (privileged user)
            throw new UserAuthenticationException(flds[3]);
        }
        if (flds.length > 7) {
            rtc = flds[0];  // get the DUZ
        } else {
            rtc = "OK";
        }
        return rtc;
    }

    public boolean setContext(String context) throws RPCException {
        if (StringUtils.isEmpty(context)) {
            throw new IllegalArgumentException("Empty context");
        }
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("XWB CREATE CONTEXT");
        vquery.addEncryptedParameter(VistaQuery.LITERAL, context);

        final String STR_EXECUTE = "execute";

        String rtn = STR_EXECUTE;
        do {
            rtn = (String) cxn.query(vquery.buildMessage());
        } while (STR_EXECUTE.equals(rtn));
        if ("1".equals(rtn)) {
            cxn.setCurrentContext(context);
        }
        return "1".equals(rtn);
    }

    public String setNewPassword(String oldVerifyCode, String newVerifyCode) throws RPCException {
        String rtc;
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("XUS CVC");
        String oldEncodedVerifyCode = vquery.encrypt(oldVerifyCode.toUpperCase(Locale.getDefault()));
        String newEncodedVerifyCode = vquery.encrypt(newVerifyCode.toUpperCase(Locale.getDefault()));
        vquery.addParameter(VistaQuery.LITERAL, oldEncodedVerifyCode + '^' + newEncodedVerifyCode + '^' + newEncodedVerifyCode);
        String rtn = (String) cxn.query(vquery.buildMessage());
        if (rtn.charAt(0) == '0') {
            rtc = "OK";
        } else {
            rtc = rtn.substring(1);
        }
        return rtc;
    }

    public void addAuthorization(String context) {
        // Nothing to do.
    }

    public String getUserId(User user) throws RPCException {
        if (StringUtils.isEmpty(user.getEncryptedFederatedUid())) {
            throw new RPCException("Empty encryptedFederatedUid");
        }

        String arg = "$O(^VA(200,\"SSN\",\"" + EncryptUtil.decrypt(user.getEncryptedFederatedUid().toString()) + "\",0))";
        String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        return RPCUtils.errMsgOrIen(response);
    }

    public String getUserId(String encryptedFederatedUid) throws RPCException {
        String arg = "$O(^VA(200,\"SSN\",\"" + EncryptUtil.decrypt(encryptedFederatedUid) + "\",0))";
        String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        return RPCUtils.errMsgOrIen(response);
    }

    public List<UserSecurityKey> getSecurityKeys(String userIen) throws RPCException {
        DdrLister query = buildGetSecurityKeysQuery(userIen);
        String[] response = query.execute();
        return toUserSecurityKeys(response);
    }

    protected DdrLister buildGetSecurityKeysQuery(String userIen) {
        DdrLister query = new DdrLister(cxn);
        query.setFile("200.051");
        query.setIens("," + userIen + ",");
        query.setFields(".01;1;2;3");
        query.setFlags("IP");
        return query;
    }

    protected List<UserSecurityKey> toUserSecurityKeys(String[] response) throws RPCException {
        ArrayList<UserSecurityKey> lst;
        if (response == null || response.length == 0) {
            lst = null;
        } else {
            LinkedHashMap<String, String> securityKeyFile = cxn.getLookupTable(VistaConstants.SECURITY_KEY);
            lst = new ArrayList<UserSecurityKey>(response.length);
            for (int i = 0; i < response.length; i++) {
                String[] flds = StringUtils.split(response[i], StringUtils.CARET);
                if (securityKeyFile.containsKey(flds[1])) {
                    UserSecurityKey key = new UserSecurityKey(flds[0], flds[1], securityKeyFile.get(flds[1]), flds[2], "", Timestamp.toDateTime(flds[3]));
                    key.setDescriptiveName(getSecurityKeyDescriptiveName(key.getId()));
                    lst.add(key);
                }
            }
        }
        return lst;
    }

    protected String getSecurityKeyDescriptiveName(String ien) throws RPCException {
        StringBuilder arg = new StringBuilder();
        arg.append("$P($G(^DIC(19.1,");
        arg.append(ien);
        arg.append(",0)),U,2)");
        return RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg.toString());
    }

    public String addSecurityKeyForContext(String userIen, String context) throws RPCException {
        String securityKey = UtilityRPCS.getSecurityKey(vistaQueryFactory, cxn, context);
        if (securityKey == null || "".equals(securityKey)) {
            throw new RPCException("No security key for context " + context);
        }
        return addSecurityKey(userIen, securityKey);
    }

    public String addSecurityKey(String userIen, String securityKey) throws RPCException {
        DdrLister query = buildAddSecurityKeyQuery(userIen, securityKey);
        String[] response = query.execute();
        String result = toAddSecurityKeyResult(response);
        String[] flds = StringUtils.split(result, StringUtils.CARET);
        if (!flds[1].equals(securityKey)) {
            throw new RPCException("Unable to add security key " + securityKey + " for user " + userIen + ": " + result);
        }
        return result;
    }

    protected DdrLister buildAddSecurityKeyQuery(String userIen, String securityKey) {
        // Can't use DDR FILER because the underlying M function requires 2 arrays and the
        // broker can only do 1.
        // Idea is to make a query that is guaranteed to find a record, then use the identifier
        // argument to add the new record.
        DdrLister query = new DdrLister(cxn);
        query.setFile("19.1");
        query.setFields(".01");
        query.setFlags("IP");
        query.setFrom(RPCUtils.adjustForNameSearch(securityKey));
        query.setPart(securityKey);
        query.setXref("B");
        query.setId("X \"S XQKUS=" + userIen + " S XQKEY=$P(^(0),U,1) S XQKF=0 D ADD^XQKEY(XQKUS,XQKEY,XQKF) D EN^DDIOL(XQKF)\"");
        return query;
    }

    protected String toAddSecurityKeyResult(String[] response) {
        String rtc;
        if (response == null || response.length == 0) {
            rtc = "";
        } else {
            rtc = response[0]; //should be an ien?
        }
        return rtc;
    }

    public String removeSecurityKey(String userIen, String securityKey) throws RPCException {
        String ien = UtilityRPCS.getSecurityKeyIen(vistaQueryFactory, cxn, securityKey);
        if ("".equals(ien)) {
            String msg = "deleteKey error: security key " + securityKey
                    + " does not exist at site " + cxn.getSiteCode() + ".";
            throw new RPCException(msg);
        }

        DdrFiler query = buildRemoveSecurityKeyQuery(userIen, ien);
        String response = query.execute();
        return toRemoveSecurityKeyResult(response);
    }

    protected DdrFiler buildRemoveSecurityKeyQuery(String userIen, String ien) {
        DdrFiler query = new DdrFiler(cxn);
        query.setOperation("EDIT");
        query.setArgs(new String[] {"200.051" + "^.01^" + ien + "," + userIen + ",^@"});
        return query;
    }

    protected String toRemoveSecurityKeyResult(String response) {
        String rtc;
        if ("".equals(response)) {
            rtc = "";
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            lines = StringUtils.trimArray(lines);
            if ("[Data]".equals(lines[0]) && lines.length == 1) {
                rtc = "OK";
            } else {
                if (!lines[1].equals(VistaConstants.BEGIN_ERRS) && !lines[lines.length - 1].equals(VistaConstants.END_ERRS)) {
                    rtc = "Unexpected return message: " + response;
                } else {
                    rtc = "deleteKey error: " + lines[lines.length - 2];
                }
            }
        }
        return rtc;
    }

    protected String buildVisitRequest(User user, boolean validSSN) throws RPCException {
        try {
            Site userSite = user.getHostSite();

            String userSiteCode;
            String userSiteName;

            //if site is an outside agency, set user site code to station 200
            if (AGENCY_VA.equalsIgnoreCase(userSite.getAgency())) {
                userSiteCode = userSite.getSiteCode();
                userSiteName = userSite.getName();
            } else {
                userSiteCode = "200";
                userSiteName = "DOD";
            }

            //VistaSite name cannot exceed 30 characters
            if (userSiteName != null && userSiteName.length() > 30) {
                userSiteName = userSiteName.substring(0, 30);
            }

            StringBuilder argSB = new StringBuilder("-31^DVBA_^")
                    .append(EncryptUtil.decrypt(user.getEncryptedFederatedUid()))
                    .append('^')
                    .append(user.getName())
                    .append('^')
                    .append(userSiteName)
                    .append('^')
                    .append(userSiteCode)
                    .append('^')
                    .append(user.getUserIen())
                    .append("^No phone");

            VistaQuery vquery = vistaQueryFactory.getVistaQuery("XUS SIGNON SETUP");
            vquery.addParameter(VistaQuery.LITERAL, argSB.toString());
            return vquery.buildMessage();
        } catch (Exception e) {
            throw new RPCException(e);
        }

    }

    protected String buildVisitRequestBse(User user) throws RPCException {
        try {
            VistaQuery vquery = vistaQueryFactory.getVistaQuery("XUS SIGNON SETUP");
            vquery.addParameter(VistaQuery.LITERAL, user.getBseToken());

            return vquery.buildMessage();
        } catch (Exception e) {
            throw new RPCException(e);
        }
    }

    public String visit(User user, String inrequestedContext, Boolean validSSN) throws RPCException {
        cxn.setUserHostSite(user.getHostSite());
        
        if (APP_CONFIG.bseTokenEnabled()) {
            if (cxn.getUserHostSite() != null && "VBA".equalsIgnoreCase(cxn.getUserHostSite().getMoniker())) {
                return visitNonBse(user, inrequestedContext, validSSN);
            } else {
                return visitBse(user);
            }
        } else {
            return visitNonBse(user, inrequestedContext, validSSN);
        }
    }

    public String visitBse(User user) throws RPCException {
        String userDUZ;
        if (!"VA".equalsIgnoreCase(user.getAgency())) {
            userDUZ =authenticateSaml(user.getSamlToken());
        } else {
            String request = buildVisitRequestBse(user);
            userDUZ = (String) cxn.query(request);
        }
        
        // TODO: check userDUZ against user.getUserIEN() or user.getUserId()

        if (!setContext(VistaConstants.JLV_CONTEXT)) {
            throw new RPCException("Cannot set JLV_CONTEXT for visit to RPC Broker");
        }

        return userDUZ;
    }


    public String visitNonBse(User user, String inrequestedContext, Boolean validSSN) throws RPCException {
        String rtc = null;
        String requestedContext = inrequestedContext;
        if (StringUtils.isEmpty(requestedContext)) {
            requestedContext = VistaConstants.CPRS_CONTEXT;
        }

        String request = buildVisitRequest(user, validSSN.booleanValue());
        cxn.query(request);

        String initialSetupContext = VistaConstants.CAPRI_CONTEXT;

        //try to initially set context to CAPRI
        try {
            setContext(initialSetupContext);
        } catch (RPCException e) {
            //try with CPRS context if CAPRI fails
            initialSetupContext = VistaConstants.CPRS_CONTEXT;
            setContext(initialSetupContext);
        }
        //Need to put user ID for this site into this site's connection
        cxn.setUserIen(getUserId(user));
        if (initialSetupContext.equals(requestedContext)) {
            rtc = cxn.getUserIen();
        } else {
            try {
                setContext(requestedContext);
                rtc = cxn.getUserIen();
            } catch (RPCException e) {
                if (e.getMessage().indexOf("context '" + requestedContext + "' does not exist on server") != -1) {
                    throw e;
                }
                // Just means user has never visited this site before
            } //String optionIEN = cxn.getOptionIen(requestedContext);
            if (rtc == null) {
                //addMenuOption requires CAPRI_CONTEXT
                setContext(VistaConstants.CAPRI_CONTEXT);

                addMenuOption(requestedContext, cxn.getUserIen());
                //Try again to set the requested option
                try {
                    setContext(requestedContext);
                } catch (RPCException e) {
                    if (e.getMessage().indexOf("does not have access to") != -1) {
                        //Some kind of xref error happened in Vista the first time, do it again.
                        setContext(VistaConstants.CAPRI_CONTEXT);
                        addMenuOption(requestedContext, cxn.getUserIen());
                        setContext(requestedContext);
                    } else {
                        throw e;
                    }
                }

                rtc = cxn.getUserIen();
            }
        }
        return rtc;
    }

    public void removeMenuOption(String optNum, String userIen) throws RPCException {
        removeOption(VistaConstants.MENU_OPTIONS_FILE, optNum, userIen);
    }

    public void removeDelegatedOption(String optNum, String userIen) throws RPCException {
        removeOption(VistaConstants.DELEGATED_OPTIONS_FILE, optNum, userIen);
    }

    protected String removeOption(String file, String optNum, String userIen) throws RPCException {
        DdrFiler query = buildRemoveOptionQuery(file, optNum, userIen);
        String response = query.execute();
        return toRemoveOptionResult(response);
    }

    protected DdrFiler buildRemoveOptionQuery(String file, String optNum, String userIen) {
        DdrFiler query = new DdrFiler(cxn);
        query.setOperation("EDIT");
        query.setArgs(new String[] {file + "^.01^" + optNum + "," + userIen + ",^@"});
        return query;
    }

    protected String toRemoveOptionResult(String response) {
        String rtc;
        if ("".equals(response)) {
            rtc = "";
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            lines = StringUtils.trimArray(lines);
            if ("[Data]".equals(lines[0]) && lines.length == 1) {
                rtc = "OK";
            } else {
                if (!lines[1].equals(VistaConstants.BEGIN_ERRS) && !lines[lines.length - 1].equals(VistaConstants.END_ERRS)) {
                    rtc = "Unexpected return message: " + response;
                } else {
                    rtc = "removeOption error: " + lines[lines.length - 2];
                }
            }
        }
        return rtc;
    }

    public UserOption getOption(String name, String userIen) throws RPCException {
        UserOption rtc = null;
        RPCUserOption[] opts = (RPCUserOption[]) getOptions(VistaConstants.MENU_OPTIONS_FILE, userIen);
        if (opts != null) {
            for (int i = 0; i < opts.length; i++) {
                if (opts[i].getName().equals(name)) {
                    rtc = opts[i];
                    break;
                }
            }
        }
        return rtc;
    }

    public String addMenuOption(String context, String userIen) throws RPCException {
        return addOption(VistaConstants.MENU_OPTIONS_FILE, context, userIen);
    }

    public String addDelegatedOption(String context, String userIen) throws RPCException {
        return addOption(VistaConstants.DELEGATED_OPTIONS_FILE, context, userIen);
    }

    protected String addOption(String file, String context, String userIen) throws RPCException {
        //Did we get a numeric userIen?
        if (!StringUtils.isNumeric(userIen)) {
            throw new RPCException("Non-numeric user ID");
        }

        //Now make sure site has the requested option.  Otherwise a record will be written with a bad pointer to
        //file 19.
        String optionIen = UtilityRPCS.getOptionIen(vistaQueryFactory, cxn, context);
        if ("".equals(optionIen)) {
            throw new RPCException("No such context");
        }

        DdrFiler query = buildAddOptionQuery(file, optionIen, userIen);
        String response = query.execute();
        return toAddOptionResult(response);
    }

    protected DdrFiler buildAddOptionQuery(String file, String optionIen, String userIen) {
        DdrFiler query = new DdrFiler(cxn);
        query.setOperation("ADD");
        query.setArgs(new String[] {file + "^.01^+1," + userIen + ",^" + optionIen});
        return query;
    }

    protected String toAddOptionResult(String response) throws RPCException {
        String rtc;
        if ("".equals(response)) {
            rtc = "";
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            lines = StringUtils.trimArray(lines);
            if ("[Data]".equals(lines[0]) && lines.length == 2) {
                rtc = parseOptionNumber(response);
            } else {
                if (!lines[1].equals(VistaConstants.BEGIN_ERRS) && !lines[lines.length - 1].equals(VistaConstants.END_ERRS)) {
                    throw new RPCException("Unexpected return message: " + response);
                }
                throw new RPCException("removeOption error: " + lines[lines.length - 2]);
            }
        }
        return rtc;
    }

    //public String[] assignMenuOptions(String[] contexts, String userIen)
    //{
    //    VistaQuery vq = vistaQueryFactory.getVistaQuery("DDR FILER");
    //    vq.addParameter(vq.LITERAL, "ADD");
    //    DictionaryHashList lst = new DictionaryHashList();
    //    for (int i = 0; i < contexts.Length; i++)
    //    {
    //        String optionIEN = cxn.getOptionIen(contexts[i]);
    //        String arg = VistaConstants.MENU_OPTIONS_FILE + "^.01^+" + Convert.ToString(i+1) + "," + userIen + ",^" + optionIEN;
    //        lst.Add(Convert.ToString(i+1), arg);
    //        vq.addParameter(vq.LIST, lst);
    //    }
    //    String rtn = (String)cxn.query(vq.buildMessage());
    //    String[] lines = StringUtils.split(rtn, StringUtils.CRLF);
    //    lines = StringUtils.trimArray(lines);
    //    if (lines[0] == "[Data]" && lines.Length == 2)
    //    {
    //        return lines;
    //    }
    //    if (lines[1] != VistaConstants.BEGIN_ERRS && lines[lines.Length - 1] != VistaConstants.END_ERRS)
    //    {
    //        throw new RPCException("Unexpected return message: " + rtn);
    //    }
    //    throw new RPCException("removeOption error: " + lines[lines.Length - 2]);
    //}

    protected String parseOptionNumber(String rtn) throws RPCException {
        String[] lines = StringUtils.split(rtn, StringUtils.CRLF);
        if (!"[Data]".equals(lines[0])) {
            throw new RPCException("Invalid return format (" + rtn + ")");
        }
        if (lines[1].startsWith("[BEGIN_diERRORS]")) {
            throw new RPCException(rtn.substring(8));
        }
        if (lines.length == 1) {
            throw new RPCException("No option number data");
        }
        int pos = lines[1].indexOf(",^");
        String optNum = lines[1].substring(pos + 2);
        if (!StringUtils.isNumeric(optNum)) {
            throw new RPCException("Non-numeric option number");
        }
        return optNum;
    }

    public LinkedHashMap<String, String> getOptionNames() throws RPCException {
//        DdrLister query = DdrLister.buildIenNameQuery(cxn, "19");
        DdrLister query = new DdrLister(cxn);
        query.setIenNameQuery("19");
        String[] response = query.execute();
        return RPCUtils.toStringDictionary(response);
    }

    protected UserOption[] getOptions(String file, String uid) throws RPCException {
        DdrLister query = buildGetOptionsQuery(file, uid);
        String[] response = query.execute();
        return toOptions(response);
    }

    protected DdrLister buildGetOptionsQuery(String file, String uid) {
        DdrLister query = new DdrLister(cxn);
        query.setFile(file);
        query.setIens("," + uid + ",");

        // E flag note
        query.setFields(".01;.01E");

        query.setFlags("IP");
        query.setScreen("I $D(^DIC(19,$P(^(0),U,1),0))'=0");
        return query;
    }

    protected UserOption[] toOptions(String[] response) throws RPCException {
        UserOption[] rtc;
        if (response == null || response.length == 0) {
            rtc = null;
        } else {
            UserOption[] result = new UserOption[response.length];
            for (int i = 0; i < response.length; i++) {
                String[] flds = StringUtils.split(response[i], StringUtils.CARET);
                result[i] = getOption(flds[1]);
                result[i].setNumber(flds[0]);
            }
            rtc = result;
        }
        return rtc;
    }

    public UserOption getOption(String ien) throws RPCException {
        DdrLister query = buildGetOptionQuery(ien);
        String[] response = query.execute();
        return toVistaOption(response);
    }

    protected DdrLister buildGetOptionQuery(String ien) {
        DdrLister query = new DdrLister(cxn);
        query.setFile("19");
        query.setFields(".01;1;3;3.01;4");
        query.setFlags("IP");
        query.setFrom(RPCUtils.adjustForNumericSearch(ien));
        query.setPart(ien);
        query.setMax("1");
        query.setXref("#");
        return query;
    }

    protected UserOption toVistaOption(String[] response) {
        UserOption rtc;
        if (response == null || response.length != 1) {
            rtc = null;
        } else {
            UserOption result = new UserOption();
            String[] flds = StringUtils.split(response[0], StringUtils.CARET);
            result.setId(flds[0]);
            result.setName(flds[1]);
            result.setDisplayName(flds[2]);
            result.setKey(flds[3]);
            result.setReverseKey(flds[4]);
            result.setType(decodeOptionType(flds[5]));
            rtc = result;
        }
        return rtc;
    }

    protected String decodeOptionType(String code) {
        String rtc = code;
        if ("A".equals(code)) {
            rtc = "action";
        }
        if ("E".equals(code)) {
            rtc = "edit";
        }
        if ("I".equals(code)) {
            rtc = "inquire";
        }
        if ("M".equals(code)) {
            rtc = "menu";
        }
        if ("P".equals(code)) {
            rtc = "print";
        }
        if ("R".equals(code)) {
            rtc = "run routine";
        }
        if ("O".equals(code)) {
            rtc = "protocol";
        }
        if ("Q".equals(code)) {
            rtc = "protocol menu";
        }
        if ("X".equals(code)) {
            rtc = "extended action";
        }
        if ("S".equals(code)) {
            rtc = "server";
        }
        if ("L".equals(code)) {
            rtc = "limited";
        }
        if ("C".equals(code)) {
            rtc = "ScreenMan";
        }
        if ("W".equals(code)) {
            rtc = "Window";
        }
        if ("Z".equals(code)) {
            rtc = "Window Suite";
        }
        if ("B".equals(code)) {
            rtc = "Broker (Client/Server)";
        }
        return rtc;
    }

    public UserOption[] getDelegatedOptions(String uid) throws RPCException {
        return getOptions(VistaConstants.DELEGATED_OPTIONS_FILE, uid);
    }

    public UserOption[] getMenuOptions(String userIen) throws RPCException {
        UserOption[] rtc;
        UserOption[] secondaryOpts = getOptions(VistaConstants.MENU_OPTIONS_FILE, userIen);
        ArrayList<UserOption> lst = new ArrayList<UserOption>();
        UserOption primaryOpt = getPrimaryMenuOption(userIen);
        if (primaryOpt != null) {
            primaryOpt = getOption(primaryOpt.getId());
            primaryOpt.setPrimaryOption(true);
            lst.add(primaryOpt);
        }
        if (secondaryOpts != null) {
            lst.addAll(Arrays.asList(secondaryOpts));
        }
        if (lst.isEmpty()) {
            rtc = null;
        } else {
            rtc = lst.toArray(new UserOption[0]);
        }
        return rtc;
    }

    protected UserOption getPrimaryMenuOption(String userIen) throws RPCException {
        UserOption rtc;
        String arg = "$P($G(^VA(200," + userIen + ",201)),U,1)";
        String pointer = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        if ("".equals(pointer)) {
            rtc = null;
        } else {
            arg = "$P($G(^DIC(19," + pointer + ",0)),U,1)";
            String name = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            UserOption result = new UserOption();
            result.setId(pointer);
            result.setName(name);
            result.setPrimaryOption(true);
            rtc = result;
        }
        return rtc;
    }

    public User[] cprsUserLookup(String target) throws RPCException {
        String request = buildCprsUserLookup(target);
        String response = (String) cxn.query(request);
        return toUsers(response);
    }

    protected String buildCprsUserLookup(String intarget) {
        String target = RPCUtils.adjustForNameSearch(intarget);
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWU NEWPERS");
        vquery.addParameter(VistaQuery.LITERAL, target.toUpperCase(Locale.getDefault()));
        vquery.addParameter(VistaQuery.LITERAL, "1");
        return vquery.buildMessage();
    }

    public User[] userLookupByName(String target, String maxRex) throws RPCException {
        DdrLister query = buildUserLookupByNameQuery(target, maxRex);
        String[] response = query.execute();
        return toUsers(response);
    }

    protected DdrLister buildBasicUserLookupQuery(String intarget, String maxrex) {
        DdrLister query = new DdrLister(cxn);
        query.setFile("200");
        query.setFields("@;.01;4;5;7;8;9;20.2;20.3");
        query.setFlags("IP");
        query.setMax(maxrex);
        String target = intarget.toUpperCase(Locale.getDefault());
        query.setFrom(RPCUtils.adjustForNameSearch(target));
        query.setPart(target);
        query.setXref("B");
        return query;
    }

    protected DdrLister buildUserLookupByNameQuery(String intarget, String maxRex) {
        DdrLister query = new DdrLister(cxn);
        query.setFile("200");

        // E flag note
        query.setFields("@;.01;.132;.137;.141;3;4;5;7;8E;9;13;20.2;20.3;29;29E;53.5;53.5E;53.6;201;201E");

        query.setFlags("IP");
        query.setMax(maxRex);
        String target = intarget.toUpperCase(Locale.getDefault());
        query.setFrom(RPCUtils.adjustForNameSearch(target));
        query.setPart(target);
        query.setXref("B");
        return query;
    }

    protected User[] toUsers(String[] inresponse) {
        String[] response;
        User[] rtc;
        if (inresponse == null || inresponse.length == 0) {
            rtc = new User[0];
        } else {
            response = StringUtils.trimArray(inresponse);
            User[] result = new User[response.length];
            for (int i = 0; i < response.length; i++) {
                String[] flds = StringUtils.split(response[i], StringUtils.CARET);
                result[i] = new User();
                result[i].setUserIen(flds[0]);
                result[i].setName(flds[1]);

                //flds[5] is FM access
                result[i].setGender(flds[6]);
                result[i].setDob(Timestamp.toUtcString(flds[7]));
                //flds[8] is disuser
                result[i].setEncryptedFederatedUid(EncryptUtil.encrypt(flds[10]));
                //flds[11] is nickname
                result[i].setSigClass(flds[12] + "\r\n" + flds[13]);
                //flds[14] is service IEN
                //flds[16] is User Class IEN
                result[i].setProviderClass(flds[17]);
                //result[i].setPrimaryPermission(new KeyValuePair(flds[19], flds[20]));
            }
            rtc = result;
        }
        return rtc;
    }

    public User userLookup(String userIen) throws RPCException {
        DdrLister query = buildUserLookupQuery(userIen);
        String[] response = query.execute();
        return toUser(response);
    }

    protected DdrLister buildGetUserQuery(String userIen) {
        DdrLister query = new DdrLister(cxn);
        query.setFile("200");
        query.setFields("@;.01;.132;.137;.138;.141;8;9;29");
        query.setFlags("IP");
        query.setMax("1");
        query.setFrom(RPCUtils.adjustForNumericSearch(userIen));
        query.setPart(userIen);
        query.setXref("#");
        return query;
    }

    protected User toUserFromGetUser(String[] response) throws RPCException {
        User rtc;
        if (response == null || response.length == 0) {
            rtc = null;
        } else {
            String[] rex = StringUtils.split(response[0], StringUtils.CRLF);
            rex = StringUtils.trimArray(rex);
            if (rex.length != 1) {
                throw new RPCException("Multiple records returned");
            }
            String[] flds = StringUtils.split(rex[0], StringUtils.CARET);
            User result = new User();
            result.setUserIen(flds[0]);
            result.setName(flds[1]);
            result.setPager(flds[3]);
            if (!"".equals(flds[4])) {
                String res;
                if ("".equals(flds[3])) {
                    res = result.getPager() + flds[4];
                } else {
                    res = result.getPager() + "/"  + flds[4];
                }
                result.setPager(res);
            }

            result.setEncryptedFederatedUid(EncryptUtil.encrypt(flds[7]));

            rtc = result;
        }
        return rtc;
    }

    protected String getUserTitle(String ien) throws RPCException {
        String arg = "$G(^DIC(3.1," + ien + ",0))";
        return RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
    }

    protected String getUserService(String ien) throws RPCException {
        String arg = "$P($G(^DIC(49," + ien + ",0)),U,1)";
        return RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
    }

    protected String getUserClass(String ien) throws RPCException {
        String arg = "$P($G(^DIC(7," + ien + ",0)),U,1)";
        return RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
    }

    protected DdrLister buildUserLookupQuery(String userIen) {
        DdrLister query = new DdrLister(cxn);
        query.setFile("200");

        // E flag note
        query.setFields("@;.01;.132;.137;.141;3;4;5;7;8E;9;13;20.2;20.3;29;29E;53.5;53.5E;53.6;201;201E");

        query.setFlags("IP");
        query.setMax("1");
        query.setFrom(RPCUtils.adjustForNumericSearch(userIen));
        query.setPart(userIen);
        query.setXref("#");
        return query;
    }

    protected User toUser(String[] response) {
        User rtc;
        if (response == null || response.length == 0) {
            rtc = null;
        } else {
            String[] flds = StringUtils.split(response[0], StringUtils.CARET);
            User result = new User();
            result.setUserIen(flds[0]);
            result.setName(flds[1]);
            //flds[5] is FM access
            result.setGender(flds[6]);
            result.setDob(Timestamp.toUtcString(flds[7]));
            //flds[8] is disuser
            result.setEncryptedFederatedUid(EncryptUtil.encrypt(flds[10]));
            //flds[11] is nickname
            result.setSigClass(flds[12] + "\r\n" + flds[13]);
            //flds[14] is service IEN
            //flds[16] is User Class IEN
            result.setProviderClass(flds[17]);
            //result.setPrimaryPermission(new KeyValuePair(flds[19], flds[20]));
            rtc = result;
        }
        return rtc;
    }

    private User[] toUsers(String rtn) {
        ArrayList<User> lst = new ArrayList<User>();
        String[] entries = StringUtils.split(rtn, StringUtils.CRLF);
        for (int i = 0; i < entries.length; i++) {
            if ("".equals(entries[i])) {
                continue;
            }
            String[] flds = StringUtils.split(entries[i], StringUtils.CARET);
            if (StringUtils.isNumeric(flds[0])) {
                User user = new User();
                user.setUserIen(flds[0]);
                user.setName(flds[1]);
                lst.add(user);
            }
        }
        return lst.toArray(new User[0]);
    }

    public String visitSite(User user, String context) throws RPCException {
        return visit(user, context, Boolean.TRUE);
    }

    public User getUserInfo(String userIen) throws RPCException {
        User result = new User();
        result.setUserIen(userIen);
        //result.setHostSiteId((String) cxn.getSiteCode().getKey());
        setUserInfo(result);
        return result;
    }

    public void setUserInfo(User user) throws RPCException {
        String userIen = user.getUserIen();
        StringBuffer arg = new StringBuffer(600);
        arg.append("$P($G(^VA(200," + userIen + ",0)),\"^\",1)");
        arg.append("_\"^\"_" + "$P($G(^VA(200," + userIen + ",1)),\"^\",9)");
        arg.append("_\"^\"_" + "$P($G(^VA(200," + userIen + ",.14)),\"^\",1)");
        arg.append("_\"^\"_" + "$P($G(^VA(200," + userIen + ",.13)),\"^\",2)");
        arg.append("_\"^\"_" + "$P($G(^VA(200," + userIen + ",.13)),\"^\",8)");
        arg.append("_\"^\"_" + "$P($G(^ECC(730,$S($D(^VA(200," + userIen + ",\"QAR\")):^VA(200," + userIen + ",\"QAR\"),1:-1),0)),\"^\",1)");
        arg.append("_\"^\"_" + "$G(^DIC(3.1,$S($P(^VA(200," + userIen + ",0),\"^\",9):$P(^VA(200," + userIen + ",0),\"^\",9),1:-1),0))");
        String response = "";
        try {
            response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg.toString());
        } catch (RPCException e) {
            if (e.getMessage().contains("M  ERROR")) {
                throw new IllegalArgumentException("No such userIen");
            } else {
                throw e;
            }
        }
        setUserProperties(user, response);
    }

    protected void setUserProperties(User user, String response) {
        if (!"".equals(response)) {
            String[] flds = StringUtils.split(response, StringUtils.CARET);
            user.setName(flds[0]);
            user.setEncryptedFederatedUid(EncryptUtil.encrypt(flds[1]));
            user.setPager(flds[4]);
        }
    }

    public boolean hasKey(String key) throws RPCException {
        return hasKey(cxn.getUserIen(), key);
    }

    public boolean hasKey(String userIen, String key) throws RPCException {
        String request = buildHasKeyRequest(userIen, key);
        String response = (String) cxn.query(request);
        return "1".equals(response);
    }

    protected String buildHasKeyRequest(String userIen, String key) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWU NPHASKEY");
        vquery.addParameter(VistaQuery.LITERAL, userIen);
        vquery.addParameter(VistaQuery.LITERAL, key);
        return vquery.buildMessage();
    }

    public boolean isValidEsig(String esig) throws RPCException {
        String request = buildIsValidEsigRequest(esig);
        String response = (String) cxn.query(request);
        return "1".equals(response);
    }

    protected String buildIsValidEsigRequest(String esig) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWU VALIDSIG");
        vquery.addEncryptedParameter(VistaQuery.LITERAL, esig);
        return vquery.buildMessage();
    }

    public User getUser(String userIen) throws RPCException {
        DdrLister query = buildGetUserQuery(userIen);
        String[] response = query.execute();
        User result = toUserFromGetUser(response);
        //result.DelegatedOptions = getDelegatedOptions(userIen);
        //result.setSecurityKeys(getSecurityKeys(userIen));
        return result;
    }

    protected LinkedHashMap<String, String> getDivisions(String userIen) throws RPCException {
        DdrLister query = buildGetDivisionsQuery(userIen);
        String[] response = query.execute();
        return RPCUtils.toStringDictionary(response);
    }

    protected DdrLister buildGetDivisionsQuery(String userIen) {
        DdrLister query = new DdrLister(cxn);
        query.setFile("200.02P");
        query.setIens("," + userIen + ",");
        query.setFields(".01E");
        query.setFlags("IP");
        query.setXref("#");
        query.setScreen("I $D(^DIC(4,$P(^(0),U,1),0))'=0");
        return query;
    }

    public String removeOption2() throws RPCException {
        //String arg = "$$GET^XPAR(\"USR^SYS^PKG\",\"ORWOR DISABLE ORDERING\",1,\"I\")";
        //String arg = "D FDA^DILF(200.03,\",18,552318650,\",.01,\"\",\"@\",\"DDRFDA\",\"DDRERR\")";
        //arg += " D FILE^DIE(\"\",\"DDRFDA\",\"DDRERR\")";
        //String arg = "$$FDASET^FILEC(\"200.03^.01^18,552318650,^@\")";
        //String result = VistaUtils.getVariableValue(cxn, arg);
        String arg = "$$FILE^DIE(\"\",$$FDASET^FILEC(\"200.03^.01^18,552318650,^@\"),\"DDRERR\")";
        return RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
    }

}


