package gov.va.med.vds;

import gov.va.med.vds.vistabeans.Endpoint;
import gov.va.med.vds.vistabeans.Site;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.*;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class TestSiteUtils
{
    private static final Logger logger = LoggerFactory.getLogger(TestSiteUtils.class);
    private static final TestSiteUtils INSTANCE = new TestSiteUtils();

    private Map<String, Site> sitesMap;

    private TestSiteUtils()
    {
        sitesMap = new LinkedHashMap<String, Site>();
        readCacheSites(TestConfig.get(TestConfig.CACHE_SITES_FILE));
        readRPCSites(TestConfig.get(TestConfig.RPC_SITES_FILE));
    }

    public static TestSiteUtils getInstance()
    {
        return INSTANCE;
    }

    public Site getSite(String siteCode)
    {
        return sitesMap.get(siteCode);
    }

    public List<Site> getAllSites()
    {
        return new ArrayList<Site>(sitesMap.values());
    }

    public Map<String, Site> getSitesMap()
    {
        return this.sitesMap;
    }

    /**
     * Reads vha sites from xml file.
     */
    private void readRPCSites(String vhaSiteFile)
    {
        final String XPATH_SITE = "//VhaSite";
        final String XPATH_SITECODE = "@ID";
        final String XPATH_SITENAME = "@name";
        final String XPATH_MONIKER = "@moniker";
        final String XPATH_ENDPT_HOST = "./DataSource/@source";
        final String XPATH_ENDPT_PORT = "./DataSource/@port";
        final String XPATH_ENDPT_MODALITY = "./DataSource/@modality";
        final String XPATH_ENDPT_STATUS = "./DataSource/@status";

        ClassLoader classLoader = this.getClass().getClassLoader();

        InputStream inputStream = null;

        if (classLoader != null)
        {
            try
            {
                inputStream = classLoader.getResourceAsStream(vhaSiteFile);
                Document xmlDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(inputStream);

                XPath xpath = XPathFactory.newInstance().newXPath();
                XPathExpression exprSite = xpath.compile(XPATH_SITE);
                XPathExpression exprSiteCode = xpath.compile(XPATH_SITECODE);
                XPathExpression exprSiteName = xpath.compile(XPATH_SITENAME);
                XPathExpression exprSiteMoniker = xpath.compile(XPATH_MONIKER);
                XPathExpression exprEndPtHost = xpath.compile(XPATH_ENDPT_HOST);
                XPathExpression exprEndPtPort = xpath.compile(XPATH_ENDPT_PORT);
                XPathExpression exprEndPtModality = xpath.compile(XPATH_ENDPT_MODALITY);
                XPathExpression exprEndPtStatus = xpath.compile(XPATH_ENDPT_STATUS);

                NodeList vhaSites = (NodeList) exprSite.evaluate(xmlDoc, XPathConstants.NODESET);

                for(int i = 0; i < vhaSites.getLength(); i++)
                {
                    Node vhaSite = vhaSites.item(i);
                    String siteCode = (String) exprSiteCode.evaluate(vhaSite, XPathConstants.STRING);
                    String siteName = (String) exprSiteName.evaluate(vhaSite, XPathConstants.STRING);
                    String siteMoniker = (String) exprSiteMoniker.evaluate(vhaSite, XPathConstants.STRING);

                    String rpcHost = (String) exprEndPtHost.evaluate(vhaSite, XPathConstants.STRING);
                    Integer rpcPort = Integer.valueOf((String) exprEndPtPort.evaluate(vhaSite, XPathConstants.STRING));
                    String rpcModality = (String) exprEndPtModality.evaluate(vhaSite, XPathConstants.STRING);
                    String rpcStatus = (String) exprEndPtStatus.evaluate(vhaSite, XPathConstants.STRING);

                    if (!sitesMap.containsKey(siteCode))
                    {
                        Site site = new Site();
                        site.setSiteCode(siteCode);
                        site.setMoniker(siteMoniker);
                        site.setName(siteName);
                        site.setAgency("VA");

                        sitesMap.put(siteCode, site);
                    }

                    Site site = sitesMap.get(siteCode);

                    Endpoint rpcEndpoint = new Endpoint();
                    rpcEndpoint.setProtocol("RPC");
                    rpcEndpoint.setSiteCode(siteCode);
                    rpcEndpoint.setHost(rpcHost);
                    rpcEndpoint.setPort(rpcPort);
                    rpcEndpoint.setModality(rpcModality);
                    rpcEndpoint.setStatus(rpcStatus);

                    if (site.getEndpoints() == null)
                        site.setEndpoints(new ArrayList<Endpoint>());

                    site.getEndpoints().add(rpcEndpoint);
                }
            }
            catch (IOException e)
            {
                logger.error(e.getMessage());
            } catch (SAXException e) {
                logger.error(e.getMessage());
            } catch (ParserConfigurationException e) {
                logger.error(e.getMessage());
            } catch (XPathExpressionException e) {
                logger.error(e.getMessage());
            } finally {
                if (inputStream != null)
                {
                    try {
                        inputStream.close();
                    } catch (IOException e) {
                        logger.error(e.getMessage());
                    }
                }
            }
        }
    }

    /**
     * Reads cache sites from xml file.
     */
    private void readCacheSites(String vhaSiteFile)
    {
        final String XPATH_SITE = "//CacheSite";
        final String XPATH_SITECODE = "@sitecode";
        final String XPATH_SITENAME = "@sitename";
        final String XPATH_MONIKER = "@moniker";
        final String XPATH_ENDPT_HOST = "@url";
        final String XPATH_ENDPT_USERNAME = "@username";
        final String XPATH_ENDPT_PASSWORD = "@password";

        ClassLoader classLoader = this.getClass().getClassLoader();

        InputStream inputStream = null;

        if (classLoader != null)
        {
            try
            {
                inputStream = classLoader.getResourceAsStream(vhaSiteFile);
                Document xmlDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(inputStream);

                XPath xpath = XPathFactory.newInstance().newXPath();
                XPathExpression exprSite = xpath.compile(XPATH_SITE);
                XPathExpression exprSiteCode = xpath.compile(XPATH_SITECODE);
                XPathExpression exprSiteName = xpath.compile(XPATH_SITENAME);
                XPathExpression exprSiteMoniker = xpath.compile(XPATH_MONIKER);
                XPathExpression exprEndPtHost = xpath.compile(XPATH_ENDPT_HOST);
                XPathExpression exprEndPtUsername = xpath.compile(XPATH_ENDPT_USERNAME);
                XPathExpression exprEndPtPassword = xpath.compile(XPATH_ENDPT_PASSWORD);

                NodeList vhaSites = (NodeList) exprSite.evaluate(xmlDoc, XPathConstants.NODESET);

                for(int i = 0; i < vhaSites.getLength(); i++)
                {
                    Node vhaSite = vhaSites.item(i);
                    String siteCode = (String) exprSiteCode.evaluate(vhaSite, XPathConstants.STRING);
                    String siteName = (String) exprSiteName.evaluate(vhaSite, XPathConstants.STRING);
                    String siteMoniker = (String) exprSiteMoniker.evaluate(vhaSite, XPathConstants.STRING);

                    String cacheHost = (String) exprEndPtHost.evaluate(vhaSite, XPathConstants.STRING);
                    String cacheUsername = (String) exprEndPtUsername.evaluate(vhaSite, XPathConstants.STRING);
                    String cachePassword = (String) exprEndPtPassword.evaluate(vhaSite, XPathConstants.STRING);

                    if (!sitesMap.containsKey(siteCode))
                    {
                        Site site = new Site();
                        site.setSiteCode(siteCode);
                        site.setMoniker(siteMoniker);
                        site.setName(siteName);
                        site.setAgency("VA");

                        sitesMap.put(siteCode, site);
                    }

                    Site site = sitesMap.get(siteCode);

                    Endpoint cacheEndpoint = new Endpoint();
                    cacheEndpoint.setProtocol("CACHE");
                    cacheEndpoint.setSiteCode(siteCode);
                    cacheEndpoint.setHost(cacheHost);
                    cacheEndpoint.setUsername(cacheUsername);
                    cacheEndpoint.setPassword(cachePassword);

                    if (site.getEndpoints() == null)
                        site.setEndpoints(new ArrayList<Endpoint>());

                    site.getEndpoints().add(cacheEndpoint);
                }
            }
            catch (IOException e)
            {
                logger.error(e.getMessage());
            } catch (SAXException e) {
                logger.error(e.getMessage());
            } catch (ParserConfigurationException e) {
                logger.error(e.getMessage());
            } catch (XPathExpressionException e) {
                logger.error(e.getMessage());
            } finally {
                if (inputStream != null)
                {
                    try {
                        inputStream.close();
                    } catch (IOException e) {
                        logger.error(e.getMessage());
                    }
                }
            }
        }
    }
}
