package gov.va.med.vds.enterprise;


import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.rpc.*;
import gov.va.med.rpc.site.VistaSiteException;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.ProblemsDao;
import gov.va.med.vds.patient.dataservice.MedicationsDataService;
import gov.va.med.vds.patient.dataservice.NotesDataService;
import gov.va.med.vds.patient.dataservice.ProblemsDataService;
import gov.va.med.vds.patient.dataservice.RadiologyDataService;
import gov.va.med.vds.vistabeans.*;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.logging.Level;

import static org.junit.Assert.*;

public class TestEnterprise
{
    private User user;
    private Patient patient;
    private QueryBean queryBean;

    @Before
    public void setup()
    {
        user = new User();
        patient = new Patient();

        String siteCode = TestConfig.get(TestConfig.ENTERPRISE_SITE);
        String userIen = TestConfig.get(TestConfig.ENTERPRISE_USER_IEN);
        String userName = TestConfig.get(TestConfig.ENTERPRISE_USER_NAME);
        String patientIen = TestConfig.get(TestConfig.ENTERPRISE_PATIENT_IEN);

        user.setAgency("VA");
        user.setUserIen(userIen);
        user.setHostSite(TestSiteUtils.getInstance().getSite(siteCode));
        user.setName(userName);
        try {
            setEncryptedFederatedUid(user);
        } catch (Exception e) {
            fail(e.getMessage());
        }

        List<KeyValuePair<String,String>> patientIens = new ArrayList<KeyValuePair<String, String>>();
        patientIens.add(new KeyValuePair<String, String>("652", patientIen));
        patient.setPatientIens(patientIens);
        List<Site> vistaSites = new ArrayList<Site>();
        vistaSites.add(user.getHostSite());
        patient.setVistaSites(vistaSites);

        queryBean = new QueryBean();
    }

    private void setEncryptedFederatedUid(User user) throws RPCException, VistaSiteException {

        User authUser = new User();
        authUser.setUserIen("4455667");
        authUser.setEncryptedFederatedUid(EncryptUtil.encrypt("123456789"));
        authUser.setName("DOD,Provider");
        authUser.setHostSite(user.getHostSite());
        authUser.setAgency("VA");

        RPCConnection cxn = setupSingleRemoteSourceQuery(authUser, authUser.getHostSite(), VistaConstants.CPRS_CONTEXT);
        VistaQueryFactory vistaQueryFactory = VistaQueryFactoryUtil.getVistaQueryFactory(cxn.getConnectionStrategy());

        String arg = "@\"^VA(200," + user.getUserIen() + ",1)\"";
        String rtn = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
//        if (StringUtils.isEmpty(rtn))
//        {
//            throw new RPCException("Unable to get federated user id");
//        }
        String[] parts = StringUtils.split(rtn, StringUtils.CARET);

        if (parts != null && parts.length > 8)
        {
            user.setEncryptedFederatedUid(EncryptUtil.encrypt(parts[8]));
        }
    }

    private static RPCConnection setupSingleRemoteSourceQuery(User user, Site site,
                                                                String context) throws RPCException, VistaSiteException
    {
        if (context == null)
        {
            context = VistaConstants.CPRS_CONTEXT;
        }

        VistaRPCConnectionManager cxnMgr = new VistaRPCConnectionManager(site);
        cxnMgr.connect();

        VistaRPCConnectionManager msq = new VistaRPCConnectionManager(cxnMgr.getConnections());
        msq.execute(VistaRPCConnectionManager.EXECUTION_CLASS_USER_RPC, VistaRPCConnectionManager.EXECUTION_METHOD_USER_RPC, new Object[] {user, context });


        return msq.getConnection(site);
    }

    @Test
    public void testUser()
    {
        assertTrue(user.getEncryptedFederatedUid() != null);
    }

    @Test
    public void testGetProblemsList()
    {
        queryBean.setUser(user);
        queryBean.setPatient(patient);
        queryBean.setStatus("ALL");
        ProblemsDataService problemsDataService = new ProblemsDataService();
        try
        {
            List<Problem> patientProblemList = problemsDataService.getPatientProblemList(queryBean);
            assertTrue(patientProblemList != null);
        }
        catch (Exception e)
        {
            fail(e.getMessage());
        }
    }

    @Test
    public void testOutpatientMedications()
    {
        queryBean.setUser(user);
        queryBean.setPatient(patient);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2013, 0, 1);
        endDate.set(2013, 5, 1);

        queryBean.setStatus("O");
        queryBean.setStartDate(startDate);
        queryBean.setEndDate(endDate);
        MedicationsDataService medsService = new MedicationsDataService();
        try
        {
            List<Medication> patientMedications = medsService.getPatientMedications(queryBean);
            assertTrue(patientMedications != null);
        }
        catch (Exception e)
        {
            fail(e.getMessage());
        }
    }

    @Test
    public void testInpatientMedications()
    {
        queryBean.setUser(user);
        queryBean.setPatient(patient);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2010, 0, 1);
        endDate.set(2013, 5, 1);

        queryBean.setStatus("I");
        queryBean.setStartDate(startDate);
        queryBean.setEndDate(endDate);
        MedicationsDataService medsService = new MedicationsDataService();
        try
        {
            List<Medication> patientMedications = medsService.getPatientMedications(queryBean);
            assertTrue(patientMedications != null);
        }
        catch (Exception e)
        {
            fail(e.getMessage());
        }
    }

    @Test
    public void testRadiologyDetails()
    {
        queryBean.setUser(user);
        queryBean.setPatient(patient);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2013, 0, 1);
        endDate.set(2013, 5, 1);

        queryBean.setStartDate(startDate);
        queryBean.setEndDate(endDate);
        RadiologyDataService radService = new RadiologyDataService();
        try
        {
            List<RadiologyReport> patientRads = radService.getPatientRads(queryBean);
            assertTrue(patientRads != null);

            RadiologyReport rad = patientRads.get(0);

            QueryBean qb = new QueryBean();
            qb.setUser(user);
            qb.setPatient(patient);
            queryBean.setItemId(rad.getExamId());
            queryBean.setStatus(null);

            queryBean.setRecordSite(rad.getSite());
            RadiologyReport report = radService.getRadiologyReport(queryBean);
            assertNotNull(report.getReportText());

        }
        catch (Exception e)
        {
            fail(e.getMessage());
        }
    }

    @Test
    public void testConsultDetails()
    {
        queryBean.setUser(user);
        queryBean.setPatient(patient);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2010, 0, 1);
        endDate.set(2013, 5, 1);

        queryBean.setStartDate(startDate);
        queryBean.setEndDate(endDate);
        NotesDataService notesService = new NotesDataService();
        try
        {
            List<Consult> consults = notesService.getPatientConsultRequests(queryBean);
            assertTrue(consults != null);

            for(Consult consult : consults)
            {
                QueryBean qb = new QueryBean();
                qb.setUser(user);
                qb.setPatient(patient);
                qb.setItemId(consult.getId());
                qb.setRecordSite(consult.getSite());
                List<FreeTextReport> reports = notesService.getConsultReport(qb);

                assertFalse(StringUtils.isEmpty(reports.get(0).getReportText()));
            }
        }
        catch (Exception e)
        {
            fail(e.getMessage());
        }
    }
}
