package gov.va.med.jmeadows.dao.hdd;


import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.DataBean;
import gov.va.med.vds.webservice.PatientDemographics;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Concept code mapping singleton.
 */
public class ConceptCodeMapper {

    protected static final String AGENCY_DOD = AppConfig.getInstance().getAgencyDOD();
    protected static final String AGENCY_VA = AppConfig.getInstance().getAgencyVA();

    private static final ConceptCodeMapper INSTANCE = new ConceptCodeMapper();


    private ConceptCodeMapper() {
        super();
    }

    public static ConceptCodeMapper getInstance() {
        return INSTANCE;
    }


    /**
     * Maps a concept codes from data bean list to a given target coding system.
     *
     * @param hddDao           Health Care Data Dictionary data access object.
     * @param dataBeans        List of data beans with concept codes to map.
     * @param targetCodeSystem Target coding system to map to.
     * @throws JMeadowsException if an error occurs.
     */
    public void mapConceptCodes(HDDDao hddDao, List dataBeans, CodingSystem targetCodeSystem) throws JMeadowsException {
        //code mapping list to populate for Health Data Dictionary query
        CodeMappingList codeMappingList = new CodeMappingList();
        Set<CodeMappingItem> codeMappingItemSet = new HashSet<CodeMappingItem>();

        //targetCodingSystem is used to translate target code systems that are used for multiple domains (ie. LOINC - vitals, lab results, etc)
        CodingSystem targetCodingSystem = null;

        //for each each data bean
        for (Object bean : dataBeans) {
            DataBean dataBean = (DataBean) bean;

            String agency = "";

            if (dataBean.getSite() != null) {
                agency = dataBean.getSite().getAgency();
            }

            if(AGENCY_DOD.equalsIgnoreCase(agency)){
                continue;
            }

            //if this bean doesn't contain a set of codes, skip it
            if (dataBean.getCodes() == null || dataBean.getCodes().isEmpty()) {
                continue;
            }

            //for each code in the data bean
            for (Code code : dataBean.getCodes()) {

                if (code.getCode() == null) {
                    continue;
                }
                //retrieve source coding system enumeration type
                CodingSystem sourceCodingSystem = CodingSystem.getCodingSystemByDisplayName(code.getSystem());

                //melcolm
                //because of disarray in mappings, use logic to set sourceCodeSystem based off of
                //target code system and agency
                String customMap = "";

                switch (targetCodeSystem) {
                    case RXNORM:
                        //System.out.println("MEDICATIONS Mapping");
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_MEDICATIONS.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_MEDICATIONS.getOID();
                        }
                        break;
                    case VITALS:
                        //System.out.println("VITALS Mapping");
                        targetCodingSystem = CodingSystem.LOINC;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_VITALS.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_VITALS.getOID();
                        }
                        break;
                    case CVX:
                        //System.out.println("IMMUNIZATION Mapping");
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_IMMUNIZATIONS.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_IMMUNIZATIONS.getOID();
                        }
                        break;
                    case NOTES:
                        //System.out.println("NOTES Mapping");
                        targetCodingSystem = CodingSystem.LOINC;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_NOTES.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_NOTES.getOID();
                        }
                        break;
                    case ALLERGIES:
                        //System.out.println("ALLERGIES Mapping");
                        targetCodingSystem = CodingSystem.RXNORM;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_ALLERGIES.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_ALLERGIES.getOID();
                        }
                        break;
                    case SNOMED:
                        //System.out.println("PROBLEMS Mapping");
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            if ("ICD9".equalsIgnoreCase(code.getSystem())) {
                                customMap = CodingSystem.VA_PROBLEMS.getOID();
                            } else {
                                customMap = CodingSystem.VA_PROBLEMS_SNOMED.getOID();
                            }
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_PROBLEMS.getOID();
                        }
                        break;
                    case LABS:
                        //System.out.println("LABS Mapping");
                        targetCodingSystem = CodingSystem.LOINC;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_LABS.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_LABS.getOID();
                        }
                        break;
                    case RADIOLOGY:
                        //System.out.println("RADIOLOGY Mapping");

                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_RADS.getOID();
                            targetCodingSystem = CodingSystem.CPT;
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_RADS.getOID();
                            targetCodingSystem = CodingSystem.LOINC;
                        }
                        break;
                    //no mapping for consults
//                    case CONSULT_ORDERS:
//                        //System.out.println("Consult Mapping");
//                        targetCodeSystem = CodingSystem.LOINC;
//                        if(AGENCY_VA.equalsIgnoreCase(agency)){
//                            customMap = CodingSystem.VA_RADS.getOID();
//                        }
//                        else if(AGENCY_DOD.equalsIgnoreCase(agency)){
//                            customMap = CodingSystem.DOD_RADS.getOID();
//                        }
//                        break;
                    case LAB_ORDERS:
                        //System.out.println("Lab Orders Mapping");
                        targetCodingSystem = CodingSystem.LOINC;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_LABS.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_LABS.getOID();
                        }
                        break;
                    case RADIOLOGY_ORDERS:
                        //System.out.println("RADIOLOGY Mapping");
                        targetCodingSystem = CodingSystem.LOINC;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_RADS.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_RADS.getOID();
                        }
                        break;
                    case MEDICATION_ORDERS:
                        //System.out.println("RADIOLOGY Mapping");
                        targetCodingSystem = CodingSystem.RXNORM;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_MEDICATIONS.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_MEDICATIONS.getOID();
                        }
                        break;
                    case X12:
                        //System.out.println("RADIOLOGY Mapping");
                        targetCodingSystem = CodingSystem.X12;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_X12.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_X12.getOID();
                        }
                        break;
                    case CDC_RACE:
                        //System.out.println("RADIOLOGY Mapping");
                        targetCodingSystem = CodingSystem.CDC_RACE;
                        if (AGENCY_VA.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.VA_RACE.getOID();
                        } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                            customMap = CodingSystem.DOD_RACE.getOID();
                        }
                        break;
                    default:
                        customMap = "";

                }

                if (sourceCodingSystem != null) {
                    //add coding mapping item to code mapping list
                    CodeMappingItem codeMappingItem = new CodeMappingItem();
                    codeMappingItem.setSiteCode(((DataBean) bean).getSite().getSiteCode());
                    codeMappingItem.setSourceSystem(sourceCodingSystem.getOID());
                    if (targetCodingSystem == null) {
                        codeMappingItem.setTargetSystem(targetCodeSystem.getOID());
                    } else {
                        codeMappingItem.setTargetSystem(targetCodingSystem.getOID());
                    }
                    codeMappingItem.setCode(code.getCode());
                    codeMappingItem.setCustomMap(customMap);

                    //override mappings for items with a code type of 'text'
//                    if(CodingSystem.VA_IMMUNIZATIONS.getOID().equals(customMap))
//                        codeMappingItem.setCode(((Immunization)bean).getName());

                    //use a Set to ensure unique values
                    codeMappingItemSet.add(codeMappingItem);
                    //codeMappingList.getCodeMappingItems().add(codeMappingItem);
                }
            }
        }

        for (CodeMappingItem codeMappingItem : codeMappingItemSet) {
            codeMappingList.getCodeMappingItems().add(codeMappingItem);
        }

        //concept code map return type
        Map<String, List<Code>> conceptCodeMap = null;

        //if code mapping list has items
        if (codeMappingList.getCodeMappingItems().size() > 0) {
            //retrieve concept codes map from Health Data Dictionary
            conceptCodeMap = hddDao.mapConceptCodes(codeMappingList, targetCodeSystem.getOID());
        }

        //if concept codes map is populated add mapped concept codes to data bean code list.
        if (conceptCodeMap == null || conceptCodeMap.isEmpty()) {
            return;
        }

        //for each data bean
        for (Object bean : dataBeans) {
            DataBean dataBean = (DataBean) bean;

            //if this bean doesn't contain a set of codes, skip it
            if (dataBean.getCodes() == null || dataBean.getCodes().isEmpty()) {
                continue;
            }

            //initialize list of mapped codes to be retrieved from concept codes map
            List<Code> mappedCodes = new ArrayList<Code>();

            //for each code in the data bean
            for (Code code : dataBean.getCodes()) {
//                CodingSystem sourceCodingSystem = null;

                //skip items that aren't defined in this system
                if (CodingSystem.getCodingSystemByDisplayName(code.getSystem()) == null) {
                    continue;
                }

                //form map retrieval key: sourceSystemId^code
                String mapKey = CodingSystem.getCodingSystemByDisplayName(code.getSystem()).getOID() + "^" + code.getCode();


                //if map contains key
                if (conceptCodeMap.containsKey(mapKey)) {
                    List<Code> conceptCodeList = conceptCodeMap.get(mapKey);

                    //loop through mapped concept code
                    for (Code conceptCode : conceptCodeList) {

                        //update target system to proper system value (ie: RADIOLOGY -> LOINC)
                        if (targetCodingSystem == null) {
                            conceptCode.setSystem(targetCodeSystem.getDisplayName());
                        } else {
                            conceptCode.setSystem(targetCodingSystem.getDisplayName());
                        }
                        //if mapped code matches existing source, update display
                        //otherwise add it to mappedCodes list.
                        if (code.getCode().equals(conceptCode.getCode())) {
                            code.setDisplay(conceptCode.getDisplay());
                        } else {
                            mappedCodes.add(conceptCode);
                        }
                    }
                }
            }

            //add mapped codes to data bean code list
            dataBean.getCodes().addAll(mappedCodes);
        }

    }

    /**
     * Maps a concept codes from data bean list to a given target coding system.
     *
     * @param hddDao          Health Care Data Dictionary data access object.
     * @param patDemographics List of PatientDemographics with race to map.
     * @throws JMeadowsException if an error occurs.
     */
    public void mapDemographics(HDDDao hddDao, List patDemographics) throws JMeadowsException {
        //code mapping list to populate for Health Data Dictionary query
        CodeMappingList codeMappingList = new CodeMappingList();
        Set<CodeMappingItem> codeMappingItemSet = new HashSet<CodeMappingItem>();

        //targetCodingSystem is used to translate target code systems that are used for multiple domains (ie. LOINC - vitals, lab results, etc)
        CodingSystem targetCodingSystem = null;

        //for each each data bean
        for (Object bean : patDemographics) {
            PatientDemographics dataBean = (PatientDemographics) bean;

            String agency = "";

            if (dataBean.getSite() != null) {
                agency = dataBean.getSite().getAgency();
            }

            if (dataBean.getEthnicity() != null && !dataBean.getEthnicity().isEmpty()) {

                //for each code in the data bean
                for (Code code : dataBean.getEthnicity()) {

                    //retrieve source coding system enumeration type
                    CodingSystem sourceCodingSystem = CodingSystem.getCodingSystemByDisplayName(code.getSystem());

                    //melcolm
                    //because of disarray in mappings, use logic to set sourceCodeSystem based off of
                    //target code system and agency
                    String customMap = "";

                    targetCodingSystem = CodingSystem.CDC_RACE;
                    if (AGENCY_VA.equalsIgnoreCase(agency)) {
                        customMap = CodingSystem.VA_RACE.getOID();
                    } else if (AGENCY_DOD.equalsIgnoreCase(agency)) {
                        customMap = CodingSystem.DOD_RACE.getOID();
                    }


                    if (sourceCodingSystem != null) {
                        //add coding mapping item to code mapping list
                        CodeMappingItem codeMappingItem = new CodeMappingItem();
                        codeMappingItem.setSiteCode(dataBean.getSite().getSiteCode());
                        codeMappingItem.setSourceSystem(sourceCodingSystem.getOID());
                        codeMappingItem.setTargetSystem(targetCodingSystem.getOID());
                        codeMappingItem.setCode(code.getCode());
                        codeMappingItem.setCustomMap(customMap);

                        //override mappings for items with a code type of 'text'
                        //                    if(CodingSystem.VA_IMMUNIZATIONS.getOID().equals(customMap))
                        //                        codeMappingItem.setCode(((Immunization)bean).getName());

                        //use a Set to ensure unique values
                        codeMappingItemSet.add(codeMappingItem);
                        //codeMappingList.getCodeMappingItems().add(codeMappingItem);
                    }
                }
            }

            if (dataBean.getRace() != null && !dataBean.getRace().isEmpty()) {

                //for each code in the data bean
                for (Code code : dataBean.getRace()) {

                    //retrieve source coding system enumeration type
                    CodingSystem sourceCodingSystem = CodingSystem.getCodingSystemByDisplayName(code.getSystem());

                    String customMap = "";

                    targetCodingSystem = CodingSystem.CDC_RACE;
                    customMap = CodingSystem.VA_RACE.getOID();

                    if (sourceCodingSystem != null) {
                        //add coding mapping item to code mapping list
                        CodeMappingItem codeMappingItem = new CodeMappingItem();
                        if(dataBean.getSite() != null) {
                            codeMappingItem.setSiteCode(dataBean.getSite().getSiteCode());
                        } else {
                            codeMappingItem.setSiteCode("none");
                        }
                        codeMappingItem.setSourceSystem(sourceCodingSystem.getOID());
                        codeMappingItem.setTargetSystem(targetCodingSystem.getOID());
                        codeMappingItem.setCode(code.getCode());
                        codeMappingItem.setCustomMap(customMap);

                        //override mappings for items with a code type of 'text'
                        //                    if(CodingSystem.VA_IMMUNIZATIONS.getOID().equals(customMap))
                        //                        codeMappingItem.setCode(((Immunization)bean).getName());

                        //use a Set to ensure unique values
                        codeMappingItemSet.add(codeMappingItem);
                        //codeMappingList.getCodeMappingItems().add(codeMappingItem);
                    }
                }
            }
        }

        for (CodeMappingItem codeMappingItem : codeMappingItemSet) {
            codeMappingList.getCodeMappingItems().add(codeMappingItem);
        }

        //concept code map return type
        Map<String, List<Code>> conceptCodeMap = null;

        //if code mapping list has items
        if (codeMappingList.getCodeMappingItems().size() > 0) {
            //retrieve concept codes map from Health Data Dictionary
            conceptCodeMap = hddDao.mapConceptCodes(codeMappingList, CodingSystem.CDC_RACE.getOID());
        }

        //if concept codes map is populated add mapped concept codes to data bean code list.
        if (conceptCodeMap == null || conceptCodeMap.isEmpty()) {
            return;
        }

        //for each data bean
        for (Object bean : patDemographics) {
            PatientDemographics dataBean = (PatientDemographics) bean;

            //if this bean doesn't contain a set of codes, skip it
            if (dataBean.getEthnicity() != null && !dataBean.getEthnicity().isEmpty()) {

                //initialize list of mapped codes to be retrieved from concept codes map
                List<Code> mappedCodes = new ArrayList<Code>();

                //for each code in the data bean
                for (Code code : dataBean.getEthnicity()) {
//                    CodingSystem sourceCodingSystem = null;

                    //skip items that aren't defined in this system
                    if (CodingSystem.getCodingSystemByDisplayName(code.getSystem()) == null) {
                        continue;
                    }

                    //form map retrieval key: sourceSystemId^code
                    String mapKey = CodingSystem.getCodingSystemByDisplayName(code.getSystem()).getOID() + "^" + code.getCode();


                    //if map contains key
                    if (conceptCodeMap.containsKey(mapKey)) {
                        List<Code> conceptCodeList = conceptCodeMap.get(mapKey);

                        //loop through mapped concept code
                        for (Code conceptCode : conceptCodeList) {

                            //update target system to proper system value (ie: RADIOLOGY -> LOINC)
                            if (targetCodingSystem != null) {
                                conceptCode.setSystem(targetCodingSystem.getDisplayName());
                            } else {
                                conceptCode.setSystem(CodingSystem.CDC_RACE.getDisplayName());
                            }
                            //if mapped code matches existing source, update display
                            //otherwise add it to mappedCodes list.
                            if (code.getCode().equals(conceptCode.getCode())) {
                                code.setDisplay(conceptCode.getDisplay());
                            } else {
                                mappedCodes.add(conceptCode);
                            }
                        }
                    }
                }

                //add mapped codes to data bean code list
                dataBean.getEthnicity().addAll(mappedCodes);
            }


            //if this bean doesn't contain a set of codes, skip it
            if (dataBean.getRace() != null && !dataBean.getRace().isEmpty()) {

                //initialize list of mapped codes to be retrieved from concept codes map
                List<Code> mappedCodes = new ArrayList<Code>();

                //for each code in the data bean
                for (Code code : dataBean.getRace()) {
//                    CodingSystem sourceCodingSystem = null;

                    //skip items that aren't defined in this system
                    if (CodingSystem.getCodingSystemByDisplayName(code.getSystem()) == null) {
                        continue;
                    }

                    //form map retrieval key: sourceSystemId^code
                    String mapKey = CodingSystem.getCodingSystemByDisplayName(code.getSystem()).getOID() + "^" + code.getCode();


                    //if map contains key
                    if (conceptCodeMap.containsKey(mapKey)) {
                        List<Code> conceptCodeList = conceptCodeMap.get(mapKey);

                        //loop through mapped concept code
                        for (Code conceptCode : conceptCodeList) {

                            //update target system to proper system value (ie: RADIOLOGY -> LOINC)
                            if (targetCodingSystem != null) {
                                conceptCode.setSystem(targetCodingSystem.getDisplayName());
                            } else {
                                conceptCode.setSystem(CodingSystem.CDC_RACE.getDisplayName());
                            }
                            //if mapped code matches existing source, update display
                            //otherwise add it to mappedCodes list.
                            if (code.getCode().equals(conceptCode.getCode())) {
                                code.setDisplay(conceptCode.getDisplay());
                            } else {
                                mappedCodes.add(conceptCode);
                            }
                        }
                    }
                }

                //add mapped codes to data bean code list
                dataBean.getRace().addAll(mappedCodes);
            }

        }

    }
}
