package gov.va.med.jmeadows.dao.hdd;

import javax.sql.DataSource;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.vds.webservice.Code;

import java.io.StringWriter;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import gov.va.med.jmeadows.common.JMeadowsException;

/**
 * Health Data Dictionary Data Access Object.
 */
public class HDDDao {

    private static final Logger LOGGER = LoggerFactory.getLogger(HDDDao.class);

    private static final AppConfig APPCONFIG = AppConfig.getInstance();

    private DataSource dataSource;

    public HDDDao(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public Map<String, List<Code>> mapConceptCodes(CodeMappingList codeMappingList, String targetSystemId) throws JMeadowsException {
        Map<String, List<Code>> rtc;
        if (APPCONFIG.useTermMappingCache()) {
            rtc = mapConceptCodesUsingHashMapCache(codeMappingList, targetSystemId);
        } else {
            rtc = mapConceptCodesUsingSQL(codeMappingList, targetSystemId);
        }
        return rtc;
    }



    /**
     * Maps concept codes to target system codes.
     *
     * @param codeMappingList Code mapping list data structure.
     * @return Concept code map. Codes are mapped to source system id and source code.
     *         Map key format: <SOURCE_SYSTEM_ID>^<SOURCE_CODE>
     * @throws JMeadowsException if an error occurs.
     */
    public Map<String, List<Code>> mapConceptCodesUsingHashMapCache(CodeMappingList codeMappingList, String targetSystemId) throws JMeadowsException {


        Map<String, List<Code>> conceptCodeMap = new LinkedHashMap<String, List<Code>>();


        for (CodeMappingItem item : codeMappingList.getCodeMappingItems()) {
            String customMap = item.getCustomMap();
            String sourceCodeStr = item.getCode();
            String targetSystem = item.getTargetSystem();
            String sourceSystemId = item.getSourceSystem();

            Code code = CodeMappingsCache.getCode(customMap, sourceCodeStr, targetSystem);

            if (code == null) {
                continue;
            }
            String name = code.getDisplay();
            //skip and log any code mappings containing an error
            if (name != null && name.contains("^")) {
                //logger.error("Error occurred during concept code mapping: {}", name);
                continue;
            }

//            String targetCodeStr = code.getCode();
            code.setSystem(CodingSystem.getCodingSystemByOID(targetSystemId).getDisplayName());

            //create map key from source system id and code
            String mapKey = sourceSystemId + "^" + sourceCodeStr;

            if (!conceptCodeMap.containsKey(mapKey)) {
                conceptCodeMap.put(mapKey, new ArrayList<Code>());
            }

            //add code to entry list
            conceptCodeMap.get(mapKey).add(code);
        }

        return conceptCodeMap;

    }



    /**
     * Maps concept codes to target system codes.
     * @param codeMappingList Code mapping list data structure.
     * @return Concept code map. Codes are mapped to source system id and source code.
     *                  Map key format: <SOURCE_SYSTEM_ID>^<SOURCE_CODE>
     * @throws JMeadowsException if an error occurs.
     */
    public Map<String, List<Code>> mapConceptCodesUsingSQL(CodeMappingList codeMappingList, String targetSystemId) throws JMeadowsException
    {
        Connection sqlConn = null;

        CallableStatement cstmt = null;
        ResultSet rset = null;

        Map<String, List<Code>> conceptCodeMap = new LinkedHashMap<String, List<Code>>();

        try {
            //generate XML representation of codeMappingList
            JAXBContext context = JAXBContext.newInstance(CodeMappingList.class);
            Marshaller marshall = context.createMarshaller();
            marshall.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, false);
            marshall.setProperty(Marshaller.JAXB_ENCODING, "UTF-16");
            StringWriter writter = new StringWriter();

            marshall.marshal(codeMappingList, writter);

            sqlConn = dataSource.getConnection();

            //call mapCodeList stored procedure
            cstmt = sqlConn.prepareCall("{call mapCodeList(?)}");
            cstmt.setString(1, writter.toString());

            rset = cstmt.executeQuery();

            //iterate through mapped codes rows
            while (rset.next()) {
                String name = rset.getString("name");

                //skip and log any code mappings containing an error
                if (name != null && name.contains("^")) {
                    //logger.error("Error occurred during concept code mapping: {}", name);
                    continue;
                }

                String sourceCodeStr = rset.getString("sourceCode");
                String sourceSystemId = rset.getString("sourceSystem");
                String targetCodeStr = rset.getString("targetCode");

                //create code with target values
                Code code = new Code();
                code.setCode(targetCodeStr);
                code.setDisplay(name);
                code.setSystem(CodingSystem.getCodingSystemByOID(targetSystemId).getDisplayName());

                //create map key from source system id and code
                String mapKey = sourceSystemId + "^" + sourceCodeStr;

                if (!conceptCodeMap.containsKey(mapKey)) {
                    conceptCodeMap.put(mapKey, new ArrayList<Code>());
                }

                //add code to entry list
                conceptCodeMap.get(mapKey).add(code);
            }

            return conceptCodeMap;

        } catch (SQLException e) {
            LOGGER.error("mapConceptCodes exception: " + e.getMessage());

            throw new JMeadowsException(e);
        } catch (JAXBException e) {
            throw new JMeadowsException(e);
        } finally {
            try {
                if (rset != null) {
                    rset.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }
    }
}
