/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;


/**
 * Audit data access object
 */
public class AuditDao extends DaoBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(AuditDao.class);

    /**
     * Sends audit log to audit sql database.
     *
     * @param siteId Site identifier
     * @param userIen user (user) identifier
     * @param userNPI user (user) national user identifier
     * @param userName user (user) name
     * @param patId  patient ien.
     * @param category audit category
     * @param requestingApp requesting application
     * @param startDate  query start date (defaults to T)
     * @param endDate    query end date   (defaults to T)
     * @param info misc query info
     * @param cardId user cardId
     * @param emailAddress user emailAddress
     * @throws JMeadowsException if an error occurs
     */
    public void setAudit(String siteId, String siteMoniker, String siteAgency, String recordID, String userIen, String userNPI,
                         String userName, String patId, String category,
                         String requestingApp, String startDate, String endDate,
                         String info, String cardId, String emailAddress, String complexTransaction) throws JMeadowsException {
        Connection sqlConn = null;
        CallableStatement cstmt = null;

        java.sql.Date stDate = null;
        java.sql.Date enDate = null;

        if (Utils.getDate(startDate) != null) {
            stDate = new java.sql.Date(Utils.getDate(startDate).getTime());
        }
        if (Utils.getDate(endDate) != null) {
            enDate = new java.sql.Date(Utils.getDate(endDate).getTime());
        }

        try {
            sqlConn = sm_dbConnectionPool.getConnection();
            cstmt = sqlConn.prepareCall("{call setAudit(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)}");
            cstmt.setDate(1, stDate);
            cstmt.setDate(2, enDate);
            cstmt.setString(3, siteId);
            cstmt.setString(4, userIen);
            cstmt.setString(5, userNPI);
            cstmt.setString(6, userName);
            cstmt.setString(7, patId);
            cstmt.setString(8, category);
            cstmt.setString(9, requestingApp);
            cstmt.setString(10, info);
            cstmt.setString(11, cardId);
            cstmt.setString(12, emailAddress);
            cstmt.setString(13, recordID);
            cstmt.setString(14, siteMoniker);
            cstmt.setString(15, siteAgency);
            cstmt.setString(16, complexTransaction);
            cstmt.executeUpdate();

        } catch (Exception e) {
            LOGGER.error("JMeadows::auditTrack; Exception - " + e.getMessage());
            throw new JMeadowsException(e);

        } finally {
            try {
                if (sqlConn != null) {
                    sqlConn.setAutoCommit(true);
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }
    }

    /**
     * Sends audit log to audit sql database.
     *
     * @param siteId Site identifier
     * @param userIen user (user) identifier
     * @param userNPI user (user) national user identifier
     * @param userName user (user) name
     * @param patId  patient ien.
     * @param category audit category
     * @param requestingApp requesting application
     * @param startDate  query start date (defaults to T)
     * @param endDate    query end date   (defaults to T)
     * @throws JMeadowsException if an error occurs
     */
    public void setAudit(String siteId, String siteMoniker, String siteAgency, String recordID, String userIen, String userNPI,
                         String userName, String patId, String category,
                         String requestingApp, String startDate, String endDate, String complexTransaction) throws JMeadowsException {
        setAudit(siteId, siteMoniker, siteAgency, recordID, userIen, userNPI, userName, patId, category, requestingApp, startDate, endDate, "", "", "", complexTransaction);
    }

    /**
     * Sends audit log to encrypted audit sql database.
     *
     * @param siteId Site identifier
     * @param userIen user (user) identifier
     * @param userNPI user (user) national user identifier
     * @param userName user (user) name
     * @param patId  patient ien.
     * @param category audit category
     * @param requestingApp requesting application
     * @param startDate  query start date (defaults to T)
     * @param endDate    query end date   (defaults to T)
     * @throws JMeadowsException if an error occurs
     */
    public void setAuditEncrypted(String siteId, String siteMoniker, String siteAgency, String recordID, String userIen, String userNPI,
                         String userName, String patId, String category,
                         String requestingApp, String startDate, String endDate, String complexTransaction) throws JMeadowsException {
        setAuditEncrypted(siteId, siteMoniker, siteAgency, recordID, userIen, userNPI, userName, patId, category, requestingApp, startDate, endDate, "", "", "", complexTransaction);
    }



    /**
     * Sends audit log to encrypted audit sql database.
     *
     * @param siteId Site identifier
     * @param siteMoniker
     * @param siteAgency
     * @param recordID
     * @param userIen user (user) identifier
     * @param userNPI user (user) national user identifier
     * @param userName user (user) name
     * @param patId  patient ien.
     * @param category audit category
     * @param requestingApp requesting application
     * @param startDate  query start date (defaults to T)
     * @param endDate    query end date   (defaults to T)
     * @param complexTransaction
     * @throws JMeadowsException if an error occurs
     */
    public void setAuditEncrypted(String siteId, String siteMoniker, String siteAgency, String recordID, String userIen, String userNPI,
                                  String userName, String patId, String category,
                                  String requestingApp, String startDate, String endDate,String info, String cardId, String emailAddress, String complexTransaction) throws JMeadowsException {
        Connection sqlConn = null;
        CallableStatement cstmt = null;

        java.sql.Date stDate = null;
        java.sql.Date enDate = null;

        if (Utils.getDate(startDate) != null) {
            stDate = new java.sql.Date(Utils.getDate(startDate).getTime());
        }
        if (Utils.getDate(endDate) != null) {
            enDate = new java.sql.Date(Utils.getDate(endDate).getTime());
        }

        try {
            sqlConn = sm_dbConnectionPoolEncrypted.getConnection();
            cstmt = sqlConn.prepareCall("{call setAudit(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)}");
            cstmt.setDate(1, stDate);
            cstmt.setDate(2, enDate);
            cstmt.setString(3, siteId);
            cstmt.setString(4, userIen);
            cstmt.setString(5, userNPI);
            cstmt.setString(6, userName);
            cstmt.setString(7, patId);
            cstmt.setString(8, category);
            cstmt.setString(9, requestingApp);
            cstmt.setString(10, info);
            cstmt.setString(11, cardId);
            cstmt.setString(12, emailAddress);
            cstmt.setString(13, recordID);
            cstmt.setString(14, siteMoniker);
            cstmt.setString(15, siteAgency);
            cstmt.setString(16, complexTransaction);
            cstmt.executeUpdate();

        } catch (Exception e) {
            LOGGER.error("JMeadows::auditTrack; Exception - " + e.getMessage());
            throw new JMeadowsException(e);

        } finally {
            try {
                if (sqlConn != null) {
                    sqlConn.setAutoCommit(true);
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }
    }

    /**
     * Sends audit log to audit sql database.
     *
     * @param siteId Site identifier
     * @param userIen user (user) identifier
     * @param cardID Smart Card ID
     * @param userNPI user (user) national user identifier
     * @param userName user (user) name
     * @param patId  patient ien.
     * @param category audit category
     * @param info misc data supporting category
     * @param requestingApp requesting application
     * @param startDate  query start date (defaults to T)
     * @param EndDate    query end date   (defaults to T)
     * @param ipAddress user's IP Address
     * @param emailAddress user's email Address (VA users)
     * @throws JMeadowsException if an error occurs
     */
    public void setLoginAudit(String cardID, String siteId, String userIen, String userNPI,
                         String userName, String patId, String category, String info,
                         String requestingApp, String startDate, String EndDate, String ipAddress, String emailAddress) throws JMeadowsException {
        Connection sqlConn = null;
        CallableStatement cstmt = null;

        java.sql.Date stDate = null;
        java.sql.Date enDate = null;

        if (Utils.getDate(startDate) != null) {
            stDate = new java.sql.Date(Utils.getDate(startDate).getTime());
        }
        if (Utils.getDate(EndDate) != null) {
            enDate = new java.sql.Date(Utils.getDate(EndDate).getTime());
        }

        try {
            sqlConn = sm_dbConnectionPool.getConnection();
            cstmt = sqlConn.prepareCall("{call setLoginAudit(?,?,?,?,?,?,?,?,?,?,?,?,?)}");
            cstmt.setDate(1, stDate);
            cstmt.setDate(2, enDate);
            cstmt.setString(3, siteId);
            cstmt.setString(4, userIen);
            cstmt.setString(5, cardID);
            cstmt.setString(6, userNPI);
            cstmt.setString(7, userName);
            cstmt.setString(8, patId);
            cstmt.setString(9, category);
            cstmt.setString(10, info);
            cstmt.setString(11, requestingApp);
            cstmt.setString(12, ipAddress);
            cstmt.setString(13, emailAddress);

            cstmt.executeUpdate();

        } catch (Exception e) {
            LOGGER.error("JMeadows::auditTrack; Exception - " + e.getMessage());
            throw new JMeadowsException(e);

        } finally {
            try {
                if (sqlConn != null) {
                    sqlConn.setAutoCommit(true);
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }
    }

}
