/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.PatientMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.smartbeans.patient.SmartClinicalReminder;
import gov.va.med.vds.webservice.ClinicalReminder;
import gov.va.med.vds.webservice.FreeTextReport;
import gov.va.med.vds.webservice.Site;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Patient clinical reminders data access object.
 */
public class ClinicalRemindersDao extends DaoBase {

    /**
     * Retrieves a list of patient clinical reminders from all available VistA patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient clinical reminders.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<ClinicalReminder> getPatientClinicalReminders(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatClinicalReminders", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {

            //only vista supports clinical reminders
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_vistaHandler)
                        .dataServiceMethod("getPatientClinicalReminders")
                        .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                        .location(loc)
                        .build()
                );
            }
        }

        Object[] reminderObjs = MultiLocationQuery.execute(
                configs, new SmartClinicalReminder(), queryBean);

        List<ClinicalReminder> reminders = null;

        if (reminderObjs != null) {
            reminders = Arrays.asList((ClinicalReminder[]) reminderObjs);
        }

        return reminders;
    }

    /**
     * Retrieves clinical reminder record details from the provided record location.
     *
     * @param queryBean Query parameters
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return Clincal reminder record details.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public FreeTextReport getClinicalReminderDetail(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

        auditQuery("ClinicalReminderDetail", queryBean);

        //only vista supports clinical reminders
        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        FreeTextReport reminderDetail = null;

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {

            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                            queryBean.getPatient().getVistaSites())) {

                reminderDetail = sm_vistaHandler.getClinicalReminderDetail(VistaBeanFactory.create(queryBean));
                break;
            }

        }

        return reminderDetail;
    }
}
