/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.hdd.ConceptCodeMapper;
import gov.va.med.jmeadows.dao.util.BHIERelayBeanFactory;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.PatientMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.smartbeans.patient.SmartImmunization;
import gov.va.med.vds.webservice.Immunization;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Patient immunizations data access object
 */
public class ImmunizationDao extends DaoBase {

    /**
     * Retrieves patient immunizations from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient immunizations.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<Immunization> getPatientImmunizations(JMeadowsQuery queryBean) throws JMeadowsException {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatImmunizations", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {

                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                includedSources.add(APP_CONFIG.getBHIECustodianOidAHLTA());

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean, includedSources, null);

                //SB: Exclude data range to retrieve entire data set
                bhieQueryBean.setStartDate(null);
                bhieQueryBean.setEndDate(null);

                configs.add(
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getPatientImmunizations")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapImmunizationsFromBHIE")
                                .location(bhieLoc)
                                .build()
                );

                if (queryBean.isIncludeVLER()) {
                    configs.add(
                            new PatientMultiLocQueryCfgBuilder()
                                    .dataServiceHandler(sm_bhieRelayServiceHandler)
                                    .dataServiceMethod("getVLERPatientImmunizations")
                                    .dataServiceMethodArgs(bhieQueryBean)
                                    .smartBeanMappingMethod("mapImmunizationsFromBHIE")
                                    .location(bhieLoc)
                                    .build()
                    );
                }


            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_vistaHandler)
                                .dataServiceMethod("getPatientImmunizations")
                                .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                                .location(loc)
                                .build()
                );

                if (queryBean.isIncludeDASVLER()) {
                    configs.add(
                            new PatientMultiLocQueryCfgBuilder()
                                    .dataServiceHandler(sm_vistaVLERHandler)
                                    .dataServiceMethod("getDASVLERImmunizations")
                                    .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                                    .smartBeanMappingMethod("mapDASVLERImmunizations")
                                    .location(loc)
                                    .build()
                    );
                }
            }
        }

        SmartImmunization smartImmunization = new SmartImmunization();

        Object[] immObjs = MultiLocationQuery.execute(
                configs, smartImmunization, queryBean);

        List<Immunization> immunizationList = null;

        if (immObjs != null) {
            immunizationList = Arrays.asList((Immunization[]) immObjs);

            //map immunization concept codes
            ConceptCodeMapper.getInstance().mapConceptCodes(hddDao, immunizationList, CodingSystem.CVX);
        }

        return immunizationList;
    }
}
