/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.common.LabLabelCache;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.hdd.ConceptCodeMapper;
import gov.va.med.jmeadows.dao.util.BHIERelayBeanFactory;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.PatientMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.smartbeans.patient.SmartLabOrder;
import gov.va.med.jmeadows.smartbeans.patient.SmartLabResult;
import gov.va.med.vds.webservice.FreeTextReport;
import gov.va.med.vds.webservice.LabOrder;
import gov.va.med.vds.webservice.LabResult;
import gov.va.med.vds.webservice.Site;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Patient laboratory data access object
 */
public class LabsDao extends DaoBase {

    /**
     * Retrieves patient lab orders from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient lab orders.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<LabOrder> getPatientLabs(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatLabs", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null) {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -3, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(new java.util.Date())));
        }

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {

                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                includedSources.add(APP_CONFIG.getBHIECustodianOidAHLTA());
                includedSources.add(APP_CONFIG.getBHIECustodianOidSHARE());

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean, includedSources, null);

                MultiLocationQueryConfig multiLocChemLabsConfig =
                    new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getPatientLabChemistries")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("mapChemistryLabsFromBHIE")
                        .location(bhieLoc)
                        .build();


                MultiLocationQueryConfig multiLocMicroLabsConfig =
                    new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getPatientLabMicrobiologies")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("mapMicrobiologyLabsFromBHIE")
                        .location(bhieLoc)
                        .build();

                MultiLocationQueryConfig multiLocAPLabsConfig =
                    new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getPatientLabAnatomicPathologies")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("mapAnatomicPathologyLabsFromBHIE")
                        .location(bhieLoc)
                        .build();

                configs.add(multiLocChemLabsConfig);
                configs.add(multiLocMicroLabsConfig);
                configs.add(multiLocAPLabsConfig);
            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod("getPatientLabs")
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(loc)
                            .build()
                );
            }
        }

        Object[] labObjs = MultiLocationQuery.execute(
                configs, new SmartLabOrder(), queryBean);

        List<LabOrder> labOrders = null;

        if (labObjs != null) {
            labOrders = Arrays.asList((LabOrder []) labObjs);
        }

        return labOrders;
    }

    /**
     * Retrieves a list of patient lab test results from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A list of patient lab test results.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<LabResult> getPatientLabTestResults(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

        auditQuery("PatLabTestResult", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);
        List<MultiLocationQueryConfig> configs =
                new ArrayList<MultiLocationQueryConfig>();

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                            queryBean.getPatient().getVistaSites())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod("getPatientLabTestResults")
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(loc)
                            .build()
                );
            }
        }

        Object[] labObjs = MultiLocationQuery.execute(
                configs, new SmartLabResult(), queryBean);

        List<LabResult> labResults = null;

        if (labObjs != null) {
            labResults = Arrays.asList((LabResult []) labObjs);
        }

        return labResults;
    }

    /**
     * Retrieves a list of lab order results from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A list of of lab order results.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<LabResult> getLabOrderResult(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

        auditQuery("LabOrderResult", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                   && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                           queryBean.getPatient().getVistaSites())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod("getLabOrderResult")
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(loc)
                            .build()
                );
            }
        }

        Object[] labObjs = MultiLocationQuery.execute(
                configs, new SmartLabResult(), queryBean);

        List<LabResult> labResults = null;

        if (labObjs != null) {
            labResults = Arrays.asList((LabResult []) labObjs);
        }

        return labResults;
    }

    /**
     * Retrieves patient lab report from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return FreeTextReport bean.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public FreeTextReport getPatientLabReport(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

        auditQuery("patientLabReport", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        FreeTextReport report = null;

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                            queryBean.getPatient().getVistaSites())) {

                report = sm_vistaHandler.getPatientLabReport(VistaBeanFactory.create(queryBean));
                break;
            }
        }

        return report;
    }

    /**
     * Retrieves patient lab results from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient lab results.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<LabResult> getPatientLabResults(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatLabResults", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null) {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -3, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(new java.util.Date())));
        }

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {

                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                includedSources.add(APP_CONFIG.getBHIECustodianOidAHLTA());
                includedSources.add(APP_CONFIG.getBHIECustodianOidSHARE());

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean, includedSources, null);

                MultiLocationQueryConfig multiLocChemLabsConfig =
                        new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_bhieRelayServiceHandler)
                            .dataServiceMethod("getPatientLabChemistries")
                            .dataServiceMethodArgs(bhieQueryBean)
                            .smartBeanMappingMethod("mapLabChemResultsFromBHIE")
                            .location(bhieLoc)
                            .build();

                MultiLocationQueryConfig multiLocMicroLabsConfig =
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getPatientLabMicrobiologies")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapLabMicroResultsFromBHIE")
                                .location(bhieLoc)
                                .build();

                MultiLocationQueryConfig multiLocAPLabsConfig =
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getPatientLabAnatomicPathologies")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapLabResultsFromBHIE")
                                .location(bhieLoc)
                                .build();

                configs.add(multiLocChemLabsConfig);
                configs.add(multiLocMicroLabsConfig);
                configs.add(multiLocAPLabsConfig);
            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod("getPatientLabResults")
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(loc)
                            .build()
                );
            }
        }

        SmartLabResult smartLabResult = new SmartLabResult();

        Object[] labObjs = MultiLocationQuery.execute(
                configs, smartLabResult, queryBean);

        List<LabResult> labResults = null;

        if (labObjs != null) {
            labResults = Arrays.asList((LabResult []) labObjs);

            //map lab result concept codes
            //ConceptCodeMapper.getInstance().mapConceptCodes(hddDao, labResults, CodingSystem.LOINC);
            //support non-HDD mappings
            ConceptCodeMapper.getInstance().mapConceptCodes(hddDao, labResults, CodingSystem.LABS);
        }

        if(labResults != null) {
            for (LabResult labResult : labResults) {
                if (labResult.getSourceProtocol() != null
                        && !labResult.getSourceProtocol().equalsIgnoreCase("RPC")
                        && labResult.getTestType() != null
                        && !labResult.getTestType().equalsIgnoreCase("external")) {
                    if (LabLabelCache.getLabel(labResult.getTestType()) != null) {
                        labResult.setTestType(LabLabelCache.getLabel(labResult.getTestType()));
                    }
                } else if (labResult.getSourceProtocol() != null
                        && labResult.getSourceProtocol().equalsIgnoreCase("RPC")
                        && labResult.getSubscript() != null) {

                    if (LabLabelCache.getLabel(labResult.getSubscript()) != null) {
                        labResult.setTestType(LabLabelCache.getLabel(labResult.getSubscript()));
                    }
                }
            }
        }

        return labResults;
    }
}
