/*
 * Janus 4.0 (c)
 * Copyright (c) 2015 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.beans.MhsGenesis;
import gov.va.med.jmeadows.dao.util.BHIERelayBeanFactory;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.PatientMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.smartbeans.patient.SmartMhsGenesis;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Patient MHS Genesis document data access object
 */
public class MhsGenesisDao extends DaoBase
{
    /**
     * Retrieves a list of patient MHS Genesis documents from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient MHS Genesis documents.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<MhsGenesis> getMhsGenesisDocuments(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatMhsGenesisDocuments", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (StringUtils.equalsIgnoreCase(SYSTEM_TYPE_DOD_ADAPTER, loc.getSystemType())) {
                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();

                gov.va.med.bhie.QueryBean bhieQueryBean = BHIERelayBeanFactory.create(queryBean, includedSources, null);

                // MHS Genesis Documents
                MultiLocationQueryConfig patMhsGenesisDocConfig = new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getMhsGenesisDocuments")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("mapMhsGenesisFromBhie")
                        .location(bhieLoc)
                        .build();

                configs.add(patMhsGenesisDocConfig);
            }
        }

        Object[] docObjs = MultiLocationQuery.execute(configs, new SmartMhsGenesis(), queryBean);

        List<MhsGenesis> genesisList = new ArrayList<MhsGenesis>();

        if (docObjs != null) {
            genesisList = Arrays.asList((MhsGenesis[])docObjs);
        }

        return genesisList;
    }
}
