/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.bhie.NoteImage;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.hdd.ConceptCodeMapper;
import gov.va.med.jmeadows.dao.util.BHIERelayBeanFactory;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.PatientMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.smartbeans.patient.SmartConsult;
import gov.va.med.jmeadows.smartbeans.patient.SmartProgressNote;
import gov.va.med.vds.webservice.Consult;
import gov.va.med.vds.webservice.FreeTextReport;
import gov.va.med.vds.webservice.Pce;
import gov.va.med.vds.webservice.ProgressNote;
import gov.va.med.vds.webservice.Site;
import gov.va.med.vhahon.common.Utils;

import java.io.ByteArrayInputStream;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;

/**
 *
 * Patient notes data access object.
 */
public class NotesDao extends DaoBase {

    /**
     * Retrieves a list of patient progress notes from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient progress notes
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<ProgressNote> getPatientProgressNotes(JMeadowsQuery queryBean) throws JMeadowsException {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatProgressNotes", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null) {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -3, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(new java.util.Date())));
        }

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {

            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {
                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                includedSources.add(APP_CONFIG.getBHIECustodianOidAHLTA());
                includedSources.add(APP_CONFIG.getBHIECustodianOidTMDS());

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean, includedSources, null);

                MultiLocationQueryConfig patNotesConfig = new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getPatientNotes")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("getProgressNotesFromBHIE")
                        .location(bhieLoc)
                        .build();

                MultiLocationQueryConfig patEncounterNotesConfig = new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getEncounterNotes")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("getProgressNotesFromBHIE")
                        .location(bhieLoc)
                        .build();

                configs.add(patNotesConfig);
                configs.add(patEncounterNotesConfig);

            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod("getPatientProgressNotes")
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(loc)
                            .build()
                );
            }
        }

        Object[] noteObjs = MultiLocationQuery.execute(
                configs, new SmartProgressNote(), queryBean);

        List<ProgressNote> progressNotes = null;

        if (noteObjs != null) {
            progressNotes = Arrays.asList((ProgressNote []) noteObjs);
            ConceptCodeMapper.getInstance().mapConceptCodes(hddDao, progressNotes, CodingSystem.NOTES);
        }

        return progressNotes;
    }

    /**
     * Retrieves a list of patient discharge summaries from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient discharge summaries.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<ProgressNote> getPatientDischargeSummaries(JMeadowsQuery queryBean) throws JMeadowsException {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);
       
        auditQuery("PatDischargeSummaries", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null) {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -3, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(new java.util.Date())));
        }

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {
                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                includedSources.add(BHIE_CUSTODIAN_OID_SHARE);

                List<String> excludedSources = new ArrayList<String>();
                excludedSources.add(BHIE_CUSTODIAN_OID_VA);

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean,  includedSources, excludedSources);

                //MultiLocationQueryConfig multiLocPatInpatientNotesConfig =


                configs.add(new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getPatientDischargeSummaries")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("getDischargeSummariesFromBHIE")
                        .location(bhieLoc)
                        .build());

                configs.add(new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_bhieRelayServiceHandler)
                        .dataServiceMethod("getGenesisInpatientSummaries")
                        .dataServiceMethodArgs(bhieQueryBean)
                        .smartBeanMappingMethod("mapInpatientSummariesFromGenesis")
                        .location(bhieLoc)
                        .build());

//                configs.add(
//                        new PatientMultiLocQueryCfgBuilder()
//                                .dataServiceHandler(sm_bhieRelayServiceHandler)
//                                //.dataServiceMethod("getPatientConsults")
//                                .dataServiceMethod("getGenesisInpatientSummaries") //workaround until DoD adaptor returns patient consults
//                                .dataServiceMethodArgs(bhieQueryBean)
//                                .smartBeanMappingMethod("getSummariesFromGENESIS")
//                                .location(bhieLoc)
//                                .build());

            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                        .dataServiceHandler(sm_vistaHandler)
                        .dataServiceMethod("getPatientDischargeSummaries")
                        .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                        .location(loc)
                        .build()
                );
            }
        }

        Object[] noteObjs = MultiLocationQuery.execute(
                configs, new SmartProgressNote(), queryBean);

        List<ProgressNote> dischargeSummaries = null;

        if (noteObjs != null) {
            dischargeSummaries = Arrays.asList((ProgressNote []) noteObjs);
            ConceptCodeMapper.getInstance().mapConceptCodes(hddDao, dischargeSummaries, CodingSystem.NOTES);
        }

        return dischargeSummaries;
    }

    /**
     * Retrieves a list of patient consult requests.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient consult requests.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<Consult> getPatientConsultRequests(JMeadowsQuery queryBean) throws JMeadowsException {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatConsults", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null) {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -3, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(new java.util.Date())));
        }

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {
                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                includedSources.add(APP_CONFIG.getBHIECustodianOidAHLTA());

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean, includedSources, null);

                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_bhieRelayServiceHandler)
                            //.dataServiceMethod("getPatientConsults")
                            .dataServiceMethod("getPatientNotes") //workaround until DoD adaptor returns patient consults
                            .dataServiceMethodArgs(bhieQueryBean)
                            .smartBeanMappingMethod("mapConsultsFromBHIE")
                            .location(bhieLoc)
                            .build()
                );

                configs.add(
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                //.dataServiceMethod("getPatientConsults")
                                .dataServiceMethod("getGenesisConsults") //workaround until DoD adaptor returns patient consults
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapConsultsFromGENESIS")
                                .location(bhieLoc)
                                .build()
                );

            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                    new PatientMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod("getPatientConsultRequests")
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(loc)
                            .build()
                );
            }
        }

        Object[] consultObjs = MultiLocationQuery.execute(
                configs, new SmartConsult(), queryBean);

        List<Consult> consults = null;

        if (consultObjs != null) {
            consults = Arrays.asList((Consult []) consultObjs);
        }

        return consults;
    }

    /**
     * Retrieves a patient consult report from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A patient consult report (a list of report fragments).
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<FreeTextReport> getConsultReport(JMeadowsQuery queryBean) throws JMeadowsException {
        List<FreeTextReport> rtc = null;
        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

        auditQuery("ConsultReport", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {

            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                            queryBean.getPatient().getVistaSites())) {

                rtc = sm_vistaHandler.getConsultReport(VistaBeanFactory.create(queryBean));
                break;
            }
        }

        return rtc;
    }

    /**
     * Retrieves a patient progress note from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A patient progress note.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public FreeTextReport getProgressNote(JMeadowsQuery queryBean) throws JMeadowsException {
        
        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

//        auditQuery("ProgressNote", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        FreeTextReport progressNote = null;

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {

            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType()) 
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                            queryBean.getPatient().getVistaSites())) {
                
                progressNote = sm_vistaHandler.getProgressNote(VistaBeanFactory.create(queryBean));
                break;
            } 
            
        }

        return progressNote;
    }

    /**
     * Retrieves a BHIE note image.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * itemId
     *
     * @return Note image
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public NoteImage getBHIENoteImage(JMeadowsQuery queryBean) throws JMeadowsException {

        validateQueryBean(queryBean,
                QueryBeanParams.USER);

//        auditQuery("getNoteImage", queryBean);

        return sm_bhieRelayServiceHandler.getNoteImage(queryBean.getComplexTransaction());
    }

    /**
     * Retrieves a BHIE note attachment.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * itemId
     *
     * @return Note image
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public NoteImage getBHIENoteAttachment(JMeadowsQuery queryBean) throws JMeadowsException {

        validateQueryBean(queryBean,
                QueryBeanParams.USER);

//        auditQuery("getNoteImage", queryBean);

        return sm_bhieRelayServiceHandler.getNoteAttachment(queryBean.getComplexTransaction());
    }

    /**
     * Retries the OID from SHARE which can be either ESSENTRIS or CHCS
     *
     * @param note
     * @return shareOID
     */
    public String getEssentrisOnlyNotes(ProgressNote note) {
        final String XPATH_CDA_SHARE_OID =
                "/ClinicalDocument/id/@root";
        try {
            String cdaXML = note.getNoteText();

            //use UTF-8 char encoding
            byte[] cdaXMLBytes = cdaXML.getBytes(Charset.forName("UTF-8"));

            ByteArrayInputStream inputStream = new ByteArrayInputStream(cdaXMLBytes);
            DocumentBuilderFactory dbf = null;

            dbf = DocumentBuilderFactory.newInstance();
            dbf.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", true);
            dbf.setFeature("http://xml.org/sax/features/external-general-entities", false);
            dbf.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
            dbf.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
            dbf.setValidating(true);
            XPath xpath = XPathFactory.newInstance().newXPath();
            Document xmlDoc = dbf.newDocumentBuilder().parse(inputStream);
            XPathExpression exprShareOID = xpath.compile(XPATH_CDA_SHARE_OID);
            String shareOID = (String)  exprShareOID.evaluate(xmlDoc, XPathConstants.STRING);
            return shareOID;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    /**
     * Retrieves the PCEs for a note (TIU ien) from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A list of PCEs
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<Pce> getPcesForNote(JMeadowsQuery queryBean) throws JMeadowsException {

        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

//        auditQuery("PcesForNote", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        List<Pce> pceList = null;

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {

            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                    queryBean.getPatient().getVistaSites())) {

                pceList = sm_vistaHandler.getPcesForNote(VistaBeanFactory.create(queryBean));
                break;
            }

        }

        return pceList;
    }
}
