/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */


package gov.va.med.jmeadows.dao.patient;

import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.AuditEntry;
import gov.va.med.jmeadows.dao.beans.AuthUserInfo;
import gov.va.med.jmeadows.dao.beans.IehrUserProfile;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.provider.AuthUserDao;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.UserLocation;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.dataservicehandler.ShareImagingServiceHandler;
import gov.va.med.vds.webservice.*;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import web.webservice.jvi.citpo.ha.osd.mil.Mil_osd_ha_citpo_jvi_webservice_beans_ProviderBean;

/**
 * User data access object.
 */
public class UserDao extends DaoBase {
    private static final String PSEUDO_VISTA_FEDERATED_UID = "123456789";

    private static final Logger LOGGER = LoggerFactory.getLogger(UserDao.class);

    /**
     * Authenticates a user against a local system.
     *
     * If successful, a qualified user object will be returned.
     *
     * Otherwise, a user object with an error message contained in the user id will be returned.
     *
     * @param siteCode  Site code associated with access and verify credentials.
     * @param accessCode user's access code.
     * @param verifyCode user's verify code.
     * @param appName Application calling method.
     * @return User profile if successful.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public User login(String siteCode, String accessCode, String verifyCode, String appName) throws JMeadowsException {

        if (StringUtils.isBlank(siteCode)) {
            throw new IllegalArgumentException("siteCode is null or empty.");
        } else if (accessCode == null || accessCode.isEmpty()) {
            throw new IllegalArgumentException("accessCode is null or empty.");
        } else if (verifyCode == null || verifyCode.isEmpty()) {
            throw new IllegalArgumentException("verifyCode is null or empty.");
        }

        User user = null;

        Site hostSite = getSite(siteCode);

        if (hostSite == null) {
            throw new IllegalArgumentException("siteCode (" + siteCode + ") is not supported.");
        }

        if (AGENCY_VA.equalsIgnoreCase(hostSite.getAgency())) {
            user = sm_vistaHandler.login(hostSite, accessCode, verifyCode);
        } else if (AGENCY_DOD.equalsIgnoreCase(hostSite.getAgency())) {
            Endpoint shareEndpoint = null;

            List<Endpoint> endpoints = hostSite.getEndpoints();
            for (Endpoint endpoint : endpoints) {
                if ("SHARE".equalsIgnoreCase(endpoint.getProtocol())) {
                    shareEndpoint = endpoint;
                    break;
                }
            }

            if (shareEndpoint == null) {
                throw new IllegalArgumentException("DoD MTF site " + hostSite.getSiteCode() + " not supported.");
            }

            ShareImagingServiceHandler shareServiceHandler = new ShareImagingServiceHandler(shareEndpoint.getHost());

            shareServiceHandler.setServiceUsername(EncryptUtil.decrypt(shareEndpoint.getUsername()));
            shareServiceHandler.setServicePassword(EncryptUtil.decrypt(shareEndpoint.getPassword()));

            user = new User();

            try {
                Mil_osd_ha_citpo_jvi_webservice_beans_ProviderBean dodProvider =
                        shareServiceHandler.authenticateProvider(accessCode, verifyCode);

                user.setHostSite(hostSite);

                //utilizing pseudo vista user ssn in interim until there is away to retrieve the real thing
                user.setEncryptedFederatedUid(EncryptUtil.encrypt(PSEUDO_VISTA_FEDERATED_UID));
                user.setName(dodProvider.getName());
                user.setUserIen(dodProvider.getUserIen());
                user.setProviderIen(dodProvider.getChcsId());
                user.setAgency(AGENCY_DOD);
            } catch (JMeadowsException e) {
                final String errorMsgTag = "msg=";

                if (e.getMessage().contains(errorMsgTag)) {
                    //catch any login error messages and insert into user ien field
                    int errMsgStartIdx = e.getMessage().indexOf(errorMsgTag) + errorMsgTag.length();
                    int errMsgEndIdx = (e.getMessage().substring(errMsgStartIdx).indexOf(';')) + errMsgStartIdx;
                    if (errMsgStartIdx > -1 && errMsgEndIdx > -1) {
                        user.setUserIen("^" + e.getMessage().substring(errMsgStartIdx, errMsgEndIdx));
                    } else {
                        user.setUserIen("^Authentication Error.");
                    }
                } else if (e.getMessage().contains("AuthenticateProvider")
                               || e.getMessage().contains("AuthenitcateProvider")) { //share exception contains misspelling
                    user.setUserIen("^Invalid access or verify code");
                } else {
                    LOGGER.error(e.getMessage(), e);
                    throw e;
                }
            }
        } else {
            throw new IllegalArgumentException("Site agency not supported: " + hostSite.getAgency());
        }

        if (user != null) {

            String requestingApp = appName;

            AuditDao auditDao = new AuditDao();

            String userIen = user.getUserIen();

            //append providerIen to userIen if one exists
            if (StringUtils.isNotBlank(user.getProviderIen())) {
                userIen += "^" + user.getProviderIen();
            }

            auditDao.setAudit(hostSite.getSiteCode(), "", "", "",
                    userIen, "", "", "", "login", requestingApp, "", "", "");
        }

        return user;
    }
//    /**
//     * Authenticates a user against a local system.
//     *
//     * If successful, a qualified user object will be returned.
//     *
//     * Otherwise, a user object with an error message contained in the user id will be returned.
//     *
//     * @param cardID  smartCard ID.
//     * @param siteCode  Site code associated with access and verify credentials.
//     * @param accessCode user's access code.
//     * @param verifyCode user's verify code.
//     * @param appName Application calling method.
//     * @param ipAddress user's IP Address.
//     * @return User profile if successful.
//     * @throws JMeadowsException if there is an error.
//     * @throws IllegalArgumentException if required parameters are missing or invalid
//     */
//    public User loginEnterprise(String cardID, String siteCode, String accessCode, String verifyCode,
//                                String appName, String ipAddress) throws JMeadowsException {
//        return loginEnterprise(cardID, siteCode, accessCode, verifyCode, appName, ipAddress, "");
//    }

    /**
     * Authenticates a user against a local system.
     *
     * If successful, a qualified user object will be returned.
     *
     * Otherwise, a user object with an error message contained in the user id will be returned.
     *
     * @param cardID  smartCard ID.
     * @param siteCode  Site code associated with access and verify credentials.
     * @param accessCode user's access code.
     * @param verifyCode user's verify code.
     * @param appName Application calling method.
     * @param ipAddress user's IP Address.
     * @param subjectDN user's subjectDN from their smartcard.
     * @return User profile if successful.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public User loginEnterprise(String cardID, String siteCode, String accessCode, String verifyCode,
                                String appName, String ipAddress, String subjectDN, String emailAddress) throws JMeadowsException {

        if (StringUtils.isBlank(siteCode)) {
            throw new IllegalArgumentException("siteCode is null or empty.");
        } else if (accessCode == null || accessCode.isEmpty()) {
            throw new IllegalArgumentException("accessCode is null or empty.");
        } else if (verifyCode == null || verifyCode.isEmpty()) {
            throw new IllegalArgumentException("verifyCode is null or empty.");
        }

        User user = null;

        Site hostSite = getSite(siteCode);

        if (hostSite == null) {
            throw new IllegalArgumentException("siteCode (" + siteCode + ") is not supported.");
        }

        if (AGENCY_VA.equalsIgnoreCase(hostSite.getAgency())) {
            user = sm_vistaHandler.login(hostSite, accessCode, verifyCode);
            user.setCardId(cardID);
            user.setEmail(emailAddress);
            user.setAgency(AGENCY_VA);
            user.setHostSite(hostSite);
        } else if (AGENCY_DOD.equalsIgnoreCase(hostSite.getAgency())) {
            Endpoint shareEndpoint = null;

            List<Endpoint> endpoints = hostSite.getEndpoints();
            for (Endpoint endpoint : endpoints) {
                if ("SHARE".equalsIgnoreCase(endpoint.getProtocol())) {
                    shareEndpoint = endpoint;
                    break;
                }
            }

            if (shareEndpoint == null) {
                throw new IllegalArgumentException("DoD MTF site " + hostSite.getSiteCode() + " not supported.");
            }

            ShareImagingServiceHandler shareServiceHandler = new ShareImagingServiceHandler(shareEndpoint.getHost());

            shareServiceHandler.setServiceUsername(EncryptUtil.decrypt(shareEndpoint.getUsername()));
            shareServiceHandler.setServicePassword(EncryptUtil.decrypt(shareEndpoint.getPassword()));

            user = new User();

            try {
                Mil_osd_ha_citpo_jvi_webservice_beans_ProviderBean dodProvider =
                        shareServiceHandler.authenticateProvider(accessCode, verifyCode);

                user.setCardId(cardID);
                user.setEmail(emailAddress);
                user.setHostSite(hostSite);

                //utilizing pseudo vista user ssn in interim until there is away to retrieve the real thing
                user.setEncryptedFederatedUid(EncryptUtil.encrypt(PSEUDO_VISTA_FEDERATED_UID));
                user.setName(dodProvider.getName());
                user.setUserIen(dodProvider.getUserIen());
                user.setProviderIen(dodProvider.getChcsId());
                user.setAgency(AGENCY_DOD);
            } catch (JMeadowsException e) {
                final String errorMsgTag = "msg=";

                if (e.getMessage().contains(errorMsgTag)) {
                    //catch any login error messages and insert into user ien field
                    int errMsgStartIdx = e.getMessage().indexOf(errorMsgTag) + errorMsgTag.length();
                    int errMsgEndIdx = (e.getMessage().substring(errMsgStartIdx).indexOf(';')) + errMsgStartIdx;
                    if (errMsgStartIdx > -1 && errMsgEndIdx > -1) {
                        user.setUserIen("^" + e.getMessage().substring(errMsgStartIdx, errMsgEndIdx));
                    } else {
                        user.setUserIen("^Authentication Error.");
                    }
                } else if (e.getMessage().contains("AuthenticateProvider")
                        || e.getMessage().contains("AuthenitcateProvider")) {  //share exception contains misspelling
                    user.setUserIen("^Invalid access or verify code");
                } else {
                    LOGGER.error(e.getMessage(), e);
                    throw e;
                }
            }
        } else {
            throw new IllegalArgumentException("Site agency not supported: " + hostSite.getAgency());
        }

        if (user != null) {

            String requestingApp = appName;

            AuditDao auditDao = new AuditDao();
            AuthUserDao aUserDao = new AuthUserDao();

            String userIen = user.getUserIen();

            //append providerIen to userIen if one exists
            if (StringUtils.isNotBlank(user.getProviderIen())) {
                userIen += "^" + user.getProviderIen();
            }

            auditDao.setLoginAudit(cardID, hostSite.getSiteCode(),
                    userIen,  "", "", "", "login", "", requestingApp, "", "", ipAddress, emailAddress);
            if(subjectDN != null && subjectDN.length() > 1) {
                AuthUserInfo aui = new AuthUserInfo();
                aui.setCardId(cardID);
                aui.setEmail(emailAddress);
                if (AGENCY_VA.equalsIgnoreCase(hostSite.getAgency())) {
                    aui.setOrganization("VA");
                } else {
                    aui.setOrganization("DOD");
                }
                aui.setSubjectDN(subjectDN);

                aUserDao.updateUserSubjectDN(aui);
            }
        }

        List<Object> restrictedSiteCodesList = user.getRestrictedSiteCodesList();
        if (CollectionUtils.isNotEmpty(restrictedSiteCodesList)) {
            // only use restrictedSiteCodes in the active sites Map
            Set<Object> activeSiteCodes = (Set) SITES.getSiteMap().keySet();
            restrictedSiteCodesList.retainAll(activeSiteCodes);
        }
        return user;
    }

    /**
     * Authenticates a user against a local system.
     *
     * If successful, a qualified user object will be returned.
     *
     * Otherwise, a user object with an error message contained in the user id will be returned.
     *
     * @param siteCode  Site code associated with access and verify credentials.
     * @param samlToken user's SAML token.
     * @param appName Application calling method.
     * @param ipAddress user's IP Address.
     * @return User profile if successful.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public User loginSaml(String siteCode, String samlToken,
                                String appName, String ipAddress, String emailAddress) throws JMeadowsException {

        if (StringUtils.isBlank(siteCode)) {
            throw new IllegalArgumentException("siteCode is null or empty.");
        } else if (StringUtils.isEmpty(samlToken)) {
            throw new IllegalArgumentException("samlToken is null or empty.");
        }

        User user = null;

        Site hostSite = getSite(siteCode);

        if (hostSite == null) {
            throw new IllegalArgumentException("siteCode (" + siteCode + ") is not supported.");
        }

        if (AGENCY_VA.equalsIgnoreCase(hostSite.getAgency())) {
            //TODO uncomment when VDS has the loginSaml webservice method
            user = sm_vistaHandler.loginSaml(hostSite, samlToken);
            user.setEmail(emailAddress);
            user.setAgency(AGENCY_VA);
            user.setHostSite(hostSite);
        } else {
            throw new IllegalArgumentException("Site agency not supported: " + hostSite.getAgency());
        }

        if (user != null) {

            String requestingApp = appName;

            AuditDao auditDao = new AuditDao();
            AuthUserDao aUserDao = new AuthUserDao();

            String userIen = user.getUserIen();

            //append providerIen to userIen if one exists
            if (StringUtils.isNotBlank(user.getProviderIen())) {
                userIen += "^" + user.getProviderIen();
            }

            auditDao.setLoginAudit("", hostSite.getSiteCode(),
                    userIen,  "", "", "", "loginSaml", "", requestingApp, "", "", ipAddress, emailAddress);
        }

        List<Object> restrictedSiteCodesList = user.getRestrictedSiteCodesList();
        if (CollectionUtils.isNotEmpty(restrictedSiteCodesList)) {
            // only use restrictedSiteCodes in the active sites Map
            Set<Object> activeSiteCodes = (Set) SITES.getSiteMap().keySet();
            restrictedSiteCodesList.retainAll(activeSiteCodes);
        }
        return user;
    }

    /**
     * Retrieve user profile that is associated with the user's user id.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @return User with duty phone, gender, pager, permission class, flag, and sig filled in.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public User getHUIProviderProfile(JMeadowsQuery queryBean) throws JMeadowsException {

        validateQueryBean(queryBean, QueryBeanParams.USER);

        User user = new User();

        auditQuery("HuiProviderProfile", queryBean);

        UserLocation loc = null;

        loc = LocationHelper.getUserLocation(queryBean.getUser());

        //get vista data
        if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
            user = sm_vistaHandler.getProviderProfile(
                    queryBean.getUser().getHostSite(),
                    queryBean.getUser().getUserIen(),
                    queryBean.getUser().getNPI());
        } else {
            throw new IllegalArgumentException("Unsupported User agency: " + user.getHostSite().getAgency());
        }

        return user;
    }

    /**
     * Retrieves a list of user alerts associated with the user's user id.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @return A list of user alerts.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<Alert> getUserAlerts(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean, QueryBeanParams.USER);

        List<Alert> alerts = null;

        UserLocation loc = LocationHelper.getUserLocation(queryBean.getUser());

        if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
            alerts = sm_vistaHandler.getUserAlerts(VistaBeanFactory.create(queryBean));
        }

        if (alerts != null && queryBean.getMax() != null && queryBean.getMax() < alerts.size())
        {
            alerts = alerts.subList(0, queryBean.getMax());
        }

        return alerts;
    }

    /**
     * Sets user's Janus GUI configuration string.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @param cfg User application configuration string.
     * @return True if configuration successfully set.
     * @throws JMeadowsException If there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public boolean setJanusGUIConfig(JMeadowsQuery queryBean, String cfg)throws JMeadowsException  {
        Connection sqlConn = null;
        CallableStatement cstmt = null;

        boolean bRet = false;

        validateQueryBean(queryBean, QueryBeanParams.USER);

        if (cfg == null || cfg.isEmpty()) {
            throw new IllegalArgumentException("Config string is null.");
        }

        try {
            auditQuery("setUserCfg", queryBean);

            sqlConn = sm_dbConnectionPool.getConnection();
            sqlConn.setAutoCommit(false);
            cstmt = sqlConn.prepareCall("{call setUserCfg(?,?)}");

            String profileId = queryBean.getUser().getUserId();

//            if (StringUtils.isBlank(profileId))
//            {
//                profileId = queryBean.getUser().getUserIen();
//            }
            cstmt.setString(1, profileId);
            cstmt.setString(2, cfg);
            cstmt.execute();
            sqlConn.commit();

            bRet = true;

        } catch (SQLException e) {

            LOGGER.error("User Configuration Save Exception: " + e.getMessage());

            try {
                if (sqlConn != null) {
                    sqlConn.rollback();
                }
            } catch (SQLException e1) {
                LOGGER.error("rollback Exception: " + e1.getMessage());
            }

            throw new JMeadowsException(e);

        } finally {
            try {
                if (sqlConn != null) {
                    sqlConn.setAutoCommit(true);
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return bRet;
    }

    /**
     * Get user's Janus GUI configuration string
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @return User application configuration string.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public String getJanusGUIConfig(JMeadowsQuery queryBean) throws JMeadowsException {
        String rtc = "";
        Connection sqlConn = null;

        validateQueryBean(queryBean, QueryBeanParams.USER);

        String cfg = null;
        CallableStatement  cstmt = null;
        ResultSet rset = null;

        //return empty string is user id is blank
        if (!StringUtils.isBlank(queryBean.getUser().getUserId())) {
            try {
                auditQuery("UserConfig", queryBean);
    
                sqlConn = sm_dbConnectionPool.getConnection();
                cstmt = sqlConn.prepareCall("{call getUserCfg(?)}");
                cstmt.setString(1, queryBean.getUser().getUserId());
    
                rset = cstmt.executeQuery();
                while (rset.next()) {
                    cfg = rset.getString("cfg");
                }
    
            } catch (SQLException e) {
                LOGGER.error("get User Config exception: " + e.getMessage());
    
                throw new JMeadowsException(e);
            } finally {
                try {
                    if (rset != null) {
                        rset.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }
    
                try {
                    if (cstmt != null) {
                        cstmt.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }
    
                try {
                    if (sqlConn != null) {
                        sqlConn.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }
            }
            rtc = cfg;
        }
        return rtc;
    }

    /**
     * Sets user application (iHER) flags string.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @param flags User application configuration string.
     * @return True if configuration successfully set.
     * @throws JMeadowsException If there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public boolean setUserFlags(JMeadowsQuery queryBean, String flags) throws JMeadowsException  {
        Connection sqlConn = null;
        CallableStatement cstmt = null;

        boolean bRet = false;

        validateQueryBean(queryBean, QueryBeanParams.USER);

        if (flags.isEmpty()) {
            throw new IllegalArgumentException("Flags string is null.");
        }

        try {
            auditQuery("setUserFlags", queryBean);

            sqlConn = sm_dbConnectionPool.getConnection();
            sqlConn.setAutoCommit(false);
            cstmt = sqlConn.prepareCall("{call setUserFlags(?,?,?)}");

            cstmt.setString(1, queryBean.getUser().getHostSite().getSiteCode());

            String profileId = queryBean.getUser().getUserId();

            if (StringUtils.isBlank(profileId)) {
                profileId = queryBean.getUser().getUserIen();
            }

            cstmt.setString(2, profileId);
            cstmt.setString(3, flags);
            cstmt.execute();
            sqlConn.commit();

            bRet = true;

        } catch (SQLException e) { // Trap Sql Exceptions
            LOGGER.error("User Flags Save Exception: " + e.getMessage());

            try {
                if (sqlConn != null) {
                    sqlConn.rollback();
                }
            } catch (SQLException e1) {
                LOGGER.error("rollback Exception: " + e1.getMessage());
            }

            throw new JMeadowsException(e);

        } finally {
            try {
                if (sqlConn != null) {
                    sqlConn.setAutoCommit(true);
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return bRet;
    }

    /**
     * Get user application (iEHR) flags string
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @return User application configuration string.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public String getUserFlags(JMeadowsQuery queryBean) throws JMeadowsException {
        Connection sqlConn = null;

        validateQueryBean(queryBean, QueryBeanParams.USER);

        String flags = null;
        CallableStatement cstmt = null;
        ResultSet rset = null;

        try {
            auditQuery("getUserFlags", queryBean);

            sqlConn = sm_dbConnectionPool.getConnection();
            cstmt = sqlConn.prepareCall("{call getUserFlags(?)}");

            cstmt.setString(1, queryBean.getUser().getUserId());

            rset = cstmt.executeQuery();
            while (rset.next()) {
                flags = rset.getString("flags");
            }

        } catch (SQLException e) {
            LOGGER.error("get User Flags exception: " + e.getMessage());

            throw new JMeadowsException(e);
        } finally {
            try {
                if (rset != null) {
                    rset.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return flags;
    }

    public boolean setIehrUserProfile(IehrUserProfile iehrUserProfile) throws JMeadowsException
    {
        Connection sqlConn = null;
        CallableStatement cstmt = null;

        String locID = iehrUserProfile.getLocID();
        String userID = iehrUserProfile.getDbUserID();
        String cfg = iehrUserProfile.getCfg();
        String flags = iehrUserProfile.getFlags();
        String scID = iehrUserProfile.getSmartCardID();
        String scAgency = iehrUserProfile.getSmartCardAgency();
        String emailAddress = iehrUserProfile.getEmailAddress();
        String subjectDN = iehrUserProfile.getSubjectDN();

        boolean bRet = false;

        if (locID == null) {
            throw new IllegalArgumentException("locID string is null.");
        }
        if (userID == null) {
            throw new IllegalArgumentException("userID string is null.");
        }
        if (cfg == null) {
            throw new IllegalArgumentException("cfg string is null.");
        }
        if (flags == null) {
            throw new IllegalArgumentException("flags string is null.");
        }
        if (scID == null) {
            throw new IllegalArgumentException("smartCardID string is null.");
        }
        if (scAgency == null || scAgency.isEmpty()) {
            throw new IllegalArgumentException("smartCardAgency string is null.");
        }
//        if (emailAddress == null) {
//            throw new IllegalArgumentException("emailAddress string is null.");
//        }
//        if (subjectDN == null) {
//            throw new IllegalArgumentException("subjectDN string is null.");
//        }

        try {
            sqlConn = sm_dbConnectionPool.getConnection();
            sqlConn.setAutoCommit(false);
            cstmt = sqlConn.prepareCall("{call setIehrUserProfile(?,?,?,?,?,?,?,?)}");

            cstmt.setString(1, locID);
            cstmt.setString(2, userID);
            cstmt.setString(3, cfg);
            cstmt.setString(4, flags);
            cstmt.setString(5, scID);
            cstmt.setString(6, scAgency);
            cstmt.setString(7, emailAddress);
            cstmt.setString(8, subjectDN);

            cstmt.execute();
            sqlConn.commit();

            bRet = true;

        } catch (SQLException e) { // Trap Sql Exceptions
            LOGGER.error("JLV User Profile Save Exception: " + e.getMessage());

            try {
                if (sqlConn != null) {
                    sqlConn.rollback();
                }
            } catch (SQLException e2) {
                LOGGER.error("rollback Exception: " + e2.getMessage());
            }

            throw new JMeadowsException(e);

        } finally {
            try {
                if (sqlConn != null) {
                    sqlConn.setAutoCommit(true);
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return bRet;
    }

    public IehrUserProfile getIehrUserProfile(String smartCardID, String smartCardAgency, String emailAddress) throws JMeadowsException
    {
        Connection sqlConn = null;

        CallableStatement  cstmt = null;
        ResultSet rset         = null;
        IehrUserProfile userProfile = new IehrUserProfile();

        try {
            sqlConn = sm_dbConnectionPool.getConnection();
            cstmt = sqlConn.prepareCall("{call getIehrUserProfile(?,?,?)}");
            cstmt.setString(1, smartCardID);
            cstmt.setString(2, smartCardAgency);
            cstmt.setString(3, emailAddress);

            rset = cstmt.executeQuery();
            while (rset.next()) {
                userProfile.setLocID(rset.getString("locID"));
                userProfile.setDbUserID(rset.getString("userID"));
                userProfile.setCfg(rset.getString("cfg"));
                userProfile.setFlags(rset.getString("flags"));
                userProfile.setSmartCardID(rset.getString("smartCardID"));
                userProfile.setSmartCardAgency(rset.getString("smartCardAgency"));
                userProfile.setEmailAddress(rset.getString("emailAddress"));
            }

        } catch (SQLException e) { // Trap Sql Exceptions
            LOGGER.error("get iEHR User Profile exception: " + e.getMessage());

            throw new JMeadowsException(e);
        } finally {
            try {
                if (rset != null) {
                    rset.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return userProfile;
    }

    /**
     * Get user's login history
     *
     * @param cardID user's smartCardID.
     *
     * @return User application configuration string.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public  List<AuditEntry>  getLoginInfo(String cardID, String emailAddress) throws JMeadowsException {
        Connection sqlConn = null;

        List<AuditEntry> audits = new ArrayList<AuditEntry>();
        CallableStatement cstmt = null;
        ResultSet rset = null;

        try {
            sqlConn = sm_dbConnectionPool.getConnection();
            cstmt = sqlConn.prepareCall("{call getLoginInfo(?,?)}");

            cstmt.setString(1, cardID);
            cstmt.setString(2, emailAddress);

            rset = cstmt.executeQuery();
            while (rset.next()) {
                AuditEntry audit = new AuditEntry();
                audit.setAuditID(rset.getString("auditID"));

                if (rset.getDate("entryDate") != null) {
                    audit.setEntryDate(rset.getTimestamp("entryDate"));
                }

                if (rset.getDate("startDate") != null) {
                    audit.setStartDate(rset.getTimestamp("startDate"));
                }

                if (rset.getDate("endDate") != null) {
                    audit.setEndDate(rset.getTimestamp("endDate"));
                }

                audit.setSystemID(rset.getString("systemID"));
                audit.setUserID(rset.getString("userID"));
                audit.setUserNPI(rset.getString("userNPI"));
                audit.setUserName(rset.getString("userName"));
                audit.setPatID(rset.getString("patID"));
                audit.setCategory(rset.getString("category"));
                audit.setQueryType(rset.getString("queryType"));
                audit.setCardID(rset.getString("cardID"));
                audit.setIpAddress(rset.getString("ipAddress"));

                audits.add(audit);
            }

        } catch (SQLException e) {
            LOGGER.error("get User Flags exception: " + e.getMessage());

            throw new JMeadowsException(e);
        } finally {
            try {
                if (rset != null) {
                    rset.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return audits;
    }

    /**
     * Add user's recently viewed patient
     * @param userId User's database ID in USERS table.
     * @param patient Recently selected patient.
     * @return boolean True if success adding record to database.
     * @throws JMeadowsException If there is an error.
     */
    public boolean addRecentlyViewedPatient(String userId, Patient patient) throws JMeadowsException
    {
        boolean success = false;
        Connection sqlConn = null;
        CallableStatement cstmt = null;

        String name = "";
        String edipi = "";
        String ssn = "";
        String fmp = "";
        String gender = "";
        String dob = "";
        String sensitive = "false";
        String icn = "";

        if (patient != null) {
            if (patient.getName() != null) {
                name = patient.getName();
            }
            if (patient.getEDIPI() != null) {
                edipi = patient.getEDIPI();
            }
            if (patient.getSSN() != null) {
                ssn = patient.getSSN();
            }
            if (patient.getFMP() != null) {
                fmp = patient.getFMP();
            }
            if (patient.getGender() != null) {
                gender = patient.getGender();
            }
            if (patient.getDob() != null) {
                dob = patient.getDob();
            }
            if (patient.isSensitive()) {
                sensitive = Boolean.toString(patient.isSensitive());
            }
            if (patient.getICN() != null) {
                icn = patient.getICN();
            }
        }

        try {
            sqlConn = sm_dbConnectionPoolEncrypted.getConnection();

            cstmt = sqlConn.prepareCall("{call addRecentlyViewedPatient(?,?,?,?,?,?,?,?,?)}");

            // userId
            cstmt.setString(1, userId);
            // Name
            cstmt.setString(2, name);
            // EDIPI
            cstmt.setString(3, edipi);
            // SSN
            cstmt.setString(4, ssn);
            // Sponsor SSN
            cstmt.setString(5, fmp);
            // Gender
            cstmt.setString(6, gender);
            // DoB
            cstmt.setString(7, dob);
            // Sensitive
            cstmt.setString(8, sensitive);
            // ICN
            cstmt.setString(9, icn);

            cstmt.executeUpdate();

            success = true;
        } catch (SQLException e) {
            LOGGER.error("Add Recently Viewed Patients Exception: " + e.getMessage());
            throw new JMeadowsException(e);
        } finally {
            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return success;
    }


    /**
     * Get user's recently viewed patients
     * @param userId User's database ID in USERS table.
     * @return List of user's recently viewed patients data.
     * @throws JMeadowsException if there is an error.
     */
    public List<Patient> getRecentlyViewedPatients(String userId) throws JMeadowsException
    {
        List<Patient> patients = new ArrayList<Patient>();
        Patient patient;

        Connection sqlConn = null;
        CallableStatement cstmt = null;
        ResultSet rset = null;

        try {
            sqlConn = sm_dbConnectionPoolEncrypted.getConnection();

            cstmt = sqlConn.prepareCall("{call getRecentlyViewedPatients(?)}");
            cstmt.setString(1, userId);

            rset = cstmt.executeQuery();

            while (rset.next()) {
                patient = new Patient();

                // Name
                patient.setName(rset.getString("name"));
                // EDIPI
                patient.setEDIPI(rset.getString("edipi"));
                // SSN
                patient.setSSN(rset.getString("ssn"));
                // Sponsor SSN
                patient.setFMP(rset.getString("sponsorSsn"));
                // Gender
                patient.setGender(rset.getString("gender"));
                // DoB
                patient.setDob(rset.getString("dob"));
                // Sensitive
                patient.setSensitive(Boolean.parseBoolean(rset.getString("sensitive")));
                // ICN
                patient.setICN(rset.getString("icn"));

                patients.add(patient);
            }
        } catch (SQLException e) {
            LOGGER.error("Get Recently Viewed Patients Exception: " + e.getMessage());
            throw new JMeadowsException(e);
        } finally {
            try {
                if (rset != null) {
                    rset.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return patients;
    }

    public void createAudit(JMeadowsQuery queryBean) throws JMeadowsException{
        try {
            auditQuery("createAudit", queryBean);
        } catch (JMeadowsException e) {
            e.printStackTrace();
        }

    }
}
