/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.bhie.BHIERelay;
import gov.va.med.bhie.BHIERelayService;
import gov.va.med.bhie.NoteImage;
import gov.va.med.bhie.QueryBean;
import gov.va.med.bhie.SnareUser;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.jmeadows.common.JMeadowsException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.MalformedURLException;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.MTOMFeature;

/**
 * BHIEServiceHandler
 */
public class BHIERelayServiceHandler extends DataServiceHandler {
    private BHIERelayService bhieRelayService;
    private static final Logger LOGGER = LoggerFactory.getLogger(BHIERelayServiceHandler.class);

    public BHIERelayServiceHandler(String url) {
        super();

        serviceURL = url;
        serviceName = "BHIERelayService";
        serviceNamespace = "http://webservice.bhie.URL       /";
        qServiceName = new QName(serviceNamespace, serviceName);

        serviceUsername = APP_CONFIG.getBHIERelayServiceUsername();
        servicePassword = APP_CONFIG.getBHIERelayServicePassword();


    }

    private BHIERelay getBHIERelayPort() throws MalformedURLException {
        return getBHIERelayPort(false);
    }

    private BHIERelay getBHIERelayPort(boolean includeMTOMFeature) throws MalformedURLException
    {
        if (bhieRelayService == null) {
            createService();
        }

        BHIERelay port = null;

        if (includeMTOMFeature) {
            port = bhieRelayService.getBHIERelayPort(new MTOMFeature());
        } else {
            port = bhieRelayService.getBHIERelayPort();
        }

        //set port endpoint address
        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        prov.getRequestContext().put(BindingProvider.USERNAME_PROPERTY, serviceUsername);
        prov.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, servicePassword);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        return port;
    }

    private synchronized void createService() {
        if (bhieRelayService == null) {
            try {
                bhieRelayService = new BHIERelayService();
            } catch (Exception e) {
                logError(e);
                throw new RuntimeException(e);
            }
        }
    }

    public V4ResultCollection getPatientAdmissions(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientAdmissions(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientAllergies(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientAllergies(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientImmunizations(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientImmunizations(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientProblemList(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientProblemList(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientRadiologyReports(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientRadiologyReports(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientVitals(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientVitals(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientAppointments(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientAppointments(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientEncounters(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientEncounters(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getEncounterNotes(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getEncounterNotes(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientNotes(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientNotes(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public NoteImage getNoteImage(String url) throws JMeadowsException {
        try {
            return getBHIERelayPort(true).getNoteImageFromUrl(url);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public NoteImage getNoteAttachment(String url) throws JMeadowsException {
        try {
            return getBHIERelayPort(true).getNoteAttachmentFromUrl(url);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientDischargeSummaries(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientDischargeSummaries(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

//    public V4ResultCollection getInpatientNotes(QueryBean queryBean) throws JMeadowsException {
//        try {
//            return getBHIERelayPort().getInpatientNotes(queryBean);
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//    }

    public V4ResultCollection getPatientConsults(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientConsults(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    //getPatientAlerts
    public V4ResultCollection getPatientManagement(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientManagement(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientMedications(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientMedications(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientLabChemistries(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientLabChemistries(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientLabMicrobiologies(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientLabMicrobiologies(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientLabAnatomicPathologies(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientLabAnatomicPathologies(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientDemographics(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientDemographics(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientSocialHistory(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientSocialHistory(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientOtherHistory(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientOtherHistory(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientFamilyHistory(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientFamilyHistory(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientOrders(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientOrders(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientInsurances(QueryBean queryBean) throws JMeadowsException
    {
        try {
            return getBHIERelayPort().getPatientInsurances(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getPatientQuestionnaires(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getPatientQuestionnaires(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getDeploymentForms(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getDeploymentForms(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getOutpatientProcedures(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getOutpatientProcedures(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientAllergies(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientAllergies(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientImmunizations(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientImmunizations(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientProblems(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientProblemList(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientProcedures(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientProcedures(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientInsurances(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientInsurances(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientVitals(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientVitals(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientDemographics(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientDemographics(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getVLERPatientCCDA(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getVLERPatientCCDA(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getHAIMSDocuments(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getHAIMSDocuments(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getMhsGenesisDocuments(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getMhsGenesisDocuments(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getGenesisEncounters(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getGenesisEncounters(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getGenesisConsults(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getGenesisConsults(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getGenesisInpatientSummaries(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getGenesisInpatientSummaries(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getGenesisProcedures(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getGenesisProcedures(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public V4ResultCollection getGenesisDocuments(QueryBean queryBean) throws JMeadowsException {
        try {
            return getBHIERelayPort().getGenesisDocuments(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public SnareUser getSnareworksUser(String certificate, String user, String credential) throws JMeadowsException {
        try {
            return getBHIERelayPort().getSnareworksUser(certificate, user, credential);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    private void logError(Exception e){
        LOGGER.error("BHIERelayService ERROR: " + e.getMessage(), e);
    }
}
