/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.*;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.foundations.utilities.Messages;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.vds.webservice.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;


/**
 * @author mgoto
 */
public class SmartFlag extends SmartBean implements Comparable {

    public static final int SITE = 1;

    /**
     * SSN OID.
     */
    public static final String SSN_OID = "2.16.840.1.113883.3.42.126.100001.2";

    /**
     * EDIPN OID.
     */
    public static final String EDI_OID = "2.16.840.1.113883.3.42.10001.100001.12";

    /**
     * Repository OID.
     */
    private static final String REP_OID = "2.16.840.1.113883.3.42.126.100001.13";

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartFlag.class);

    public Flag m_flag;


    public SmartFlag() {
        super();
        m_flag = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartFlag(Flag flag, String sort) {
        super();
        m_flag = flag;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartFlag) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_flag);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }


    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartFlag[] temp = new SmartFlag[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartFlag newAll = new SmartFlag((Flag) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartFlag newAll = new SmartFlag((Flag) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Flag[] ret = new Flag[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_flag;
            }

            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartFlag[] temp = new SmartFlag[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartFlag newAll = new SmartFlag((Flag) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Flag[] ret = new Flag[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_flag;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Flag[] mapPatientFlagsFromBHIE(Object resultCollection) throws JMeadowsException {
        V4ResultCollection patFlagsResultCollection = (V4ResultCollection) resultCollection;
        List<Flag> vistaBeans = new ArrayList<Flag>();

        //If data record is empty, inform user.
        if (patFlagsResultCollection.getDataRecordList() == null || patFlagsResultCollection.getDataRecordList().getPatientManagement() == null) {
            PatientLocation bhieLoc = new PatientLocation();
            bhieLoc.setSystemType(SYSTEM_TYPE_DOD_ADAPTER);

            vistaBeans.addAll(Arrays.asList(
                    (Flag[]) generateBHIEPatFlagErrorBean(bhieLoc)));
        } else {
            try {
                for (PatientManagementV4 patMan : patFlagsResultCollection.getDataRecordList().getPatientManagement()) {
                    for (FlagV4 bhieFlag : patMan.getFlags()) {
                        Flag flag = new Flag();
                        flag.setName(bhieFlag.getName());
                        boolean excludeFlag = false;

                        if (bhieFlag.getValues() != null) {
                            for (SpdcBeanV4 value : bhieFlag.getValues()) {
                                if (value != null && value.isPrimary() && value.getDisplay() != null) {
                                    if (value.getDisplay().equalsIgnoreCase("No")
                                            || value.getDisplay().equalsIgnoreCase("N")
                                            || value.getDisplay().equalsIgnoreCase("false")
                                            || value.getDisplay().equalsIgnoreCase("F")
                                            || value.getDisplay().equalsIgnoreCase("0")
                                            || (value.getDisplay().toLowerCase().indexOf("false") > -1)) {
                                        excludeFlag = true;
                                    }
                                }
                            }
                        }

                        if(!excludeFlag) {
                            flag.setShortName(bhieFlag.getShortName());
                            flag.setComment(bhieFlag.getComment());
                            String repositoryId = "";
                            String repositoryName = "";
                            String facilityName = "";
                            String eventId = "";
                            if (patMan.getRepository() != null && patMan.getRepository().getIds() != null && !patMan.getRepository().getIds().isEmpty()) {
                                repositoryId = patMan.getRepository().getIds().get(0).getId();
                            }
                            if (patMan.getRepository() != null) {
                                repositoryName = patMan.getRepository().getName();
                            }
                            if (patMan.getFacility() != null) {
                                facilityName = patMan.getFacility().getName();
                            }
                            if (patMan.getRecordId() != null) {
                                eventId = patMan.getRecordId().getId();
                            }
                            if (repositoryId == null || repositoryId.length() == 0) {
                                repositoryId = REP_OID;
                            }
                            if (repositoryName == null || repositoryName.length() == 0) {
                                repositoryName = "ahlta";
                            }

                            addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                                    facilityName, eventId, flag);

                            vistaBeans.add(flag);
                        }
                    }


                }
            } catch (Exception ex) {
                LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
                throw new JMeadowsException(ex);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(patFlagsResultCollection.getStatusListv4(),
                patFlagsResultCollection.getErrorListv4(), patFlagsResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Flag[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Flag[vistaBeans.size()]);
    }

    public Object[] generateBHIEPatFlagErrorBean(Location location) {

        Flag flag = new Flag();
        flag.setName(Messages.get("demographics.dod.results.empty").toUpperCase());
        flag.setContent(Messages.get("demographics.dod.results.helpdesk").toUpperCase());
        return new Flag[]{flag};
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Flag flag = new Flag();
        flag.setName(errorMsg);
        flag.setId("idHere");
        flag.setSite(getConnectionUnavailableSite(location));
        return new Flag[]{flag};
    }

    public int compareTo(Object obj) {
        SmartFlag tmp = (SmartFlag) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_flag.getSite() != null && tmp.m_flag.getSite() != null) {
            retVal = compareString(this.m_flag.getSite().getSiteCode(), tmp.m_flag.getSite().getSiteCode());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    //TODO: Why is sort by always SITE?
    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = SITE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }
    }
}
