/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.webservice;


import gov.va.med.bhie.NoteImage;
import gov.va.med.bhie.SnareUser;
import gov.va.med.foundations.utilities.ARMLog;
import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.common.LabLabelCache;
import gov.va.med.jmeadows.common.LanguageCache;
import gov.va.med.jmeadows.dao.beans.*;
import gov.va.med.jmeadows.dao.hdd.CodeMappingsCache;
import gov.va.med.jmeadows.dao.hdd.VLERSiteCache;
import gov.va.med.jmeadows.dao.patient.*;
import gov.va.med.jmeadows.dao.provider.AuthUserDao;
import gov.va.med.jmeadows.dao.provider.ProviderAdmissionsDao;
import gov.va.med.jmeadows.dao.provider.ProviderAppointmentsDao;
import gov.va.med.jmeadows.dao.provider.ProviderConsultsDao;
import gov.va.med.jmeadows.dao.provider.ProviderLabsDao;
import gov.va.med.jmeadows.dao.provider.ProviderNotesDao;
import gov.va.med.jmeadows.dao.provider.ProviderOrdersDao;
import gov.va.med.jmeadows.heartbeat.ServiceErrors;
import gov.va.med.jmeadows.site.Sites;
import gov.va.med.vds.webservice.*;

import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Locale;

import javax.annotation.Resource;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.soap.MTOM;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * JMeadows Data Service
 */
@MTOM
@WebService
public class JMeadowsData {
    private static final Logger LOGGER = LoggerFactory.getLogger(JMeadowsData.class);
    private static final AppConfig APPCONFIG = AppConfig.getInstance();

    /**
     * Web context.
     */
    @Resource
    private WebServiceContext wsContext;


    public JMeadowsData() throws JMeadowsException {
        if (APPCONFIG.useTermMappingCache()) {
            LOGGER.debug("load terminology mapping data when JMeadows starts.");
            CodeMappingsCache.loadDataFromDatabase();
            LOGGER.debug("finish loading data from database. Total " + CodeMappingsCache.getSize());
        }

        if (APPCONFIG.loadVLERSiteCache()) {
            VLERSiteCache.loadDataFromDatabase();
        }

        LabLabelCache.loadDataFromConfig();

        LanguageCache.loadDataFromConfig();
    }

//    /**
//     * Authenticates a user against a local system.
//     *
//     * If successful, a qualified user object will be returned.
//     *
//     * Otherwise, a user object with an error message contained in the user id will be returned.
//     *
//     * @param cardID  SmartCard ID
//     * @param siteCode  Site code associated with access and verify credentials.
//     * @param accessCode user's access code.
//     * @param verifyCode user's verify code.
//     * @param requestingApp Application calling method.
//     * @param ipAddress user's IP address
//     * @return ResponseLogin login response object.
//     * @throws JMeadowsException if a serious error occurs
//     * @throws IllegalArgumentException if required parameters are missing or invalid
//     */
//    @WebMethod(operationName = "loginEnterprise")
//    public User loginEnterprise(@WebParam(name = "cardID")
//                                String cardID, @WebParam(name = "siteCode")
//                                String siteCode, @WebParam(name = "accessCode")
//                                String accessCode, @WebParam(name = "verifyCode")
//                                String verifyCode, @WebParam(name = "requestingApp")
//                                String requestingApp, @WebParam(name = "ipAddress")
//                                String ipAddress) throws JMeadowsException
//    {
//        try
//        {
//            UserDao userDao = new UserDao();
//            return userDao.loginEnterprise(cardID, siteCode, accessCode, verifyCode, requestingApp, ipAddress);
//        }
//        catch (JMeadowsException ex)
//        {
//            logger.error(ex.getMessage(), ex);
//            throw ex;
//        }
//    }

    /**
     * Retrieves authorized user information
     * @param authUserInfoQuery Requires userId and organization
     * @return Authorized user info result
     * @throws JMeadowsException If there is an exception
     */
    @WebMethod(operationName = "updateUserSubjectDN")
    public boolean updateUserSubjectDN(
            @WebParam(name = "authUserInfoQuery") AuthUserInfo authUserInfoQuery) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AuthUserDao authUserDao = new AuthUserDao();
            boolean rtc = authUserDao.updateUserSubjectDN(authUserInfoQuery);
            if (rtc) {
                logResponse(null, begin, "", 1);
            } else {
                logResponse(null, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException e) {
            logResponse(null, begin, e.getMessage(), 0);
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    /**
     * Authenticates a user against a local system.
     *
     * If successful, a qualified user object will be returned.
     *
     * Otherwise, a user object with an error message contained in the user id will be returned.
     *
     * @param siteCode  Site code associated with access and verify credentials.
     * @param accessCode user's access code.
     * @param verifyCode user's verify code.
     * @param requestingApp Application calling method.
     * @return User profile if successful.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "login")
    public User login(@WebParam(name = "siteCode")
                      String siteCode, @WebParam(name = "accessCode")
                      String accessCode, @WebParam(name = "verifyCode")
                      String verifyCode, @WebParam(name = "requestingApp")
                      String requestingApp) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            User rtc = null;
            String container = "";
            Site site = Sites.getInstance().getSite(siteCode);
            if (site != null) {
                container = site.getContainer();
            } else {
                site = new Site();
            }
            if (APPCONFIG.useSnareworks() && !"VA".equals(site.getAgency())) {
                LOGGER.debug("login container retrieved = " + container);
                if (container == null || container.length() == 0) {
                    container = "unknown";
                    LOGGER.error("EMPTY Container passed in.");
                }
                AuthUserDao authUserDao = new AuthUserDao();
                SnareUser suser = authUserDao.getSnareworksUser(null, container + "@" + accessCode, verifyCode);
                if (suser != null) {
                    String snaremsg = suser.getSnareMessage();
                    rtc = new User();
                    if (snaremsg == null) {
                        snaremsg = "";
                    }
                    if ("No Snare Servers available".equalsIgnoreCase(snaremsg)) {
                        throw new JMeadowsException("Snareworks is down.");
                    } else if (snaremsg.toLowerCase(Locale.getDefault()).indexOf("login failed") >= 0) {
                        rtc.setUserIen("^Not a valid User ID /  Password pair.");
                    } else if (!suser.isValid()) {
                        rtc.setUserIen("^You are not an authorized user.");
                    } else if (snaremsg.length() > 0) {
                        throw new JMeadowsException(snaremsg);
                    } else {
                        rtc.setAgency("DOD");
                        rtc.setDmisId(site.getDmisId());
                        rtc.setHostSite(site);
                        rtc.setUserId(suser.getNcid());
                        rtc.setEncryptedAvCode("");
                        rtc.setUserIen(suser.getNcid());
                        rtc.setName(suser.getCn());
                        rtc.setEncryptedFederatedUid(EncryptUtil.encrypt("123456789"));
                    }
                }
            } else {
                UserDao userDao = new UserDao();
                rtc = userDao.login(siteCode, accessCode, verifyCode, requestingApp);
                if (rtc != null) {
                    logResponse(null, begin, "", 1);
                } else {
                    logResponse(null, begin, "", 0);
                }
            }
            return rtc;
        } catch (JMeadowsException ex) {
            String exMsg = ex.toString();

            //do not log access & verify failures
            if (exMsg != null && !exMsg.contains("^")) {
                logResponse(null, begin, ex.getMessage(), 0);
                logError(ex);
            }

            throw ex;
        }
    }

    /**
     * Authenticates a user against a local system.
     *
     * If successful, a qualified user object will be returned.
     *
     * Otherwise, a user object with an error message contained in the user id will be returned.
     *
     * @param cardID  SmartCard ID
     * @param siteCode  Site code associated with access and verify credentials.
     * @param accessCode user's access code.
     * @param verifyCode user's verify code.
     * @param requestingApp Application calling method.
     * @param ipAddress user's IP address
     * @param subjectDN user's subjectDN from their smartcard
     * @return ResponseLogin login response object.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "loginEnterprise")
    public User loginEnterprise(@WebParam(name = "cardID")
                                String cardID, @WebParam(name = "siteCode")
                                String siteCode, @WebParam(name = "accessCode")
                                String accessCode, @WebParam(name = "verifyCode")
                                String verifyCode, @WebParam(name = "requestingApp")
                                String requestingApp, @WebParam(name = "ipAddress")
                                String ipAddress, @WebParam(name = "subjectDN")
                                String subjectDN, @WebParam(name = "emailAddress")
                                String emailAddress) throws JMeadowsException {
        User rtc = null;
        long begin = System.currentTimeMillis();
        try {
            String container = "";
            Site site = Sites.getInstance().getSite(siteCode);
            if (site != null) {
                container = site.getContainer();
            } else {
                site = new Site();
            }
            if (APPCONFIG.useSnareworks() && !"VA".equals(site.getAgency())) {
                LOGGER.debug("login container retrieved = " + container);
                if (container == null || container.length() == 0) {
                    container = "unknown";
                    LOGGER.error("EMPTY Container passed in.");
                }
                AuditDao auditDao = new AuditDao();
                AuthUserDao authUserDao = new AuthUserDao();

                SnareUser suser = authUserDao.getSnareworksUser(null, container + "@" + accessCode, verifyCode);

//                //melcolm test
//
//                SnareUser suser = new SnareUser();
//                suser.setCn("SHMOE,JOE");
//                suser.setNcid("666");
//                suser.setValid(true);
//                //end melcolm test
                if (suser != null) {
                    String snaremsg = suser.getSnareMessage();
                    rtc = new User();
                    if (snaremsg == null) {
                        snaremsg = "";
                    }
                    if ("No Snare Servers available".equalsIgnoreCase(snaremsg)) {
                        throw new JMeadowsException("Snareworks is down.");
                    } else if (snaremsg.toLowerCase(Locale.getDefault()).indexOf("login failed") >= 0) {
                        rtc.setUserIen("^Not a valid User ID /  Password pair.");
                    } else if (!suser.isValid()) {
                        rtc.setUserIen("^You are not an authorized user.");
                    } else if (snaremsg.length() > 0) {
                        throw new JMeadowsException(snaremsg);
                    } else {
                        rtc.setAgency("DOD");
                        rtc.setCardId(cardID);
                        rtc.setDmisId(site.getDmisId());
                        rtc.setHostSite(site);
                        rtc.setUserId(suser.getNcid());
                        rtc.setEncryptedAvCode("");
                        rtc.setUserIen(suser.getNcid());
                        rtc.setName(suser.getCn());
                        rtc.setEncryptedFederatedUid(EncryptUtil.encrypt("123456789"));
                        auditDao.setLoginAudit(cardID, siteCode,
                                suser.getNcid(), "", suser.getCn(), "", "login", "SnareworksManual", requestingApp, "", "", ipAddress, emailAddress);
                    }
                }
            } else {
                UserDao userDao = new UserDao();
                rtc = userDao.loginEnterprise(cardID, siteCode, accessCode, verifyCode, requestingApp, ipAddress, subjectDN, emailAddress);
                if (rtc != null) {
                    logResponse(null, begin, "", 1);
                } else {
                    logResponse(null, begin, "", 0);
                }
            }
        } catch (JMeadowsException ex) {
            logResponse(null, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
        return rtc;
    }

    /**
     * Authenticates a user against a local system.
     *
     * If successful, a qualified user object will be returned.
     *
     * Otherwise, a user object with an error message contained in the user id will be returned.
     *
     * @param siteCode  Site code associated with access and verify credentials.
     * @param samlToken user's SAML token.
     * @param requestingApp Application calling method.
     * @param ipAddress user's IP address
     * @return ResponseLogin login response object.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "loginSaml")
    public User loginSaml(@WebParam(name = "siteCode")
                                        String siteCode, @WebParam(name = "samlToken")
                                        String samlToken, @WebParam(name = "requestingApp")
                                        String requestingApp, @WebParam(name = "ipAddress")
                                        String ipAddress, @WebParam(name = "emailAddress")
                                        String emailAddress) throws JMeadowsException {
        User rtc = null;
        long begin = System.currentTimeMillis();
        try {
            String container = "";
            Site site = Sites.getInstance().getSite(siteCode);
            if (site != null) {
                container = site.getContainer();
            } else {
                site = new Site();
            }
            UserDao userDao = new UserDao();
            rtc = userDao.loginSaml(siteCode, samlToken, requestingApp, ipAddress, emailAddress);
            if (rtc != null) {
                logResponse(null, begin, "", 1);
            } else {
                logResponse(null, begin, "", 0);
            }
        } catch (JMeadowsException ex) {
            logResponse(null, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
        return rtc;
    }

    /**
     * Performs a patient lookup against the provided site location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * recordSiteCode - site to perform search against
     * itemId - patient name
     *
     * @return A list of patients that match search criteria.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     * @deprecated Use patient lookupPatientPDWS
     */
    @Deprecated
    @WebMethod(operationName = "lookupPatient")
    public List<Patient> lookupPatient(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            List<Patient> rtc = patientDao.lookupPatient(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Performs a patient lookup against the DMDC Patient Discovery Web Service (PDWS).
     *
     * @param queryBean String parameters.
     *
     * @return A list of patients that match search criteria.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "lookupPatientPDWS")
    public ResponsePatientQuery lookupPatientPDWS(@WebParam(name = "queryBean") PDWSQueryBean queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            ResponsePatientQuery rtc = patientDao.lookupPatientPDWS(queryBean);
            //if (rtc != null) {
            logResponse(null, begin, "", 1);
//            } else {
//                logResponse(null, begin, "", 0);
//            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(null, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Performs a patient lookup against the Master Veteran Index (MVI).
     *
     * @param queryBean String parameters.
     *
     * @return A list of patients that match search criteria.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "lookupPatientMVI")
    public ResponsePatientQuery lookupPatientMVI(@WebParam(name = "queryBean") PDWSQueryBean queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            ResponsePatientQuery rtc = patientDao.lookupPatientMVI(queryBean);
            //if (rtc != null) {
            logResponse(null, begin, "", 1);
//            } else {
//                logResponse(null, begin, "", 0);
//            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(null, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     *  Performs patient selection against the provided site location.
     *
     *
     *  If the patient is from the VA or has a corresponding VistA identifier the following will occur:
     *
     *      VistA level patient selection will be performed.
     *
     *      All sensitive VA patient selections will be flagged.
     *
     *      Patient object will contain VistA ICN if one exists.
     *
     *      VistaLocations site code list will contain all VistA locations that contain records for this patient.
     *
     *
     *  There are two patient selection methods MSI and PIX.
     *
     *  PIX patient selection is only performed if the configuration flag has been set, MSI is the default.
     *
     *  PIX service-based patient selection details:
     *
     *      PIX service based patient selection will query the PIX service to
     *      retrieve available corresponding identifiers.
     *
     *      The following Patient object fields if the PIX service-based selection method is utilized:
     *          localVAIen
     *          localDoDIen
     *          dodEdipn
     *          vistaIcn
     *
     *       If the selection method cannot confirm or deny the existence of a corresponding identifier,
     *       the PIX based patient selection method will set the corresponding
     *       identifier to the IDENTIFIER_UNAVAILABLE tag.
     *
     *       The patientId, and patientSiteCode will be set to the itemId and recordSiteCode parameters.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * itemId - The identifier of the patient to select
     * recordSiteCode - The itemId's associated site code (VA or DoD facility).
     *
     * @return A qualified selected patient.
     *
     *
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid.
     * @deprecated Use selectPatientMVI
     */
    @Deprecated
    @WebMethod(operationName = "selectPatient")
    public Patient selectPatient(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        throw new UnsupportedOperationException("This operation is no longer supported");
    }

    /**
     * Performs patient selection against the MVI service using and EDIPI retrieved from the DMDC PDWS.
     *
     * MVI service-based patient selection details:
     *
     *      MVI service based patient selection will query the MVI service to
     *      retrieve available corresponding VA identifiers.
     *
     *      The following Patient object fields if the MVI service-based selection method is utilized:
     *          edipi
     *          icn
     *          patientIens
     *
     *       If the selection method cannot confirm or deny the existence of a corresponding identifier,
     *       the MVI based patient selection method will set the corresponding
     *       identifier to the IDENTIFIER_UNAVAILABLE tag.
     *
     *       The edipi will be set to the itemId parameter.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * itemId - The edipi identifier of the patient to select
     *
     * @return The selected patient.
     * @throws JMeadowsException if there is an error.
     */
    @WebMethod(operationName = "selectPatientMVI")
    public ResponsePatientSelect selectPatientMVI(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            ResponsePatientSelect rtc = patientDao.selectPatientMVI(queryBean);
            //if (rtc != null) {
            logResponse(queryBean, begin, "", 1);
//            } else {
//                logResponse(queryBean, begin, "", 0);
//            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Performs patient selection to VistA for a sensitive patient after user has accepted sensitive patient warning.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @throws JMeadowsException if there is an error.
     */
    @WebMethod(operationName = "selectPatientForVASensitive")
    public void selectPatientForVASensitive(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            patientDao.selectPatientForVASensitive(queryBean);
            logResponse(queryBean, begin, "", 0);
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets the list of VA sensitive patient alerts.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @throws JMeadowsException if there is an error.
     */
    @WebMethod(operationName = "getVASensitivePatientAlerts")
    public List<Alert> getVASensitivePatientAlerts(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            List<Alert> rtc = patientDao.getVASensitivePatientAlerts(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }


    /**
     * Performs audit on VA user who accepts to access a DoD-only patient record
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @throws JMeadowsException if there is an error.
     */
    @WebMethod(operationName = "auditVARestrictedAccess")
    public void auditVARestrictedAccess(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            patientDao.auditVARestrictedAccess(queryBean);
            logResponse(queryBean, begin, "", 0);
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Performs audit on VA user who accepts to access a DoD-only patient record
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @throws JMeadowsException if there is an error.
     */
    @WebMethod(operationName = "auditSensitiveRecordAccess")
    public void auditSensitiveRecordAccess(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            patientDao.auditSensitiveRecordAccess(queryBean);
            logResponse(queryBean, begin, "", 0);
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }


    /**
     * Retrieves a list of patient admission records from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient admission records.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientAdmissions")
    public List<PatientAdmission> getPatientAdmissions(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionDao admissionDao = new AdmissionDao();
            List<PatientAdmission> rtc = admissionDao.getPatientAdmissions(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    @WebMethod(operationName = "getPatientHistory")
    public List<PatientHistory> getPatientHistory(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            HistoryDao historyDao = new HistoryDao();
            List<PatientHistory> rtc = historyDao.getPatientHistory(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }


    /**
     * Retrieves a list of patient allergies from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient allergies.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientAllergies")
    public List<Allergy> getPatientAllergies(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AllergyDao allergyDao = new AllergyDao();
            List<Allergy> rtc = allergyDao.getPatientAllergies(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of DOD VLER patient allergies
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient allergies.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getDODVLERPatientAllergies")
    public List<Allergy> getDODVLERPatientAllergies(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AllergyDao allergyDao = new AllergyDao();
            List<Allergy> rtc = allergyDao.getDODVLERPatientAllergies(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }


    /**
     * Retrieves a patient allergy details from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return AllergyDetail containing allergy report text.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getAllergyDetail")
    public AllergyDetail getAllergyDetail(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AllergyDao allergyDao = new AllergyDao();
            AllergyDetail rtc = allergyDao.getAllergyDetail(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves Patient Encounters report  from the provided record location.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * startDate
     * endDate
     *
     * @return Encounters Report.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientEncountersReport")
    public FreeTextReport getPatientEncountersReport(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AppointmentsDao apptsDao = new AppointmentsDao();
            FreeTextReport rtc = apptsDao.getPatientEncountersReport(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a report of patient Encounters
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T + 1 year).
     *
     * @return A list of patient appointments.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientAppointments")
    public List<PatientAppointments> getPatientAppointments(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AppointmentsDao appointmentsDao = new AppointmentsDao();
            List<PatientAppointments> rtc = appointmentsDao.getPatientAppointments(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }


    /**
     * Retrieves a list of patient encounters from AHLTA.
     *
     * @param queryBean Query parameters
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient encounters from AHLTA.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientEncounters")
    public List<Encounter> getPatientEncounters(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AppointmentsDao appointmentsDao = new AppointmentsDao();
            List<Encounter> rtc = appointmentsDao.getPatientEncounters(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient clinical reminders from all available VistA patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient clinical reminders.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientClinicalReminders")
    public List<ClinicalReminder> getPatientClinicalReminders(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ClinicalRemindersDao remindersDao = new ClinicalRemindersDao();
            List<ClinicalReminder> rtc = remindersDao.getPatientClinicalReminders(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves clinical reminder record details from the provided record location.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return Clincal reminder record details.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getClinicalReminderDetail")
    public FreeTextReport getClinicalReminderDetail(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ClinicalRemindersDao remindersDao = new ClinicalRemindersDao();
            FreeTextReport rtc = remindersDao.getClinicalReminderDetail(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient demographics from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     *
     * @return A list of patient demographics.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientDemographics")
    public List<PatientDemographics> getPatientDemographics(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            List<PatientDemographics> rtc = patientDao.getPatientDemographics(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient demographics details from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     *
     * @return A list of patient demographics details.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientDemographicsDetail")
    public List<PatientDemographicsDetail> getPatientDemographicsDetail(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientDao patientDao = new PatientDao();
            List<PatientDemographicsDetail> rtc = patientDao.getPatientDemographicsDetail(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves patient immunizations from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient immunizations.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientImmunizations")
    public List<Immunization> getPatientImmunizations(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ImmunizationDao immunizationDao = new ImmunizationDao();
            List<Immunization> rtc = immunizationDao.getPatientImmunizations(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves patient lab orders from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient lab orders.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientLabs")
    public List<LabOrder> getPatientLabs(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            LabsDao labsDao = new LabsDao();
            List<LabOrder> rtc = labsDao.getPatientLabs(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves patient lab results from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient lab results.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientLabResults")
    public List<LabResult> getPatientLabResults(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            LabsDao labsDao = new LabsDao();
            List<LabResult> rtc = labsDao.getPatientLabResults(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves patient lab report from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return FreeTextReport bean.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientLabReport")
    public FreeTextReport getPatientLabReport(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            LabsDao labsDao = new LabsDao();
            FreeTextReport rtc = labsDao.getPatientLabReport(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient medications from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * status (possible options: O - outpatient, I - inpatient, defaults to "O"
     * active (defaults to 1)
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return List of patient medications.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientMedications")
    public List<Medication> getPatientMedications(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            MedicationsDao medsDao = new MedicationsDao();
            List<Medication> rtc = medsDao.getPatientMedications(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

//    /**
//     * Retrieves patient questionnaires.
//     *
//     * @param queryBean Query parameters.
//     *
//     * {@link JMeadowsQuery} required parameters:
//     *
//     * {@link gov.va.med.vds.webservice.User}
//     * {@link gov.va.med.vds.webservice.Patient}
//     *
//     * Optional:
//     * startDate (defaults to T-3 months)
//     * endDate   (defaults to T).
//     *
//     * @return List of patient questionnaires.
//     * @throws JMeadowsException if a serious error occurs
//     * @throws IllegalArgumentException if required parameters are missing or invalid
//     */
//    @WebMethod(operationName = "getPatientQuestionnaires")
//    public List<QuestionnaireBean> getPatientQuestionnaires(@WebParam(name = "queryBean")JMeadowsQuery queryBean) throws JMeadowsException
//    {
//        long begin = System.currentTimeMillis();
//        try
//        {
//            QuestionnaireDao questDao = new QuestionnaireDao();
//            List<QuestionnaireBean> rtc = questDao.getPatientQuestionnaires(queryBean);
//            logResponse(queryBean, begin, "", rtc);
//            return rtc;
//        }
//        catch (JMeadowsException ex)
//        {
//            logResponse(queryBean, begin, ex.getMessage(), 0);
//            logger.error(ex.getMessage(), ex);
//            throw ex;
//        }
//    }

//    /**
//     * Retrieves patient deployment forms.
//     *
//     * @param queryBean Query parameters.
//     *
//     * {@link JMeadowsQuery} required parameters:
//     *
//     * {@link gov.va.med.vds.webservice.User}
//     * {@link gov.va.med.vds.webservice.Patient}
//     *
//     * Optional:
//     * startDate (defaults to T-3 months)
//     * endDate   (defaults to T).
//     *
//     * @return List of patient questionnaires.
//     * @throws JMeadowsException if a serious error occurs
//     * @throws IllegalArgumentException if required parameters are missing or invalid
//     */
//    @WebMethod(operationName = "getDeploymentForms")
//    public List<DeploymentForm> getDeploymentForms(@WebParam(name = "queryBean")JMeadowsQuery queryBean) throws JMeadowsException
//    {
//        long begin = System.currentTimeMillis();
//        try
//        {
//            DeploymentFormsDao dao = new DeploymentFormsDao();
//            List<DeploymentForm> rtc = dao.getDeploymentForms(queryBean);
//            logResponse(queryBean, begin, "", rtc);
//            return rtc;
//        }
//        catch (JMeadowsException ex)
//        {
//            logResponse(queryBean, begin, ex.getMessage(), 0);
//            logger.error(ex.getMessage(), ex);
//            throw ex;
//        }
//    }

    /**
     * Retrieves patient questionnaires.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return List of patient questionnaires.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientForms")
    public List<Form> getPatientForms(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            QuestionnaireDao questDao = new QuestionnaireDao();
            List<Form> rtc = questDao.getPatientForms(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient progress notes from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient progress notes
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientProgressNotes")
    public List<ProgressNote> getPatientProgressNotes(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao notesDao = new NotesDao();
            List<ProgressNote> rtc = notesDao.getPatientProgressNotes(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a BHIE note image.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * itemId
     *
     * @return Note image
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getBHIENoteImage")
    public NoteImage getBHIENoteImage(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao noteDao = new NotesDao();
            NoteImage rtc = noteDao.getBHIENoteImage(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a BHIE note attachment.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * itemId
     *
     * @return Note image
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getBHIENoteAttachment")
    public NoteImage getBHIENoteAttachment(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao noteDao = new NotesDao();
            NoteImage rtc = noteDao.getBHIENoteAttachment(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient documents from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient progress notes
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientDocuments")
    public List<PatientDocument> getPatientDocuments(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            DocumentDao docDao = new DocumentDao();
            List<PatientDocument> rtc = docDao.getPatientDocuments(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient discharge summaries from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient discharge summaries.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientDischargeSummaries")
    public List<ProgressNote> getPatientDischargeSummaries(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao notesDao = new NotesDao();
            List<ProgressNote> rtc = notesDao.getPatientDischargeSummaries(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient consult requests.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient consult requests.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientConsultRequests")
    public List<Consult> getPatientConsultRequests(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao notesDao = new NotesDao();
            List<Consult> rtc = notesDao.getPatientConsultRequests(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient orders from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient orders.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientOrders")
    public List<Order> getPatientOrders(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            OrdersDao ordersDao = new OrdersDao();
            List<Order> rtc = ordersDao.getPatientOrders(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient problems from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient problems.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientProblemList")
    public List<Problem> getPatientProblemList(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProblemDao problemDao = new ProblemDao();
            List<Problem> rtc = problemDao.getPatientProblemList(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient radiology reports from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient radiology reports.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientRads")
    public List<RadiologyReport> getPatientRads(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            RadiologyDao radiologyDao = new RadiologyDao();
            List<RadiologyReport> rtc = radiologyDao.getPatientRads(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient vitals from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient vitals.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientVitals")
    public List<Vitals> getPatientVitals(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            VitalsDao vitalsDao = new VitalsDao();
            List<Vitals> rtc = vitalsDao.getPatientVitals(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient lab test results from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A list of patient lab test results.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientLabTestResults")
    public List<LabResult> getPatientLabTestResults(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            LabsDao labsDao = new LabsDao();
            List<LabResult> rtc = labsDao.getPatientLabTestResults(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of lab order results from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A list of of lab order results.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getLabOrderResult")
    public List<LabResult> getLabOrderResult(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            LabsDao labsDao = new LabsDao();
            List<LabResult> rtc = labsDao.getLabOrderResult(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

//    /**
//     * Retrieves a list of non-VA patient medications from available patient locations.
//     *
//     * @param queryBean Query parameters.
//     *
//     * {@link JMeadowsQuery} required parameters:
//     *
//     * {@link gov.va.med.vds.webservice.User}
//     * {@link gov.va.med.vds.webservice.Patient}
//     *
//     * Optional:
//     * startDate (defaults to T-3 months)
//     * endDate   (defaults to T).
//     *
//     *
//     * @return A list of non-VA patient medications.
//     * @throws JMeadowsException if a serious error occurs
//     * @throws IllegalArgumentException if required parameters are missing or invalid
//     */
//    @WebMethod(operationName = "getPatientMedicationsNVA")
//    public List<Medication> getPatientMedicationsNVA(@WebParam(name = "queryBean")JMeadowsQuery queryBean) throws JMeadowsException
//    {
//        try
//        {
//            MedicationsDao medsDao = new MedicationsDao();
//            return medsDao.getPatientMedicationsNVA(queryBean);
//        }
//        catch (JMeadowsException ex)
//        {
//            logger.error(ex.getMessage(), ex);
//            throw ex;
//        }
//    }

    /**
     * Retrieves patient admission record details from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return AdmissionDetail bean.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getAdmissionDetails")
    public AdmissionDetail getAdmissionDetails(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionDao admissionDao = new AdmissionDao();
            AdmissionDetail rtc = admissionDao.getAdmissionDetails(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of admission diagnosis from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of admission diagnosis records.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getAdmissionDiagnosis")
    public List<Diagnosis> getAdmissionDiagnosis(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionDao admissionDao = new AdmissionDao();
            List<Diagnosis> rtc = admissionDao.getAdmissionDiagnosis(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of admission procedures from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of admission procedures.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getAdmissionProcedures")
    public List<Procedure> getAdmissionProcedures(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionDao admissionDao = new AdmissionDao();
            List<Procedure> rtc = admissionDao.getAdmissionProcedures(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a patient consult report from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A patient consult report (a list of report fragments).
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getConsultReport")
    public List<FreeTextReport> getConsultReport(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao notesDao = new NotesDao();
            List<FreeTextReport> rtc = notesDao.getConsultReport(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieve user profile that is associated with the user's user id.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @return User with duty phone, gender, pager, permission class, flag, and sig filled in.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderProfile")
    public User getProviderProfile(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            UserDao userDao = new UserDao();
            User rtc = userDao.getHUIProviderProfile(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves medication record report from the provided VistA location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return Inpatient medication record report.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getMedicationDetail")
    public FreeTextReport getMedicationDetail(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            MedicationsDao medsDao = new MedicationsDao();
            FreeTextReport rtc = medsDao.getMedicationDetail(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a patient order from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return Order details string.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getOrderDetail")
    public FreeTextReport getOrderDetail(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            OrdersDao ordersDao = new OrdersDao();
            FreeTextReport rtc = ordersDao.getOrderDetail(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

//    /**
//     * Retrieves a code gray message for patient from a given location.
//     *
//     * {@link JMeadowsQuery} required parameters:
//     *
//     * {@link gov.va.med.vds.webservice.User}
//     * {@link gov.va.med.vds.webservice.Patient}
//     * recordSiteCode
//     *
//     * @return A patient code gray message.
//     * @throws JMeadowsException if a serious error occurs
//     * @throws IllegalArgumentException if required parameters are missing or invalid
//     */
//    @WebMethod(operationName = "getPatientCodeGreen")
//    public String getPatientCodeGreen(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException
//    {
//        try
//        {
//            PatientDao patientDao = new PatientDao();
//            return patientDao.getPatientCodeGreen(queryBean);
//        }
//        catch (JMeadowsException ex)
//        {
//            logger.error(ex.getMessage(), ex);
//            throw ex;
//        }
//    }

    /**
     * Retrieves a list of patient's current vitals from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     *
     * @return A list of patient's current current vitals.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientCurrentVitals")
    public List<Vitals> getPatientCurrentVitals(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            VitalsDao vitalsDao = new VitalsDao();
            List<Vitals> rtc = vitalsDao.getPatientCurrentVitals(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient's Health Summary Types from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient's Health Summary Types.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid.
     */
    @WebMethod(operationName = "getPatientHealthSummaryTypes")
    public List<HealthSummaryTypeList> getPatientHealthSummaryTypes(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            HealthSummariesDao healthSummariesDao = new HealthSummariesDao();
            List<HealthSummaryTypeList> rtc = healthSummariesDao.getPatientHealthSummaryTypes(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a patient heath summary from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A patient health summary note.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPatientHealthSummary")
    public FreeTextReport getPatientHealthSummary(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            HealthSummariesDao healthSummariesDao = new HealthSummariesDao();
            FreeTextReport rtc = healthSummariesDao.getPatientHealthSummary(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient's insurances from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient's insurances.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid.
     */
    @WebMethod(operationName = "getPatientInsurances")
    public List<InsuranceBean> getPatientInsurances(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            InsuranceDao insuranceDao = new InsuranceDao();
            List<InsuranceBean> rtc = insuranceDao.getPatientInsurances(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of outpatient's procedures from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of outpatient's procedures
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid.
     */
    @WebMethod(operationName = "getOutpatientProcedures")
    public List<Procedure> getOutpatientProcedures(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            OutpatientProcedureDao outpatientProcedureDao = new OutpatientProcedureDao();
            List<Procedure> rtc = outpatientProcedureDao.getOutpatientProcedures(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

//    /**
//     * Retrieves patient prescription fills from the provided record location.
//     *
//     * @param queryBean Query parameters.
//     *
//     * {@link JMeadowsQuery} required parameters:
//     *
//     * {@link gov.va.med.vds.webservice.User}
//     * {@link gov.va.med.vds.webservice.Patient}
//     * recordSiteCode
//     * itemId
//     *
//     * @return Patient prescription bean.
//     * @throws JMeadowsException if a serious error occurs
//     * @throws IllegalArgumentException if required parameters are missing or invalid
//     */
//    @WebMethod(operationName = "getPrescriptionFills")
//    public Prescription getPrescriptionFills(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException
//    {
//        try
//        {
//            MedicationsDao medsDao = new MedicationsDao();
//            return medsDao.getPrescriptionFills(queryBean);
//        }
//        catch (JMeadowsException ex)
//        {
//            logger.error(ex.getMessage(), ex);
//            throw ex;
//        }
//    }

    /**
     * Retrieves a list of patient MHS Genesis documents from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient MHS Genesis documents.
     * @throws JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getMhsGenesisDocuments")
    public List<MhsGenesis> getMhsGenesisDocuments(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException
    {
        long begin = System.currentTimeMillis();
        try {
            MhsGenesisDao mhsDao = new MhsGenesisDao();
            List<MhsGenesis> rtc = mhsDao.getMhsGenesisDocuments(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        }
        catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a patient problem list detail from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return Problem record detail.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProblemDetail")
    public ProblemDetail getProblemDetail(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProblemDao problemDao = new ProblemDao();
            ProblemDetail rtc = problemDao.getProblemDetail(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a patient progress note from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A patient progress note.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProgressNote")
    public FreeTextReport getProgressNote(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao notesDao = new NotesDao();
            FreeTextReport rtc = notesDao.getProgressNote(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a patient radiology report from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return RadiologyReport bean.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getRadiologyReport")
    public RadiologyReport getRadiologyReport(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            RadiologyDao radiologyDao = new RadiologyDao();
            RadiologyReport rtc = radiologyDao.getRadiologyReport(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of patient c32 document from the NwHIN VLER interface.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return List of VlerDocument.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getVLERDocumentList")
    public List<VlerDocument> getVLERDocumentList(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            VLERDao vlerDao = new VLERDao();
            List<VlerDocument> rtc = vlerDao.getVLERDocumentList(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a VlerDocument (contains c32) from the NwHIN VLER interface.
     *
     * @param vDoc Vler Document c32 location information.
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return List of VlerDocument.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getVLERDocument")
    public VlerDocument getVLERDocument(@WebParam(name = "vDoc") VlerDocument vDoc,
                                        @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            VLERDao vlerDao = new VLERDao();
            VlerDocument rtc = vlerDao.getVLERDocument(vDoc, queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a DOD VLER CCDA
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of CCDAs.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getDODVLERPatientCCDA")
    public List<VLERCCDA> getDODVLERPatientCCDA(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            VLERDao vlerDao = new VLERDao();
            List<VLERCCDA> rtc = vlerDao.getDODVLERPatientCCDA(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a VA VLER DAS Data
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of CCDAs.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getDASPatientData")
    public List<VLERCCDA> getDASPatientData(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            VLERDao vlerDao = new VLERDao();
            List<VLERCCDA> rtc = vlerDao.getDASVLERData(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a list of user alerts associated with the user's user id.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     *
     * @return A list of user alerts.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getUserAlerts")
    public List<Alert> getUserAlerts(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            UserDao userDao = new UserDao();
            List<Alert> rtc = userDao.getUserAlerts(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves a Patient visit note from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A patient visit note.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getVisitNotes")
    public FreeTextReport getVisitNotes(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            PatientApptDao patientApptDao = new PatientApptDao();
            FreeTextReport rtc = patientApptDao.getVisitNotes(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

//    /**
//     * Logs audit to audit sql database.
//     *
//     * @param siteCode Site code
//     * @param userIen user (user) identifier
//     * @param providerIen user's user identifier
//     * @param userNPI user (user) national user identifier
//     * @param userName user (user) name
//     * @param patId  patient ien.
//     * @param category audit category
//     * @param requestingApp requesting application
//     * @param startDate  query start date (defaults to T)
//     * @param endDate    query end date   (defaults to T)
//     * @throws JMeadowsException if an error occurs
//     */
//    @WebMethod(operationName = "logAudit")
//    public void logAudit(@WebParam(name = "siteCode") String siteCode,
//                         @WebParam(name = "userIen") String userIen,
//                         @WebParam(name = "providerIen") String providerIen,
//                         @WebParam(name = "userNPI") String userNPI,
//                         @WebParam(name = "userName") String userName,
//                         @WebParam(name = "patId") String patId,
//                         @WebParam(name = "category") String category,
//                         @WebParam(name = "requestingApp") String requestingApp,
//                         @WebParam(name = "startDate") String startDate,
//                         @WebParam(name = "endDate") String endDate) throws JMeadowsException {
//        String usrIen = userIen;
//        try {
//            AuditDao auditDao = new AuditDao();
//
//            //append providerIen to userIen if one exists
//            if (StringUtils.isNotBlank(providerIen)) {
//                usrIen += "^" + providerIen;
//            }
//
//            auditDao.setAudit(siteCode, usrIen, userNPI, userName,
//                    patId, category, requestingApp, startDate, endDate);
//        } catch (JMeadowsException ex) {
//            LOGGER.error(ex.getMessage(), ex);
//
//        }
//
//    }

    /**
     * Logs audit to audit sql database.
     *
     * @param siteCode Site code
     * @param siteMoniker
     * @param siteMoniker
     * @param recordID
     * @param userIen user (user) identifier
     * @param providerIen user's user identifier
     * @param userNPI user (user) national user identifier
     * @param userName user (user) name
     * @param patId  patient ien.
     * @param category audit category
     * @param requestingApp requesting application
     * @param startDate  query start date (defaults to T)
     * @param endDate    query end date   (defaults to T)
     * @param info  query misc info.
     * @param cardId  user card id.
     * @param emailAddress  user email address.
     * @param complexTransaction
     * @throws JMeadowsException if an error occurs
     */
    @WebMethod(operationName = "logAudit")
    public void logAudit(@WebParam(name = "siteCode") String siteCode,
                         @WebParam(name = "siteMoniker") String siteMoniker,
                         @WebParam(name = "siteAgency") String siteAgency,
                         @WebParam(name = "recordID") String recordID,
                         @WebParam(name = "userIen") String userIen,
                         @WebParam(name = "providerIen") String providerIen,
                         @WebParam(name = "userNPI") String userNPI,
                         @WebParam(name = "userName") String userName,
                         @WebParam(name = "patId") String patId,
                         @WebParam(name = "category") String category,
                         @WebParam(name = "requestingApp") String requestingApp,
                         @WebParam(name = "startDate") String startDate,
                         @WebParam(name = "endDate") String endDate,
                         @WebParam(name = "info") String info,
                         @WebParam(name = "cardId") String cardId,
                         @WebParam(name = "emailAddress") String emailAddress,
                         @WebParam(name = "complexTransaction") String complexTransaction) throws JMeadowsException {
        String usrIen = userIen;
        try {
            AuditDao auditDao = new AuditDao();

            //append providerIen to userIen if one exists
            if (StringUtils.isNotBlank(providerIen)) {
                usrIen += "^" + providerIen;
            }

            auditDao.setAudit(siteCode, siteMoniker, siteAgency, recordID, usrIen, userNPI, userName,
                    patId, category, requestingApp, startDate, endDate, info, cardId, emailAddress, complexTransaction);
        } catch (JMeadowsException ex) {
            logError(ex);

        }

    }

    /**
     * Gets a list of past and/or future appointments which are assigned to the
     * user passed in the queryBean.
     * @param queryBean requires a user, startDate and endDate. Start and
     * end dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * @return a list of past and/or future patient appointments
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderAppointments")
    public List<PatientAppointments> getProviderAppointments(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderAppointmentsDao appointmentsDao =
                    new ProviderAppointmentsDao();
            List<PatientAppointments> rtc = appointmentsDao.getProviderAppointments(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }


    /**
     * Gets a list of patients who have been admitted or are being attended
     * by the user passed in the queryBean.
     * @param queryBean requires a user
     * @return a list of patients who were admitted or are being attended
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderAdmissions")
    public List<PatientAdmission> getProviderAdmissions(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderAdmissionsDao admissionsDao = new ProviderAdmissionsDao();
            List<PatientAdmission> rtc = admissionsDao.getProviderAdmissions(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets a list of patients who are admitted in the specified ward(s). This
     * method <b>only</b> queries the local VistA system.
     * @param queryBean requires itemId. itemId is a ^ delimited ward ien
     * string, ie "24^47" for two wards or just "24" for a single ward.
     * @return a list of patients in the specified ward(s)
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getWardAdmissions")
    public List<PatientAdmission> getWardAdmissions(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderAdmissionsDao admissionsDao = new ProviderAdmissionsDao();
            List<PatientAdmission> rtc = admissionsDao.getWardAdmissions(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets a list of all unsigned notes which belong to the user passed
     * in the queryBean.
     * @param queryBean requires a user
     * @return unsigned notes for a given user
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderUnsignedNotes")
    public List<ProgressNote> getProviderUnsignedNotes(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderNotesDao notesDao = new ProviderNotesDao();
            List<ProgressNote> rtc = notesDao.getProviderUnsignedNotes(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets a list of all consults which have been requested by the user
     * passed in the queryBean. All statuses will be returned.
     * @param queryBean requires a user, startDate and endDate. Start and
     * end dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * @return a list of all consults requested
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderConsultsRequested")
    public List<Consult> getProviderConsultsRequested(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderConsultsDao consultDao = new ProviderConsultsDao();
            List<Consult> rtc = consultDao.getProviderConsultsRequested(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets a list of all consults which a user must complete. Ie,
     * user X orders an echocardiogram; user Y receives the consult
     * to be completed. This method enables a user (in this example Y) to
     * see what consults need to be completed on a longitudinal patient domain
     * rather than a patient by patient basis.
     * @param queryBean requires user, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and
     * stop searching. The wider the date range the longer it takes to process.
     * @return a list of consults that a user has received for a given
     * user and date range.
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderConsultsReceived")
    public List<Consult> getProviderConsultsReceived(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderConsultsDao consultDao = new ProviderConsultsDao();
            List<Consult> rtc = consultDao.getProviderConsultsReceived(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets a list of orders pending which have been placed by the user
     * passed in the queryBean.
     * @param queryBean requires user, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * @return a list of orders pending for a given user and date range.
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderOrdersPending")
    public List<Order> getProviderOrdersPending(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderOrdersDao orderDao = new ProviderOrdersDao();
            List<Order> rtc = orderDao.getProviderOrdersPending(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets a list of orders which have results which have been placed by the
     * user passed in the queryBean.
     * @param queryBean requires user, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * @return a list of orders which have results for a given user and
     * date range. For each order, the actual result is also included.
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderOrdersResulted")
    public List<Order> getProviderOrdersResulted(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderOrdersDao orderDao = new ProviderOrdersDao();
            List<Order> rtc = orderDao.getProviderOrdersResulted(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Gets a list of abnormal lab results which were ordered by the user
     * passed in the queryBean.
     * @param queryBean requires user, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * @return a list of abnormal lab results for given user and date
     * range. For each abnormal lab result, the abnormal result is included.
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderLabAbnormalResults")
    public List<LabResult> getProviderLabAbnormalResults(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            ProviderLabsDao labDao = new ProviderLabsDao();
            List<LabResult> rtc = labDao.getProviderAbnormalLabResults(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * retrieve the user's Janus GUI configuration (JSON)
     * @param queryBean requires user.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getJanusGUIConfig")
    public String getJanusGUIConfig(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {

        String cfg = "";

        long begin = System.currentTimeMillis();
        try {
            UserDao provDao = new UserDao();
            cfg = provDao.getJanusGUIConfig(queryBean);
            logResponse(queryBean, begin, "", 0);
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
        }
        return cfg;
    }

    /**
     * set the user's Janus GUI configuration (JSON)
     * @param queryBean requires user.
     * @param cfg - the user's configuration in JSON.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "setJanusGUIConfig")
    public boolean setJanusGUIConfig(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean,
            @WebParam(name = "cfg") String cfg) throws JMeadowsException {
        boolean result = false;
        long begin = System.currentTimeMillis();
        try {
            UserDao provDao = new UserDao();
            result = provDao.setJanusGUIConfig(queryBean, cfg);
            logResponse(queryBean, begin, "", 0);
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);

        }
        return result;
    }

    /**
     * retrieve the user's iEHR permission flags
     * @param queryBean requires user.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getProviderFlags")
    public String getUserFlags(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {

        String flags = "";

        long begin = System.currentTimeMillis();
        try {
            UserDao provDao = new UserDao();
            flags = provDao.getUserFlags(queryBean);
            logResponse(queryBean, begin, "", 0);
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
        }
        return flags;
    }

    /**
     * set the user's iEHR permission flags
     * @param queryBean requires user.
     * @param flags - the user's flags.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "setProviderFlags")
    public boolean setUserFlags(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean,
            @WebParam(name = "flags") String flags) throws JMeadowsException {
        boolean result = false;
        long begin = System.currentTimeMillis();
        try {
            UserDao provDao = new UserDao();
            result = provDao.setUserFlags(queryBean, flags);
            logResponse(queryBean, begin, "", 0);
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);

        }
        return result;
    }

    /**
     * Gets jMeadows webservice version: <MAJOR>.<MINOR>.<BUILD>
     * @return jMeadows version
     */
    @WebMethod(operationName = "getVersion")
    public String getVersion() {
        return APPCONFIG.getAppVersion();
    }

    /**
     * Retrieves an iEHR user's profile based on the information that is read off of
     * their smart card (CAC/PIV, etc.)
     *
     * @param smartCardID Person identification read from their smart card.
     * @param smartCardAgency Smart card issuer's agency. To differentiate between two or more user's having the same
     *                        person ID, but from different agencies.
     * @return IehrUserProfile containing the iEHR user's profile.
     * @throws JMeadowsException if a serious error occurs
     */
    @WebMethod(operationName = "getIehrUserProfile")
    public IehrUserProfile getIehrUserProfile(
            @WebParam(name = "smartCardID") String smartCardID,
            @WebParam(name = "smartCardAgency") String smartCardAgency,
            @WebParam(name = "emailAddress") String emailAddress) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        UserDao provDao = new UserDao();
        IehrUserProfile rtc = provDao.getIehrUserProfile(smartCardID, smartCardAgency, emailAddress);
        // if (rtc != null) {
        logResponse(null, begin, "", 1);
//            } else {
//                logResponse(null, begin, "", 0);
//            }
        return rtc;
    }

    /**
     * Sets an iEHR user's profile back into the sql server database
     *
     * @param userProfile Allergy form options request object.
     * @return True if operation was successful, otherwise false.
     * @throws JMeadowsException if a serious error occurs
     */
    @WebMethod(operationName = "setIehrUserProfile")
    public boolean setIehrUserProfile(
            @WebParam(name = "iehrUserProfile") IehrUserProfile userProfile) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        UserDao provDao = new UserDao();
        boolean rtc = provDao.setIehrUserProfile(userProfile);
        logResponse(null, begin, "", 0);
        return rtc;
    }

    /**
     * Retrieves VA/MTF Site map. Map is keyed by VA/MTF SiteCode.
     * @return VA/MTF Site map.
     */
    @WebMethod(operationName = "getSites")
    public List<Site> getSites() {
        long begin = System.currentTimeMillis();
        List<Site> rtc = Sites.getInstance().getSiteList();
        logResponse(null, begin, "", rtc);
        return rtc;
    }

    /**
     * Retrieves last unsuccessful and successful login audit entries
     * @return List of audit entries.
     */
    @WebMethod(operationName = "getLoginInfo")
    public List<AuditEntry> getLoginInfo(
            @WebParam(name = "cardID") String cardID,
            @WebParam(name = "emailAddress") String emailAddress) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            UserDao provDao = new UserDao();
            List<AuditEntry> rtc = provDao.getLoginInfo(cardID, emailAddress);
            logResponse(null, begin, "", rtc);
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(null, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }

    /**
     * Retrieves the status information of JLV's external services
     * @return List of service statuses
     */
    @WebMethod(operationName = "getServiceErrors")
    public String getServiceErrors() {
        long begin = System.currentTimeMillis();
        String rtc = ServiceErrors.getInstance().getServiceErrors();
        logResponse(null, begin, "", 0);
        return rtc;
    }

    /**
     * Retrieves authorized user information
     * @param authUserInfoQuery Requires userId and organization
     * @return Authorized user info result
     * @throws JMeadowsException If there is an exception
     */
    @WebMethod(operationName = "getAuthUser")
    public AuthUserInfo getAuthUser(
            @WebParam(name = "authUserInfoQuery") AuthUserInfo authUserInfoQuery) throws JMeadowsException {
        AuthUserInfo rtc = null;
        long begin = System.currentTimeMillis();
        try {
            if ("DOD".equalsIgnoreCase(authUserInfoQuery.getOrganization()) && authUserInfoQuery.isSnareworksFlag()) {
                rtc = new AuthUserInfo();
                rtc.setSnareworksFlag(true);
                try {
                    AuthUserDao authUserDao = new AuthUserDao();
                    SnareUser suser = authUserDao.getSnareworksUser(authUserInfoQuery.getCertificate(), null, null);

                    if (suser != null && suser.isValid()) {
                        logResponse(null, begin, "", 1);
                    } else {
                        logResponse(null, begin, "", 0);
                    }

                    if (suser != null) {
                        if ("No Snare Servers available".equalsIgnoreCase(suser.getSnareMessage())) {
                            throw new JMeadowsException("Snareworks is down.");
                        }
                        if (suser.isValid()) {
                            rtc.setCardId(authUserInfoQuery.getCardId());
                            rtc.setCertificate(authUserInfoQuery.getCertificate());
                            rtc.setEmail("");
                            rtc.setFacility("0000");
                            rtc.setName(suser.getCn());
                            rtc.setOrganization("DOD");
                            rtc.setSubjectDN(authUserInfoQuery.getSubjectDN());
                            rtc.setNcid(suser.getNcid());
                            rtc.setSnareworksValid(true);
                        }
                    }
                } catch (JMeadowsException jex) {
                    logError(jex);
                    throw jex;
                } catch (Exception ex) {
                    logError(ex);
                    throw new JMeadowsException("Snareworks is down.");
                }
            } else {
                AuthUserDao authUserDao = new AuthUserDao();
                rtc = authUserDao.getEntry(authUserInfoQuery);
                if (rtc != null) {
                    logResponse(null, begin, "", 1);
                } else {
                    logResponse(null, begin, "", 0);
                }
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(null, begin, ex.getMessage(), 0);
            logError(ex);
            throw new JMeadowsException(ex);
        }
    }

    @WebMethod(operationName = "resetStatus")
    public void resetStatus() {
        JMeadowsStatus.resetStatus();
    }

    @WebMethod(operationName = "getStatus")
    public String getStatus() {
        return JMeadowsStatus.getStatus();
    }


    /**
     * Get user's recently viewed patients
     * @param userId User's database ID in USERS table.
     * @return List of user's recently viewed patients data.
     * @throws JMeadowsException if there is an error.
     */
    @WebMethod(operationName = "getRecentlyViewedPatients")
    public List<Patient> getRecentlyViewedPatients(
            @WebParam(name = "userId") String userId) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            UserDao userDao = new UserDao();
            List<Patient> rtc = userDao.getRecentlyViewedPatients(userId);
//            if (rtc != null) {
//                logResponse(null, begin, "", 1);
//            }
//            else {
//                logResponse(null, begin, "", 0);
//            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(null, begin, ex.getMessage(), 0);
            logError(ex);
            throw new JMeadowsException(ex);
        }
    }

    @WebMethod(operationName = "getPatientDocumentVixStudyViewerUrl")
    public String getPatientDocumentVixStudyViewerUrl(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean,
            @WebParam(name = "document") PatientDocument document) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        VixDao vixDao = new VixDao();
        String studyViewerUrl = vixDao.getPatientDocumentVixStudyViewerUrl(queryBean, document);
        if (studyViewerUrl != null) {
            logResponse(queryBean, begin, "", 1);
        } else {
            logResponse(queryBean, begin, "", 0);
        }
        return studyViewerUrl;
    }

    @WebMethod(operationName = "getEncounterDocumentVixStudyViewerUrl")
    public String getEncounterDocumentVixStudyViewerUrl(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean,
            @WebParam(name = "documentId") String documentId,
            @WebParam(name = "documentSiteId") String documentSiteId) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        VixDao vixDao = new VixDao();
        String studyViewerUrl = vixDao.getEncounterDocumentVixStudyViewerUrl(queryBean, documentId, documentSiteId);
        if (studyViewerUrl != null) {
            logResponse(queryBean, begin, "", 1);
        } else {
            logResponse(queryBean, begin, "", 0);
        }
        return studyViewerUrl;
    }

    @WebMethod(operationName = "getRadiologyVixStudyViewerUrl")
    public String getRadiologyVixStudyViewerUrl(
            @WebParam(name = "queryBean") JMeadowsQuery queryBean,
            @WebParam(name = "radiologyReport") RadiologyReport radiologyReport) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        VixDao vixDao = new VixDao();
        String studyViewerUrl = vixDao.getRadiologyVixStudyViewerUrl(queryBean, radiologyReport);
        if (studyViewerUrl != null) {
            logResponse(queryBean, begin, "", 1);
        } else {
            logResponse(queryBean, begin, "", 0);
        }
        return studyViewerUrl;
    }

//    private void logResponse(long begin, String err, int rcount) {
//        logResponse(null, begin, err, rcount);
//    }

    private void logResponse(JMeadowsQuery queryBean, long begin, String err, List<?> rtc) {
        if (rtc != null) {
            logResponse(queryBean, begin, err, rtc.size());
        } else {
            logResponse(queryBean, begin, err, 0);
        }
    }

    private void logResponse(JMeadowsQuery queryBean, long begin, String err, int rcount) {
        long end = System.currentTimeMillis();
        String patientId = "Patient ID was not set";
        String queryInfo = "";

        try {
            if (queryBean != null && queryBean.getPatient() != null) {
                if(StringUtils.isNotEmpty(queryBean.getPatient().getEDIPI())){
                    patientId = queryBean.getPatient().getEDIPI();
                } else if("ICN".equalsIgnoreCase(queryBean.getStatus())){
                    patientId = queryBean.getItemId();
                }

                queryInfo = getKey(queryBean);
            }
            MessageContext msgctx = wsContext.getMessageContext();
            String operation = "" + msgctx.get(MessageContext.WSDL_OPERATION);
            String[] pieces = operation.split("\\}");
            if (pieces.length > 1) {
                operation = pieces[1];
            }

            String dsource = AppConfig.getInstance().getDataSource();
            ARMLog dbl = new ARMLog(dsource);
            dbl.logEvent(patientId, "jMeadows." + operation, err, end - begin, rcount, "", queryInfo);
            dbl.close();
            boolean error = false;
            if (err != null && err.length() > 0) {
                error = true;
            }
            JMeadowsStatus.updateStatistics(operation, error, end - begin);
        } catch (Exception ex) {
            logError(ex);
        }
    }

    private String getKey(JMeadowsQuery queryBean) {
        StringBuilder rtc = new StringBuilder();
        if (queryBean != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss", Locale.getDefault());
            if(StringUtils.isNotEmpty(queryBean.getPatient().getEDIPI())){
                rtc.append(queryBean.getPatient().getEDIPI());
            } else if("ICN".equalsIgnoreCase(queryBean.getStatus())){
                rtc.append(queryBean.getItemId());
            }

            rtc.append('|');
//            rtc.append(type);
            rtc.append('|');
            if (queryBean.getStartDate() != null) {
                rtc.append(sdf.format(queryBean.getStartDate().getTime()));
            }
            rtc.append('|');
            if (queryBean.getEndDate() != null) {
                rtc.append(sdf.format(queryBean.getEndDate().getTime()));
            }
            rtc.append('|');
            rtc.append(queryBean.getRecordSiteCode());
            rtc.append('|');
            rtc.append(queryBean.getRequestingApp());
        }
        return rtc.toString();
    }

    /**
     * Retrieves the PCEs for the note (TIU ien) from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link JMeadowsQuery} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A list of PCEs.
     * @throws JMeadowsException if a serious error occurs
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    @WebMethod(operationName = "getPcesForNote")
    public List<Pce> getPcesForNote(@WebParam(name = "queryBean") JMeadowsQuery queryBean) throws JMeadowsException {
        long begin = System.currentTimeMillis();
        try {
            NotesDao notesDao = new NotesDao();
            List<Pce> rtc = notesDao.getPcesForNote(queryBean);
            if (rtc != null) {
                logResponse(queryBean, begin, "", 1);
            } else {
                logResponse(queryBean, begin, "", 0);
            }
            return rtc;
        } catch (JMeadowsException ex) {
            logResponse(queryBean, begin, ex.getMessage(), 0);
            logError(ex);
            throw ex;
        }
    }


    private void logError(Exception e){
        LOGGER.error("jMeadows ERROR: " + e.getMessage(), e);
    }
}

